#
# $Id: Email.pm,v 1.52 1999/02/26 22:48:06 duncan Exp $
#
# Written by
#   Will DeHaan  will@cobaltnet.com
#   Kevin Chiu   kevin@cobaltnet.com
#
# Copyright 1998 Cobalt Networks http://www.cobaltnet.com/
#

package Cobalt::Email;
use AutoLoader 'AUTOLOAD';
BEGIN {
require Exporter;
@ISA    = qw(Exporter);
@EXPORT = qw(mail_add_relay
	     mail_get_relay
	     mail_del_relay
	     mail_set_privacy
	     mail_set_AutoAlias
	     mail_set_max_document
	     mail_get_max_document
	     mail_virtuser_add_alias
	     mail_virtuser_del_alias
	     mail_virtuser_add_route
	     mail_virtuser_del_route
	     mail_virtuser_set_byuser
	     mail_virtuser_get_byuser
	     mail_virtuser_del_byuser
	     mail_virtuser_list_alias
	     mail_virtuser_list_route
	     mail_virtuser_change_domain
	     mail_list_cw
	     mail_add_cw
	     mail_del_cw
	     mail_set_masq
	     mail_get_masq
	     mail_set_deny
	     mail_get_deny
	     lock_sendmail
	     mail_build_virtusertable
	     mail_sendmail_hup
	     sendmail_set_delivery_mode
	     sendmail_set_queue_interval
	     sendmail_get_queue_interval
	    );

require Cobalt::Netutil;
import Cobalt::Netutil qw(:DEFAULT);
}

use integer;
#eval("use Cobalt::Product;");
#use strict;

1;
__END__

sub mail_add_relay
# adds a relay to the access database, format:  "host/network/domain RELAY"
# Enables email relaying by host/network/domain only
# Arguments: array of network/domains for whom to add relaying permissions
# Return value: status string of the form "1288 Cheap irony not supported
#   in this release."
{
        my (@new) = @_;
        my ($relay,%exists);
	my ($notend) = 1;

        foreach $relay (@new)
        {
                $exists{$relay} = 1;
        }

        return "4999 email_cant_lock_accessDB" if ( -e "$Lockdir/access" );

        open (DBWRITE, "> $Lockdir/access")
          || return "4999 email_cant_write_accessDB";
        open (DBREAD, "$MailAccessDB")
          || return "4999 email_cant_read_accessDB";

        while (<DBREAD>)
        {
		if ($notend)
		{
	                if (/Mail rejected due to possible SPAM/o)
	                {
			       # not for us...
			       print DBWRITE;
	                }
	                elsif (/^(\S+)\s+RELAY/o)
	                {
			        # there's a relay line here...but it may
			        # be the same as one we're adding
	                        print DBWRITE "$1\tRELAY\n" unless ($exists{$1});
	                }
	                elsif (/^# Put custom additions below/o)
	                {
			        # Now add all our new relay hosts
	                        foreach $relay (@new)
	                        {
	                                print DBWRITE "$relay\tRELAY\n";
	                        }
	                        print DBWRITE "# Put custom additions below (Do not change/remove this line).\n";
	                        $notend = 0;
	                }
			else
			{
				print DBWRITE;
			}
		}
                else
                {
                        print DBWRITE;
                }
        }

        close DBWRITE;
        close DBREAD;

	unless (-w $MailAccessDB) {
	  # For some reason, we can't write to this file...eek!
	  unlink("$Lockdir/access");
	  return "4999 email_cant_write_accessDB";
	}
	    

	# Now that the flat files have been built, make the database file
	print STDERR "$MakeMap_cmd hash $MailAccessDB < $Lockdir/access\n" 
	  if ($Debug);
	system("$MakeMap_cmd hash $MailAccessDB < $Lockdir/access >>$Sauce_log 2>&1")
	  && return "4999 email_cant_buildhash";

	# keep the text version around, too
	rename("$Lockdir/access","$MailAccessDB") 
	  || return "4999 email_cant_unlock_accessDB";

	# notify sendmail of the change.
	Cobalt::Email::mail_sendmail_hup();

	return "2999 email_ok_relay_on"
}

sub mail_get_relay
# Obtains the networks/domains allowed to use sendmail to relay mail
# Arguments: none
# Return value: array of networks/domains
{
        my (@relays);
	my ($notend) = 1;
        if (-r $MailAccessDB)
        {
                open (ACCESSDB,"$MailAccessDB")
                  || return "4999 email_cant_read_accessDB";
                while (<ACCESSDB>)
                {
                        chomp ($_);
			if ($notend)
			{
				if ( /(\S+)\s+RELAY/o )
				{
					push (@relays, $1);
				}
                        	elsif ( /^# Put custom additions below/o )
                        	{
					$notend = 0;
                        	}
			}
                }
                close (ACCESSDB);
        }
        return(@relays);
}

sub mail_del_relay
# Removes networks/domains from sendmail's ip_allow and name_allow files
# Disables  email relaying by network/domain only
# Arguments: array of networks/domains
# Return value: string of the form "3100 IiinnNPPuuuTtt"
{
        my (@del) = @_;
        my ($relay,%exists);
        my ($notend) = 1;

        foreach $relay (@del)
        {
                $exists{$relay} = 1;
        }

        return "4999 email_cant_lock_accessDB" if ( -e "$Lockdir/access" );

        open (DBWRITE, "> $Lockdir/access")
          || return "4999 email_cant_write_accessDB";
        open (DBREAD, "$MailAccessDB")
          || return "4999 email_cant_read_accessDB";

        while (<DBREAD>)
        {
		if ($notend)
		{
	                if (/Mail rejected due to possible SPAM/o && $notend)
	                {
	                        print DBWRITE;
	                }
	                elsif (/^(\S+)\s+RELAY/o)
	                {
	                        print DBWRITE "$1\tRELAY\n" unless ($exists{"$1"});
	                }
	                elsif (/^# Put custom additions below/o)
	                {
	                        print DBWRITE "# Put custom additions below (Do not change/remove this line).\n";
	                        $notend = 0;
	                }
			else
			{
				print DBWRITE;
			}
		}
                else
                {
                        print DBWRITE;
                }
        }

        close DBWRITE;
        close DBREAD;

	unless (-w $MailAccessDB) {
	  unlink("$Lockdir/access");
	  return "4999 email_cant_write_accessDB";
	}

	# Build the database version of the access database
        print STDERR "$MakeMap_cmd hash $MailAccessDB < $Lockdir/access" if ($Debug);
        system("$MakeMap_cmd hash $MailAccessDB < $Lockdir/access >>$Sauce_log 2>&1")
          && return "4999 email_cant_buildhash";

	# keep the text version, too
        rename("$Lockdir/access", "$MailAccessDB") || return "4999 email_cant_unlock_accessDB";

        Cobalt::Email::mail_sendmail_hup();

	return "2999 email_ok_relays_removed";
}

sub mail_set_privacy
# Enables/Disables both expn and vrfy smtp commands in /etc/sendmail.cf
# Arguments: 1 to enable, 0 to disable
# Return value: string like "4000 No.  Not now.  Not ever."
# Site effects: edits /etc/sendmail.cf
{
	my($privacies) = $_[0];
	return "4999 email_cant_sendmailcf_locked" if (-e "$Lockdir/sendmail.cf");
	open(TMPCF,"> $Lockdir/sendmail.cf") || return "4999 email_cant_open_sendmailcf";
	open(SNDCF,"$Sendmail_conf")
	  || return "4999 email_cant_read_sendmailcf";
	while(<SNDCF>) 
	{
		if(/O PrivacyOptions=/o)
		{
			print TMPCF "O PrivacyOptions=authwarnings,noexpn,novrfy\n" if($privacies);
			print TMPCF "O PrivacyOptions=authwarnings\n" unless($privacies);
		} else {
			print TMPCF;
		}
	}
	close(SNDCF);
	close(TMPCF);

	rename("$Lockdir/sendmail.cf","$Sendmail_conf") || return "4999 email_cant_write_sendmailcf";

	return "2999 email_ok_privacy_set" if ($privacies);
	return "2999 email_ok_privacy_off" unless ($privacies);
}

sub mail_set_AutoAlias
# Enables/Disables AutoRebuildAliases in /etc/sendmail.cf
# Arguments: 1 to enable, 0 to disable
# Return value: string like "2000 I am starting an ISP and.."
# Site effects: edits /etc/sendmail.cf
{
	my($auto_rebuild) = $_[0];
	return "4999 email_cant_sendmailcf_locked" if (-e "$Lockdir/sendmail.cf");
	open(TMPCF,"> $Lockdir/sendmail.cf") || return "4999 email_cant_open_sendmailcf";
	open(SNDCF,"$Sendmail_conf")
	  || return "4999 email_cant_read_sendmailcf";
	while(<SNDCF>) 
	{
		if(/AutoRebuildAliases/)
		{
			print TMPCF "O AutoRebuildAliases\n" if($auto_rebuild);
			print TMPCF "#O AutoRebuildAliases\n" unless($auto_rebuild);
		} else {
			print TMPCF;
		}
	}
	close(SNDCF);
	close(TMPCF);

	rename("$Lockdir/sendmail.cf","$Sendmail_conf") || return "4999 email_cant_write_sendmailcf";

	return "2999 email_ok_autorebuild_set" if ($auto_rebuild);
	return "2999 email_ok_autorebuild_off" unless ($auto_rebuild);
}

sub mail_set_max_document
# Sets the maximum document size in bytes that sendmail will accept
# Arguments: natural byte value, 0 for unlimited
# Return value: string like "2000 It does taste like butter"
# Site effects: edits /etc/sendmail.cf
{
        # local variables should be lowercased for consistency (and for
        # the preprocessor, unfortunately...)
	my($max_doc_size) = $_[0];
	return "4999 email_invalid_max_doc_size" if ($max_doc_size < 0);
	return "4999 email_cant_sendmail_not_configured" unless (-e "$Sendmail_conf");
	return "4999 email_cant_sendmailcf_locked" if (-e "$Lockdir/sendmail.cf");
	open(TMPCF,"> $Lockdir/sendmail.cf") || return "4999 email_cant_open_sendmailcf";
	open(SNDCF,"$Sendmail_conf")
	  || return "4999 email_cant_read_sendmailcf";
	while(<SNDCF>) 
	{
		if(/MaxMessageSize/o)
		{
			print TMPCF "O MaxMessageSize=$max_doc_size\n" if($max_doc_size);
			print TMPCF "#O MaxMessageSize=1000000\n" if(!$max_doc_size);
		} else {
			print TMPCF;
		}
	}
	close(SNDCF);
	close(TMPCF);

	rename("$Lockdir/sendmail.cf","$Sendmail_conf") || return "4999 email_cant_write_sendmailcf";

	Cobalt::Email::mail_sendmail_hup();

	return "2999 email_ok_max_doc_set";
}

sub mail_get_max_document
# Gets the maximum document size in bytes that sendmail will accept
# Arguments: none
# Return value: natural byte value, 0 for unlimited
{
	return "4999 email_cant_sendmail_not_configured" unless (-e "$Sendmail_conf");
	open(SNDCF,"$Sendmail_conf")
	  || return "4999 email_cant_read_sendmailcf";
	while(<SNDCF>) 
	{
		if(/^O MaxMessageSize=(\d+)/)
		{
			return $1;
		} elsif (/^#O MaxMessageSize/) {
			return 0;
		}
	}
	return 0;
	close SNDCF;
}

sub mail_virtuser_add_alias
# Adds a hash of login name and fully qualified email addresses or @domains
# Arguments: hash  (alias => user)
# Return value: string like "2000 It's happy hour"
# Side effects: Rebuilds /etc/virtusertable.db from modified /etc/virtusertable
{
	my(%add_alias) = @_;
	
	my(%aliases) = Cobalt::Email::mail_virtuser_list_alias();
	my(%routes) = Cobalt::Email::mail_virtuser_list_route();

	my($alias);
	foreach $alias (keys %add_alias)
	{
	    if( $aliases{ $alias } ) {
		return "4999 email_cant_alias_used";
	    }
	    $aliases{"$alias"} = $add_alias{"$alias"}; # No duplicate aliases allowed
	}
	
	Cobalt::Email::mail_build_virtusertable(\%aliases, \%routes)
	    || return "4999 email_cant_modify_list";

	return "2999 email_ok_virtusers_added";
}

sub mail_virtuser_del_alias
# Deletes a hash of login name and fully qualified email addresses or @domains
# Arguments: hash  (alias => user)
# Return value: string like "2000 Mmm, smells like BBQ and beer"
# Side effects: Rebuilds /etc/virtusertable.db from modified /etc/virtusertable
{
	my(%delAlias) = @_;
	
	my(%aliases) = Cobalt::Email::mail_virtuser_list_alias();
	my(%routes) = Cobalt::Email::mail_virtuser_list_route();

	my($alias);

	foreach $alias (keys %delAlias)
	{
	    delete $aliases{$alias};
	}

	Cobalt::Email::mail_build_virtusertable(\%aliases, \%routes)
	    || return "4999 email_cant_modify_list";

	return "2999 email_ok_virtusers_removed";
}

sub mail_virtuser_add_route
# Adds a hash of "domain" and "fqdn to route to"
# Arguments: hash  (domain => fqdn to route to)
# Return value: string like "2000 Itchy and scratchy"
# Side effects: Rebuilds /etc/virtusertable.db from modified /etc/virtusertable
{
	my(%add_route) = @_;
	
	my(%aliases) = Cobalt::Email::mail_virtuser_list_alias();
	my(%routes) = Cobalt::Email::mail_virtuser_list_route();

	my($r);
	foreach $r (keys %add_route)
	{
	    if( $routes{ $r } ) {
		return "4999 email_cant_route_used";
	    }
	    $routes{"$r"} = $add_route{"$r"}; # No duplicate aliases allowed
	}
	
	Cobalt::Email::mail_build_virtusertable(\%aliases, \%routes)
	    || return "4999 email_cant_modify_list";

	return "2999 email_ok_virtusers_added";
}

sub mail_virtuser_del_route
# Deletes a hash of "domain" and "fqdn to route to"
# Arguments: hash  (domain => fqdn to route to)
# Return value: string like "2000 Mmm.. forbidden dounut..."
# Side effects: Rebuilds /etc/virtusertable.db from modified /etc/virtusertable
{
	my(%del_route) = @_;
	
	my(%aliases) = Cobalt::Email::mail_virtuser_list_alias();
	my(%routes) = Cobalt::Email::mail_virtuser_list_route();
	my($r);

	foreach $r (keys %del_route)
	{
	    delete $routes{$r};
	}

	Cobalt::Email::mail_build_virtusertable(\%aliases, \%routes)
	    || return "4999 email_cant_modify_list";

	return "2999 email_ok_virtusers_removed";
}

sub mail_virtuser_set_byuser
# Adds/Removes aliases as necessary
# Arguments: username, array of aliases
# Return value: string of the form "4200 Intellect overflow line 0"
{
	my($username,@alii) = @_;
	my($alias,%newalii);
	foreach $alias (@alii) { $newalii{"$alias"} = $username; }
	
	my(@oldalias) = Cobalt::Email::mail_virtuser_get_byuser($username);
	my($alias,%oldalii);
	foreach $alias (@oldalias) { $oldalii{"$alias"} = $username; }

	my($ret) = Cobalt::Email::mail_virtuser_del_alias(%oldalii);
	return $ret if ($ret !~ /^2/o);

	my($ret) = Cobalt::Email::mail_virtuser_add_alias(%newalii);
	if ($ret !~ /^2/o) {
	    # try to restore to the initial value
	    # this is NOT guaranteed to work, though
	    Cobalt::Email::mail_virtuser_add_alias(%oldalii);

	    return $ret;
	}

	return "2999 email_ok_virtalias_updated";
}

sub mail_virtuser_get_byuser
# Parses a list of aliases a user owns
# Arguments: username
# Return value: Array of aliases
{
	my($username) = @_;
	my(%virtuser) = Cobalt::Email::mail_virtuser_list_alias();
	my($alias,$user,@alii);
	foreach $alias (keys %virtuser)
	{
		push(@alii,$alias) if ($virtuser{"$alias"} eq $username);
	}
	return(@alii);
}

sub mail_virtuser_del_byuser
# Sames as VirtuserSetByUser with no aliases specified
#   This will iterate VirtuserSetByUser calls when there are multiple arguments
# Arguments: a username or an array of usernames
# Return value: String like usual
{
	my(@username) = @_;
	my($user);
	foreach $user (@username) 
	{
		my($ret) = Cobalt::Email::mail_virtuser_set_byuser($user);
		return $ret if ($ret !~ /^2/);
	}
	return "2999 email_ok_virtaliase_removed";
}

sub mail_virtuser_list_alias
# Returns a hash of fully qualified email addresses or @domains and target user names or error messages
# Arguments: none
# Return value: an alias hash (alias => user)
# Example: my(%aliases) = Cobalt::Email::mail_virtuser_list_alias();
{
	my(%aliases); 

	sleep 2 if (-e "$Lockdir/virtusertable");
	
	return "4999 email_cant_list_locked" 
	  if (-e "$Lockdir/virtusertable");
	
	open(VRT,"$EmailVirtUser") || return ''; # Null return if no flat file
	while(<VRT>) 
	{
            if( /custom additions below/o ) { last; }
	    chop;
	    next if (/^\s*\#/o);
	    next if (/^\s*\@\S+\s+\%1\@\S+/o); # skip accept-email-at-domain routes
	    $aliases{"$1"} = $2 if (/^\s*(\S+)\s*(.*)$/o); 
	}
	close (VRT);

	return(%aliases);
}

sub mail_virtuser_list_route
# Returns a hash of fully qualified "domain" and target "fqdn" routes, or error messages
# Arguments: none
# Return value: a routes hash (domain => fqdn to route to)
# Example: my(%routes) = Cobalt::Email::mail_virtuser_list_route();
{
	my(%routes); 

	sleep 2 if (-e "$Lockdir/virtusertable");
	
	return "4999 email_cant_list_locked" 
	  if (-e "$Lockdir/virtusertable");
	
	open(VRT,"$EmailVirtUser") || return ''; # Null return if no flat file
	while(<VRT>) 
	{
            if( /custom additions below/o ) { last; }
	    chop;
	    next if (/^\s*\#/o);
	    next unless (/^\@/o);
	    $routes{$1} = $2 if (/^\s*\@(\S+)\s+\%1\@(.*)$/o);
	}
	close (VRT);

	return(%routes);
}

sub mail_virtuser_change_domain
# Changes email alias domains for when any site host or domain name changes
# Arguments: old qualified hostname, new qualified host name
#              or, hash of (old qualified hostname -> new qualified host name)
# Return value: string of the regular sort
# Rebuilds /etc/virtusertable
{
        my(%aliasdelta) = @_;
        my($alias,$route,@splut,$foundModification);

        my(%Alii) = Cobalt::Email::mail_virtuser_list_alias();
	my(%routes) = Cobalt::Email::mail_virtuser_list_route();

        foreach $alias (keys %Alii)
        {
                next unless ($alias =~ /\@/o); # Not willing to mess with custom entries

                @splut = split('@',$alias); # (alias name,alias domain)
                if ($aliasdelta{ $splut[1] })
                {
                        $splut[1] = $aliasdelta{ $splut[1] };
                        $Alii{ join('@',@splut) } = $Alii{ $alias };
                        delete $Alii{$alias}; # Delete old record
                        $foundModification = 1;
                }
        }

        if ($foundModification) # Rebuild aliases
        {
                Cobalt::Email::mail_build_virtusertable(\%Alii, \%routes) || return "4999 email_cant_modify_list";
        }


        return "2999 email_ok_virtusers_modified";
}

sub mail_list_cw
# Reports a sorted list of hostnames that sendmail is willing to accept mail to
# Arguments: none
# Return value: sorted array of domain/host names
# Example: my(@sendmail_server_aliases) = sendmail_cw_list;
{
    my @aliases;

    if( -r $Sendmail_cw ) {
	open( SND, "$Sendmail_cw" ) || 
	  return "4999 email_cant_open_sendmailcw";
	while( <SND> ) {
	    if( /include all aliases for your machine here/o ) { last; }
	    if( /^\s*\#/o ) { next; }
	    if( /^$/o ) { next; }

	    chomp;  # chomp is safer than chop
	    push( @aliases, $_ );
	}
	close SND;
    }

    return sort( @aliases );
}

sub mail_add_cw 
# Adds alias(s) to sendmail.cw, unless it's already defined
# Arguments: an array of host/domain names
# Return value: a string like "2000 I can't believe it's not butter"
# Example: &good_stuff if (Sendmail_cw_add('mail.com','me.mail.com'));
# Note: Does NOT restart sendmail
{
    my @additions=@_;
    my @aliases=Cobalt::Email::mail_list_cw();

    # build hash
    my $alias;
    my %aliasesHash;
    foreach $alias ( @aliases ) { $aliasesHash{ $alias }=1; }
    foreach $alias ( @additions ) {
	if( !Cobalt::Netutil::netutil_test_domainname( $alias ) ) {
	    my $name=$alias;
	    return "4999 email_invalid_hostname";
	}

	$aliasesHash{ $alias }=1;
    }

    Cobalt::Email::lock_sendmail(1) || return "4999 email_cant_list_locked";

    # read in user additions
    my $userArea;
    my $userAdditions;
    open( SND, "<$Sendmail_cw" ) || 
      return "4999 email_cant_open_sendmailcw";
    while( <SND> ) {
	if( /include all aliases for your machine here/ ) { $userArea=1; }
	if( $userArea ) { $userAdditions.=$_; }
    }
    close( SND );

    open( SND, ">$Sendmail_cw" ) || 
      return "4999 email_cant_open_sendmailcw";
    foreach $alias ( keys %aliasesHash ) { print SND "$alias\n"; }
    print SND "\n$userAdditions";
    close( SND );

    Cobalt::Email::lock_sendmail(0);

    Cobalt::Email::mail_sendmail_hup();

    return "2999 email_ok_server_added";
}

sub mail_del_cw 
# Removes alias(s) from sendmail.cw
# Arguments: an array of host/domain names
# Return value: a string like "4000 I smell smoke"
# Example: &good_stuff if (Sendmail_cw_del('nomore.mail.com','less.mail.isp'));
# Note: Does NOT restart sendmail
{
    my @subtractions=@_;

    # build hash
    my %aliasesHash;
    my $alias;
    my @aliases=Cobalt::Email::mail_list_cw();
    foreach $alias ( @aliases ) { $aliasesHash{ $alias }=1; }
    foreach $alias ( @subtractions ) { delete $aliasesHash{ $alias }; }

    foreach $alias ( keys %aliasesHash ) {
	if( !Cobalt::Netutil::netutil_test_domainname( $alias ) ) {
	    my $name=$alias;
	    return "4999 email_invalid_hostname";
	}
    }

    Cobalt::Email::lock_sendmail(1) || return "4999 email_cant_list_locked";

    my $userArea=0;
    my $userAdditions;
    open( SND, "<$Sendmail_cw" ) || 
      return "4999 email_cant_open_sendmailcw";
    while( <SND> ) {
	if( /include all aliases for your machine here/ ) { $userArea=1; }
	if( $userArea ) { $userAdditions.= $_; }
    }
    close( SND );

    open( SND, ">$Sendmail_cw" ) || 
      return "4999 email_cant_write_sendmailcw";
    foreach $alias ( keys %aliasesHash ) { print SND "$alias\n"; }
    print SND "\n$userAdditions";
    close( SND );

    Cobalt::Email::lock_sendmail(0);

    Cobalt::Email::mail_sendmail_hup();

    return "2999 email_ok_server_removed";	
}

sub mail_set_masq
# Set the domain that we want to masquerade as
# Arguments: domain name, or blank for no masquerading
# Side effects: edits sendmail.cf, restarts sendmail
# Return value: status string of the form "2743 Fly, monkeys, fly!"
# Example: Cobalt::Email::mail_set_masq("cobaltnet.com");
{
    my ($masqas) = @_;
    $masqas ||= '';
    return "4999 email_cant_sendmail_locked"
	if (-e "$Lockdir/sendmail.cf");
    unless (open(SO,">$Lockdir/sendmail.cf"))
    {
	return "4999 email_cant_lock_sendmailcf";
    }
    unless (open(SI,$Sendmail_conf))
    {
	unlink("$Lockdir/sendmail.cf");
	return "4999 email_cant_read_sendmailcf";
    }
    while (<SI>)
    {
	if (/^DM/o)
	{
	    print SO "DM$masqas\n";
	}
	else
	{
	    print SO;
	}
    }
    close SI;
    close SO;
    unless (rename("$Lockdir/sendmail.cf",$Sendmail_conf))
    {
	unlink("$Lockdir/sendmail.cf");
	return "4999 email_cant_read_sendmailcf";
    }
    return "2999 email_ok_set_masq";
}

sub mail_get_masq
# Get the name of the domain we masquerade as
# Arguments: none
# Return value: a domain name, or blank for no masquerading
# Side effects: none
# Example: my $domain = get_mail_masq();
{
    my $masqas = '';
    unless (open(SI,$Sendmail_conf))
    {
	return "4999 email_cant_read_sendmailcf";
    }
    while (<SI>)
    {
	if (/^DM(.*)/o)
	{
	    $masqas = $1;
	    last;
	}
    }
    close SI;
    return $masqas;
}

sub mail_set_deny
# Set the list of domains we refuse to accept mail from
# Arguments: list of domain or host names
# Side effects: modifies /etc/mail/deny and rebuilds deny.db
# Example: Cobalt::Email::mail_set_deny("cybersell.com","owlsnest.com");
{
        my (@denys) = @_;
	my ($notend) = 1;
	my ($i);

	# check for existing lockfile, bail if one found
        return "4999 email_cant_lock_accessDB" if ( -e "$Lockdir/access" );

	# open the lockfile for writing, the real file for reading
	open (DBREAD, "$MailAccessDB")
	  || return "4999 email_cant_read_accessDB";
        open (DBWRITE, "> $Lockdir/access")
          || return "4999 email_cant_write_accessDB";

	while (<DBREAD>)
	{
		if ($notend)
		{
			if (/Mail rejected due to possible SPAM/o)
			{
			        # clear out all existing denies
				next;
			}
			elsif (/^# Put custom additions below/o)
			{
				foreach $i (@denys)
				{
					print DBWRITE "$i\t550 Mail rejected due to possible SPAM\n";
				}
				print DBWRITE "# Put custom additions below (Do not change/remove this line).\n";
				$notend = 0;
			}
			else
			{
				print DBWRITE;
			}
		}
		else
		{
			print DBWRITE;
		}
	}

	close (DBREAD);
	close (DBWRITE);

	unless (-w $MailAccessDB) {
	  unlink("$Lockdir/access");
	  return "4999 email_cant_write_accessDB";
	}

        print STDERR "$MakeMap_cmd hash $MailAccessDB < $Lockdir/access" if ($Debug);
        system("$MakeMap_cmd hash $MailAccessDB < $Lockdir/access >>$Sauce_log 2>&1")
          && return "4999 email_cant_buildhash";

        rename("$Lockdir/access","$MailAccessDB") || return "4999 email_cant_unlock_accessDB";

        Cobalt::Email::mail_sendmail_hup();

	return "2999 email_ok_set_blocking";
}

sub mail_get_deny
# Get the list of site from which we refuse mail
# Arguments: none
# Return value: a list of sites we block
# Example: @spammers = Cobalt::Email::mail_get_deny();
{
        my (@denys);
	
        if (-r $MailAccessDB)
        {
                open (ACCESSDB,"$MailAccessDB")
                  || return "4999 email_cant_read_accessDB";
                while (<ACCESSDB>)
                {
		  if ( /(\S+)\s+550 Mail rejected/o )
		  {
		    push(@denys,$1);
		  }
		  elsif ( /^\# Put custom additions below/o )
		  {
		    # leave the loop
		    last;
		  }
                }
                close (ACCESSDB);
        }
        return(@denys);
}

# = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =


sub lock_sendmail
# Arguments: 1 to lock, 0 to unlock
# Return Value: 1 for success, 0 for failure
{
        my($state) = $_[0];
        if ($state)
        {
        return 0 if (-e "$Lockdir/sendmail.cw.lock"); # Already in use
        open(LOCK,"> $Lockdir/sendmail.cw.lock") || return 0;
	print LOCK "$$\n$0"; 
	close(LOCK);
        return 1 if (-e "$Lockdir/sendmail.cw.lock");
        } else {
        unlink("$Lockdir/sendmail.cw.lock") if (-e "$Lockdir/sendmail.cw.lock");
        return 0 if (-e "$Lockdir/sendmail.cw.lock");
        return 1;
        }
}



sub mail_build_virtusertable
# Build the virtusertable database based on the entries in the
# virtusertable file 
# Arguments: 
#       1. a set of aliases to put in the table, in the form
#          of a reference to a hash table, $aliasname => aliasto
#       2. a set of domains and their fqdn routes to put in the
#          table, in the form of a reference to a hash table, $domain => fqdnroute
# Rewrites the virtusertable with whatever aliases it is given as an
#   argument, plus whatever it finds below the "custom additions below"
#   lines at the bottom.
# Return value: 0 on failure, 1 on success
# Side effects: little green men on acid
{
	my($aliases,$routes) = @_;
	my($table_data,$key,$val,$end);
	my($lockfile) = "$Lockdir/virtusertable";

                                                # do the aliases
	while (($key,$val) = each %$aliases) {
	    $table_data .= "$key\t$val\n";
	}

	$table_data .= "\n# accept-email-at-domain routes\n";

	                                        # do the routes
	while (($key,$val) = each %$routes) {
	    $table_data .= "\@".$key."\t\%1\@".$val."\n";
	}

	sleep 2 if (-e $lockfile );             # make an attempt to wait for it
        return 0 if ( -e $lockfile );           # give up

        open( DBWRITE, "> $lockfile" )
	    || return 0;
        open( DBREAD, "$EmailVirtUser" )
	    || return 0;

	print DBWRITE "# This file is automatically generated\n";
	print DBWRITE "# Please put custom changes at the end\n";
	print DBWRITE "# Cobalt Networks 1998\n\n";
	print DBWRITE "$table_data\n";

	                                        # handle custom lines
        while (<DBREAD>) {
	    unless($end) {
		if (/^# Put custom additions below/o) {
		    print DBWRITE;
		    $end = 1;
		}
	    } else {
		print DBWRITE;
	    }
	}
        close DBWRITE;
        close DBREAD;

	                                        # we can't write to this file...eek!
	unless (-w $EmailVirtUser) {
	  unlink("$lockfile");
	  return 0;
	}
	    
	                                        # build the database, save the text
	system("$MakeMap_cmd hash $EmailVirtUser.db < $lockfile >>$Sauce_log 2>&1")
	    && return 0;
	rename("$lockfile","$EmailVirtUser") 
	    || return 0;

	                                        # if we've come this far...
	return 1;
}


sub mail_sendmail_hup
# Sends sendmail a HUP so it rereads its configuration
# Arguments: none
# Side effects: kill -HUP's sendmail
# Return value: none
{
  return if (! -e $Sendmail_pidfile);  # if it's not running, don't HUP...
    open( PID, "<$Sendmail_pidfile" ) || return;
    my $pid=<PID>;
    close( PID );

    chomp $pid;
    kill 'HUP', $pid if ($pid);
}


sub sendmail_set_delivery_mode
# sets the delivery mode of sendmail
#
# Arguments: $mode = (i)nteractive, (b)background, (q)ueue 
{
  my($mode) = shift;
  
  my %delivery_modes = (
			'i' => 'interactive',
			'interactive' => 'interactive',
			'b' => 'background',
			'background' => 'background',
			'q' => 'queue',
			'queue' => 'queue'
		       );
  
  return "4999 email_cant_invalid_mode" if ( ! $delivery_modes{$mode});
  
  return "4999 email_cant_sendmailcf_locked" if (-e "$Lockdir/sendmail.cf");
  open(TMPCF,"> $Lockdir/sendmail.cf") || return "4999 email_cant_open_sendmailcf";
  open(SNDCF,"$Sendmail_conf")
    || return "4999 email_cant_read_sendmailcf";
  while(<SNDCF>) 
    {
      if(/O DeliveryMode=/o)
	{
	  print TMPCF "O DeliveryMode=$delivery_modes{$mode}\n";
	} else {
	  print TMPCF;
	}
    }
  close(SNDCF);
  close(TMPCF);
  
  rename("$Lockdir/sendmail.cf","$Sendmail_conf") || return "4999 email_cant_write_sendmailcf";
  
  return "2999 email_ok_deliverymode_set";
}

sub sendmail_set_queue_interval
# sets the sendmail queue flushing interval by setting a symbolic link
# from the correct /etc/cron.* directory to the flush_sendmail_queue script.
# we flush the sendmail queue this way rather than letting sendmail do it
# to synchronize the flushing with the fetchmail retrieval
# args: $interval = fetchmail interval
{
  my ($interval) = shift;
  my $crondir;

  my $found = 0;
  
  # if $interval is 'never' set $found = 1
  # this causes any existing interval to be removed
  $found = 1 if ($interval eq 'never');

  foreach $crondir (@Cron_Intervals) {
    # sanity check to make sure not more than one symlink exists
    if (-e "$Cron_basedir$crondir/flush_sendmail_queue") {
      unlink("$Cron_basedir$crondir/flush_sendmail_queue");
    }
    if ($interval eq $crondir) {
      symlink($Sendmail_flush_script, "$Cron_basedir$crondir/flush_sendmail_queue");
      $found = 1;
    }
  }

  if ($found) {
    return "2999 email_ok_set_interval";
  } else {
    return "4999 email_cant_unsupported_interval";
  }
}


sub sendmail_get_queue_interval
# retrieves the fetchmail frequency by checking for a symbolic link in each
# of the /etc/cron.* directories.
# args: none
# returns the string name of the cron interval, or 'never' if not yet set
{
  my $found = 'never';
  my $crondir;

  foreach $crondir (@Cron_Intervals) {
    if (-e "$Cron_basedir$crondir/flush_sendmail_queue") {
      return $crondir;
    }
  }

  return $found;
}


1;

