#
# $Id: Fpx.pm,v 1.14 1999/03/12 21:56:06 will Exp $
#
# Written by Will DeHaan <will@cobaltnet.com>
#
# Copyright 1999 Cobalt Networks http://www.cobaltnet.com/
#

package Cobalt::Fpx;
use AutoLoader 'AUTOLOAD';
BEGIN {
require Exporter;
@ISA    = qw(Exporter);
@EXPORT = qw(fpx_set_web           fpx_get_web
             fpx_hose_apacheconf   fpx_recurse_chmod
             fpx_rewrite_htaccess
	    );

require File::Copy;
import File::Copy qw(move);
require Cobalt::Language;
import Cobalt::Language qw(:DEFAULT);
require Cobalt::User;
import Cobalt::User qw(user_exist);
require Cobalt::Vsite;
import Cobalt::Vsite qw(vsite_list vsite_get_bygroup vsite_getgroup vsite_get_fpx vsite_set_fpx);
}

1;
__END__

sub fpx_set_web
# Enable or disable FrontPage Server Extensions 
# Arguments: group or username, 1 for on/0 for off
# Return value: a status string of the usual form form "2534 ho hum" 
# Side effects: Calls MS's fpsrvadm.exe, modifies /etc/htpasswd using Vsite.pm, 
#
# Complications:
# fpadmsrv.exe expects an Apache configuration syntax that's wrong.
# M$ doesn't respect server syntax, fpadmsrv.exe is a joke.
# We're generating an alternate [fp98.]httpd.conf that would
# never work with Apache but is laid out how FPX would like
# it to be.  The silver lining is that we never let fpadmsrv.exe 
# modify any Apache configuration file. (See vsite_set/get_fpx) 
#
# Examples: $good = Cobalt::Fpx::fpx_set_web($group,0);
#            $bad = Cobalt::Fpx::fpx_set_web($user,1);
{
  my($group,$state)=@_; # $group may also be a user

  return "1999 fpx_invalid_unsupported" unless ($Features{'fpx'});
  return "4999 fpx_cant_fpxnotavail" unless (-x $Fpx_admsrvexe);

  # Current state
  my $fpx_active = fpx_get_web($group); 
  return "2999 fpx_ok_alreadyon" if ($state && $fpx_active);
  return "2999 fpx_ok_alreadyoff" unless ($state || $fpx_active);
  
  # Test user vs group/site
  my($user,$could_not_install);
  if(user_exist($group))  ### User web
  {
    $user=$group; # Clean variable names
    $group='';

    my($site)=vsite_getgroup($user);
    return "4999 fpx_invalid_baduser" unless ($site);

    # No user webs unless the site has a root web
    return "4999 fpx_cant_sitedisabled" unless(fpx_get_web($site));

    # fpadmsrv.exe matches $siteName in httpd.conf
    my($siteName)=(vsite_get_bygroup($site))[1];

    my($user_home,$user_uid)=(getpwnam($user))[7,2];

    if ($state) # Enable
    {
      # Rename .htaccess files so fpadmsrv.exe doesn't die from confusion
      my(@htax) = fpx_rename_htaccess("$user_home/$Webdir");

      # FPX user web won't install without a dummy .htaccess file sitting
      #   in the MAIN SITE WEB ROOT.  'Guess nobody at Microsoft went to 
      #   sysadmin school.  Learned to program from a lawyer maybe?
      my($redmond)=("$Userdir/$site/$Webdir/.htaccess");
      if (-e $redmond) 
      {
        $redmond=0;
      } else
      {
        open(MSBS,">$redmond"); # touch 
        close(MSBS); 
      }

      # Call fpadmsrv.exe (ugh)
      system("$Fpx_admsrvexe -o install -p 80 -t apache -m \"$siteName\" -w \"\~$user\" -u \"$user\" -g \"$site\" -pw \"$$\" -s \"$Htconf/fp98.httpd.conf\" -xu \"$user\" -xg \"$site\" > /dev/null") && ($could_not_install = 1);

      # ignore anything that fpx did to this file
      unlink($redmond) if ($redmond);  

      # FPX creates a bunch of .htaccess files that need to be re-written for
      # pam/shadow support.. So, find the new .htaccess files and edit each one
      # We do this before restoring original .htaccess files
      fpx_rewrite_htaccess((getpwnam($user))[7]."/$Webdir",'user',"$user $Adminuser");

      fpx_restore_htaccess((getpwnam($user))[7]."/$Webdir",@htax); # Fix what we've started
      return "4999 fpx_cant_install" if ($could_not_install);

      # Update Apache conf manually
      my(@fpx_users)=vsite_get_fpx($site);
      push(@fpx_users,$user); # Add user to site fpx users
      my($return)=vsite_set_fpx($site,1,@fpx_users);
      return $return if ($return !~ /^2/);

      # Restore lost sticky bits
      fpx_recurse_chmod( 02775, "$user_home/$Webdir");

      # SUID page counter 
      chmod(04755,(getpwnam($user))[7]."/$Webdir/_vti_bin/fpcount.exe");

      return "2999 fpx_ok_installed";
    } else # Disable
    {
      # fpx "delete" actually rm -rf's the user web tree.  Unacceptable, so
      # we first move the user home directory out of harms way, then restore it.
      rename("$user_home/$Webdir","$user_home/fp98-$Webdir");
      
      # Uninstall
      my $fpx_failure = 0; 
      system("$Fpx_admsrvexe -o delete -p 80 -t apache -m \"$siteName\" -w \"~$user\" -s \"$Htconf/fp98.httpd.conf\" > /dev/null") && ($fpx_failure = 1); 

      # Restore the user web directory
      rename("$user_home/fp98-$Webdir","$user_home/$Webdir");

      # Now bail since we've restored functionality
      return "4999 fpx_cant_uninstall" if ($fpx_failure);

      # Update Apache conf explicitly
      my(@old_users)=vsite_get_fpx($site);
      my(@fpx_users,$fpx_user);
      foreach $fpx_user (@old_users)
      {
        push(@fpx_users,$fpx_user) unless ( ($fpx_user eq $user) ||
                                            ($fpx_user eq $site) );
      }
      my($return)=vsite_set_fpx($site,1,@fpx_users);
      return $return if ($return !~ /^2/);

      # FPX files have been made unavailable to fpadmsrv.exe
      system("/bin/rm -rf $user_home/$Webdir/_vti* > /dev/null 2>&1; /bin/rm -rf $user_home/$Webdir/_private > /dev/null 2>&1\n");
      
      return "2999 services_fpx_ok_uninstalled";
    }

  } else                  ### Site web
  {
    # fpadmsrv.exe matches $siteName in fp98.httpd.conf
    my($siteName)=(vsite_get_bygroup($group))[1];
    return "4999 fpx_invalid_badsite" unless ($siteName);
    my($ret)=fpx_hose_apacheconf();
    return "4999 fpx_cant_editconf" unless ($ret);

    if ($state) # Enable
    {
      # Rename .htaccess files so fpadmsrv.exe doesn't die from confusion
      my(@htax) = fpx_rename_htaccess("$Userdir/$group/$Webdir");

      my($user)=$WWWuser; # For lack of a better default

      # Call fpadmsrv.exe (ugh)
      system("$Fpx_admsrvexe -o install -p 80 -t apache -m \"$siteName\" -u \"$user\" -pw \"$$\" -s \"$Htconf/fp98.httpd.conf\" -xu \"$user\" -xg \"$group\" > /dev/null") && ($could_not_install = $!);

      # FPX creates a bunch of .htaccess files that need to be re-written for
      # pam/shadow support.. So, find the new .htaccess files and edit each one
      # We do this before restoring original .htaccess files
      fpx_rewrite_htaccess( "$Userdir/$group/$Webdir", 'group', "$group $Adminuser");

      # Fix what we've started for clean aborts
      fpx_restore_htaccess( "$Userdir/$group/$Webdir", @htax); 
      return "4999 fpx_cant_install" if ($could_not_install);

      # Update Apache conf manually
      my($return)=vsite_set_fpx($group,1);
      return $return if ($return !~ /^2/);

      # Add SMTP capability to FPX forms
      if (-w "$Fpx_cnfdir/$siteName:80.cnf")
      {
        my($orig_cnf);
	open(CNF);
        while(<CNF>) { $orig_cnf .= $_; }
        close(CNF);

	unless($orig_cnf =~ /smtphost/i)
	{
	  open(NUCNF,">> $Fpx_cnfdir/$siteName:80.cnf");
	  print NUCNF "SMTPHost: 127.0.0.1\n";
	  close(NUCNF);
	}

	# Must be readable by $WWWuser
	chmod(0644,"$Fpx_cnfdir/$siteName:80.cnf");
      }

      # Restore lost sticky bits
      fpx_recurse_chmod( 02775, "$Userdir/$group/$Webdir");

      # Enable the page counter (SGID)
      chmod(02755,"$Userdir/$group/$Webdir/_vti_bin/fpcount.exe");

      return "2999 fpx_ok_installed";

    } else # Disable
    {
      # Uninstall
      # Call fpadmsrv.exe (ugh)
      system("$Fpx_admsrvexe -o uninstall -p 80 -t apache -m \"$siteName\" -s \"$Htconf/fp98.httpd.conf\" > /dev/null") && return "4999 fpx_cant_uninstall";

      # Update Apache conf manually
      my($return)=vsite_set_fpx($group,0);
      return $return if ($return !~ /^2/);

      # fpadmsrv should do this automatically 
      #  ..but we don't trusts its now does we..
      unlink("$Fpx_cnfdir/$siteName:80.cnf");

      # fpadmsrv.exe won't delete all stub cgis or old logs
      system("/bin/rm -rf $Userdir/$group/$Webdir/_vti* > /dev/null 2>&1; /bin/rm -rf $Userdir/$group/$Webdir/_private > /dev/null 2>&1\n");

      return "2999 services_fpx_ok_uninstalled";
    }

    return "4999 fpx_cant_install" if ($could_not_install);
    return "2999 fpx_ok_installed";
  }
}                                               

sub fpx_get_web
# Are FrontPage Server Extensions enabled?
# Returns 1 if FP web is present, 0 otherwise
# Arguments: user or site group name
# Side effects: none
{
  my($group)=@_;
  
  return 0 unless ($Features{'fpx'});
  return 0 unless (-x $Fpx_admsrvexe);

  # Test user vs group/site
  my($user);
  if(user_exist($group))  ### User web
  {
    $user=$group; # Clean variable names
    $group=''; 

    my($site)=vsite_getgroup($user);
    return 0 unless ($site);

    # Get fpx status from Apache conf, much faster than fpadmsrv.exe
    my(@fpxusers)=vsite_get_fpx($site);
    my($userinstance);
    foreach $userinstance (@fpxusers)
    {
      return 1 if ($userinstance eq $user);
    }
  } else
  {
    my($siteName)=(vsite_get_bygroup($group))[1];
    # If the site root web exists it's going to have a config
    return 1 if (-e "$Fpx_cnfdir/$siteName:80.cnf");
  }
  return 0; 
}

sub fpx_rename_htaccess
# Rename .htaccess files to something else
# Arguments: directory root
# Return value: array of qualified files moved
{
  my($basedir)=@_;

  my(@htax,$hta);
  open(HTAS,"/usr/bin/find $basedir -name .htaccess -print |")
    || return();
  while(<HTAS>) { chop; push(@htax,$_); }
  foreach $hta (@htax) 
  {
    move("$hta","$hta.FP98") unless ($hta eq "$Htgroup_dir/$MainGroup/$Webdir/.htaccess");
  }
  close HTAS;
  return(@htax);
}

sub fpx_restore_htaccess
# Restores ftp_rename_htaccess action
# Arguments: FP Web directory, array of qualified files to move
# Return value: true or false
# Side affect: Removes .htaccess file from web root if FP added one
#   This seems to be a common bug, where a corrupt .htaccess appears
#   out of nowhere.
{
  my($webroot,@htax)=@_;

  my($hta);
  foreach $hta (@htax) 
  {
    move("$hta.FP98","$hta");
  }
  unlink("$webroot/.htaccess") if (-e "$webroot/.htaccess"); # risky business
  return 1;
}

sub fpx_rewrite_htaccess
# Rewrites fpx-generated .htaccess files so they'll do a Cobalt config
# Arguments: directory root, literal group/user, group or username string
# Return value: array of edited files
{
  my($basedir,$flavour,$user)=@_;

  my(@htax,@edited,$hta,$newfile);
  open(HTAS,"/usr/bin/find $basedir -name .htaccess -print |")
    || return();
  while(<HTAS>) { chop; push(@htax,$_); }
  foreach $hta (@htax) 
  {
    open(HTA,$hta) || next;
    while(<HTA>)
    {
      s/^AuthUserFile.*$//g;
      s/^AuthGroupFile.*$//g;
      s/^require.*$/require $flavour $user\n/g;
      $newfile.=$_;
    }
    close(HTA);

    # Clobber old file
    open(HTN,">$hta") || next;
    print HTN $newfile;
    close(HTN); 
 
    push(@edited,$hta);
    $newfile=''; # ..primitive
  }
  close HTAS;
  return(@edited);
}

sub fpx_recurse_chmod
# Resets web directory permissions post-FPX install
# Arguments: directory mode, root directory
{
  my( $file_mode, $root_dir)=@_;
  $file_mode ||= "02775"; # Defaults are good
  return unless ( -d $root_dir);

  chmod( $file_mode, $root_dir);

  opendir( SBDR, $root_dir) || return;
  map fpx_recurse_chmod($file_mode, "$root_dir/$_"),
    grep +(-d "$root_dir/$_" and !/^\.{1,2}$/), readdir(SBDR);
  close (SBDR);
}

sub fpx_hose_apacheconf
# Generates an alternate Apache-like configuration file
# that uses incorrect syntax.  This alternate configuration
# is what fpadmsrv.exe would like to see, and would not
# work AS an Apache configuration file.
#
# Arguments: none
# Return value: true for success, false for failure
# Side effects: Writes new configuration file
{
  my(@vsites)=vsite_list();
  open(HTMUNGE,"> $Htconf/fp98.httpd.conf") || return 0;
  open(HTCNF,"$Htconf/httpd.conf") || return 0;
  while(<HTCNF>)
  {
    unless(/^<VirtualHost/.../^<\/VirtualHost/)
    {
      print HTMUNGE;
    }
  }
  close(HTCNF);

  my($vsite_instance);
  foreach $vsite_instance (@vsites)
  {
    print HTMUNGE <<EOF;
<VirtualHost @$vsite_instance[1]>
ServerName @$vsite_instance[1]
ServerAdmin admin
DocumentRoot $Htgroup_dir/@$vsite_instance[2]/$Webdir
</VirtualHost>

EOF
  }

  close(HTMUNGE);

  # httpd must be able to read this on FP client hits
  chmod(0644,"$Htconf/fp98.httpd.conf");

  return 1;
}










