#
# $Id: Group.pm,v 1.44 1999/03/11 08:08:36 gsri Exp $
#
# Package for managing groups
#
# Written by Brion Moss (brion@cobaltnet.com, brion@nda.com)
#
# Copyright 1998 Cobalt Networks http://www.cobaltnet.com
#

package Cobalt::Group;
use AutoLoader 'AUTOLOAD';
BEGIN {
require Exporter;
@ISA    = qw(Exporter);
@EXPORT = qw(group_add group_add_user group_del_user group_del
	     group_show group_list group_set_user group_exist
	     group_set_owner group_get_owner
	     group_set_quota group_get_quota
	     group_add_nohome group_del_nohome group_set_membership
	    );

require Cobalt::Quota;
import Cobalt::Quota qw(:DEFAULT);
require Cobalt::List;
import Cobalt::List qw(:DEFAULT);
require Cobalt::Admserv;
import Cobalt::Admserv qw(admserv_set_access);
require Cobalt::Netutil;
import Cobalt::Netutil qw(:DEFAULT);
require Cobalt::Utility;
import Cobalt::Utility qw(:DEFAULT);
require Cobalt::Atalk;
import Cobalt::Atalk qw(:DEFAULT);
}

use integer;
#use strict;
#use Cobalt::Paths;
1;

__END__

#sub group_set_owner
# group_set_owner: set the owner for a group
# Arguments: group name, owner
# Side effects: creates the list
# Return value: the usual
# Not used in the current release
#{
#    my ($group,$owner) = @_;
#    return Cobalt::List::list_add("$group+owner",$owner);
#}

#sub group_get_owner
# group_get_owner: return the owner for the group
# Arguments: group name
# Side effects: none
# Return value: the group owner name
# Not used in the current release
#{
#    my ($group) = @_;
#    return Cobalt::List::list_show("$group+owner");
#}

sub group_del
# group_del: delete a group
# Arguments: the name of the group to delete
# Side effects: Removes the group from the group and htgroup file
# Return value: a status string of the form "604 Gotta dance!"
{
    my($name) = @_;
    return "4282 group $name does not exist" unless (getgrnam($name));
    my $return;
    return "4200 Cannot modify $name: prohibited group"
	if (($System_groups{$name}) || ($name eq $MainGroup));
    return $return unless (($return = Cobalt::Group::lock_group()) =~ /^2/o);

    print STDERR "Cleaning from $Gtmp\n" if ($Debug);
    open(GROUP,$Gtmp) || return Cobalt::Group::group_abort("4203 Cannot open $Gtmp: $!");
    open(GTMP,">$Lockdir/group") || 
	return Cobalt::Group::group_abort("4204 Cannot clobber $Lockdir/group: $!");
    "$name:" =~ /^$name:/;   # pre-compile the pattern
    while (<GROUP>)
    {
	print GTMP unless (//);
    }
    close GTMP;
    close GROUP;
    rename("$Lockdir/group",$Gtmp) ||
	return Cobalt::Group::group_abort("4205 Cannot rename $Gtmp: $!");

    unless ($Features{'pam'}) {
      open(GROUP,$Htgtmp) 
	|| return Cobalt::Group::group_abort("4206 Cannot open $Htgtmp: $!");
      open(GTMP,">$Lockdir/htgroup") || 
	return Cobalt::Group::group_abort("4207 Cannot clobber $Lockdir/htgroup: $!");
      "$name:" =~ /^$name:/;   # pre-compile, not in the loop
      while (<GROUP>)
      {
	print GTMP unless (//);
      }
      close GTMP;
      close GROUP;
      rename("$Lockdir/htgroup",$Htgtmp) ||
	return Cobalt::Group::group_abort("4208 Cannot rename $Htgtmp: $!");
    }

    Cobalt::Utility::sauce_deltree("$Htgroup_dir/$name");
    Cobalt::Group::samba_del($name) if ($Features{'smb'});
    if ($Features{'atalk'})
    {
      my $atalk_ret = Cobalt::Atalk::atalk_del_volume("$Htgroup_dir/$name");
      return $atalk_ret unless ($atalk_ret =~ /^2/o);
    }

    return $return unless (($return = Cobalt::Group::commit_group()) =~ /^2/o);

    # Remove the discussion group and mailing list
    Cobalt::List::list_delete($name);
    # not useful for new discussion group
#    if (-d "$Discuss_html_dir/$name")
#    {
	# rm -r $Discuss_dir/$name $Discuss_html_dir/$name
#	Cobalt::Utility::sauce_deltree("$Discuss_dir/$name");
#	Cobalt::Utility::sauce_deltree("$Discuss_html_dir/$name");
#    }

    print STDERR "Completed deletion of $name\n" if ($Debug);
    return "2235 group deleted successfully";
}


sub group_del_nohome
# group_del:_nohome delete a group that does not have a home directory
# Arguments: the name of the group to delete
# Side effects: Removes the group from the group and htgroup file
# Return value: a status string of the form "604 Gotta dance!"
{
    my($name) = @_;
    return "4201 group $name does not exist" unless (getgrnam($name));
    my $return;
    return "4202 Cannot modify $name: prohibited group"
        if (($System_groups{$name}) || ($name eq $MainGroup));
    return $return unless (($return = Cobalt::Group::lock_group()) =~ /^2/o);

    print STDERR "Cleaning from $Gtmp\n" if ($Debug);
    open(GROUP,$Gtmp) || return Cobalt::Group::group_abort("4283 Cannot open $Gtmp: $!");
    open(GTMP,">$Lockdir/group") ||
        return Cobalt::Group::group_abort("4284 Cannot clobber $Lockdir/group: $!");
    "$name:" =~ /^$name:/;   # pre-compile the pattern
    while (<GROUP>)
    {
        print GTMP unless (//);
    }
    close GTMP;
    close GROUP;
    rename("$Lockdir/group",$Gtmp) ||
        return Cobalt::Group::group_abort("4285 Cannot rename $Gtmp: $!");

    unless ($Features{'pam'}) {
      open(GROUP,$Htgtmp) || return Cobalt::Group::group_abort("4286 Cannot open $Htgtmp: $!");
      open(GTMP,">$Lockdir/htgroup") ||
        return Cobalt::Group::group_abort("4287 Cannot clobber $Lockdir/htgroup: $!");
      "$name:" =~ /^$name:/;   # pre-compile, not in the loop
      while (<GROUP>)
      {
        print GTMP unless (//);
      }
      close GTMP;
      close GROUP;
      rename("$Lockdir/htgroup",$Htgtmp) ||
        return Cobalt::Group::group_abort("4288 Cannot rename $Htgtmp: $!");
    }

    return $return unless (($return = Cobalt::Group::commit_group()) =~ /^2/o);


    print STDERR "Completed deletion of $name\n" if ($Debug);
    return "2231 group deleted successfully";
}



sub group_del_user
# group_del_user: delete a member from a group
# Arguments: the group name and a list of users to delete
# Side effects: calls &group_set_membership
# Return value: a status string of the form "725 plane out of fuel, can't land"
{
    my $return;
    return $return unless (($return = Cobalt::Group::lock_group()) =~ /^2/o);
    my ($group,@victims) = @_;
    print STDERR "Removing @victims from group $group\n" if ($Debug);
    my $victim;
    my @members = Cobalt::Group::group_show($group);
    foreach $victim (@victims)
    {
	@members = grep(!/^$victim$/,@members);
    }
    return $return unless (($return = Cobalt::Group::group_set_membership($group,@members))
			   =~ /^2/o);
    return $return unless (($return = Cobalt::Group::commit_group()) =~ /^2/o);
    return "2236 users removed successfully";
}

sub group_show
# group_show: list who is in this group
# Arguments: group name
# Side effects: none
# Return value: a sorted array of group members
# Doesn't include the admin or the web server user
{
    my ($group) = @_;
    my @users = sort(split(/\s+/o,(getgrnam($group))[3]));

    @users = (grep(!/^($Adminuser|$WWWuser)$/o,sort(@users)));
    return @users;
}

sub group_list
# group_list: Returns a list of groups, not including system groups
# Arguments: none
# Side effects: none
# Return value: a sorted array of group names
{
    my ($name,@return);
    setgrent();
    while ($name = getgrent)
    {
	push(@return,$name) unless ($System_groups{$name});
    }
    endgrent();
    return sort(@return);
}

sub group_exist
# group_exist: does a group exist?
# Arguments: login name
# Side effects: none
# Return value: true if they do, false if they don't
{
  setgrent; # flush
  (getgrnam($_[0])) ? (return 1) : (return 0);
}

sub group_set_quota
# Set the quota for a group
# Arguments: group name, quota
# Side effects: sets the quota for the group
# Return value: string of the form "816 This onion already has a definition"
{
    my ($group,$quota) = @_;
    # Error message in MB, not KB
    my $warn_quota = ($MaxQuota / 1024);
    return "4281 Group quotas may not be set above $warn_quota"
	if ($quota > $MaxQuota);
    return Cobalt::Quota::setqlim($Userpart,(getgrnam($group))[2],$quota,
		   ($quota + $Quota_harder),1);
}

sub group_get_quota
# Get quota information for a group
# Arguments: group name
# Side effects: none
# Return value: array containing used, limit
{
    my ($group) = @_;
    return (Cobalt::Quota::repquota($group,1))[0,1];
}

sub group_add
# group_add: Create a group
# Arguments: the name of the group, group quota, and (optional) list of members
# Side effects: adds entries to the group file
# Return value: the usual status string of format "816 I said NO camels"
{
    my ($return,$gid);
    return "1253 Groups not enabled" unless ($Features{'groups'});
    my ($group,$quota,@members) = @_;
    return "4234 Invalid group name $group" if ($group !~ /^[\w\-\.]+$/o);
    return "4242 Group names must be no longer than $MaxNameLen characters."
	if (length($group) > $MaxNameLen);
    my $warn_quota = ($MaxQuota / 1024);
    return "4243 Group quotas may not exceed $warn_quota"
	if ($quota > $MaxQuota);
    return "4267 The name $group is reserved by the sytem" 
	if ($System_groups{$group});
    return "4299 Group $group already exists"
	if (getgrnam($group));
    return "4240 Name $group already in use as a user" 
	if (getpwnam($group));
    return "4241 Name $group is already in use as a mailing list or alias"
	if (Cobalt::List::alias_exist($group));
    return 
	Cobalt::Group::group_abort("4244 Too many members; you may only have $MaxMembers members per group")
	    if ($#members >= $MaxMembers);

    return $return unless (($return = Cobalt::Group::lock_group()) =~ /^2/o);

    ### Add to the files in /etc
    setgrent();
    for ($gid = $Firstgid; getgrgid($gid); $gid++) {;}
    my $line = "${group}::${gid}:" . join(',',@members,$Adminuser) . "\n";
    print STDERR "Adding to $Gtmp: $line" if ($Debug);
    open(GTMP,">>$Gtmp") 
	|| return Cobalt::Group::group_abort("4222 Cannot append to $Gtmp: $!");
    print GTMP $line;
    close GTMP;
    unless ($Features{'pam'}) {
      my $htline = "${group}:" . join(' ',@members,$Adminuser). "\n";
      open(GTMP,">>$Htgtmp") 
	|| return Cobalt::Group::group_abort("4223 Cannot append to $Htgtmp: $!");
      print GTMP $htline;
      close GTMP;
    }

    ### Make the directory
    if ($Features{'groupdirs'}) {
      my $home = "$Htgroup_dir/$group";
      print STDERR "Creating directories under $home\n" if ($Debug);
      return Cobalt::Group::group_abort("4226 Group home $home already exists") if (-e $home);
      
      # Make sure the top-level group directory exists
      unless (-d $Htgroup_dir)	{
	# this line doesn't chmod sometimes
	mkdir($Htgroup_dir,0755);
	chmod 0755, $Htgroup_dir;

	if ($Features{'atalk'}) {
	  # AppleTalk seems to feel some odd need for this directory.
	  mkdir("$Htgroup_dir/Network Trash Folder",01777)
	    || return Cobalt::Group::group_abort("4247 Cannot make $Htgroup_dir/Network Trash Folder: $!\n");
	  chmod(01777,"$Htgroup_dir/Network Trash Folder");
	}
      }
      
      mkdir($home,02775) 
	|| return Cobalt::Group::group_abortbort("4999 group_cant_create_home");
      chown($WWWuid,$gid,$home);
      # The setgid bit means that all files or subdirectories will be created
      # also owned by this group.  The user writable bit has to be set or
      # appletalk won't work
      chmod(02775,$home);
      
      mkdir("$home/$Webdir",02775) 
	|| return Cobalt::Group::group_abort("4999 group_cant_create_home");
      chown($WWWuid,$gid,"$home/$Webdir");
      chmod(02775,"$home/$Webdir");
      
      if ($Features{'privatedirs'})	{
	mkdir("$home/$Webdir/$Private_html",02770) 
	  || return Cobalt::Group::group_abort("4228 Cannot create group private area $home/$Webdir/$Private_html: $!");
	chown($WWWuid,$gid,"$home/$Webdir/$Private_html");
	chmod(02770,"$home/$Webdir/$Private_html");
      }
      
      # Copy *.html from the skel directory into the new group home area
      opendir(SKEL,$Group_skel)
    	|| return Cobalt::Group::group_abort("3228 Cannot read $Group_skel: $!");
      my ($htmlfile);
      foreach $htmlfile (grep(/\.html$/o,readdir(SKEL)))	{
	next if ($htmlfile eq "index.html"); # special case, do later
	print STDERR "Copying $htmlfile to $home/$Webdir/$htmlfile\n"
	  if ($Debug);
	
	# inline the cp subroutine for speed
	open(SRC,"$Group_skel/$htmlfile") 
	  || return Cobalt::Group::group_abort("4224 Cannot open source file $Group_skel/$htmlfile: $!");
	open(DEST,">$home/$Webdir/$htmlfile") 
	  || return Cobalt::Group::group_abort("4225 Cannot open dest file $home/$htmlfile: $!");
	while (<SRC>)
	  {
	    # No substitutions
	    print DEST;
	  }
	close DEST;
	close SRC;
	
	chown($WWWuid,$gid,"$home/$Webdir/$htmlfile");
	chmod(0664,"$home/$Webdir/$htmlfile");
      }
      closedir SKEL;

      # now do index.html; this is a special case because we do some macro
      # substitution on it, e.g. for the group name
      my $cgroup = ucfirst($group);
      
      my $index_skel = "$Group_skel/index.html";
      if ($Features{'specialHomeSkel'})	{
	$index_skel = "$Group_skel/$MainGroup/index.html" 
	  if ($group eq $MainGroup);
      }
      
      open(SRC,$index_skel) 
	|| return Cobalt::Group::group_abort("4277 Cannot open source file $Group_skel/index.html: $!");
      open(DEST,">$home/$Webdir/index.html") 
	|| return Cobalt::Group::group_abort("4278 Cannot open dest file $home/$Webdir/index.html: $!");
      while (<SRC>) {
	if (/\[/o)	  {
	  s/\[GROUP\]/$cgroup/;
          s/\[RGROUP\]/$group/;
	  s/\[PATH\]/$home\/index.html/;
	}
	print DEST;
      }
      close DEST;
      close SRC;
      
      chown($WWWuid,$gid,"$home/$Webdir/index.html");
      chmod(0664,"$home/$Webdir/index.html");
      
      if ($Features{'privatedirs'}) {
	# lock down access to the private directory
	Cobalt::Admserv::admserv_set_access("$home/$Webdir/$Private_html","require","group",$group);
	
	# now do index.html for the private area
	if (-e "$Group_skel/private.tmpl") {
	  my $cgroup = ucfirst($group);
	  open(SRC,"$Group_skel/private.tmpl") 
	    || return Cobalt::Group::group_abort("4279 Cannot open source file $Group_skel/private.tmpl: $!");
	  open(DEST,">$home/$Webdir/$Private_html/index.html") 
	    || return Cobalt::Group::group_abort("4280 Cannot open dest file $home/$Webdir/$Private_html/index.html: $!");
	  while (<SRC>) {
	    if (/\[/o) {
	      s/\[GROUP\]/$cgroup/;
	      s/\[PATH\]/$home\/$Private_html\/index.html/;
	    }
	    print DEST;
	  }
	  close DEST;
	  close SRC;
	  
	  chown($WWWuid,$gid,"$home/$Webdir/$Private_html/index.html");
	  chmod(0664,"$home/$Webdir/$Private_html/index.html");
	}
      }

      Cobalt::Group::samba_add($home,$group) if $Features{'smb'};

      if ($Features{'atalk'})
      {
        my $atalk_ret = Cobalt::Atalk::atalk_set_volume($home, $group);
        return Cobalt::Group::group_abort($atalk_ret) if ($atalk_ret !~ /^2/o);
      }      

    } # if groupdirs

    # not useful for new discussion group
#    Cobalt::Group::group_mkdiscuss($group,$gid) if ($Features{'discuss'});

    Cobalt::List::list_create($group,@members);  # no checkee returnee

    ### end game
    return $return unless (($return = Cobalt::Group::commit_group()) =~ /^2/o);
    Cobalt::Group::group_set_quota($group,$quota);

    print STDERR "Completed adding $group\n" if ($Debug);

    # We restart the web server so it re-reads its group membership;
    # otherwise it may not be able to read group files properly.
    # If we just hup'ed, there would be no permissions set
    # For a virtual machine model a group is only added 
    #   with a virtual interface, which then restarts
    #   the web server, no need to restart twice
    unless($Features{'vmachine'})
    {
        # Restart the web server, if it's up
        if (-f "$Subsys/httpd")
        {
            system("$Htscript stop >>$Sauce_log 2>&1");
            system("$Htscript start >>$Sauce_log 2>&1");
            system("$Lcdshowscript >>$Sauce_log 2>&1") if (-x $Lcdshowscript);
        }
    }
    return "2237 group $group created successfully";
}


sub group_add_nohome
# group_add_nohome: Create a group
# Arguments: the name of the group, group quota, and (optional) list of members
# Side effects: adds entries to the group file
# Return value: the usual status string of format "816 I said NO camels"
{
    my ($return,$gid);
    my ($group,$quota,@members) = @_;
    return "4289 Invalid group name $group" if ($group !~ /^[\w\-\.]+$/o);
    return "4290 Group names must be no longer than $MaxNameLen characters."
        if (length($group) > $MaxNameLen);
    my $warn_quota = ($MaxQuota / 1024);
    return "4291 Group quotas may not exceed $warn_quota"
        if ($quota > $MaxQuota);
    return "4293 Group $group already exists"
        if (getgrnam($group));
    return "4294 Name $group already in use as a user"
        if (getpwnam($group));
    return "4295 Name $group is already in use as a mailing list or alias"
        if (Cobalt::List::alias_exist($group));
    return
        Cobalt::Group::group_abort("4296 Too many members; you may only have $MaxMembers members per group")
            if ($#members >= $MaxMembers);

    return $return unless (($return = Cobalt::Group::lock_group()) =~ /^2/o);

    ### Add to the files in /etc
    setgrent();
    for ($gid = $Firstgid; getgrgid($gid); $gid++) {;}
    my $members = join(',',@members,$Adminuser);
    my $line = "${group}::${gid}:${members}\n";
    print STDERR "Adding to $Gtmp: $line" if ($Debug);
    open(GTMP,">>$Gtmp")
        || return Cobalt::Group::group_abort("4297 Cannot append to $Gtmp: $!");
    print GTMP $line;
    close GTMP;
    unless ($Features{'pam'}) {
      my $hmembers = join(' ',@members,$Adminuser);
      my $htline = "${group}:${hmembers}\n";
      open(GTMP,">>$Htgtmp")
        || return Cobalt::Group::group_abort("4298 Cannot append to $Htgtmp: $!");
      print GTMP $htline;
      close GTMP;
    }

    ### end game
    return $return unless (($return = Cobalt::Group::commit_group()) =~ /^2/o);
    Cobalt::Group::group_set_quota($group,$quota);

    ## gsri: believe this is necessary here as
    ##       group_add_user or group_set_user could be called later
    Cobalt::List::list_create($group,@members);  # no checkee returnee

    print STDERR "Completed adding $group\n" if ($Debug);
    return "2230 Group $group created successfully";
}



sub group_add_user
# group_add_user: Adds a user to the group
# Arguments: the name of the group and a list of new users to add
# Side effects: modifies the entry in the group file by calling
#   &group_set_membership
# Return value: usual status string of format "567 take back your mink"
{
    my $return;
    return $return unless (($return = Cobalt::Group::lock_group()) =~ /^2/o);
    my($group,@newguys) = @_;
    my @members = Cobalt::Group::group_show($group);
    push(@members,@newguys);
    print STDERR "Putting @newguys in $group\n" if ($Debug);
    return $return unless (($return = Cobalt::Group::group_set_membership($group,@members))
			   =~ /^2/o);
    return $return unless (($return = Cobalt::Group::commit_group()) =~ /^2/o);
    return "2234 users added to group $group successfully";
}

sub group_set_user
# group_set_user: set the members list for a group
# Arguments: the name of the group and an array of members
# Side effects: modifies /etc/group, /etc/htgroup
{
    my $return;
    return $return unless (($return = Cobalt::Group::lock_group()) =~ /^2/o);
    my($group,@members) = @_;
    return $return unless (($return = Cobalt::Group::group_set_membership($group,@members))
			   =~ /^2/o);
    return $return unless (($return = Cobalt::Group::commit_group()) =~ /^2/o);
    return "2222 membership of group $group set successfully";
}

sub group_set_membership
# group_set_membership: set the members list for a group
# Arguments: the name of the group and an array of members
# Side effects: modifies /etc/group, /etc/htgroup
#   This function ensures that the list contains no duplicate names.
{
    my ($group,@list) = @_;
    return Cobalt::Group::group_abort("4210 No such group: $group") unless (getgrnam($group));
    return 
	Cobalt::Group::group_abort("4245 Too many members; you may only have $MaxMembers members per group")
	    if ($#list >= $MaxMembers);

    print STDERR "Membership of $group is being set to @list\n" if ($Debug);

    # Using the hash table is a cute way to guarantee no repeat
    # entries
    my %temp = map { $_, 1} @list;
    $temp{$Adminuser} = 1; # ditto for admin
    my $list = join(',',sort(keys(%temp)));

    # Don't go to any trouble if nothing is changing
    my $oldmembers = join(',',Cobalt::Group::group_show($group));
    if ($oldmembers eq $list)
    {
	return "2224 No change to group membership";
    }

    # fix the group file
    open(GROUP,$Gtmp) ||  return Cobalt::Group::group_abort("4211 Cannot read $Gtmp: $!");
    open(GTMP,">$Lockdir/group") 
	|| return Cobalt::Group::group_abort("4212 Cannot append to $Lockdir/group: $!");
    "$group:x:" =~ /^$group:(.+):/;   # pre-compiled regexps are better
    while (<GROUP>)
    {
	if (//)
	{
	    print GTMP "$group:$1:$list\n";
	}
	else
	{
	    print GTMP;
	}
    }
    close GTMP;
    close GROUP;
    rename("$Lockdir/group",$Gtmp) ||
	return Cobalt::Group::group_abort("4213 Cannot replace $Gtmp: $!");

    unless ($Features{'pam'}) {
      # Fix the htgroup file
      $list = join(' ',keys(%temp));
      open(GROUP,$Htgtmp) ||  return Cobalt::Group::group_abort("4214 Cannot read $Htgtmp: $!");
      open(GTMP,">$Lockdir/htgroup") 
	|| return Cobalt::Group::group_abort("4215 Cannot append to $Lockdir/htgroup: $!");
      "$group:" =~ /^$group:/;   # pre-compile for speed
      while (<GROUP>)
      {
	if (//)
	{
	    print GTMP "$group:$list\n";
	}
	else
	{
	    print GTMP;
	}
      }
      close GTMP;
      close GROUP;
      rename("$Lockdir/htgroup",$Htgtmp) ||
	return Cobalt::Group::group_abort("4216 Cannot replace $Htgtmp: $!");
    }

    # Fix the list membership to match the group membership
    Cobalt::List::list_set_mem($group,@list);
    return "2225 Group membership set successfully";
}

##############################

sub lock_group
# Locks the group file so that nobody can collide with us.
# Arguments: none
# Side effects: copies /etc/group to /etc/gtmp
{
    my ($return);
    if (-e $Gtmp)
    {
	return Cobalt::Group::group_abort("4217 The group file is locked.");
    }
    else
    {
	# Lock the group file by creating the gtmp file as a
	# copy of the group file.
	print STDERR "Copying group file $Group to $Gtmp\n" if ($Debug);

	# inline for better performance
	open(SRC,$Group) 
	    || return Cobalt::Group::group_abort("4236 Cannot open source file $Group: $!");
	open(DEST,">$Gtmp") 
	    || return Cobalt::Group::group_abort("4237 Cannot open dest file $Gtmp: $!");
	print DEST while (<SRC>);
	close DEST;
	close SRC;

	open(LOCK,">$Lockdir/group-$$")
	  || return Cobalt::Group::group_abort("4265 Cannot get group lock: $!");
	print LOCK "$$\n$0";
	close LOCK;

    	unless ($Features{'pam'}) {
	  if (! -e $Htgroup)
	  {
	    # this looks like a fresh system
	    print STDERR "Creating new htgroup file $Htgroup\n" if ($Debug);
	    open(HTGROUP,">$Htgroup")
	      || return "4248 Cannot write $Htgroup";
	    print HTGROUP "${Adminuser}:$Adminuser\n";
	    close HTGROUP;
	  }
	  
	  open(SRC,$Htgroup) 
	    || return Cobalt::Group::group_abort("4238 Cannot open source file $Htgroup: $!");
	  open(DEST,">$Htgtmp") 
	    || return Cobalt::Group::group_abort("4239 Cannot open dest file $Htgtmp: $!");
	  print DEST while (<SRC>);
	  close DEST;
	  close SRC;
    	}
    }
    print STDERR "Locked $Group with $Gtmp\n" if ($Debug);
    return "2226 Group file locked";
}

sub commit_group
# Commits the changes to the group file by moving gtmp back onto group
# Arguments: none
# Side effects: rebuilds the group files based on the aliases file, and
#   replaces /etc/group with /etc/gtmp
{
    rename($Gtmp,$Group) 
	|| return Cobalt::Group::group_abort("4219 Cannot replace $Group with $Gtmp: $!");
    chmod(0644,$Group);
    print STDERR "Unlocked $Gtmp\n" if ($Debug);
    if ($Features{'pam'}) {
        # currently sauce supports shadow group only
        # support for radius/nis/etc requires libpwdb
	system("$Grpconv 2>/dev/null") &&
	    return ("3999 group_cant_grpconv");
    }
    else {
    rename($Htgtmp,$Htgroup) 
	|| return Cobalt::Group::group_abort("4218 Cannot replace $Htgroup with $Htgtmp: $!");
    chmod(0644,$Htgroup);
    }

    unlink("$Lockdir/group-$$");
    endgrent();   # sync
    return "2227 Group changes committed successfully";
}

sub unlock_group
# Unlocks the group file by wiping $Gtmp
# Arguments: none
# Side effects: deletes $Gtmp and $Htgtmp
{
    unless ($Features{'pam'}) {
      unlink($Htgtmp) || return "4220 Cannot delete gtmp $Gtmp: $!";
    }
    unlink($Gtmp) || return "4221 Cannot delete gtmp $Gtmp: $!";
    print STDERR "Unlocked $Gtmp\n" if ($Debug);

    unlink("$Lockdir/group-$$");
    endgrent();   # sync
    return "2229 Group file unlocked";
}

sub group_abort
# leave in a graceful manner
# Arguments: a string to print
# Side effects: unlocks anything we've locked
{
    Cobalt::Group::unlock_group() if (-e "$Lockdir/group-$$");
    return join('',@_);
}

sub samba_del
# Removes a share from the samba configuration file
# Arguments: share name (generally the group name)
# Side effects: modifies smb.conf
{
    my ($share) = @_;
    return "1256 Samba is not enabled on this system" 
      unless ($Features{'smb'});
    return "4235 smb.conf is locked" if (-e "$Lockdir/smb.conf");
    open(SMB,"$Smbconf") 
	|| return "4230 Cannot modify samba config file $Smbconf: $! ";
    open(SNB,">$Lockdir/smb.conf") 
	|| return "4231 Cannot write samba config file $Lockdir/smb.conf: $! ";
    # compile the expression now, not each time we loop
    ";$share BEGIN" =~ /;$share BEGIN/; 
    while (<SMB>)
    {
	last if (//);
	print SNB;
    }
    # compile the expression now, not each time we loop
    ";$share END" =~ /;$share END/;  
    while (<SMB>)
    {
	last if (//);
    }
    print SNB while (<SMB>);
    close SMB;
    close SNB;
    rename("$Lockdir/smb.conf",$Smbconf) ||
	return "4232 Cannot update $Smbconf: $!";
    return "2258 Removed share";
}

sub samba_add
# Add a share to the samba config file
# Arguments: directory name, share name
# Side effects: modifies smb.conf
{
    my ($dir,$share) = @_;
    return "1255 Samba is not enabled on this system" 
      unless ($Features{'smb'});
    return "4209 smb.conf is locked" if (-e "$Lockdir/smb.conf");
    open(SMB,">>$Smbconf") 
	|| return "4229 Cannot modify samba config file $Smbconf: $! ";
    print SMB ";$share BEGIN (do not delete this line)\n";
    print SMB "[$share]\n  path = $dir\n  public = no\n  browseable = yes\n";
    print SMB "  writable = yes\n  printable = no\n  create mask = 0775\n";
    print SMB "  valid users = $Adminuser \@$share\n";
    print SMB "  force create mode = 0664\n";
    print SMB "  force directory mode = 0775\n";
    print SMB "  hide dot files = yes\n";
    print SMB ";$share END (do not delete this line)\n";
    close SMB;
    return "2257 Added share";
}

sub group_mkdiscuss
# Creates a discussion group; sets up all the directories and files
# Arguments: group name, group id
# Side effects: creates the directories and copies HTML files
# Return value: the usual status string
{
    my ($group,$gid) = @_;
    my $cgroup = ucfirst($group);
    my $sfile;
    return "1254 Discussion groups not enabled"
      unless ($Features{'discuss'});

    # Make sure the top-level directories for discussion stuff
    # are all there and in place
    mkdir ($Discuss_html_dir,0755) unless (-d $Discuss_html_dir);
    mkdir ($Discuss_dir,0755) unless (-d $Discuss_dir);

    if ((-d "$Discuss_html_dir/$group") ||
	(-d "$Discuss_dir/$group"))
    {
	return "4246 Discussion directory for $group already exists";
    }

    # make directories
    mkdir("$Discuss_dir/$group",0755)
	|| print STDERR "Cannot make discussion directory: $!\n";
    mkdir("$Discuss_html_dir/$group",0775);
    mkdir("$Discuss_html_dir/$group/private",0775);
    chown($WWWuid,$gid,"$Discuss_html_dir/$group",
	  "$Discuss_html_dir/$group/private");
    chmod(0775,"$Discuss_html_dir/$group",
	  "$Discuss_html_dir/$group/private");
    
    # touch the data.txt file
    open(D,">$Discuss_html_dir/$group/data.txt")
      || return "4292 Cannot create $Discuss_html_dir/$group/data.txt: $!";
    close D;
    chmod(0664,"$Discuss_html_dir/$group/data.txt");
    chown($WWWuid,$gid,"$Discuss_html_dir/$group/data.txt");

    # HTML templates
    foreach $sfile (@Discuss_html)
    {
	open(IN,"$Discuss_skel/$sfile")
	    || return "4276 Cannot read discussion $sfile template: $!";
	open(OUT,">$Discuss_html_dir/$group/$sfile")
	    || return "4275 Cannot write discussion $sfile template: $!";
	while (<IN>)
	{
	    if (/XXX|GROUP/o)
	    {
		$_ =~ s/XXX/$cgroup/g;
		$_ =~ s/GROUP/$group/g;
	    }
	    print OUT;
	}
	close IN;
	close OUT;
	chown($WWWuid,$gid,"$Discuss_html_dir/$group/$sfile");
	chmod(0664,"$Discuss_html_dir/$group/$sfile");
    }

    # CGI templates
    foreach $sfile (@Discuss_cgis)
    {
	open(IN,"$Discuss_skel/$sfile")
	    || return "4249 Cannot read discussion $sfile template: $!";
	open(OUT,">$Discuss_dir/$group/$sfile")
	    || return "4269 Cannot write discussion $sfile template: $!";
	while (<IN>)
	{
	    if (/ROUP/o)
	    {
		$_ =~ s/CROUP/$cgroup/g;
		$_ =~ s/GROUP/$group/g;
	    }
	    print OUT;
	}
	close IN;
	close OUT;
	chmod(0755,"$Discuss_dir/$group/$sfile");
    }
    Cobalt::Admserv::admserv_set_access("$Discuss_html_dir/$group","require","group",$group);
    Cobalt::Admserv::admserv_set_access("$Discuss_dir/$group","require","group",$group);
    return "2223 Discussion group created successfully";
}

1;
