#
# $Id: Network.pm,v 1.55 1999/02/19 20:52:05 brion Exp $
#
# Written by Brion Moss (brion@cobaltnet.com, brion@nda.com)
#
# Copyright 1998 Cobalt Networks http://www.cobaltnet.com
#

# Return codes: 2000 = success, 3000 = abort in the middle, 4000 = error

package Cobalt::Network;
use AutoLoader 'AUTOLOAD';
BEGIN {
require Exporter;
@ISA    = qw(Exporter);
@EXPORT = qw(net_set_hostname net_get_hostname net_set_ipaddr net_get_ipaddr
	     net_set_dnsclient net_get_dnsclient net_set_gateway 
	     net_get_gateway net_get_broadcast net_get_network
	     net_get_interfaces net_get_alladdrs net_convert_netmask
	     net_lock_network net_unlock_network net_abort
	     net_network_ismember net_down_interface
	     net_set_forwarding net_get_forwarding
	     net_set_vinterface net_del_vinterface
	    );

require Cobalt::Dhcpd;
import Cobalt::Dhcpd qw(:DEFAULT);
require Cobalt::Email;
import Cobalt::Email qw(mail_add_relay mail_virtuser_change_domain);
require Cobalt::Services;
import Cobalt::Services qw(get_dns_server_on);
require Cobalt::Utility;
import Cobalt::Utility qw(:DEFAULT);
}

use integer;
#use strict;
#use Cobalt::Paths;
1;
__END__

sub net_set_hostname
# Set the hostname
# Arguments: the new name for the computer, a boolean of wether
#    to force a reconfig (normally there will be none if the
#    hostname hasn't changed)
# Return value: a string containing a return code and a message
#   (e.g. "908 Name change complete")
# Side effects: edits the network settings in /etc/sysconfig, the
#   /etc/HOSTNAME file, and in the hosts file; then tells the system
#   its new name (which then takes effect immediately)
#   Also reconfigure the web server and admin server for the new names
#   (and turn hostnamelookups off while we're there)
#   Restarts the web servers and the mail server
{
    my ($name,$force) = @_;
    return "4372 Cannot reconfigure network: system busy"
	unless(Cobalt::Network::net_lock_network());

    if(length( $name ) > $MaxHostnameLen ) { return Cobalt::Network::net_abort("4388 Hostname cannot be longer than $MaxHostnameLen characters"); }

    if ($name =~ /^([\w\-\.]+)$/o)
    {
	# untaint (we used to do thorough taint checking.  Those were
	#          the days...)
	$name = $1;
    }
    else
    {
	return Cobalt::Network::net_abort("4370 invalid hostname $name");
    }

    # Localhost is blocked by the UI; down at this level it's okay
    #return Cobalt::Network::net_abort("4371 reserved name: localhost")
	#if ($name eq "localhost");

    my ($fullname,$shortname,$oldname,$domain,$changed);
    # fullname is the fully qualified domain name
    $domain = (Cobalt::Network::net_get_dnsclient())[0];
    if ($domain && ($name !~ /\.$domain$/))
    {
	$fullname = "$name.$domain";
	$shortname = $name;
    }
    else
    {
	$fullname = $name;
	$shortname = $name;
	$shortname =~ s/\.$domain$//;
    }

    # Localhost should never have a domain name
    if ($shortname =~ /^localhost(\..+)?$/o)
    {
      $fullname = "localhost";
      $shortname = "localhost";
    }

    return Cobalt::Network::net_abort("4362 $Net_cfg is locked, cannot set hostname")
	if (-e "$Lockdir/network");
    open(O,">$Lockdir/network")
	|| return Cobalt::Network::net_abort("4361 Cannot modify $Net_cfg: $!");
    open(I,$Net_cfg) 
	|| return Cobalt::Network::net_abort("4350 Cannot read $Net_cfg: $!","$Lockdir/network");
    while (<I>)
    {
        if (/^HOSTNAME=(.*)$/o)
        {
	    $oldname = $1;
	    $_ = "HOSTNAME=$fullname\n";
        }
	print O;
    }
    close O;
    close I;
    rename("$Lockdir/network",$Net_cfg) ||
	return Cobalt::Network::net_abort("3407 Cannot update network file: $!","$Lockdir/network");

    # If nothing changed, don't do anything
    # Special case: if the new name is "localhost" we must be
    # doing a reset
    if ((($fullname eq $oldname) && ($fullname ne "localhost"))
	&& (! $force))
    {
	Cobalt::Network::net_unlock_network();
	return "2067 No update to hostname or domain name needed";
    }

    my ($ip,$mask) = Cobalt::Network::net_get_ipaddr();

    # Rebuild the hosts file
    # Actually, we don't fix the hosts file; we clobber it entirely.
    if (($ip ne "127.0.0.1") && ($name ne "localhost"))
    {
        return Cobalt::Network::net_abort("3406 hosts file is locked")
	    if (-e "$Lockdir/hosts");
	open(O,">$Lockdir/hosts")
	    || return Cobalt::Network::net_abort("3405 Cannot modify hosts file: $!");
	open(I,$Hostfile)
	  || return Cobalt::Network::net_abort("3437 Cannot read hosts file: $!");
	my $printed = 0;
	while (<I>) 
	{
	  if ((/^$ip\s/) || (/\s$fullname(\s|$)/)) {
	    # found a line with old info about this host
	    # the linux dnsdomainname command gets its domainname from
	    # reading /etc/hosts.
	    print O "$ip\t$fullname \t# main interface\n" unless ($printed);
	    #print O "$ip\t$shortname $fullname \t# main interface\n" unless ($printed);
	    $printed = 1;
	  } else {
	    print O;
	  }
	}
	print O "$ip\t$fullname \t# main interface\n" unless ($printed);
	close O;
	close I;
	rename("$Lockdir/hosts",$Hostfile) ||
	    return Cobalt::Network::net_abort("3417 Cannot update hosts file: $!","$Lockdir/hosts");
    }
	
    # I don't know who uses this file.  I don't really much care.
    if (open (HN,">$Hostnamefile"))
    {
	print HN "$fullname\n";
	close HN;
    }
    else
    {
	return Cobalt::Network::net_abort("3412 Cannot change hostname file: $!");
    }
    
    # Change the servername for the web server, if there is one
    if ($Features{'www'}) {
      return Cobalt::Network::net_abort("3418 $Htconf/httpd.conf is locked")
	if (-e "$Lockdir/httpd.conf");
      open(O,">$Lockdir/httpd.conf")
        || return Cobalt::Network::net_abort("3420 Cannot modify $Htconf/httpd.conf: $!");
      open(I,"$Htconf/httpd.conf")
	|| return Cobalt::Network::net_abort("3419 Cannot read $Htconf/httpd.conf: $!",
			"$Lockdir/httpd.conf");
      my $nomore;
      while (<I>) {
	if ($nomore) {
	  print O;
	  next;
	}
        elsif (/^ServerName /o) {
	  $_ = "ServerName $fullname\n";
        }
        elsif (/^ServerAdmin /o) {
	  $_ = "ServerAdmin $Adminuser\@$fullname\n";
        }
        elsif (/^\#ServerName new.host.name/o) {
	  $_ = "ServerName $fullname\n";
        }
	# take advantage of the typo, and fix it as a semaphore
	elsif (/^\# See alto the BindAddress entry./o) {
	  print O "# See also the BindAddress entry.\n\n";
	  print O "### Anything below this will be left untouched by\n";
	  print O "### The Cobalt administration interface.\n";
	  print O "### If adding virtual hosts, also add Listen directives\n";
	  $_ = "### as described above.\n\n";
	  $nomore = 1;
	}
	elsif (/<VirtualHost/) {
	  # We're in the virtual host area, leave it alone
	  $nomore = 1;
	}
	elsif (/^\#\#\# Anything below this will be left untouched by/o) {
	  # We're in the virtual host area, leave it alone
	  $nomore = 1;
	}
	print O;
      }
      close I;
      close O;
      rename("$Lockdir/httpd.conf","$Htconf/httpd.conf") ||
	return Cobalt::Network::net_abort("3404 Cannot update $Htconf/httpd.conf: $!");
    } # end changes to $Htconf
	
    # Change the servername for the admin server, if any
    if ($Features{'adm'}) {
      return Cobalt::Network::net_abort("3422 $AdminConf/httpd.conf is locked")
	if (-e "$Lockdir/adm_httpd.conf");
      open(O,">$Lockdir/adm_httpd.conf")
        || return Cobalt::Network::net_abort("3424 Cannot modify $AdminConf/httpd.conf: $!");
      open(I,"$AdminConf/httpd.conf")
        || return Cobalt::Network::net_abort("3423 Cannot read $AdminConf/httpd.conf: $!",
			"$Lockdir/httpd.conf");
      while (<I>) {
	# We fix the ServerAdmin, but we don't restart the web server.
	# Restarting the web server causes badness, because we're in
	# the middle of talking to the bloody thing.
        if (/^ServerAdmin /o) {
	    $_ = "ServerAdmin $Adminuser\@$fullname\n";
        }
	elsif (/^ServerName /o) { # replace literal localhost
	  $_ = "ServerName $fullname\n";
	}
	
	print O;
      }
      close I;
      close O;
      rename("$Lockdir/adm_httpd.conf","$AdminConf/httpd.conf") ||
	return Cobalt::Network::net_abort("3425 Cannot update $AdminConf/httpd.conf: $!",
		     "Lockdir/httpd.conf");
    } # end admin server changes

    # Set the hostname for the running system
    system("$Hostname_cmd $fullname 1>&2") &&
	return Cobalt::Network::net_abort("3413 Cannot change name without rebooting: $!");

    # Update aliases from sendmail's /etc/virtusertable
    my($ret) = Cobalt::Email::mail_virtuser_change_domain($oldname,$fullname);
    return Cobalt::Network::net_abort($ret) if ($ret !~ /^2/o);

    # Update the server identifier in dhcpd.conf
    if ($Features{'dhcpd'} && (-e $Dhcpd_conf)) {
      my($ret) = Cobalt::Dhcpd::dhcpd_set_name($fullname);
      return Cobalt::Network::net_abort($ret) if ($ret !~ /^2/o);
    }

    # then fork a child to restart the daemons
    my $pid;
    if (!defined($pid = fork)) {
	die "cannot fork: $!";
    } elsif ($pid) {
	# i'm the parent
	print STDERR "forking a child to restart daemons\n" if ($Debug);
	return "2064 daemons restarted";
    }
    else
    {
	# otherwise I'm the child
	open(STDERR,">>$Sauce_log");
	open(STDOUT,">>$Sauce_log");

	# fix the DNS entries and restart
	if (Cobalt::Services::get_dns_server_on())
	{
	  # Signal the script to rebuild the files and update the server
	  # The CGI will take care of restarting the server
	  system("$Dns_cgi -update >>$Sauce_log 2>&1");
	}

	# restart appletalk
	if (-e "$Subsys/atalk")
	{
	  my $netif;
	  foreach $netif (Cobalt::Network::net_get_interfaces())
	    {
	      # Turn off atalk and reset the interface
	      my ($gw,$gwif) = Cobalt::Network::net_get_gateway();
	      my ($addr,$mask) = Cobalt::Network::net_get_ipaddr($netif);
	      my ($broadcast) = Cobalt::Network::net_get_broadcast($addr,$mask);
	      my $network = Cobalt::Network::net_get_network($addr,$mask);
	      system("$Atalkscript stop >>$Sauce_log 2>&1");
	      system("$Ifconfig_cmd $netif down 1>&2");
	      system("$Ifconfig_cmd $netif inet $addr netmask $mask broadcast $broadcast 1>&2");
	      system("$Route_cmd add -net $network netmask $mask $netif 1>&2");
	      Cobalt::Network::net_set_gateway($gw,$gwif) if ($gw);
	    }

	    system("$Atalkscript start >>$Sauce_log 2>&1");
	}

	# Everything else can be concurrent
	Cobalt::Network::net_unlock_network();

	# restart the web server
	if (-f "$Subsys/httpd")
	{
	    my ($pid,$l);
	    # Note that we die instead of returning because we're a child
	    # process...return()ing from here could result in some really
	    # bad juju.
	    open(PID,$Htpid) 
	        || die Cobalt::Network::net_abort("4017 Cannot read httpd PID file $Htpid: $!");
	    $l = <PID>;
	    if ($l =~ /^(\d+)$/o)
	    {
	        $pid = $1;
	    }
	    close PID;
	    kill('HUP',$pid) if ($pid);
	}

	# restart mail
	if (-f "$Subsys/sendmail")
	{
	    system("$Mailscript stop >>$Sauce_log 2>&1 ");
	    system("$Mailscript start >>$Sauce_log 2>&1 ");
	}

	# restart samba
	if (-f "$Subsys/smb")
	{
	    system("$Smbscript stop >>$Sauce_log 2>&1 ");
	    system("$Smbscript start >>$Sauce_log 2>&1 ");
	}

	system ("$Lcdshowscript start >>$Sauce_log 2>&1")
	    if (-x $Lcdshowscript);
	exit;
    }
    return "2412 Name sucessfully changed from $oldname to $fullname";
}

sub net_get_hostname
# Get the hostname
# Arguments: optional interface (e.g., "eth1")
# Return value: a string containing the hostname (unqualified)
# Side effects: none
{
  my ($netif) = @_;
  my $name;
  if ((defined($Features{'noinventhostnames'}) && $Features{'noinventhostnames'})
	|| ((! $netif) || ($netif eq $Netif))) {
    # if (noinventhostnames), always return official host name;  also, 
    # primary interface gets the official host name
    open(NAME,"/proc/sys/kernel/hostname") ||
	return "4351 Cannot read /proc/sys/kernel/hostname: $!";
    chomp($name = <NAME>);
    close NAME;
  }
  else {
    # I'm not the primary interface, look my name up
    my ($addr) = Cobalt::Network::net_get_ipaddr($netif);
    # Whoops, did I just hardcode AF_INET as 2?  Damn...
    ($name) = gethostbyaddr(pack('C4',split('\.',$addr)),2);
    $name ||= "cobalt-$netif";
  }
  my ($domain);
  if (($domain = (Cobalt::Network::net_get_dnsclient())[0])
      && ($name =~ /$domain$/))
  {
    $name =~ s/\.?$domain$//;
  }

  # Just in case...we should always return a name.
  $name ||= $DefaultName;

  return $name;
}

sub net_set_ipaddr
# Set the IP address and netmask
# Arguments: the new IP address and netmask (in dotted quad), and an 
#   optional network interface name (e.g. "eth1").  The network 
#   interface argument will default to the primary (eth0) interface.
# Side effects: edits the files in /etc/sysconfig
#   Reconfigures the network interface for the new address
#   Sets the .user redirects in the admin server, and restarts the admin server
{
    my ($addr,$mask,$netif) = @_;
    my $broadcast = Cobalt::Network::net_get_broadcast($addr,$mask);
    my $network = Cobalt::Network::net_get_network($addr,$mask);
    my ($dida,$didn,$didm,$didb);

    $netif ||= $Netif; # default the interface

    my ($oldaddr,$oldmask) = Cobalt::Network::net_get_ipaddr($netif);
    return "2999 network_ok_same_address"
      if (($addr eq $oldaddr) && ($mask eq $oldmask));

    ## Check arguments
    $addr =~ s/(\.|^)0+(\d)/$1$2/go;  # get rid of excess leading zeros
    # If we're given a weird IP and it's not via the web, wake the LCD
    if ($addr !~ /^(\d+\.\d+\.\d+\.\d+)$/o)
    {
	print STDERR "Got invalid IP address $addr\n" if ($Debug);
	exec("/sbin/lcdstart") unless $ENV{'REMOTE_ADDR'};
	return "4360 Invalid ip address $addr";  # not reached
    }
    if (Cobalt::Network::net_network_ismember($addr,"224.0.0.0","240.0.0.0")) {
      # This is an address on the multicast network
      return "4999 network_invalid_ismulticast";
    }

    ## Get background information
    my $name = Cobalt::Network::net_get_hostname($netif);
    my ($gw,$gwif) = Cobalt::Network::net_get_gateway();
    print STDERR "My name is $name and my gateway is $gw on $gwif\n" if ($Debug);

    my ($fullname,$domain);
    if (($domain = (Cobalt::Network::net_get_dnsclient())[0])
	&& ($name !~ /$domain$/))
    {
	$fullname = "$name.$domain";
    }
    else
    {
	$fullname = "";
    }

    ## Rewrite the interface configuration file
    # need the general lock NO MATTER WHAT
    return "4375 Cannot reconfigure network: system busy"
      unless(Cobalt::Network::net_lock_network());
    # get the lock specific to this interface
    # the general lock this shouldn't be an issue)
    return Cobalt::Network::net_abort("4364 Cannot set IP address: ifcfg-$netif is locked")
      if (-e "$Lockdir/ifcfg-$netif");
    open(O,">$Lockdir/ifcfg-$netif")
	|| return Cobalt::Network::net_abort("4366 Cannot modify interface config ifcfg-$netif: $!");
    if (! -e "$If_cfg_dir/ifcfg-$netif") {
      # This interface hasn't been configured yet
      print O "DEVICE=$netif\n";
      print O "ONBOOT=yes\n";
    }
    else {
      open(I,"$If_cfg_dir/ifcfg-$netif") 
	|| return Cobalt::Network::net_abort("4365 Cannot read interface config ifcfg-$netif: $!");
      while (<I>) {
        if (/^IPADDR=(.*)$/o) {
	  $oldaddr = $1;
	  print O "IPADDR=$addr\n";
	  $dida = 1;
        }
        elsif (/^NETWORK=(.*)$/o) {
	  print O "NETWORK=$network\n";
	  $didn = 1;
        }
        elsif (/^NETMASK=(.*)$/o) {
	  print O "NETMASK=$mask\n";
	  $didm = 1;
        }
        elsif (/^BROADCAST=(.*)$/o) {
	    print O "BROADCAST=$broadcast\n";
	    $didb = 1;
        }
	elsif (/^BOOTPROTO/o) {
	  # get rid of this; if we're in here, we're hard-coded
	  ;
	}
	else {
	  print O;
	}
      }
      close I;
    }
    print O "IPADDR=$addr\n" unless ($dida);
    print O "NETWORK=$network\n" unless ($didn);
    print O "NETMASK=$mask\n" unless ($didm);
    print O "BROADCAST=$broadcast\n" unless ($didb);
    close O;

    ## Rebuild the hosts file
    if (($name ne "localhost") && ($addr ne "127.0.0.1"))
    {
      my %hostlist;
      return Cobalt::Network::net_abort("3402 Cannot write hosts file: hosts file is locked") 
	if (-e "$Lockdir/hosts");
      open(HOUT,">$Lockdir/hosts")
	|| return Cobalt::Network::net_abort("3415 Cannot modify hosts file: $!");
      open(HIN,$Hostfile)
	|| return Cobalt::Network::net_abort("3401 Cannot read hosts file: $!");
      #while (<HIN>) {
      #  next if (/^\#/o);  # we don't deal well with comments
      #  my ($addr,$name) = split(/\s+/o,$_,2);
      #  $hostlist{$addr} = $name;
      #}
      #undef $hostlist{$oldaddr} if ($oldaddr ne $addr);
      #unless ($hostlist{$addr}) {
      #  $hostlist{$addr} = "$name $fullname \t# interface $netif\n";
      #}
      #print HOUT "# This file is generated and clobbered by the Cobalt interface.\n";
      #print HOUT "127.0.0.1\t", $hostlist{"127.0.0.1"};
      #undef $hostlist{"127.0.0.1"};
      #my $a;
      #foreach $a (sort keys %hostlist) {
      #  print HOUT "$a\t", $hostlist{$a} if ($hostlist{$a});
      #}
      my $printed;
      while (<HIN>) 
      {
	if ((/^$addr\s/) || (/\s$fullname(\s|$)/)) {
	  # found a line with old info about this host
	  #print HOUT "$addr\t$name $fullname \t# main interface\n" unless ($printed);
	  print HOUT "$addr\t$fullname \t# main interface\n" unless ($printed);
	  $printed = 1;
	} else {
	  print HOUT;
	}
      }
      #print HOUT "$addr\t$name $fullname \t# main interface\n" unless ($printed);
      print HOUT "$addr\t$fullname \t# main interface\n" unless ($printed);
      close HOUT;
      close HIN;
      rename("$Lockdir/hosts",$Hostfile) ||
	return Cobalt::Network::net_abort("3408 Cannot update hosts file: $!");
    }

    ## Reconfigure the interface
    print STDERR "$Ifconfig_cmd $netif inet $addr netmask $mask broadcast $broadcast\n"
	if ($Debug);
    print STDERR "$Route_cmd add -net $network netmask $mask $netif\n"
	if ($Debug);
    system("$Ifconfig_cmd $netif down 1>&2");
    system("$Ifconfig_cmd $netif inet $addr netmask $mask broadcast $broadcast 1>&2");
    system("$Route_cmd add -net $network netmask $mask $netif 1>&2");
    # We need to re-add the broadcast route if we reset eth0
    system("$Route_cmd add -net 255.255.255.255 netmask 255.255.255.255 $Netif 1>&2") if ($netif eq $Netif);
    print STDERR "Now, My name is $name and my gateway is $gw\n" if ($Debug);
    Cobalt::Network::net_set_gateway($gw,$gwif) if ($gw);

    print STDERR "Changed IP address for $netif to $addr from $oldaddr\n"
      if ($Debug);

    # Change the ip of the web server Listen directive, if there is one
    if ($Features{'www'}) {
    
      return Cobalt::Network::net_abort("3421 $Htconf/httpd.conf is locked")
        if (-e "$Lockdir/httpd.conf");
     
      open(O,">$Lockdir/httpd.conf")
        || return Cobalt::Network::net_abort("3432 Cannot modify $Htconf/httpd.conf: $!");
      open(I,"$Htconf/httpd.conf")
        || return Cobalt::Network::net_abort("3430 Cannot read $Htconf/httpd.conf: $!",
			"$Lockdir/httpd.conf");

      # There are cases when we don't know the $oldaddr, 
      #   after a lcd reset network for one, so we relax this a bit..
      my $searchstring = "Listen ";
      $searchstring = "Listen $oldaddr:80" if ($oldaddr);

      my $nomore;
      while (<I>) {
        if ($nomore) {
          print O;
	  next;
        }
        elsif (/^$searchstring/o) {
          $_ = "Listen $addr:80\n";
	  $nomore = 1;
        }
        print O;
      }
      close I;
      close O;
      rename("$Lockdir/httpd.conf","$Htconf/httpd.conf") ||
        return Cobalt::Network::net_abort("3414 Cannot update $Htconf/httpd.conf: $!");
    } # end changes to $Htconf

    # Bring all the virtual interfaces associated with this
    # physical interface back up
    my $iface;
    opendir(IFDIR,$If_cfg_dir) || 
      return Cobalt::Network::net_abort("4175 Cannot read network script dir $If_cfg_dir: $!");
    my @ifcfgs = grep(/^ifcfg-$netif:\d+/,readdir(IFDIR));
    closedir IFDIR;
    foreach $iface (@ifcfgs) {
      $iface =~ s/^ifcfg-//o;

      # Set the netmask, broadcast, and network for each virtual interface
      open(VIRTIN,"$If_cfg_dir/ifcfg-$iface") || 
	  Cobalt::Network::net_abort("4172 Cannot read $If_cfg_dir/ifcfg-$iface: $!");
      open(VIRTOUT,">$Lockdir/ifcfg-$iface") ||
	  Cobalt::Network::net_abort("4174 Cannot write $Lockdir/ifcfg-$iface: $!");
      my $vaddr; # the IP of the virtual site
      while (<VIRTIN>) {

	  # This counts on the fact that IPADDR shows up in the file before
	  # NETWORK and BROADCAST.  As the sauce is written right now, we
	  # can count on that, assuming nobody hand-edits the files.
	  # $mask is the $mask we were using for the main interface
	  if (/^IPADDR=([\d\.]+)/o) {
	      $vaddr = $1;
	      print VIRTOUT "IPADDR=$vaddr\n";
	  }
	  elsif (/^NETMASK/o) {
	      print VIRTOUT "NETMASK=$mask\n";
	  }
	  elsif (/^NETWORK/o) {
	      my $network = Cobalt::Network::net_get_network($vaddr,$mask);
	      print VIRTOUT "NETWORK=$network\n";
	  }
	  elsif (/^BROADCAST/o) {
	      my $broadcast = Cobalt::Network::net_get_broadcast($vaddr,$mask);
	      print VIRTOUT "BROADCAST=$broadcast\n";
	  }
	  else {
	      print VIRTOUT;
	  }
      }

      close VIRTIN;
      close VIRTOUT;
      rename("$Lockdir/ifcfg-$iface","$If_cfg_dir/ifcfg-$iface") ||
	  Cobalt::Network::net_abort("4173 Cannot set new ifcfg-$iface file: $!");

      # Tell the mail server to allow relaying from our network and domain
      my($ret) = Cobalt::Email::mail_add_relay($network);
      return Cobalt::Network::net_abort($ret) if ($ret !~ /^2/o);

      # Now bring the interface up (unless we're still booting up)
      system("$Ifup $iface >> $Sauce_log") if (Cobalt::Utility::sauce_booted());
    }

    # unlock
    rename("$Lockdir/ifcfg-$netif","$If_cfg_dir/ifcfg-$netif") ||
	return Cobalt::Network::net_abort("3403 Cannot update interface config file: $!");
    Cobalt::Network::net_unlock_network();
    return "2410 Changed IP address from $oldaddr to $addr";
}

sub net_get_ipaddr
# Returns the IP address, netmask, and MAC (ethernet) address of the 
#   running system, as set in the kernel.
# Arguments: an optional interface name (defaults to eth0)
# Return value: a list containing ("IP address","netmask","MAC address").
# Side effects: none
{
  # my $netif = (@_);
  my $netif = $_[0];
  $netif ||= $Netif;
  my ($ipaddr,$netmask,$hwaddr);
  open(IFCFG,"$Ifconfig_cmd $netif|")
    || return "4353 Cannot run $Ifconfig_cmd: $!";
  while (<IFCFG>)
    {
      chomp;
      if (/inet addr:([\d\.]+).+Mask:([\d\.]+)/o)
	{
	  $ipaddr = $1;
	  $netmask = $2;
	}
      elsif (/HWaddr (.+)$/o)
	{
	  $hwaddr = $1;  # get the MAC address
	}
    }
  close IFCFG;
  return ($ipaddr,$netmask,$hwaddr);
}

sub net_set_dnsclient
# Configure the DNS client settings
# Arguments: the domain name and a list of nameservers
#   If given an argument of "none", all DNS client information is wiped
# Return value: a status message of the form "600 success"
# Side effects: modifies resolv.conf
{
    my ($domain,@nameservers) = @_;
    my ($server);
    return "4373 Invalid domain name $domain"
	if ($domain !~ /^([\w\-\.]+)$/o);
    return "4363 Invalid character in name server address"
	if (grep(/[^\w\-\.]/o,@nameservers));

    # We should lock, but since we're just clobbering we're probably okay
    open (RC,">$Resolvconf") 
	|| return "4354 Cannot write $Resolvconf";
    if ($domain eq "none")
    {
	close RC;
	return "2419 emptied resolv.conf";
    }
    elsif ($domain && @nameservers)
    {
	print RC "$Domainword $domain\n";
	foreach $server (@nameservers)
	{
	    $server =~ s/(\.|^)0+(\d)/$1$2/go;  # get rid of excess leading zeros
	    print RC "nameserver $server\n" if ($server);
	}
	close RC;

	# Tell the mail server to allow relaying from our and domain
	my($ret) = Cobalt::Email::mail_add_relay($domain);
	return Cobalt::Network::net_abort($ret) if ($ret !~ /^2/o);

	return "2418 wrote resolv.conf";
    }
    close RC;

    return "2417 No change to DNS client configuration";
}

sub net_get_dnsclient
# Returns the DNS information
# Arguments: none
# Return value: the current settings in resolv.conf, as a list of the
#   form ($domain,@nameservers)
# Side effects: none
{
    my ($domain,@nameservers);
    return "" if (! -e $Resolvconf);
    open(RC,$Resolvconf) || return "4355 Cannot read $Resolvconf: $!\n";
    while (<RC>)
    {
	if (/^(domain|search) (.+)/o)
	{
	    $domain = $2;
	}
	elsif (/^nameserver (.+)/o)
	{
	    push(@nameservers,$1);
	}
    }
    close RC;
    return ($domain,@nameservers);
}

sub net_set_gateway
# Sets the default gateway
# Arguments: the address of the gateway, optional the network interface.
#   If given an argument of "none", undefine the gateway.
#   The network interface will default to eth0 if not given.
# Return value: a status string of the form "984 The dentist escaped"
# Side effects: edits the network configuration file and modifies
#   the routing information
{
    my ($gaw,$netif) = @_;

    if ($gaw =~ /^[0\.]+$/o) { return "4371 Invalid gateway $gaw"; }

    my ($gw,$gwline,$gwdevline,$oldgw,$oldgwdev,$setgw,$setgwdev);
    $netif ||= $Netif;

    print STDERR "Setting gateway to $gaw ...\n " if ($Debug);
    # Note that "0.0.0.0" is considered a valid gateway, because
    # that's the gateway the modem points things to.  0.0.0.0's
    # therefore have to be filtered out as bad input by the UI or
    # the LCD--we can't do it.
    if ($gaw eq "none")
    {
	$gw = "";
	$gwline = undef;
	$gwdevline = undef;
    }
    elsif ($gaw =~ /^(\d+\.\d+\.\d+\.\d+)$/o)
    {
	$gw = $1;
	$gwline = "GATEWAY=$gw\n";
	$gwdevline = "GATEWAYDEV=$netif\n";
    }
    else
    {
	print STDERR "Invalid gateway $gaw\n" if ($Debug);
	return "4356 Invalid gateway $gaw";
    }
    $gw =~ s/(\.|^)0+(\d)/$1$2/go;  # get rid of excess leading zeros
    my ($ipee) = Cobalt::Network::net_get_ipaddr();
    return "4357 Host cannot be its own gateway" if ($gw eq $ipee);

    return "4367 $Net_cfg is locked, cannot set gateway" 
	if (-e "$Lockdir/network");
    open(O,">$Lockdir/network")
	|| return "4369 Cannot modify $Net_cfg: $!";
    open(I,$Net_cfg) 
	|| return Cobalt::Network::net_abort("4368 Cannot read $Net_cfg: $!");
    while (<I>)
    {
        if (/^GATEWAY=(.*)$/o)
        {
	    $oldgw = $1;
	    $_ = $gwline;
	    $setgw = 1;
        }
	if (/^GATEWAYDEV=(.*)$/o) {
		$oldgwdev = $1;
		$_ = $gwdevline;
		$setgwdev = 1;
	}
	print O;
    }
    print O $gwline if ((! $setgw) && $gwline);
    print O $gwdevline if ((! $setgwdev) && $gwdevline);
    close O;
    close I;

    print STDERR "$Route_cmd del default\n" if ($Debug);
    system("$Route_cmd del default >>$Sauce_log 2>&1");
    if ($gw)
    {
	print STDERR "$Route_cmd add default gw $gw $netif\n" if ($Debug);
	# Make sure we remove the lock file before returning
	if (system("$Route_cmd add default gw $gw $netif 1>&2")) {
	  unlink("$Lockdir/network");
	  return "3416 Cannot add routing table entry";
	}
    }

    # unlock and save
    rename("$Lockdir/network",$Net_cfg) ||
	return "3409 Cannot update network file: $!";

    return "2411 Gateway changed from $oldgw to $gw";
}

sub net_get_gateway
# Returns the current gateway.
# Arguments: optional network interface argument
# Return value: the IP address of the gateway, the network interface
# Side effects: none
{
  my ($gw,$gwif);
  open(ROUTE,"/proc/net/route") 
    || return "4359 Cannot read routing table: $!";
  while (<ROUTE>)
    {
      if (/^(\w+)\s+00000000\s+(\w\w)(\w\w)(\w\w)(\w\w)\s/)
	{
	  my ($a,$b,$c,$d) = map(hex,($5,$4,$3,$2));
	  $gw = "$a.$b.$c.$d";
	  $gwif = $1;
	  last;
	}
    }
  close ROUTE;
  undef $gwif unless ($gw);  # can't return a interface w/o an address
  return ($gw,$gwif);
}

sub net_get_interfaces
# Returns a list of physical network interfaces
# Arguments: none
# Return value: a list of network interfaces
{
  # we use a hash table to make sure we get a unique list
  # (eth0 doesn't show up many times if we have virtual interfaces)
  my %ifs;
  open(NS,"/proc/net/dev") 
    || return;
    # callers expects real interface names
#    || return "4358 Cannot read /proc/net/dev: $!";
  while (<NS>)
    {
      if (/^\s*(eth\d+):/o)
        {
	  $ifs{$1} = 1 unless(/No statistics available/o);
        }
    }
  close NS;
  return keys(%ifs);
}

sub net_get_alladdrs
# Lists all IP addresses to which this system answers
# This includes all physical and virtual addresses (but not loopback).
# Arguments: none
# Side effects: none
# Return value: a list of IP addresses
{
  my @addrs;
  open(IFCFG,"$Ifconfig_cmd -a|")
    || return "4999 network_cant_ifconfig";
  while (<IFCFG>)
    {
      if (/inet addr:([\d\.]+).+Mask:/o)
	{
	  push(@addrs,$1) unless ($1 eq "127.0.0.1");
	}
    }
  close IFCFG;
  return @addrs;
}

sub net_down_interface
# bring down a network interface
# Arguments: network interface
# Side effects: ifconfig down the interface and
#               remove interface associated files
# Return value: a status message of the form "600 success"
{
    my ( $netif )=@_;

    # lock it first
    if( !Cobalt::Network::net_lock_network() ) {
	return "4393 Cannot reconfigure network: system busy";
    }

    if( system( "$Ifconfig_cmd $netif down 1>&2" )!=0 ) {
	return Cobalt::Network::net_abort( "4394 Cannot bring down network interface $netif" );
    }

    # clean up network script
    unlink "$If_cfg_dir/ifcfg-$netif";

    Cobalt::Network::net_unlock_network();

    return "2068 Network interface brought down successfully";
}

#### Internal functions

sub net_get_broadcast
# Returns the standard broadcast address which would go with this network
#   We assume that we should use the high address, because everyone does now.
# Arguments: IP address and netmask, in dotted quad notation
# Return value: broadcast address, in dotted quad notation
{
    my ($addr,$netmask) = @_;
    my ($a1,$a2,$a3,$a4) = map(pack('C',$_),split(/\./o,$addr));
    my ($m1,$m2,$m3,$m4) = map(pack('C',$_),split(/\./o,$netmask));
    my ($n1,$n2,$n3,$n4,$b1,$b2,$b3,$b4,$f,$bcast);
    $n1 = unpack('C',($m1 & $a1));
    $n2 = unpack('C',($m2 & $a2));
    $n3 = unpack('C',($a3 & $m3));
    $n4 = unpack('C',($a4 & $m4));
    $f = pack('C',0xff);   # 0xff = 255
    $b1 = $n1 + unpack('C',($m1 ^ $f));
    $b2 = $n2 + unpack('C',($m2 ^ $f));
    $b3 = $n3 + unpack('C',($m3 ^ $f));
    $b4 = $n4 + unpack('C',($m4 ^ $f));
    $bcast = "$b1.$b2.$b3.$b4";
    return $bcast;
}

sub net_get_network
# Returns the network address which would go with this IP and netmask
# Arguments: IP address and netmask, in dotted quad notation
# Return value: network address, in dotted quad notation
{
    my ($addr,$netmask) = @_;
    my ($a1,$a2,$a3,$a4) = map(pack('C',$_),split(/\./o,$addr));
    my ($m1,$m2,$m3,$m4) = map(pack('C',$_),split(/\./o,$netmask));
    my ($n1,$n2,$n3,$n4,$network);
    $n1 = unpack('C',($m1 & $a1));
    $n2 = unpack('C',($m2 & $a2));
    $n3 = unpack('C',($a3 & $m3));
    $n4 = unpack('C',($a4 & $m4));
    $network = "$n1.$n2.$n3.$n4";
    return $network;
}

sub net_lock_network
# We only want one person or process mucking with the network configuration
# at a time.  Use this to lock things.
# Arguments: none
# Return value: true if we get the lock, false if we can't get the lock
# Side effects: creates a lock file in $Lockdir
{
    # sometimes there are race conditions where another net_set function
    # still hasn't returned by the time we get called.  Give that extra
    # pause to let things settle if things look bad.
    my $i;
    # Poll for the netlock to go away
    for ($i = 0; $i < 15; $i++) {
      last if (! -e $Netlock);
      sleep 2;
    } 
    return 0 if (-e $Netlock);
    open(L,">$Netlock") || return 0;
    print L "$$\n$0\n";
    close L;
    return 1;
}

sub net_unlock_network
# When we're done, we unlock
# Arguments: none
# Return value: none
# Side effects: removes lock files
{
  my $netif;
  foreach $netif (Cobalt::Network::net_get_interfaces()) {
    unlink("$Lockdir/ifcfg-$netif");
  }
  unlink($Netlock);
}

sub net_abort
# Remove locks as we abort
# Arguments: error string, list of additional lockfiles to clean
# Return value: the error string which it was passed
# Example: return Cobalt::Network::net_abort("Help me!","$Lockdir/other.lock");
{
    my ($message,@locks) = @_;
    unlink($Netlock,@locks) || print STDERR "Help!  Can't unlock $Netlock\n";
    open(LOG,">>$Sauce_log") || open(LOG,">>$Console");
    print LOG "$0 aborting with $message\n";
    close LOG;
    return $message;
}

sub net_convert_netmask
# Convert netmask from decimal notation to dotted quad (16 => 255.255.0.0) or vice versa
# Arguments: netmask in decimal, or netmask in dotted quad
# Return value; netmask in dotted quad if decimal argument given, or netmask in
#      decimal if dotted quad argument is given
# Example: $quadmask = Cobalt::Network::net_convert_netmask(26);
{
  my ($mask) = @_;
  # I should be able to do this in some cool mathematical way.
  # If only I knew how.
  my %netmasks = (
		  32 => "255.255.255.255", 31 => "255.255.255.254",
		  30 => "255.255.255.252", 29 => "255.255.255.248",
		  28 => "255.255.255.240", 27 => "255.255.255.224",
		  26 => "255.255.255.192", 25 => "255.255.255.128",
		  24 => "255.255.255.0",   23 => "255.255.254.0",
		  22 => "255.255.252.0",   21 => "255.255.248.0",
		  20 => "255.255.240.0",   19 => "255.255.224.0",
		  18 => "255.255.192.0",   17 => "255.255.128.0",
		  16 => "255.255.0.0",     15 => "255.254.0.0",
		  14 => "255.252.0.0",     13 => "255.248.0.0",
		  12 => "255,240.0.0",     11 => "255.224.0.0",
		  10 => "255.192.0.0",      9 => "255.128.0.0",
		   8 => "255.0.0.0",        7 => "254.0.0.0",
		   6 => "252.0.0.0",        5 => "248.0.0.0",
		   4 => "240.0.0.0",        3 => "224.0.0.0",
		   2 => "192.0.0.0",        1 => "128.0.0.0",
		   0 => "0.0.0.0",

		  "0.0.0.0" => 0,          "128.0.0.0" => 1,
		  "192.0.0.0" => 2,        "224.0.0.0" => 3,
		  "240.0.0.0" => 4,        "248.0.0.0" => 5,
		  "252.0.0.0" => 6,        "254.0.0.0" => 7,
		  "255.0.0.0" => 8,        "255.128.0.0" => 9,
		  "255.192.0.0" => 10,     "255.224.0.0" => 11,
		  "255,240.0.0" => 12,     "255.248.0.0" => 13,
		  "255.252.0.0" => 14,     "255.254.0.0" => 15,
		  "255.255.0.0" => 16,     "255.255.128.0" => 17,
		  "255.255.192.0" => 18,   "255.255.224.0" => 19,
		  "255.255.240.0" => 20,   "255.255.248.0" => 21,
		  "255.255.252.0" => 22,   "255.255.254.0" => 23,
		  "255.255.255.0" => 24,   "255.255.255.128" => 25,
		  "255.255.255.192" => 26, "255.255.255.224" => 27,
		  "255.255.255.240" => 28, "255.255.255.248" => 29,
		  "255.255.255.252" => 30, "255.255.255.254" => 31,
		  "255.255.255.255" => 32,
		 );
  
  return $netmasks{$mask};
}

sub net_network_ismember
# Is the given IP within the given network?
# Arguments: IP, network address, network mask (either decimal or dotted)
# Side effects: none
# Return value: 1 for yes, 0 for no
{
  my ($ip,$net,$mask) = @_;

  # deal with things properly whatever form the netmask is in
  $mask = Cobalt::Network::net_convert_netmask($mask) if ($mask =~ /^\d+$/o);
  return (Cobalt::Network::net_get_broadcast($ip,$mask) eq Cobalt::Network::net_get_broadcast($net,$mask)) ?
    (1) : (0);
}

sub net_get_forwarding
# Find out wether IP forwarding is set to on or off
# Arguments: none
# Return value: 1 if forwarding is allowed, 0 if we do not forward
# Side effects: none
{
  open(I,$Net_cfg) || return 0;
  while (<I>) {
    if (/^FORWARD_IPV4=false/o) {
      # we don't forward
      close I;
      return 0;
    } elsif (/^FORWARD_IPV4=true/o) {
      # we do forward
      close I;
      return 1;
    }
  }
  # defaults to no if its not explicitly set
  close I;
  return 0;
}

sub net_set_forwarding
# Turn IP forwarding between interfaces on or off
# Arguments: 1 to turn forwarding on, 0 to not forward packets
# Return value: status string of the form "4934 Never leave the room with
#   your hand still glued to the telephone."
# Side effects: Modifies /etc/sysconfig/network to set boot-time configuration,
#   and sets the kernel forwarding flag via writing to the appropriate file
#   in /proc.
{
  my ($we_forward_packets) = @_;
  return Cobalt::Network::net_abort("4999 network_cant_netcfg_locked")
    if (-e "$Lockdir/network");
  open(O,">$Lockdir/network")
    || return Cobalt::Network::net_abort("4999 network_cant_write_netcfg");
  open(I,$Net_cfg) 
    || return Cobalt::Network::net_abort("4999 network_cant_read_netcfg");
  my $done = 0;
  while (<I>) {
    if (/^FORWARD_IPV4=(\w+)$/o) {
      ($we_forward_packets) ? 
	($_ = "FORWARD_IPV4=true\n") : ($_ = "FORWARD_IPV4=false\n");
      $done = 1;
    }
    print O;
  }
  close I;
  unless ($done) {
    ($we_forward_packets) ? 
      (print O "FORWARD_IPV4=true\n") : (print O "FORWARD_IPV4=false\n");
  }
  close O;
  rename("$Lockdir/network",$Net_cfg) ||
    return Cobalt::Network::net_abort("4999 network_cant_update_netcfg");
  open(F,">$ForwardFile") || return "3999 network_cant_forward_kernel";
  print F "$we_forward_packets\n";
  close F;
  return "2999 network_ok_forwarding";
}

sub net_set_vinterface
# Configure virtual network interfaces - Automatically invoked by Vsite.pm
# Arguments: the new IP address, netmask, 
#   optional physical interface name (defaults to eth0)
# Side effects: edits the files in /etc/sysconfig
#   Reconfigures the network interface for the new address
# Return value: status string of the form "4346 You don't get a wish
#   because you don't have your own brain."
{
    my ($addr,$mask,$netif) = @_;

    $netif ||= $Netif;

    # $addr =~ s/(\.|^)0+(\d)/$1$2/go;  # get rid of excess leading zeros
    
    if( !Cobalt::Netutil::netutil_test_ipaddr( $addr ) )
    {
        print STDERR "Got invalid IP address $addr\n" if ($Debug);
        return "4140 $addr is not a valid IP address";
    }
    if (Cobalt::Network::net_network_ismember($addr,"224.0.0.0","240.0.0.0")) {
      # This is an address on the multicast network
      return "4999 vsite_invalid_ismulticast";
    }

    my $broadcast = Cobalt::Network::net_get_broadcast($addr,$mask);
    my $network = Cobalt::Network::net_get_network($addr,$mask);
    my $main_addr = (Cobalt::Network::net_get_ipaddr())[0];

    # Verify $addr is not already in use
    my(@all_IPs) = Cobalt::Network::net_get_alladdrs();
    my($ip_instance,%active_IPs);
    foreach $ip_instance (@all_IPs) { $active_IPs{"$ip_instance"}=1; }
    return "4999 network_cant_ipalreadyinuse" 
      if ($active_IPs{$addr});

    # This is a new interface
    my @siteslist = Cobalt::Vsite::vsite_list();
    return "4598 This system is limited to $Sites_Max virtual sites."
      if (($#siteslist + 1) >= $Sites_Max);  # plus one to un-zero-index

    # Get the number for the next interface to add
    my ($ifnum,$virtif);
    opendir(IFDIR,$If_cfg_dir) ||
      return "4143 Cannot read network script dir $If_cfg_dir: $!";
    my @ifcfgs = sort {$a <=> $b } (grep(/^ifcfg-$netif:\d+$/o,readdir(IFDIR)));
    closedir IFDIR;
    for ($ifnum = 0; $ifnum <= $#ifcfgs; $ifnum++)
    {
        if ($ifcfgs[$ifnum] ne "ifcfg-$netif:$ifnum")
        {
            $virtif = "$netif:$ifnum";
            last;
        }
    }
    $virtif ||= "$netif:" . ($#ifcfgs + 1);
    print STDERR "Allocating new interface $virtif for $addr\n"
      if ($Debug);

    print STDERR "New settings for $virtif: $addr/$mask\n"
        if ($Debug);
    # need the general lock NO MATTER WHAT
    return "4379 Cannot reconfigure network: system busy"
      unless(Cobalt::Network::net_lock_network());
    return Cobalt::Network::net_abort("4144 Cannot configure virtual interface: ifcfg-$virtif is locked")
      if (-e "$Lockdir/ifcfg-$virtif");
    open(O,">$Lockdir/ifcfg-$virtif")
      || return Cobalt::Network::net_abort("4145 Cannot get lock for interface $virtif: $!");

    print O "DEVICE=$virtif\n";
    print O "IPADDR=$addr\n";
    print O "NETMASK=$mask\n";
    print O "NETWORK=$network\n";
    print O "BROADCAST=$broadcast\n";
    print O "ONBOOT=yes\nALIAS=yes\nBOOTPROTO=none\n";
    close O;

    # unlock
    unless (rename("$Lockdir/ifcfg-$virtif","$If_cfg_dir/ifcfg-$virtif"))
    {
        return Cobalt::Network::net_abort("3145 Cannot update virtual interface config file: $!",
                         "$Lockdir/ifcfg-$virtif");
    }

    # Bring the interface up
    print STDERR "$Ifup $virtif\n" if ($Debug);
    system("$Ifup $virtif >>$Sauce_log 2>&1\n");

    Cobalt::Network::net_unlock_network();

    return "2140 Set up virtual interface for address $addr";
}

sub net_del_vinterface
# Deletes a virtual interface - Automatically invoked by Vsite.pm
# Arguments: IP address of network interface
# Return value: return code + string of the form "2000 Ok"
# Example: $ret = Cobalt::Network::net_del_vinterface('eth0:4');
{
    my ($addr) = @_;

    # Determine the virtual network interface
    my($virtif,$netif);
    open(IFCFG,"$Ifconfig_cmd -a|")
      || return "4999 network_cant_ifconfig";
    while (<IFCFG>)
    {
	$netif = $1 if (/^(eth[\d\:]+)\s+Link/);
	$virtif = $netif if (/^\s*inet addr:$addr\s+/);
    }
    close IFCFG;

    # Avoid the ugly..
    undef $virtif if ($virtif eq 'lo'); 
    return "4999 network_cant_ipnotinuse" unless ($virtif);
    return "2999 network_invalid_notvinterface" if ($virtif eq 'eth0');
    return "2999 network_invalid_notvinterface" if ($virtif eq 'eth1');
    return "4154 Cannot read $If_cfg_dir/ifcfg-$virtif: $!" unless (-r "$If_cfg_dir/ifcfg-$virtif");

    # get the main network lock file (and the rest of our locks)
    return "4386 Cannot reconfigure network: system busy"
      unless(Cobalt::Network::net_lock_network());
    return "4150 Cannot remove interface: ifcfg-$virtif is locked"
        if (-e "$Lockdir/ifcfg-$virtif");
    open(O,">$Lockdir/ifcfg-$virtif")
        || return "4151 Cannot lock interface config ifcfg-$virtif: $!";
    close O;

    system("$Ifdown $virtif >>$Sauce_log 2>&1");
    unlink("$If_cfg_dir/ifcfg-$virtif");
    unlink("$Lockdir/ifcfg-$virtif");

    Cobalt::Network::net_unlock_network();

    return "2142 Virtual interface deleted successfully";
}

1;
