#
# Written by Brion Moss (brion@cobaltnet.com, brion@nda.com)
#
# Cobalt Networks http://www.cobaltnet.com
#

package Cobalt::Quota;
use AutoLoader 'AUTOLOAD';
BEGIN {
require Exporter;
@ISA    = qw(Exporter);
@EXPORT = qw(setqlim getqcarg repquota);
}

#use Cobalt::Paths;
#use strict;
1;
__END__

sub QCMD
# From quota.h
{
    my ($cmd, $type) = @_;
    $type = 0 unless ($type);  # default to type user
    return ((($cmd) << $SUBCMDSHIFT) | (($type) & $SUBCMDMASK));
}

sub setqlim
# Set the quota for a user or group
# Arguments: device on which to set quotas, uid, soft limit, hard limit,type
#   (type = 0 for users, 1 for groups)
# Side effects: modifies the quota tables
{
    my ($dev,$uid,$soft,$hard,$type) = @_;
    return "1999 quota_unsupported_noquotas" unless ($Features{'quotas'});
    my ($stat);
    my ($dqblk)  = pack("LLLLLLLL",0,0,0,0,0,0,0,0); 
    $stat=syscall($SYS_quotactl,Cobalt::Quota::QCMD($Q_GETQUOTA,$type),$dev,$uid+0,$dqblk); 
    return "4999 quota_cant_get_user_quota" if ($stat);
    my (@vals) = unpack("LLLLLLLL",$dqblk);
    $vals[1] = $soft;
    $vals[0] = $hard if ($soft);
    $dqblk  = pack("LLLLLLLL",@vals); 
    $stat=syscall($SYS_quotactl,Cobalt::Quota::QCMD($Q_SETQLIM,$type),$dev,$uid+0,$dqblk); 
    return "4999 quota_cant_set_user_quota" if ($stat);
    $stat=syscall($SYS_quotactl,Cobalt::Quota::QCMD($Q_SYNC,$type),$dev,$uid+0,$dqblk); 
    return "4999 quota_cant_sync_user_quota" if ($stat);
    return "2999 quota_ok_user_set";
}

sub getqcarg
# converts a directory name into the matching device on which to set quotas
# Arguments: a directory
# Return value: a device name (e.g. /dev/hda1)
# Side effects: calls df
# This function isn't used; I hard-coded everything into Paths.pm
{
    my ($dir) = @_;
    $dir = $Htusers_dir if (! -d $dir);
    open(DF,"$Df_cmd $dir|") 
	|| return "4999 quota_cant_df";
    while (<DF>)
    {
	if (/^(\/dev\/\w+)/o)
	{
	    close DF;
	    return $1;
	}
    }
    return "4999 quota_cant_getfs";
}

sub repquota
# Report on the quota for a user or group
# Arguments: user/group name and type (type=0 for user, 1 for group)
#   Optional args: ID, homedir
#   If ID and homedir are not given, read them from the password or
#   group file.
# Return value: array: (used, soft limit, hard limit)
# In practice, hard limit and soft limit will always be the same
# Example: ($used,$soft,$hard) = Cobalt::Quota::repquota("john",0,9124,"/home/users/john");
{
    my ($user,$type,$id,$home) = @_;
    return "1999 quota_unsupported_noquotas" unless ($Features{'quotas'});
    unless ($id) {
      if ($type)
      {
	($id) = (getgrnam($user))[2];
	$home = "$Htgroup_dir/$user";
      }
      else
      {
	($id,$home) = (getpwnam($user))[2,7];
      }
    }
    my ($dqblk)  = pack("LLLLLLLL",0,0,0,0,0,0,0,0); 
    my $stat=syscall($SYS_quotactl,Cobalt::Quota::QCMD($Q_GETQUOTA,$type+0),
		     $Userpart,$id+0,$dqblk); 
    return "4999 quota_cant_get_quota" if ($stat);
    my (@vals) = unpack("LLLLLLLL",$dqblk);
    # The UI doesn't understand that 0 means no quota, so instead of returning
    # 0 to indicate no quota, we return a really big number (e.g. the size of 
    # the disk)
    unless ($vals[0] && $vals[1]) {
      my $maxquota;
      $home = $Htgroup_dir unless (-d $home);
      if (open (DF, "$Df_cmd $home|")) {
	my $header = <DF>; # toss the header line
	while (<DF>) {
	  $maxquota = (split(/\s+/o))[3];
	  # make sure it's divisible by the size of a block
	  $maxquota = int($maxquota / $BytesPerBlock) * $BytesPerBlock; 
	}
	close DF;
      }
      $maxquota ||= $MaxQuota; # couldn't df for some reason (no home directory?)
      $vals[0] = $maxquota unless ($vals[0]);
      $vals[1] = $maxquota unless ($vals[1]);
    }
    return($vals[2],$vals[1],$vals[0]);
}

1;
