#
# $Id: Security.pm,v 1.9 1999/01/21 20:00:17 gsri Exp $
#
# Written by Brion Moss (brion@cobaltnet.com, brion@nda.com)
#
# Copyright 1998 Cobalt Networks http://www.cobaltnet.com
#

package Cobalt::Security;
use AutoLoader 'AUTOLOAD';
BEGIN {
require Exporter;
@ISA    = qw(Exporter);
@EXPORT = qw(set_user_management set_group_management);
}

#use Cobalt::Paths;
#use strict;
1;
__END__

sub security_set_user_management
# Allow or disallow anyone in the world to manage users
# Arguments: 1 to allow anyone in the world to manage users,
#   0 to allow only user "admin" to manage users
# Return value: usual string of form "785 Green is a pretty color"
# Side effects: creates or deletes .htaccess files as appropriate
{
    my ($setting) = @_;
    return "1400 Users not implemented" unless ($Features{'users'});
    if ($setting)
    {
	# allow anyone
	if (! -e "$Users_dir/$Access_file")
	{
	    return "2441 No change to user management";
	}
	else 
	{
	    unlink("$Users_dir/$Access_file") ||
		return "4401 Cannot delete Users_dir/$Access_file: $!";
	    return "2447 Anyone in the world can manage users";
	}
    }
    else
    {
	# allow admin only
	if (-e "$Users_dir/$Access_file")
	{
	    return "2443 No change to user management";
	}
	else 
	{
	    open(UXS,">$Users_dir/$Access_file") ||
		return "4402 Cannot write Users_dir/$Access_file: $!";
	    print UXS "<LIMIT GET>\norder allow,deny\nallow from all\n";
	    print UXS "require user $Adminuser\n</LIMIT>\n";
	    print UXS "Authname $Adm_Authname\n";
	    print UXS "Authtype Basic\n";
    	    unless ($Features{'pam'}) {
	    print UXS "AuthUserFile $Htpasswd\n";
	    print UXS "AuthGroupFile $Htgroup\n";
	    }
	    close UXS;
	    return "2449 Only the administrator may manage users";
	}
    }
}


sub security_set_group_management
# Allow or disallow anyone in the world to manage groups
# Arguments: 1 to allow anyone in the world to manage groups,
#   0 to allow only user "admin" to manage groups
# Return value: usual string of form "924 I eat chalk"
# Side effects: creates or deletes .htaccess files as appropriate
{
    my ($setting) = @_;
    return "1401 Groups not implemented" unless ($Features{'groups'});
    if ($setting)
    {
	# allow anyone
	if (! -e "$Groups_dir/$Access_file")
	{
	    return "2442 No change to group management";
	}
	else 
	{
	    unlink("$Groups_dir/$Access_file") ||
		return "4403 Cannot delete Groups_dir/$Access_file: $!";
	    return "2440 Anyone in the world can manage groups";
	}
    }
    else
    {
	# allow admin only
	if (-e "$Groups_dir/$Access_file")
	{
	    return "2444 No change to group management";
	}
	else 
	{
	    open(GXS,"$Groups_dir/$Access_file") ||
		return "4404 Cannot write Groups_dir/$Access_file: $!";
	    print GXS "<LIMIT GET>\norder allow,deny\nallow from all\n";
	    print GXS "require user $Adminuser\n</LIMIT>\n";
	    print GXS "Authname $Adm_Authname\n";
	    print GXS "Authtype Basic\n";
            unless ($Features{'pam'}) {
            print GXS "AuthUserFile $Htpasswd\n";
	    print GXS "AuthGroupFile $Htgroup\n";
            }
	    close GXS;
	    return "2448 Only the administrator may manage groups";
	}
    }
}

1;
