#
# $Id: Services.pm,v 1.38 1999/03/04 22:58:20 gsri Exp $
#
# Written by Brion Moss (brion@cobaltnet.com, brion@nda.com)
#            Kevin K.M. Chiu (kevin@cobaltnet.com)
#
# Copyright 1998 Cobalt Networks http://www.cobaltnet.com
#

package Cobalt::Services;
use AutoLoader 'AUTOLOAD';
BEGIN {
require Exporter;
@ISA    = qw(Exporter);
@EXPORT = qw(service_set_active   service_get_active
	     set_atalk_server_on  get_atalk_server_on
	     set_dhcp_server_on   get_dhcp_server_on
	     set_dns_server_on    get_dns_server_on
	     set_ftp_server_on    get_ftp_server_on
	     set_fpx_on           get_fpx_on
	     set_gated_on         get_gated_on
             set_gated_interfaces get_gated_interfaces
	     set_mail_server_on   get_mail_server_on
	     set_networker_client_on get_networker_client_on
	     set_nfs_server_on    get_nfs_server_on
	     set_nfs_perms        get_nfs_perms
	     set_ntp_server_on    get_ntp_server_on
	     set_smb_server_on    get_smb_server_on
	     set_smb_workgroup    get_smb_workgroup
	     set_smb_controller   get_smb_controller
	     set_snmp_server_on   get_snmp_server_on
	     set_snmp_community   get_snmp_community
	     set_telnet_server_on get_telnet_server_on
	     set_telnet_server_open get_telnet_server_open
	    );

require Cobalt::Language;
import Cobalt::Language qw(:DEFAULT);
require Cobalt::Network;
import Cobalt::Network qw(net_get_ipaddr net_set_ipaddr);
}

#use Cobalt::Paths;
#use strict;
1;
__END__

sub service_set_active
# Turn the given service on or off
# Arguments: service name, 1 for on/0 for off
# Side effects: sets/breaks symlinks for the boot scripts, starts up
#   the server
# Return value: status string of the form "2821 Walk on the grass"
# Example: $ret = service_enable("dns",1);
{
  my ($service,$state) = @_;
  if ($service eq "mail") {
    return Cobalt::Services::set_mail_server_on($state);
  }
  if ($service eq "atalk") {
    return Cobalt::Services::set_atalk_server_on($state);
  }
  if ($service eq "dhcp") {
    return Cobalt::Services::set_dhcp_server_on($state);
  }
  if ($service eq "dns") {
    return Cobalt::Services::set_dns_server_on($state);
  }
  if ($service eq "ftp") {
    return Cobalt::Services::set_ftp_server_on($state);
  }
  if ($service eq "gated_on") {
    return Cobalt::Services::set_gated_on($state);
  }
  if ($service eq "networker") {
    return Cobalt::Services::set_networker_client_on($state);
  }
  if ($service eq "nfs") {
    return Cobalt::Services::set_nfs_server_on($state);
  }
  if ($service eq "ntp") {
    return Cobalt::Services::set_ntp_server_on($state);
  }
  if ($service eq "smb") {
    return Cobalt::Services::set_smb_server_on($state);
  }
  if ($service eq "snmp") {
    return Cobalt::Services::set_snmp_server_on($state);
  }
  if ($service eq "telnet") {
    return Cobalt::Services::set_telnet_server_on($state);
  }
}

sub service_get_active
# Get the state of the service
# Arguments: service name
# Side effects: none
# Return value: 1 for on, 0 for off
# Example: $ret = service_enable("dns",1);
{
  my ($service) = @_;
  if ($service eq "mail") {
    return Cobalt::Services::get_mail_server_on();
  }
  if ($service eq "atalk") {
    return Cobalt::Services::get_atalk_server_on();
  }
  if ($service eq "dhcp") {
    return Cobalt::Services::get_dhcp_server_on();
  }
  if ($service eq "dns") {
    return Cobalt::Services::get_dns_server_on();
  }
  if ($service eq "ftp") {
    return Cobalt::Services::get_ftp_server_on();
  }
  if ($service eq "gated_on") {
    return Cobalt::Services::get_gated_on();
  }
  if ($service eq "networker") {
    return Cobalt::Services::get_networker_client_on();
  }
  if ($service eq "nfs") {
    return Cobalt::Services::get_nfs_server_on();
  }
  if ($service eq "ntp") {
    return Cobalt::Services::get_ntp_server_on();
  }
  if ($service eq "smb") {
    return Cobalt::Services::get_smb_server_on();
  }
  if ($service eq "snmp") {
    return Cobalt::Services::get_snmp_server_on();
  }
  if ($service eq "telnet") {
    return Cobalt::Services::get_telnet_server_on();
  }
}

sub set_mail_server_on
# Turn the mail server on or off
# Arguments: 1 for on, 0 for off
# Return value: a status string of the form "778 can't tapdance"
# Side effects: creates or breaks symlinks in /etc/rc.d/rc3.d
{
    my ($state) = @_;
    return "1100 Mail service not implemented in this release."
      unless ($Features{'mail'});

    if ($state && (! -e $Maillink))
    {
	symlink($Mailscript,$Maillink) ||
	    return "4420 Cannot create link to $Mailscript: $!";
	return "4470 $Inetd_conf is locked"
	    if (-e "$Lockdir/inetd.conf");
	open(O,">$Lockdir/inetd.conf")
	    || return "4433 Cannot write $Lockdir/inetd.conf: $!";
	open(I,$Inetd_conf)
	    || return "4432 Cannot read $Inetd_conf: $!";
	while (<I>)
	{
	    if (/^\#(pop-3|imap)/o)
	    {
		$_ =~ s/^\#//o;
	    }
	    print O;
	}
	close I;
	close O;
	rename("$Lockdir/inetd.conf",$Inetd_conf)
	    || return "4434 Cannot replace $Inetd_conf: $!";
	open(PID,$Inetd_pid) || return "3458 Network server down: $!";
	my $line = <PID>;
	my $pid;
	if ($line =~ /^(\d+)$/o)
	{
	    $pid = $1;
	}
	close PID;
	kill(1,$pid) if ($pid);
	system ("$Mailscript start >> $Sauce_log 2>&1") &&
	    return "3455 Cannot start mail server: $!";
	system("$Newaliases_cmd >>$Sauce_log 2>&1");
	system ("$Lcdshowscript start >> $Sauce_log 2>&1")
	    if (-x $Lcdshowscript);
	return "2450 Mail server started";
    }
    elsif ((! $state) && (-e $Maillink))
    {
	unlink($Maillink) ||
	    return "3457 Cannot break link $Maillink: $!";
	return "4416 $Inetd_conf is locked"
	    if (-e "$Lockdir/inetd.conf");
	open(O,">$Lockdir/inetd.conf")
	    || return "4437 Cannot write $Lockdir/inetd.conf: $!";
	open(I,$Inetd_conf)
	    || return "4436 Cannot read $Inetd_conf: $!";
	while (<I>)
	{
	    if (/^(pop-3|imap)/o)
	    {
		$_ =~ s/^/\#/o;
	    }
	    print O;
	}
	close I;
	close O;
	rename("$Lockdir/inetd.conf",$Inetd_conf)
	    || return "4438 Cannot replace $Inetd_conf: $!";
	open(PID,$Inetd_pid) || return "3456 Network server down: $!";
	my ($pid);
	my $line = <PID>;
	if ($line =~ /^(\d+)$/o)
	{
	    $pid = $1;
	}
	close PID;
	kill(1,$pid) if ($pid);
	system ("$Mailscript stop >> $Sauce_log 2>&1") &&
	    return "4421 Cannot stop mail server: $!";
	system ("$Lcdshowscript start >> $Sauce_log 2>&1")
	    if (-x $Lcdshowscript);
	return "2452 Mail server stopped";
    }
    else
    {
	my $off;
	($state) ? ($off = Cobalt::Language::msg_get('On_string')) : 
	  ($off = Cobalt::Language::msg_get('Off_string'));
	return "2451 No change -- mail server was already $off";
    }
}

sub get_mail_server_on
# Is the mail server set to be on?
# Returns 1 if the mail server is activated, 0 if deactivated
#   This reflects what ought to be, *not* what is
# Arguments: none
# Side effects: none
{
    (-e $Maillink) ? (return 1) : (return 0);
}

sub set_smb_server_on
# Turn the smb server on or off
# Arguments: 1 for on, 0 for off
# Return value: a status string of the form "682 insufficient depth perception"
# Side effects: creates or breaks symlinks in /etc/rc.d/rc3.d
{
    my ($state) = @_;

    return "1101 Windows filesharing is not implemented in this release"
      unless ($Features{'smb'});

    if ($state && (! -e $Smblink))
    {
	my $pid;
	if (!defined($pid = fork)) {
	    die "cannot fork: $!";
	} elsif ($pid) {
	    # i'm the parent
	    print STDERR "Forking to restart samba\n" if ($Debug);
	    return "2493 Smb server started";
	}
	else
	{
	    # otherwise I'm the child
	    open(STDERR,">>$Sauce_log");
	    open(STDOUT,">>$Sauce_log");
	    # Note that we die() instead of return() because we are
	    # a child process, and if we return we'll end up back in
	    # the CGI, executing the same code our parent process
	    # is executing.  That would be Bad.
	    symlink($Smbscript,$Smblink) ||
	      die "4423 Cannot create link $Smblink: $!";
	    system ("$Smbscript start >> $Sauce_log 2>&1") &&
	      die "3493 Cannot start smb server: $!";
	    system ("$Lcdshowscript start >> $Sauce_log 2>&1")
	      if (-x $Lcdshowscript);
	    exit;
	}
    }
    elsif ((! $state) && (-e $Smblink))
    {
	unlink($Smblink) ||
	    return "3497 Cannot break link $Smblink: $!";
	system ("$Smbscript stop >> $Sauce_log 2>&1") &&
	    return "4424 Cannot stop smb service: $!";
	system ("$Lcdshowscript start >> $Sauce_log 2>&1")
	    if (-x $Lcdshowscript);
	return "2497 Smb server stopped";
    }
    else
    {
	my $off;
        ($state) ? ($off = Cobalt::Language::msg_get('On_string')) : 
		    ($off = Cobalt::Language::msg_get('Off_string'));
	return "2491 No change -- smb server was already $off";
    }
}

sub get_smb_server_on
# Is the smb server set to be on?
# Returns 1 if the smb server is activated, 0 if deactivated
#   This reflects what ought to be, *not* what is
# Arguments: none
# Side effects: none
{
    (-e $Smblink) ? (return 1) : (return 0);
}

sub set_nfs_server_on
# Turn the nfs server on or off
# Arguments: 1 for on, 0 for off
# Return value: a status string of the form "882 insufficient depth perception"
# Side effects: creates or breaks symlinks in /etc/rc.d/rc3.d
{
    my ($state) = @_;
    
    return "1102 NFS is not implemented in this release"
      unless($Features{'nfs'});
    
    if ($state && (! -e $Nfslink))
    {
	my $pid;
	if (!defined($pid = fork)) {
	    die "cannot fork: $!";
	} elsif ($pid) {
	    # i'm the parent
	    print STDERR "Forking to start NFS\n" if ($Debug);
	    return "2494 Nfs server started";
	}
	else
	{
	    # otherwise I'm the child
	    open(STDERR,">>$Sauce_log");
	    open(STDOUT,">>$Sauce_log");
	    symlink($Nfsscript,$Nfslink) ||
	      die "4425 Cannot create nfs link: $!";
	    symlink($Portmap_script,$Portmap_link) ||
	      die "3434 Cannot create portmap link: $!";
	    system("$Portmap_script start >>$Sauce_log 2>&1") &&
	      die "3435 Cannot start portmapper: $!";
	    system ("$Nfsscript start") &&
	      die "3494 Cannot start nfs server: $!";
	    system ("$Lcdshowscript start >> $Sauce_log 2>&1")
	      if (-x $Lcdshowscript);
	    exit;
	}
    }
    elsif ((! $state) && (-e $Nfslink))
    {
	my $pid;
	if (!defined($pid = fork)) {
	    die "cannot fork: $!";
	} elsif ($pid) {
	    # i'm the parent
	    print STDERR "Forking to stop NFS\n" if ($Debug);
	    return "2496 Nfs server stopped";
	}
	else
	{
	    # otherwise I'm the child
	    open(STDERR,">>$Sauce_log");
	    open(STDOUT,">>$Sauce_log");
	    unlink($Nfslink) ||
	      die "3496 Cannot break nfs link: $!";
	    unlink($Portmap_link) ||
	      die "3436 Cannot break portmap link: $!";
	    system ("$Nfsscript stop") &&
	      die "4426 Cannot stop nfs server: $!";
	    system("$Portmap_script stop >>$Sauce_log 2>&1") &&
	      die "4452 Cannot stop portmapper: $!";
	    system ("$Lcdshowscript start >> $Sauce_log 2>&1")
	      if (-x $Lcdshowscript);
	    exit;
	}
    }
    else
    {
	my $off;
        ($state) ? ($off = Cobalt::Language::msg_get('On_string')) : 
		    ($off = Cobalt::Language::msg_get('Off_string'));
	return "2498 No change -- nfs server was already $off";
    }
}

sub get_nfs_server_on
# Is the nfs server set to be on?
# Returns 1 if the nfs server is activated, 0 if deactivated
#   This reflects what ought to be, *not* what is
# Arguments: none
# Side effects: none
{
    (-e $Nfslink) ? (return 1) : (return 0);
}

sub set_dns_server_on
# Turn the dns server on or off
# Arguments: 1 for on, 0 for off
# Side effects: creates or removes /etc/named.boot, runs or kills named
{
    my ($state) = @_;

    return "1103 DNS service is not implemented in this release"
      unless ($Features{'dns'});

    # Supports both Bind 4 and 8 formats
    my($bind_conf)=$Named_boot;
    $bind_conf=$Named_conf if ($Features{'bind8'});

    if ($state)
    {
        if (-e "$Named_disabled")
	{
            rename($Named_disabled,$bind_conf)
		|| return "4850 Cannot copy $Named_disabled to $bind_conf: $!";
            unless (system("$Named_script start >>$Sauce_log 2>&1"))
	    {
                return "2850 DNS server started";
	    }
            else
	    {
                return "4851 DNS server start failed: $!";
	    }
	}
        elsif (-e "$bind_conf")
        {
            return "2856 No change -- DNS server was already on";
        }
        else
        {
            # Create fresh new named.boot
            my($named_boot,$srv);
	    if ($Features{'bind8'}) {
		$named_boot = <<EOF;
options { directory "$Named_dir"; };
zone "." { type hint; file "db.cache"; };
EOF
	    } else {
		$named_boot = "directory\t$Named_dir\n\ncache\t.\tdb.cache\n";
            }
            open(NAMED,"> $bind_conf")
		|| return "4852 Cannot write to $bind_conf: $!";
            print NAMED $named_boot;
            close(NAMED);
            $srv = system("$Named_script start >>$Sauce_log 2>&1");

            if (!$srv)
	    {
		return "2852 DNS server started";
	    }
            else
	    {
                return "4853 DNS server start failed: $!";
	    }
	}
    }
    else
    {
        if (-e "$bind_conf")
	{
	    my $srv = system("$Named_script stop >>$Sauce_log 2>&1");
	    unlink( "$Subsys/named" ) if (-f "$Subsys/named" );
	    
	    rename($bind_conf,$Named_disabled)
		|| return "4856 Cannot rename $bind_conf to $Named_disabled: $!";
	    if (!$srv)
	    {
		return "2854 DNS server stopped";
	    }
	    else
	    {
		return "4854 DNS server stop failed: $!";
	    }
	}
        else
	{
	    return "2855 No change -- DNS server was already off";
	}
    }
    return 1;
}

sub get_dns_server_on
# Checks for an active DNS server
# Returns 1 if the dns service is active, 0 if it is disabled
#   This reflects what ought to be, *not* what is
# Arguments: none
# Side effects: none
{
    # Supports both Bind 4 and 8 formats
    my($bind_conf)=$Named_boot;
    $bind_conf=$Named_conf if ($Features{'bind8'});

    (-e "$bind_conf") ? (return 1) : (return 0);
}

sub set_atalk_server_on
# Turn the atalk server on or off
# Arguments: 1 for on, 0 for off
# Return value: a status string of the form "582 insufficient depth perception"
# Side effects: creates or breaks symlinks in /etc/rc.d/rc3.d
{
    my ($state) = @_;

    return "1104 AppleShare service is not implemented in this release"
      unless ($Features{'atalk'});

    if ($state && (! -e $Atalklink))
    {
	symlink($Atalkscript,$Atalklink) ||
	    return "4427 Cannot create link $Atalklink: $!";
	my $pid;
	if (!defined($pid = fork)) {
	    die "cannot fork: $!";
	} elsif ($pid) {
	    # i'm the parent
	    print STDERR "Forking to restart appletalk\n" if ($Debug);
	    return "2431 Atalk server started";
	}
	else
	{
	    # otherwise I'm the child
	    open(STDERR,">>$Sauce_log");
	    open(STDOUT,">>$Sauce_log");
	    system ("$Atalkscript start >>$Sauce_log 2>&1 ") &&
		warn "3431 Cannot start atalk server: $!";
	    system ("$Lcdshowscript start >> $Sauce_log 2>&1")
	      if (-x $Lcdshowscript);
	    exit;
	}
    }
    elsif ((! $state) && (-e $Atalklink))
    {
	my $pid;
	if (!defined($pid = fork)) {
	    die "cannot fork: $!";
	} elsif ($pid) {
	    # i'm the parent
	    print STDERR "Forking to stop appletalk\n" if ($Debug);
	    return "2433 Atalk server stopped";
	}
	else
	{
	    # otherwise I'm the child
	    open(STDERR,">>$Sauce_log");
	    open(STDOUT,">>$Sauce_log");
	    unlink($Atalklink) ||
	      die "3433 Cannot break link $Atalklink: $!";
	    system ("$Atalkscript stop 1>&2") &&
	      die "4428 Cannot stop atalk server: $!";
	    system ("$Lcdshowscript start >> $Sauce_log 2>&1")
	      if (-x $Lcdshowscript);
	    Cobalt::Network::net_set_ipaddr(Cobalt::Network::net_get_ipaddr());
	    exit;
	}
    }
    else
    {
	my $off;
        ($state) ? ($off = Cobalt::Language::msg_get('On_string')) : 
	  ($off = Cobalt::Language::msg_get('Off_string'));
	return "2454 No change -- atalk server was already $off";	
    }
}

sub get_atalk_server_on
# Is the atalk server set to be on?
# Returns 1 if the atalk server is activated, 0 if deactivated
#   This reflects what ought to be, *not* what is
# Arguments: none
# Side effects: none
{
    (-e $Atalklink) ? (return 1) : (return 0);
}

sub set_dhcp_server_on
# Turn the dhcp server on or off
# Arguments: 1 for on, 0 for off
# Return value: a status string of the form "4325 Massive ego impeding development"
# Side effects: creates or breaks symlink
{
    my ($state) = @_;

    return "1110 Dhcpd service is not implemented in this release"
      unless ($Features{'dhcpd'});

    return "4333 DHCP Server Parameters are not yet configured" if ($state && !(-e $Dhcpd_conf));

    if ($state && (! -e $Dhcpdlink))
    {
	unless (-e $Dhcpd_leases)
	{
	    open(LEES,"> $Dhcpd_leases") || return "4334 Can not create lease database";
	    close(LEES);
	}
	open(DHCPSTART,"$Dhcpdscript start 2>&1 |") ||
	  return "4999 dhcpd_cant_runscript";
	open(LOG,">>$Sauce_log") || return "4999 dhcpd_cant_log";
	# The error message comes right before the "exiting" line...
	# so we remember the error and then return it if we see
	# "exiting"
	my ($line,$line1,$line2);
	while (<DHCPSTART>) {
	  if (/exiting/o) {
	    print LOG;
	    close LOG;
	    close DHCPSTART;
	    $line = "$line1 $line2";
	    return "4999 dhcpd_cant_start";
	  }
	  unless (/^[\s\^]*$/o) {  # ignore blank lines
	    $line1 = $line2;
	    chomp($line2 = $_);
	  } 
	  print LOG;
	}
	close DHCPSTART;
	close LOG;
	system ("$Lcdshowscript start >> $Sauce_log 2>&1")
	  if (-x $Lcdshowscript);
	symlink($Dhcpdscript,$Dhcpdlink) ||
	    return "4335 Cannot create link $Dhcpdlink: $!";
	return "2335 Dhcp server started successfully";
    }
    elsif ((! $state) && (-e $Dhcpdlink))
    {
	my $pid;
	if (!defined($pid = fork)) {
	    die "cannot fork: $!";
	} elsif ($pid) {
	    # i'm the parent
	    print STDERR "Forking to stop Dhcpd\n" if ($Debug);
	    return "2332 Dhcp server stopped";
	}
	else
	{
	    # otherwise I'm the child
	    open(STDERR,">>$Sauce_log");
	    open(STDOUT,">>$Sauce_log");
	    unlink($Dhcpdlink) ||
	      die "3503 Cannot break link $Dhcpdlink: $!";
	    system ("$Dhcpdscript stop 1>&2") &&
	      die "4336 Cannot stop Dhcp server: $!";
	    system ("$Lcdshowscript start >> $Sauce_log 2>&1")
	      if (-x $Lcdshowscript);
	    exit;
	}
    }
    elsif ($state)
    {
      return "2333 No change -- Dhcp server was already on";	
    }
    else 
    {
      return "2334 No change -- Dhcp server was already off";	
    }
}

sub get_dhcp_server_on
# Is the dhcp server set to be on?
# Returns 1 if the atalk server is activated, 0 if deactivated
#   This reflects what ought to be, *not* what is
# Arguments: none
# Side effects: none
{
    (-e $Dhcpdlink) ? (return 1) : (return 0);
}

sub set_ftp_server_on
# Turn the ftp server on or off
# Arguments: 1 for on, 0 for off
# Return value: a status string of the form "638 smokey bacon"
# Side effects: modifies inetd.conf
{
    my ($state) = @_;
    if ($state)
    {
	# turn it on!
	return "4439 $Inetd_conf is locked"
	    if (-e "$Lockdir/inetd.conf");
	open(O,">$Lockdir/inetd.conf")
	    || return "4441 Cannot write $Lockdir/inetd.conf: $!";
	open(I,$Inetd_conf)
	    || return "4440 Cannot read $Inetd_conf: $!";
	while (<I>)
	{
	    if (/^\#ftp/o)
	    {
		$_ =~ s/^\#ftp/ftp/o;
	    }
	    print O;
	}
	close I;
	close O;
	rename("$Lockdir/inetd.conf",$Inetd_conf)
	    || return "4442 Cannot replace $Inetd_conf: $!";
	open(PID,$Inetd_pid) || return "3459 Network server down: $!";
	my ($pid);
	my $line = <PID>;
	if ($line =~ /^(\d+)$/o)
	{
	    $pid = $1;
	}
	close PID;
	kill(1,$pid) if ($pid);
	return "2453 Ftp server started";
    }
    else
    {
	return "4443 $Inetd_conf is locked"
	    if (-e "$Lockdir/inetd.conf");
	open(O,">$Lockdir/inetd.conf")
	    || return "4445 Cannot write $Lockdir/inetd.conf: $!";
	open(I,$Inetd_conf)
	    || return "4444 Cannot read $Inetd_conf: $!";
	while (<I>)
	{
	    if (/^ftp/o)
	    {
		$_ =~ s/^ftp/\#ftp/o;
	    }
	    print O;
	}
	close I;
	close O;
	rename("$Lockdir/inetd.conf",$Inetd_conf)
	    || return "4446 Cannot replace $Inetd_conf: $!";

	open(PID,$Inetd_pid) || return "3464 Network server down: $!";
	my $pid;
	my $line = <PID>;
	if ($line =~ /^(\d+)$/o)
	{
	    $pid = $1;
	}
	close PID;
	kill(1,$pid) if ($pid);
	return "2464 Ftp server stopped";
    }
}

sub get_ftp_server_on
# Is the ftp server set to be on?
# Returns 1 if the ftp server is activated, 0 if deactivated
#   This reflects what ought to be, *not* what is
# Arguments: none
# Side effects: none
{
    open(INETD,$Inetd_conf) || return "4430 Cannot read $Inetd_conf: $!";
    while (<INETD>)
    {
	if (/^ftp\s/o)
	{
	    close INETD;
	    return 1;
	}
    }
    close INETD;
    return 0;
}

sub set_nfs_perms
# Define the list of hosts which are allowed to connect to the NFS server.
# Arguments: up to two strings specifying hosts which should be granted
#   access.  Empty arguments will clear the lines from the exports file.
#   The strings may include wilcards (*.foo.com) or IP addresses.
# Side effects: modifies /etc/exports and calls exportfs
# Return value: usual error string
{
    return "1105 NFS is not implemented in this release"
      unless ($Features{'nfs'});

    my ($allow1,$allow2) = @_;
    return "4449 Invalid export host $allow1"
	unless ($allow1 =~ /^[\w\*\.\-]*$/o);
    return "4462 Invalid export host $allow2"
	unless ($allow2 =~ /^[\w\*\.\-]*$/o);
    open(EXPORTS,">$Exports_file") ||
	return "4447 Cannot write exports file $Exports_file: $!";
    if ($allow1)
    {
	print EXPORTS "$Htgroup_dir\t$allow1(rw)\n";
	print EXPORTS "$Htusers_dir\t$allow1(rw)\n";
    }
    if ($allow2)
    {
	print EXPORTS "$Htgroup_dir\t$allow2(rw)\n";
	print EXPORTS "$Htusers_dir\t$allow2(rw)\n";
    }
    close EXPORTS;
    # exportfs sends signals to the NFS processes.  If the processes aren't
    # running, then don't bother with the signal
    if (Cobalt::Services::get_nfs_server_on())
    {
	system($Exports_cmd) && return "4448 Cannot reread exports list: $!";
    }
    return "2455 Updated NFS exports list";
}

sub get_nfs_perms
# Find out who we are exporting to
# Arguments: none
# Return value: a list of (up to two) hosts or host-patterns to which
#   we are exporting NFS
# Side effects: none
{
    my @exportees;
    open(EXPORTS,$Exports_file) ||
	return "4463 Cannot read exports file $Exports_file: $!";
    while (<EXPORTS>)
    {
	# Only count the user lines.  Assume that the group line
	# to match will be there
	if (/users\t(.+)\(rw\)$/o)
	{
	    push (@exportees,$1);
	}
    }
    close EXPORTS;
    return @exportees;
}


sub set_telnet_server_on
# Turn the telnet server on or off
# Arguments: 1 for on, 0 for off
# Return value: a status string of the form "619 Major Strasser's been shot"
# Side effects: modifies inetd.conf
{
    my ($state) = @_;
    if ($state)
    {
	# turn it on!
	return "4453 $Inetd_conf is locked"
	    if (-e "$Lockdir/inetd.conf");
	open(O,">$Lockdir/inetd.conf")
	    || return "4455 Cannot write $Lockdir/inetd.conf: $!";
	open(I,$Inetd_conf)
	    || return "4454 Cannot read $Inetd_conf: $!";
	while (<I>)
	{
	    if (/^\#$SecretTelnetPort/o)
	    {
		$_ =~ s/^\#$SecretTelnetPort/$SecretTelnetPort/o;
	    }
	    print O;
	}
	close I;
	close O;
	rename("$Lockdir/inetd.conf",$Inetd_conf)
	    || return "4456 Cannot replace $Inetd_conf: $!";
	open(PID,$Inetd_pid) || return "3465 Network server down: $!";
	my ($pid);
	my $line = <PID>;
	if ($line =~ /^(\d+)$/o)
	{
	    $pid = $1;
	}
	close PID;
	kill(1,$pid) if ($pid);
	return "2456 Telnet server started";
    }
    else
    {
	return "4457 $Inetd_conf is locked"
	    if (-e "$Lockdir/inetd.conf");
	open(O,">$Lockdir/inetd.conf")
	    || return "4459 Cannot write $Lockdir/inetd.conf: $!";
	open(I,$Inetd_conf)
	    || return "4458 Cannot read $Inetd_conf: $!";
	while (<I>)
	{
	    if (/^$SecretTelnetPort/o)
	    {
		$_ = "#$_";
	    }
	    print O;
	}
	close I;
	close O;
	rename("$Lockdir/inetd.conf",$Inetd_conf)
	    || return "4460 Cannot replace $Inetd_conf: $!";

	open(PID,$Inetd_pid) || return "3466 Network server down: $!";
	my $pid;
	my $line = <PID>;
	if ($line =~ /^(\d+)$/o)
	{
	    $pid = $1;
	}
	close PID;
	kill(1,$pid) if ($pid);
	return "2457 Telnet server stopped";
    }
}

sub get_telnet_server_on
# Is the telnet server set to be on?
# Returns 1 if the telnet server is activated, 0 if deactivated
#   This reflects what ought to be, *not* what is
# Arguments: none
# Side effects: none
{
    open(INETD,$Inetd_conf) || return "4461 Cannot read $Inetd_conf: $!";
    while (<INETD>)
    {
	if (/^$SecretTelnetPort\s/o)
	{
	    close INETD;
	    return 1;
	}
    }
    close INETD;
    return 0;
}

sub set_telnet_server_open
# Set wether only root can telnet in, or anyone at all
# Arguments: 1 for anyone, 0 for root only
# Side effects: creates or removes /etc/nologin, modifies
#   /etc/rc.d/rc.sysinit to remove /etc/nologin or not
# Return value: usual status thingy
# Example: Cobalt::Services::set_telnet_server_open(1);
{
    my ($newstate) = @_;
    my ($nowstate) = (! -e $Nologin);
    if (($newstate && $nowstate) ||
	((! $newstate) && (! $nowstate)) )
    {
	return "2458 No change to telnet state";
    }
    elsif ($newstate)
    {
	# open 'er up!
	unlink($Nologin);

	# fix the boot script so that nologin gets removed at
	# boot time
	return "3411 rc.sysinit is in use; boot state not set."
	  if (-e "$Lockdir/rc.sysinit");
	open(SO,">$Lockdir/rc.sysinit")
	    || return "4419 Cannot lock $Sysinit_script: $!";
	open(SI,"$Sysinit_script")
	    || return "4418 Cannot read rc.sysinit: $!";
	while (<SI>)
	{
	    if (/^rm -f \/etc\/mtab~ \/fastboot/o)
	    {
		chomp $_;
		$_ .= " $Nologin\n";
	    }
	    print SO;
	}
	close SI;
	close SO;
	rename("$Lockdir/rc.sysinit","$Sysinit_script")
	    || return "4417 Cannot write $Sysinit_script: $!";
	chmod(0755,"$Sysinit_script");
	return "2460 Telnet access opened to all users";
    }
    else
    {
	# close it down
	open(N,">$Nologin") 
	    || return "4429 Cannot create $Nologin: $!";
	close N;

	# fix the boot script so that nologin won't get clobbered
	# on bootup
	return "3410 rc.sysinit is in use; boot state not set."
	  if (-e "$Lockdir/rc.sysinit");
	open(SO,">$Lockdir/rc.sysinit") 
	    || return "4435 Cannot lock $Sysinit_script: $!";
	open(SI,"$Sysinit_script") 
	    || return "4431 Cannot read $Sysinit_script: $!";
	while (<SI>)
	{
	    if (/$Nologin/o)
	    {
		$_ =~ s/\s*$Nologin//og;
	    }
	    print SO;
	}
	close SI;
	close SO;
	rename("$Lockdir/rc.sysinit","$Sysinit_script")
	    || return "4422 Cannot write $Sysinit_script: $!";
	chmod(0755,"$Sysinit_script");
	return "2459 Telnet access restricted";
    }
}

sub get_telnet_server_open
# Check wether telnet access is open
# Arguments: none
# Return value: 0 if only root can log in, 1 if anyone can
# Example: print "Anyone can log in\n" if (Cobalt::Services::get_telnet_server_open());
{
    (-e $Nologin) ? (return 0) : (return 1);
}

sub set_smb_workgroup
# Set the workgroup for the SMB server (samba)
# Arguments: the workgroup name
# Side effects: modifies smb.conf and restarts samba
# Return value: usual status thingy
# Example: Cobalt::Services::set_smb_workgroup("workgroup");
{
    return "1106 Windows filesharing is not implemented in this release"
      unless ($Features{'smb'});

    my ($workgroup) = @_;
    return "4464 smb.conf is locked" if (-e "$Lockdir/smb.conf");
    open(SNB,">$Lockdir/smb.conf") 
	|| return "4471 Cannot write samba config file $Lockdir/smb.conf: $! ";
    open(SMB,"$Smbconf") 
	|| return "3475 Cannot read samba config file $Smbconf: $! ";
    while (<SMB>)
    {
	# Suppose you're thinking of a plate of shrimp...
	if (/^\s*workgroup = (\w+)/o)
	{
	    # $oldworkgroup = $1;
	    $_ = "   workgroup = $workgroup\n";
	}
	print SNB;
    }
    close SMB;
    close SNB;
    rename("$Lockdir/smb.conf",$Smbconf) ||
	return "3467 Cannot update $Smbconf: $!";

    # Start a child process to restart the samba server
    my $pid;
    if (!defined($pid = fork)) {
	die "cannot fork: $!";
    } elsif ($pid) {
	# i'm the parent
	print STDERR "forking a child to restart daemons\n" if ($Debug);
    }
    else
    {
	# otherwise I'm the child
	open(STDERR,">>$Sauce_log");
	open(STDOUT,">>$Sauce_log");
	# restart samba
	if (-f "$Subsys/smb")
	{
	    system("$Smbscript stop >>$Sauce_log 2>&1 ");
	    system("$Smbscript start >>$Sauce_log 2>&1 ");
	}
	system ("$Lcdshowscript start >>$Sauce_log 2>&1")
	    if (-x $Lcdshowscript);
	exit;
    }

    # done!
    return "2413 Set samba workgroup to $workgroup";
}

sub get_smb_workgroup
# Get the workgroup for the SMB server (samba)
# Arguments: none
# Side effects: modifies smb.conf and restarts samba
# Return value: the name of the workgroup
# Example: $workgroup = Cobalt::Services::get_smb_workgroup();
{
    my ($workgroup);
    open(SMB,"$Smbconf") 
	|| return "3471 Cannot read samba config file $Smbconf: $! ";
    while (<SMB>)
    {
	if (/\s*workgroup = (\w+)/o)
	{
	    $workgroup = $1;
	    last;
	}
    }
    close SMB;
    return $workgroup;
}

sub set_smb_controller
# Set the domain server for the SMB server (samba), or disable domain logins
# Arguments: a boolean of wether to use a domain server (1 means do use it), 
#   the domain server name
# Side effects: modifies smb.conf and restarts samba
# Return value: usual status thingy
# Examples: Cobalt::Services::set_smb_controller(1,"corp-pdc");
#           Cobalt::Services::set_smb_controller(0,"");
{
    return "1107 Cannot set domain server: not implemented in this release"
      unless ($Features{'dns'});

    my ($usecontroller,$server) = @_;
    my $security;
    ($usecontroller) ? ($security = "server") : ($security = "user");
    return "4467 smb.conf is locked" if (-e "$Lockdir/smb.conf");
    open(SNB,">$Lockdir/smb.conf") 
	|| return "4468 Cannot write samba config file $Lockdir/smb.conf: $! ";
    open(SMB,"$Smbconf") 
	|| return "3469 Cannot read samba config file $Smbconf: $! ";
    while (<SMB>)
    {
	# Joe, you're kind of a Philistine, aren't you?
	if (/^\s*security = (.+)/o)
	{
	    $_ = "   security = $security\n";
	}
	elsif (/^;?\s*password server = (.*)/o)
	{
	    my $oldserver = $1;
	    if ($usecontroller)
	    {
		$_ = "   password server = $server\n";
	    }
	    else
	    {
		$_ = ";   password server = $server\n";  
	    }
	}
	print SNB;
    }
    close SMB;
    close SNB;
    rename("$Lockdir/smb.conf",$Smbconf) ||
	return "3470 Cannot update $Smbconf: $!";

    # Start a child process to restart the samba server
    my $pid;
    if (!defined($pid = fork)) {
	die "cannot fork: $!";
    } elsif ($pid) {
	# i'm the parent
	print STDERR "forking a child to restart daemons\n" if ($Debug);
    }
    else
    {
	# otherwise I'm the child
	open(STDERR,">>$Sauce_log");
	open(STDOUT,">>$Sauce_log");
	# restart samba
	if (-f "$Subsys/smb")
	{
	    system("$Smbscript stop >>$Sauce_log 2>&1 ");
	    system("$Smbscript start >>$Sauce_log 2>&1 ");
	}
	system ("$Lcdshowscript start >>$Sauce_log 2>&1")
	    if (-x $Lcdshowscript);
	exit;
    }

    # done!
    $server ||= "off"; # default so the message makes sense
    return "2414 Set samba domain controller to $server";
}

sub get_smb_controller
# Get the domain controller for the SMB server (samba)
# Arguments: none
# Side effects: modifies smb.conf and restarts samba
# Return value: an array containing a boolean of wether samba is using a
#    server, and the name of the server
# Example: ($using_domain_controller,$server) = Cobalt::Services::get_smb_controller();
#          print "Server is $server\n" if ($using_domain_controller);
{
    my ($security,$server);
    open(SMB,"$Smbconf") 
	|| return "3472 Cannot read smb config file $Smbconf: $! ";
    while (<SMB>)
    {
	if (/^\s*security = (.+)/o)
	{
	    ($1 eq "user") ? ($security = 0) : ($security = 1);
	}
	elsif (/^\;?\s*password server = (.+)/o)
	{
	    $server = $1;
	}
    }
    close SMB;
    $server = "" if ($server =~ /\</o);
    return ($security,$server);
}

sub set_snmp_server_on
# Turn the snmp server on or off
# Arguments: 1 for on, 0 for off
# Return value: a status string of the form "911 Invalid hormone balance."
# Side effects: creates or breaks symlinks in /etc/rc.d/rc3.d
{
    my ($state) = @_;

    return "1403 snmp service not available in this release"
      unless ($Features{'snmp'});

    if ($state && (! -e $SNMPlink))
    {
        my $pid;
        if (!defined($pid = fork)) {
            die "cannot fork: $!";
        } elsif ($pid) {
            # i'm the parent
            print STDERR "Forking to restart snmpd\n" if ($Debug);
            return "2461 SNMP server started";
        }
        else
        {
            # otherwise I'm the child
            open(STDERR,">>$Sauce_log");
            open(STDOUT,">>$Sauce_log");
            symlink($SNMPscript,$SNMPlink) ||
	      die "4487 Cannot create link $SNMPlink: $!";
            system ("$SNMPscript start >> $Sauce_log 2>&1") &&
	      die "3498 Cannot start snmp server: $!";
            system ("$Lcdshowscript start >> $Sauce_log 2>&1")
	      if (-x $Lcdshowscript);
            exit;
        }
    }
    elsif ((! $state) && (-e $SNMPlink))
    {
        unlink($SNMPlink) ||
            return "4488 Cannot break link $SNMPlink: $!";
        system ("$SNMPscript stop >> $Sauce_log 2>&1") &&
            return "3499 Cannot stop snmp service: $!";
        system ("$Lcdshowscript start >> $Sauce_log 2>&1")
	      if (-x $Lcdshowscript);
        return "2462 SNMP server stopped";
    }
    else
    {
        my $off;
        ($state) ? ($off = Cobalt::Language::msg_get('On_string')) : 
	  ($off = Cobalt::Language::msg_get('Off_string'));
        return "2463 No change -- snmp server was already $off";
    }
}

sub get_snmp_server_on
# Is the snmp server set to be on?
# Returns 1 if the snmp server is activated, 0 if deactivated
#   This reflects what ought to be, *not* what is
# Arguments: none
# Side effects: none
{
    (-e $SNMPlink) ? (return 1) : (return 0);
}

sub set_snmp_community
# Sets the community with read and write access
# Arguments: read-only community, read/write community
#   If a community name is set to empty, then disable that community
# Return value: a status string of the form "666 Got a raise. Blow a hole in the account."
# Example: $ret = set_snmp_community("mypublic","");  (this disables the 
#   r/w community and sets the read-only community to "mypublic").
{
    return "1404 SNMP not implemented in this release"
	unless ($Features{'snmp'});

    my ( $read_community, $write_community )=@_;

    # The SNMP RFC just says that a community should be an "octet string";
    # but I know that would break the config file, so I'm going to
    # restrict to alphanumeric, underscore, dash, and dots until I find 
    # documenation to say otherwise.
    return "4999 services_snmp_invalid_read_community"
      if ($read_community !~ /^[\w\.\-]*$/o);
    return "4999 services_snmp_invalid_write_community"
      if ($write_community !~ /^[\w\.\-]*$/o);

    return "4472 $SNMPconf is locked" if ( -e "$Lockdir/snmp.conf" );
    open( SNMPLOCK, ">$Lockdir/snmp.conf" )
	|| return "4473 Cannot write SNMP config file $Lockdir/snmp.conf: $! ";
    open( SNMP, "$SNMPconf") 
	|| return "3477 Cannot read SNMP config file $SNMPconf: $! ";
    my $done=0;
    while( <SNMP> ) {
      if (/^[\#\s]*community\s+(\S+)\s+(xmini.+)/o ) {
	# Change the read-only community
	if ($read_community) {
	  $_ = "community\t$read_community\t$2\n";
	}
	elsif (/^c/o) {
	  # only comment it out if it isn't already
	  $_ = "#$_";
	}
      } 
      elsif (/^[\#\s]*community\s+(\S+)\s+(mini.+)/o) {
	# Change the read/write community (commented out by default)
	if ($write_community) {
	  $_ = "community\t$write_community\t$2\n";
	}
	elsif (/^c/o) {
	  # only comment it out if it isn't already
	  $_ = "#$_";
	}
      }
      elsif (/^trap community:/o) {
	# Trap community is not documented.  We don't want to just
	# leave it as "public" if we changed community names...
	my $trap_community;
	# first try setting it to the write community
	$trap_community = $write_community;
	# if write_community wasn't set, set it to the read community
	$trap_community ||= $read_community;
	# otherwise...well, set it to something.  No communities are
	# set, so we'll set it to the default of "public".
	$trap_community ||= "public";

	$_ = "trap community:\t$trap_community\n";
      }
      print SNMPLOCK;
    }
    close SNMPLOCK;
    close SNMP;
    rename( "$Lockdir/snmp.conf", $SNMPconf ) ||
	return "3478 Cannot update $SNMPconf: $!";
    chmod(0600, "$SNMPconf");

    # Start a child process to restart the SNMP agent
    my $pid;
    if( !defined( $pid=fork ) ) { die "cannot fork: $!"; }
    elsif( $pid ) {
	# i'm the parent
	print STDERR "forking a child to restart daemons\n" if ($Debug);
    }
    else {
	# otherwise I'm the child
	open( STDERR, ">>$Sauce_log" );
	open( STDOUT, ">>$Sauce_log" );
	# restart SNMP
	if( -f "$Subsys/snmpd" ) {
	    system( "$SNMPscript stop >>$Sauce_log 2>&1 " );
	    system( "$SNMPscript start >>$Sauce_log 2>&1 " );
	}
	system( "$Lcdshowscript start >>$Sauce_log 2>&1" )
	    if( -x $Lcdshowscript );
	exit;
    }

    # done!
    return "2999 services_snmp_ok_community_set";
}

sub get_snmp_community
# Gets the names of the SNMP communities
# Arguments: none
# Return value: the read-only community, the read/write community
#   If a community name is not set, then we return 0.
{
    my ($read_community,$write_community);
    $read_community ||= '';   # default in case there is none set
    $write_community ||= '';  # default in case there is none set
    open( SNMP, "<$SNMPconf" )
	|| return "3479 Cannot read SNMP config file $SNMPconf: $! ";
    while( <SNMP> ) {
	if( /^\s*community\s+(\S+)\s+xmini/o ) {
	    $read_community = $1;
	    # if we know both, then we're done
	    last if ($write_community);
	}
	elsif( /^\s*community\s+(\S+)\s+mini/o ) {
	    $write_community = $1;
	    # if we know both, then we're done
	    last if ($read_community);
	}
    }
    close SNMP;
    return ($read_community,$write_community);
}

sub set_ntp_server_on
# Turn the ntp server on or off
# Arguments: 1 for on, 0 for off
# Return value: a status string of the form "1132 hacks not supported in this release"
# Side effects: creates or breaks symlinks in /etc/rc.d/rc3.d
{
  my ($state) = @_;

  if (! $Features{'ntp'} )
  {
    return "1109 ntp service not available in this release";
  } 
  else 
  {

    if ($state && (! -e $NTPlink))
    {
        my $pid;
        if (!defined($pid = fork)) {
            die "cannot fork: $!";
        } elsif ($pid) {
            # i'm the parent
            print STDERR "Forking to restart NTP server\n" if ($Debug);
            return "2502 NTP server started";
        }
        else
        {
            # otherwise I'm the child
            open(STDERR,">>$Sauce_log");
            open(STDOUT,">>$Sauce_log");
            symlink($NTPscript,$NTPlink) ||
	      die "4489 Cannot create link $NTPlink: $!";
            system ("$NTPscript start >> $Sauce_log 2>&1") &&
	      die "3500 Cannot start ntp server: $!";
            system ("$Lcdshowscript start >> $Sauce_log 2>&1")
	      if (-x $Lcdshowscript);
            exit;
        }
    }
    elsif ((! $state) && (-e $NTPlink))
    {
        unlink($NTPlink) ||
            return "4490 Cannot break link $NTPlink: $!";
        system ("$NTPscript stop >> $Sauce_log 2>&1") &&
            return "3501 Cannot stop ntp service: $!";
        system ("$Lcdshowscript start >> $Sauce_log 2>&1")
	  if (-x $Lcdshowscript);
        return "2503 NTP server stopped";
    }
    else
    {
        my $off;
        ($state) ? ($off = Cobalt::Language::msg_get('On_string')) : 
	  ($off = Cobalt::Language::msg_get('Off_string'));
        return "2504 No change -- NTP server was already $off";
    }
  }
}

sub get_ntp_server_on
# Is the ntp server set to be on?
# Returns 1 if the ntp server is activated, 0 if deactivated
#   This reflects what ought to be, *not* what is
# Arguments: none
# Side effects: none
{
    (-e $NTPlink) ? (return 1) : (return 0);
}

sub set_networker_client_on
# Turn the Legato Networker client on or off
# Arguments: 1 for on, 0 for off
# Return value: a status string of the form "2700 Carpe diem, punc"
# Side effects: creates or breaks symlinks in /etc/rc.d/rc3.d
{
  my ($state) = @_;

  if (! $Features{'networker'} )
  {
    return "1999 services_networker_unsupported";
  } 
  else 
  {
    if ($state && (! -e $Networker_link))
    {
        my $pid;
        if (!defined($pid = fork)) {
            die "cannot fork: $!";
        } elsif ($pid) {
            # i'm the parent
            print STDERR "Forking to restart Legato Networker client\n"
	      if ($Debug);
            return "2999 services_networker_ok_started";
        }
        else
        {
            # otherwise I'm the child
            open(STDERR,">>$Sauce_log");
            open(STDOUT,">>$Sauce_log");
            symlink($Networker_script,$Networker_link) ||
	      die "4999 services_networker_cant_link";
            system ("$Networker_script start >> $Sauce_log 2>&1") &&
	      die "3999 services_networker_cant_start";
            system ("$Lcdshowscript start >> $Sauce_log 2>&1")
	      if (-x $Lcdshowscript);
            exit;
        }
    }
    elsif ((! $state) && (-e $Networker_link))
    {
        unlink($Networker_link) ||
            return "4999 services_networker_cant_unlink";
        system ("$Networker_script stop >> $Sauce_log 2>&1") &&
            return "3999 services_networker_cant_stop";
        system ("$Lcdshowscript start >> $Sauce_log 2>&1")
	  if (-x $Lcdshowscript);
        return "2999 services_networker_ok_stopped";
    }
    elsif ($state)
    {
      return "2999 services_networker_ok_alreadyon";
    }
    else
    {
      return "2999 services_networker_ok_alreadyoff";
    }
  }
}

sub get_networker_client_on
# Is the Legato Networker client set to be on?
# Returns 1 if the client is activated, 0 if deactivated
#   This reflects what ought to be, *not* what is
# Arguments: none
# Side effects: none
{
    (-e $Networker_link) ? (return 1) : (return 0);
}

sub set_fpx_on
# Enable or disable FrontPage Server Extensions on the main site
# Arguments: 1 for on/0 for off
# Return value: a status string of the usual form form "2534 ho hum" 
# Side effects: Calls MS's fpsrvadm.exe, cats /etc/htpasswd
# Examples: $good = Cobalt::Services::set_fpx_on(0);
#            $bad = Cobalt::Services::set_fpx_on(1);
{
  my $state = shift(@_);
  my $fpx_active = get_fpx_on();

  if (! $Features{'fpx'} )
  {
    return "1999 services_fpx_unsupported";
  }

  unless (-x "$Fpx_admsrvexe")
  {
    return "4999 services_fpx_bad_fpxnotavail";
  } 
  else 
  {

    if ($state && (! $fpx_active))
    {
	# Install

	# Temporarily move .htaccess files
	my(@htax,$hta);
	open(HTAS,"/usr/bin/find $Htgroup_dir/$MainGroup -name .htaccess -print |");
	while(<HTAS>) { chop; push(@htax,$_); }
	foreach $hta (@htax) 
	{
	    system("/bin/mv $hta $hta.FP98") unless ($hta eq "$Htgroup_dir/$MainGroup/.htaccess");
	}

	# Can't return after failure without .htaccess restoer
	my($could_not_install);
	
	# Call fpadmserv.exe (ugh)
	system("$Fpx_admsrvexe -o install -p 80 -t apache -m \"\" -u \"$Adminuser\" -pw \"$$\" -s \"$Htconf/httpd.conf\" -xu \"$WWWuser\" -xg \"$MainGroup\" > /dev/null")
	    && ($could_not_install = 1);

	# Restore .htaccess files asap!
	foreach $hta (@htax) 
	{
	    system("/bin/mv $hta.FP98 $hta") if (-e "$hta.FP98");
	}
	return "4999 services_fpx_bad_could_not_install" if ($could_not_install);

	# chmod u+s g+s fpcount.exe
	chmod(06555,"$Htgroup_dir/$MainGroup/_vti_bin/fpcount.exe");

	# qube2/raq2 fpx configurations are different! don't mess w/ fpx
        # if you hate m-soft.
	unless ($Features{'pam'}) {
	  # Use admin's password, only sure user
	  open(HTPAS,"$Htpasswd") 
	    || return "3999 services_fpx_bad_nosetpassword";
	  open(MSHT,"> $Htgroup_dir/$MainGroup/_vti_pvt/service.pwd") 
	    || return "3999 services_fpx_bad_nosetpassword";
	  print MSHT "# -FrontPage-\n";
	  while(<HTPAS>) {
	    print MSHT if (/^$Adminuser\:/o);
	  }
	  close(MSHT);
	  close(HTPAS);
	  chown ( (getpwnam($WWWuser))[2], (getgrnam($MainGroup))[2], 
		  "$Htgroup_dir/$MainGroup/_vti_pvt/service.pwd" );
	}

	# Configure SMTP sender for fpx cgi files
	if (-w $Fpx_we80cnf && !$could_not_install)
	{ 
		# We first check to see if fpadmsrv could somehow have configured smtp
		# It's never happened before, but who knows how it may behave in the future
		# (feature requested by Kevin 01/11/99 /wdh)
		my($SMTPrelay)=0;
		open(FPCNF,"$Fpx_we80cnf") || return "4999 services_fpx_bad_nosmtp";
		while(<FPCNF>) { $SMTPrelay=1 if (/SMTPHost/o); }
		
		unless($SMTPrelay) # Append the SMTPHost: directive
		{
			open(FPCNF,">>$Fpx_we80cnf") || return "4999 services_fpx_bad_nosmtp";
			print FPCNF "SMTPHost:127.0.0.1\n";
			close(FPCNF);
		}
	}

        return "2999 services_fpx_ok_installed";
    }
    elsif ((! $state) && $fpx_active)
    {
	# Uninstall
	# Call fpadmserv.exe (ugh)
	system("$Fpx_admsrvexe -o uninstall -p 80 -t apache -m \"\" -s \"$Htconf/httpd.conf\" > /dev/null")
	    && return "4999 services_fpx_bad_could_not_uninstall";
	
	# Already done by fpadmsrv unless something went wrong AND it returned 0 anyways
	# (feature requested by Kevin 01/11/99 /wdh)
	unlink($Fpx_we80cnf) if (-e $Fpx_we80cnf); 
        
	return "2999 services_fpx_ok_uninstalled";
    }
    elsif ($state)
    {
      return "2999 services_fpx_ok_alreadyon";
    }
    else
    {
      return "2999 services_fpx_ok_alreadyoff";
    }
  }
}

sub get_fpx_on
# Are FrontPage Server Extensions enabled on the main site?
# Returns 1 if installed, 0 if uninstalled or never installed 
#   Tests for the existance of we80.cnf
# Arguments: none
# Side effects: none
{
    (-e $Fpx_we80cnf) ? (return 1) : (return 0);
}

sub set_gated_on
# Turn the Gate Daemon (gated) on or off
# Arguments: 1 for on/0 for off; optional list of interfaces on 
#    which to listen
# Return value: a status string of the form "2534 The doggy in the 
#    window is $10"
# Side effects: creates or breaks symlinks in /etc/rc.d/rc3.d
# Examples: $ret = Cobalt::Services::set_gated_on(0);
#           $ret = Cobalt::Services::set_gated_on(1,"eth0","eth1");
{
  my $state = shift(@_);

  if (! $Features{'gated'} )
  {
    return "1999 services_gated_unsupported";
  } 
  else 
  {
    # If we're given a list of interfaces, fix gated.conf
    if ($#_ >= 0) {
      my $ret = Cobalt::Services::set_gated_interfaces(@_);
      return $ret if ($ret !~ /^2/o);
    }

    if ($state && (! -e $Gated_link))
    {
        my $pid;
        if (!defined($pid = fork)) {
            die "cannot fork: $!";
        } elsif ($pid) {
            # i'm the parent
            print STDERR "Forking to restart gated\n"
	      if ($Debug);
            return "2999 services_gated_ok_started";
        }
        else
        {
            # otherwise I'm the child
            open(STDERR,">>$Sauce_log");
            open(STDOUT,">>$Sauce_log");
            symlink($Gated_script,$Gated_link) ||
	      die "4999 services_gated_cant_link";
            system ("$Gated_script start >> $Sauce_log 2>&1") &&
	      die "3999 services_gated_cant_start";
            system ("$Lcdshowscript start >> $Sauce_log 2>&1")
	      if (-x $Lcdshowscript);
            exit;
        }
    }
    elsif ((! $state) && (-e $Gated_link))
    {
        unlink($Gated_link) ||
            return "4999 services_gated_cant_unlink";
        system ("$Gated_script stop >> $Sauce_log 2>&1") &&
            return "3999 services_gated_cant_stop";
        system ("$Lcdshowscript start >> $Sauce_log 2>&1")
	  if (-x $Lcdshowscript);
        return "2999 services_gated_ok_stopped";
    }
    elsif ($state)
    {
      return "2999 services_gated_ok_alreadyon";
    }
    else
    {
      return "2999 services_gated_ok_alreadyoff";
    }
  }
}

sub get_gated_on
# Is gated set to be on?
# Returns 1 if the daemon is activated, 0 if deactivated
#   This reflects what ought to be, *not* what is
# Arguments: none
# Side effects: none
{
    (-e $Gated_link) ? (return 1) : (return 0);
}

sub set_gated_interfaces
# Set the list of interfaces on which gated will listen
# Arguments: list of interfaces
# Side effects: modifies gated.conf
# Return value: return string of the form "2149 My tongue is glowing"
# Example: $ret = Cobalt::Services::set_gated_interfaces("eth0","eth1");
{
  my %netifs = map { $_, 1 } @_;   # make a hash of the interface list
  return "4999 services_gated_cant_locked"
    if (-e "$Lockdir/gated.conf");
  open(OUT,">$Lockdir/gated.conf") || return "4999 services_gated_cant_lock";
  unless (open(IN,$Gated_conf)) {
    unlink("$Lockdir/gated.conf");
    return "4999 services_gated_cant_read_config";
  }
  my $inrip;
  while (<IN>) {
    if (/^rip yes/o) {
      $inrip = 1;
      print OUT;
    }
    elsif ($inrip && /\};/o) {
      my $netif;
      foreach $netif (sort keys %netifs) {
	print OUT "        interface $netif ripin ripout version 2 broadcast;\n"
	  if ($netifs{$netif} == 1);
      }
      $inrip = 0;
      print OUT;
    }
    elsif ($inrip && /^\s*interface all/o) {
      # the default interface entry
      print OUT;
    }
    elsif ($inrip && /^\s*interface (\S+)\s/o) {
      print OUT if ($netifs{$1} == 1);
      $netifs{$1} = 0;
    }
    else {
      print OUT;
    }
  }
  close IN;
  close OUT;
  rename("$Lockdir/gated.conf",$Gated_conf);
  return "2999 services_gated_ok_setifs";
}

sub get_gated_interfaces
# Get the list of interfaces on which gated will listen
# Arguments: none
# Side effects: reads gated.conf
# Return value: list of interfaces ("eth0","eth1")
# Example: @netifs = Cobalt::Services::get_gated_interfaces();
{
  open(IN,$Gated_conf) || 
    return "4999 services_gated_cant_read_config";
  my $inrip;
  my @netifs;
  while (<IN>) {
    if (/^rip yes/o) {
      $inrip = 1;
    }
    elsif ($inrip && /\};/o) {
      last;
    }
    elsif ($inrip && /^\s*interface all/o) {
      # the default interface entry
      next;
    }
    elsif ($inrip && /^\s*interface (\S+)/o) {
      push(@netifs,$1);
    }
  }
  close IN;
  return @netifs;
}

1;
