#
# $Id: Time.pm,v 1.20 1999/03/05 01:44:55 will Exp $
#
# Written by Brion Moss (brion@cobaltnet.com, brion@nda.com)
# ntp-werks by Will DeHaan (will@cobaltnet.com)
#
# Copyright 1998 Cobalt Networks http://www.cobaltnet.com
#

package Cobalt::Time;
use AutoLoader 'AUTOLOAD';
BEGIN {
require Exporter;
@ISA    = qw(Exporter);
@EXPORT = qw(set_time time_set_zone time_get_zone ntp_set_servers ntp_get_servers ntpc_set ntpc_get);

require Cobalt::Netutil;
import Cobalt::Netutil qw(:DEFAULT);
}

#use Cobalt::Paths;
#use strict;
1;
__END__

sub ntpc_get
# Checks whether a ntpdate fetch is scheduled, and what server it queries
# Arguments: none
# Return value: two values, the first is 1 or 0 for on or off,
#    the second is the target server or null
{
	my($server);

	if (-e "$NtpcConf")
	{
		open(NTPC,"$NtpcConf");
		chomp($server = <NTPC>);
		close(NTPC);
	} else
	{
		return(0,'');
	}
	if (-e "$NtpcCron")
	{
		return(1,$server);
	} else
	{
		return(0,$server);
	}
}

sub ntpc_set
# Schedules a ntpdate update, remembers the ntp server if updates are disabled\
# Arguments: 1 or 0 for enable updates or disable, ntp server name
# Return value: String, of the form "4324 That's it. I'm moving to Jamaica."
# Calling this with status of 1 ntpdates immediately to test the server
{
	my($status,$server) = @_;

	# Use old server if none specified but enabled updates
	if ($status && !$server && (-e "$NtpcConf"))
	{
                open(NTPC,"$NtpcConf");
                chomp($server = <NTPC>);
                close(NTPC);
	}

	if (Cobalt::Netutil::netutil_test_netaddr("$server")) 
	{
		open(NTPC,"> $NtpcConf");
		print NTPC "$server\n";
		close(NTPC);
	} elsif ($status) 
	{
		return "4633 Invalid ntp server name";
	}

	if ($status)
	{
		if( Cobalt::Netutil::netutil_ping( $server )!~/^2/ ) {
		    return "4659 NTP server is unreachable";
		}

		my($ret) = `$NtpDate $server 2>&1`;
		if ($ret =~ /no server suitable for synchronization/)
		{
			return "4657 The NTP server could not be contacted";
		} elsif ($ret =~ /t adjust the time of day: Invalid/)
		{
			return "4658 NTP update failed, the current date is too inaccurate.  The date and time must be updated manually.";
		}
		return "4632 Could not refresh time from $server" unless ($ret =~ /[adjust|step] time server/);

		# Fix the timestamp in /etc/issue, because monitor uses it
		# to tell how long ago we finished booted up
		utime($^T,$^T,$Issuefile);
		# Fix the timestamp on the aliases db, because we auto-rebuild 
		# based on it being older than /etc/aliases
		utime($^T,$^T,$Alias_db) if ($Features{'mail'});

		# Avoid a network slam when 200 RaQs all ntp fetch at the same
		# time.  Now they'll do it over a ten minute window
		my($ntpdelay) = int (rand(600)) + 1;

		# Now make the cute little shell script that keeps us synced
		open(NTCRON,"> $NtpcCron");
		print NTCRON <<EOF;
#!/bin/sh

# NTPdate cron event
# Configured through the administrative web interface
# Copyright (C) 1998, Cobalt Networks, Inc
# All rights reserved

sleep $ntpdelay
$NtpDate $server >>$Sauce_log 2>&1
EOF
		close(NTCRON);
		chmod(0700,"$NtpcCron");

		return "2714 ntpc updates scheduled successfully";
	} else
	{
		unlink("$NtpcCron") if (-e "$NtpcCron");
		unlink("$NtpcConf") unless ($server);
		return "2715 ntpc scheduled updates disabled successfully";
	}
}

sub ntp_set_servers
# Define a list of ntp servers
# Arguments: array of fully qualified time server names or ip addresses
# Return value: string of the form "2123 Please use open policy servers"
# Side effects: edits /etc/ntp.conf
{
    my(@NtpServers) = reverse(@_);
    return "4625 ntp is not available in this release" unless ($Features{'ntp'});
    return "4626 ntp configuration is not available" unless (-e "$NtpConf");
    return "4627 ntp server is locked" if (-e "$Lockdir/ntp.conf");

    my($ntpServer,$NewServers,%ExistServer);
    while ($#NtpServers >= 0) 
    {
	$ntpServer = pop(@NtpServers); # foreach $ntpServer (@NtpServers

	next if ($ExistServer{"$ntpServer"});

	if ($ntpServer =~ /[a-zA-Z]/)
	{
		return "4628 Invalid ntp server name: $ntpServer" unless (Cobalt::Netutil::netutil_test_domainname($ntpServer));
		$NewServers .= "server\t$ntpServer\n";
	} elsif ($ntpServer =~ /\d/) {
		return "4629 Invalid ntp server IP address: $ntpServer" unless (Cobalt::Netutil::netutil_test_ipaddr($ntpServer));
		$NewServers .= "server\t$ntpServer\n";
	}
	$ExistServer{"$ntpServer"} = 1; # Don't want duplicate entries
    }
    return "4630 Please specify a valid NTP server" unless ($NewServers);

    open(NTPLCK,"> $Lockdir/ntp.conf");
    open(NTPCNF,"$NtpConf");
    while(<NTPCNF>)
    {
	if (/^server/)
	{
	    print NTPLCK $NewServers;
	    $NewServers = '';  
	} else {
	    print NTPLCK;
	}
    }
    close(NTPCNF);
    close(NTPLCK);
    rename("$Lockdir/ntp.conf","$NtpConf") || return "4631 Could not update ntp configuration file";

    return "2630 NTP servers updated";
}

sub ntp_get_servers
{
    return "0" unless ($Features{'ntp'});
    return "0" unless (-e "$NtpConf");
    my(@NtpServers);
    open(NTPCF,"$NtpConf");
    while(<NTPCF>) 
    {
	push(@NtpServers,$1) if (/^server\s+([a-zA-Z\d\.]+)/o);
    }
    return(@NtpServers);
    close NTPCF;
}

sub set_time
# Set the time and date
# Arguments: An array containing the month (0-11), the date (1-31), the
#   year (19xx or 20xx or whatever), the hour (0-23) and the minute (0-59)
# Side effects: sets the system clock
# Return value: Returns a string of the form "889 Not enough string"
{
    return "4486 Invalid time format" if (grep(/[^\d]/o,@_));
    my ($month,$date,$year,$hour,$minute) = @_;
    $month++;
    system("$Date_cmd --set='$month/$date/$year $hour:$minute' >>$Sauce_log 2>&1") &&
	return "4480 Cannot set date: $!";
    system("$Clock_cmd -w >>$Sauce_log 2>&1") &&
	return "4481 Cannot save date: $!";
    $^T = time();
    # Fix the timestamp in /etc/issue, because monitor uses it
    # to tell how long ago we finished booted up
    utime($^T,$^T,$Issuefile);
    # Fix the timestamp on the aliases db, because we auto-rebuild 
    # based on it being older than /etc/aliases
    utime($^T,$^T,$Alias_db) if ($Features{'mail'});

    # Restart crond in a child process
    my $pid;
    if (!defined($pid = fork)) {
	die "cannot fork: $!";
    } elsif ($pid) {
	# i'm the parent
	print STDERR "Forking to restart crond\n" if ($Debug);
	return "2039 Date set";
    }
    else
    {
	# otherwise I'm the child
	open(STDERR,">>$Sauce_log");
	open(STDOUT,">>$Sauce_log");
	system("$Cron_script stop >>$Sauce_log 2>&1");
	system("$Cron_script start >>$Sauce_log 2>&1");

	# Signal custodiat to wake up so we avoid eternal sleep...
	open(CUSTOPID,"$Custo_pid");
	my $cpid = <CUSTOPID>;
	close CUSTOPID;
	kill('ALRM',$cpid) if ($cpid);
	exit;
    }
}

sub time_set_zone
# Set our timezone
# Arguments: A string of the form GMT plus/minus number of hours
#   (e.g., GMT-3, or GMT+12)
# Side effects: sets our local time (using the /etc/localtime link)
# Return value: the usual string
{
    my ($zone) = @_;
    if ($Zones{$zone})
    {
	$zone = $Zones{$zone};
    }
    else
    {
	# Not mapped, use the Etc/GMT+# format
	$zone = "Etc/$zone";
    }

    return "4483 Invalid time zone $zone"
	unless (-f "$Zone_dir/$zone");
    unlink($Localtime_link) ||
	return "4484 Cannot change timezone: $!";
    symlink("$Zone_dir/$zone",$Localtime_link) ||
	return "4485 Cannot set timezone: $!";
    return "2040 timezone set to $zone";
}

sub time_get_zone
# Get our timezone
# Arguments: none
# Return value: zone in terms of GMT +/- some number (e.g., "GMT+3")
# Side effects: none
{
    my $zone = readlink($Localtime_link);
    $zone =~ s/^$Zone_dir\///o;
    if ($Zones{$zone})
    {
	return($Zones{$zone});
    }
    else
    {
	# not mapped, return in GMT+# format
	return (split('/',$zone))[1];
    }
}

1;
