#
# $Id: User.pm,v 1.62 1999/03/12 14:57:31 gsri Exp $
#
# User.pm: Module for managing users
#
# Written by Brion Moss (brion@cobaltnet.com, brion@nda.com)
#            Kevin K.M. Chiu (kevin@cobaltnet.com)
#	     Will DeHaan (will@cobaltnet.com)
#            John D. Blair (jdblair@cobaltnet.com) (SMB & ATALK hooks)
#
# Copyright 1998 Cobalt Networks http://www.cobaltnet.com
#

package Cobalt::User;
use AutoLoader 'AUTOLOAD';
BEGIN {
require Exporter;
@ISA    = qw(Exporter);
@EXPORT = qw(user_add user_mod user_del user_show user_list user_exist
	     user_list_groups user_list_site user_info_sorted);

require Cobalt::Admserv;
import Cobalt::Admserv qw(admserv_set_access);
require Cobalt::Email;
import Cobalt::Email qw(:DEFAULT);
require Cobalt::Group;
import Cobalt::Group qw(group_del_user);
require Cobalt::List;
import Cobalt::List qw(alias_exist list_get_mem list_sites_del_mem list_lists_del_mem list_lists list_sites_del_mem alias_delete list_sites_lists alias_get_bytarget alias_delete);
require Cobalt::Quota;
import Cobalt::Quota qw(:DEFAULT);
require Cobalt::Netutil;
import Cobalt::Netutil qw(:DEFAULT);
require Cobalt::Network;
import Cobalt::Network qw(:DEFAULT);
require Cobalt::Vacation;
import Cobalt::Vacation qw(:DEFAULT);
require Cobalt::Vsite;
import Cobalt::Vsite qw(vsite_get_bygroup);
require Cobalt::Utility;
import Cobalt::Utility qw(:DEFAULT);
require Cobalt::Samba;
import Cobalt::Samba qw(:DEFAULT);
require Cobalt::Atalk;
import Cobalt::Atalk qw(:DEFAULT);
}

use integer;
1;
__END__

sub user_exist
# user_exist: does a user exist?
# Arguments: login name
# Side effects: none
# Return value: true if they do, false if they don't
# Example: $return = Cobalt::User::user_exist("name");
{
    my ($user) = @_;
    setpwent;   # flush
    ((getpwnam($user))[0]) ? (return 1) : (return 0);
}

sub user_add
# user_add: Create a new user
# Arguments: login, full name, cleartext password, quota, shell
#            optional group (necessary for RaQ),
#            optional home directory (if specified, don't create or touch)
# Side effects: adds the user to the password file and the htpassword file, 
#   and creates a home directory.
# Return value: a string of the form "526 The wicked witch is dead"
# Example: Cobalt::User::user_add("rblaine","Rick Blaine","SeCrEt",2048,$Usershell);
{
    my ($username,$descr,$passwd,$quota,$shell,$base_group,$user_home) = @_;
    my ($group) = $base_group;
    my ($id,$uid,$return,$cryptpw,$htmlfile);
    my ($user_group,$user_dirgroup);
    my ($homedir);

    return "4543 Invalid user name $username" if ($username !~ /^[\w\-\.]+$/o);
    return "4554 Names must be no longer than $MaxNameLen characters."
	if (length($username) > $MaxNameLen);
    # Print out the max in megs, not kilobytes
    return "4544 User quotas must be a positive integer"
      if ($quota !~ /^\d+$/o);
    my $warn_quota = ($MaxQuota / 1024);
    return "4555 User quotas may not exceed $warn_quota"
	if ($quota > $MaxQuota);
    return "4546 $username is a reserved name" if ($System_users{$username});
    return "4500 User $username already exists" if (Cobalt::User::user_exist($username));
    return "4501 User names cannot contain colons"
	if ($descr =~ /:/o);
    return "4557 The name $username is reserved by the system"
	if ($System_groups{$username} && $username ne $Adminuser);
    return "4530 Name $username is already in use as a group"
	if (getgrnam($username));
    return "4545 Name $username is already in use as a mailing list or alias"
	if (Cobalt::List::alias_exist($username));
    return $cryptpw                    # error messages contain spaces
	if (($cryptpw = Cobalt::User::verify_password($passwd)) =~ /\s/o);

    if ($base_group && $Features{'vmachine'}) 
    {
 	unless (-d "$Htgroup_dir/$base_group/users")
	{
	    mkdir("$Htgroup_dir/$base_group/users",02775);
	    chown( (getpwnam($WWWuser))[2], (getgrnam($base_group))[2], "$Htgroup_dir/$base_group/users");
	    chmod(02775,"$Htgroup_dir/$base_group/users"); # the mkdir above doesn't work.
	}
	# all the files should be owned by the site group
	$user_group = (getgrnam($base_group))[2];
	$user_dirgroup = $user_group;
	$base_group = "/$base_group/users";
    }
    else {
	# the files should be owned by the generic users group
	$user_group = $WWWgid;
	$user_dirgroup = $WWWgid;
    }
    print STDERR "$username in group $user_group\n" if ($Debug);

    if ($user_home) { $homedir = $user_home; }
    else {
	if( $base_group ) {
	    $homedir = "$Userdir$base_group/$username";

	    # Make sure the parent directory exists
	    if( ! -d "$Userdir$base_group" ) { mkdir( "$Userdir$base_group", 02775 ); }
	}
	else {
	    $homedir = "$Userdir/$username";

	    # Make sure the parent directory exists
	    if( ! -d $Userdir ) { mkdir( $Userdir, 02775 ); }
	}
    }

    if ( !$user_home && -e $homedir) {
	return "4503 Cannot create $username: directory $homedir already exists.";
    }

    setpwent();

    # set default shell
    if( !$shell ) { $shell=$Usershell; }

    # Truncate the GCOS if it's too long
    $descr = substr($descr,0,$MaxGcosLen) if (length($descr) > $MaxGcosLen);
    $descr =~ s/>/&gt;/og;  # No HTML!
    $descr =~ s/</&lt;/og;
    $descr =~ s/\'/&\#039;/og;  # Replace quotes with HTML

    return $return unless (($return = Cobalt::User::lock_passwd()) =~ /^2/o);

    ### Add the entry to the password file

    # Keep track of what UIDs we've given and don't reassign them
    # (and don't forget to wipe this in sysreset if we use it)
    if (open(UID,$Uid_file))
    {
	# UID file exists
	$id = <UID>;
	close UID;
    }
    $id ||= $Firstuid;
    #setpwent();
    #for($id; getpwuid($id); $id++) {;}  # Get next UID
    #if ($id =~ /^(\d+)$/o)  { $uid = $1; }
    #else                    { return Cobalt::User::user_abort("4514 Malformed UID");  }
    $uid = Cobalt::User::user_get_first_available_uid($id);
    setpwent();
    if (getpwuid($uid)) {
      return Cobalt::User::user_abort("4506 getpwuid($uid) says true -- get_first_available_uid is hosed\n");
    }

    open(UID,">$Uid_file")
      || return Cobalt::User::user_abort("4556 Cannot write $Uid_file: $!");
    print UID $uid+1;
    close UID;
    
    my $line = "$username:$cryptpw:$uid:$Usergroup:$descr:$homedir:$shell\n";
    print STDERR "Adding to $Ptmp:\n$line" if ($Debug);
    open(PTMP,">>$Ptmp") 
	|| return Cobalt::User::user_abort("4515 Cannot append to $Ptmp: $!");
    print PTMP $line;
    close PTMP;
    
    unless ($Features{'pam'}) {
      # Appends to the htpasswd file
      open(HTP,">>$Htptmp") 
	|| return Cobalt::User::user_abort("4516 Cannot append to $Htptmp: $!");
      print HTP "$username:$cryptpw\n";
      close HTP;
    }
    
    unless ($user_home) {
      ### Create the directory and files for the user
      print STDERR "Creating directories under $homedir\n" if ($Debug);

      mkdir($homedir,$UserDirPermissions) 
	|| return Cobalt::User::user_abort("4999 user_cant_create_homedir");
      chown($uid,$user_dirgroup,$homedir)
	|| return Cobalt::User::user_abort("3999 user_cant_set_ownership");
      chmod($UserDirPermissions,$homedir);

      # Add public web directory
      mkdir("$homedir/$Webdir",$UserWebPermissions) 
	|| return Cobalt::User::user_abort("4999 user_cant_create_webdir");
      chown($uid,$user_dirgroup,"$homedir/$Webdir")
	|| return Cobalt::User::user_abort("3999 user_cant_set_ownership");
      chmod($UserWebPermissions,"$homedir/$Webdir");
    
      # Copy the HTML templates
      opendir(SKEL,$User_skel)
	|| return Cobalt::User::user_abort("3592 Cannot read $User_skel: $!");
      foreach $htmlfile (grep(/\.html$/o,readdir(SKEL)))
      {
	next if ($htmlfile eq "index.html"); # special case, do later
	print STDERR "Copying $htmlfile to $homedir/$htmlfile\n"
	  if ($Debug);
	open(SRC,"$User_skel/$htmlfile") 
	  || return Cobalt::User::user_abort("4539 Cannot open source file $User_skel/$htmlfile: $!");
	open(DEST,">$homedir/$Webdir/$htmlfile") 
	  || return Cobalt::User::user_abort("4542 Cannot open dest file $homedir/$Webdir/$htmlfile: $!");
	print DEST while (<SRC>);
	close DEST;
	close SRC;
	
	chown($uid,$user_group,"$homedir/$Webdir/$htmlfile");
	chmod(0664,"$homedir/$Webdir/$htmlfile") if ($Features{'vsites'});
      }
      closedir SKEL;

      # Now do index.html
      my $domain;
      if ($group) {
	  if ($group eq $MainGroup) { $domain = Cobalt::Netutil::netutil_get_fulldomain(); }
	  else { $domain = (Cobalt::Vsite::vsite_get_bygroup($group))[ 1 ]; }
      }
      print STDERR "Copying index.html to $homedir/$Webdir/index.html\n"
	if ($Debug);
      open(SRC,"$User_skel/index.html") 
	|| return Cobalt::User::user_abort("4550 Cannot open source file $User_skel/index.html: $!");
      open(DEST,">$homedir/$Webdir/index.html") 
	|| return Cobalt::User::user_abort("4553 Cannot open dest file $homedir/$Webdir/index.html: $!");
      while (<SRC>)
      {
	if (/\[/o)
	{
	  s/\[DOMAIN\]/$domain/go;
	  s/\[PATH\]/$homedir\/index.html/o;
	  s/\[USER\]/$descr/o;
	  s/\[USERNAME\]/$username/go;
	}
	print DEST;
      }
      close DEST;
      close SRC;
      chown($uid,$user_group,"$homedir/$Webdir/index.html");
      chmod(0664,"$homedir/$Webdir/index.html") if ($Features{'vsites'});
      
      if ($Features{'privatedirs'})
      {
	mkdir("$homedir/$Webdir/$Private_html",$UserPrivateDirPermissions)
	  || return Cobalt::User::user_abort("4528 Cannot create $homedir/$Webdir/$Private_html: $!");
	chown($uid,$user_dirgroup,"$homedir/$Webdir/$Private_html");
	chmod(02750,"$homedir/$Webdir/$Private_html");
	Cobalt::Admserv::admserv_set_access("$homedir/$Webdir/$Private_html","require","user",$username);
	
	# Now do index.html for the private directory
	if (-e "$User_skel/private.tmpl")
	{
	  print STDERR "Copying private.tmpl to $homedir/$Webdir/$Private_html/index.html\n"
	    if ($Debug);
	  open(SRC,"$User_skel/private.tmpl") 
	    || return Cobalt::User::user_abort("4570 Cannot open source file $User_skel/private.tmpl: $!");
	  open(DEST,">$homedir/$Webdir/$Private_html/index.html") 
	    || return Cobalt::User::user_abort("4571 Cannot open dest file $homedir/$Webdir/index.html: $!");
	  while (<SRC>)
	  {
	    if (/\[/o)
	    {
	      s/\[USER\]/$descr/;
	      s/\[PATH\]/$homedir\/$Private_html\/index.html/;
	    }
	    print DEST;
	  }
	  close DEST;
	  close SRC;
	  chown($uid,$user_group,"$homedir/$Private_html/index.html");
	}
      }

      # AppleTalk seems to feel some odd need for this directory.
      # Let the baby have its way.
      if ($Features{'atalk'}) {
	mkdir("$homedir/Network Trash Folder",$UserDirPermissions)
	  || return Cobalt::User::user_abort("4999 user_cant_create_trash");
	chown($uid,$user_dirgroup,"$homedir/Network Trash Folder");
	chmod($UserDirPermissions,"$homedir/Network Trash Folder");
	mkdir("$homedir/$Webdir/Network Trash Folder",$UserDirPermissions)
	  || return Cobalt::User::user_abort("4999 user_cant_create_trash");
	chown($uid,$user_dirgroup,"$homedir/$Webdir/Network Trash Folder");
	chmod($UserDirPermissions,"$homedir/$Webdir/Network Trash Folder");
      }
    }
    ### End dir creation

    # Touch the user mail spool and set ownership
    open(MLSPL,"> $Mailspool/$username")
      || return Cobalt::User::user_abort("4596 Cannot create mail spool for $username: $!"); 
    close(MLSPL);
    chmod(0600,"$Mailspool/$username");
    chown($uid, $user_group ,"$Mailspool/$username");

    # update the netatalk passwd file (will they ever end?)
    if ($Features{'atalk'}) {
      my $atalk_ret = Cobalt::Atalk::atalk_set_passwd($username, $passwd);
      return $atalk_ret if ($atalk_ret !~ /^2/);
    }

    # lastly, add an entry to the smbpasswd file
    if ($Features{'smb'}) {
      $ENV{'SMBPASSWD'} = $passwd;
      open(GETHASH, "$Smb_gethash|")
	|| return Cobalt::User::user_abort("4584 Cannot run $Smb_gethash: $!");
      my $smbhashes = <GETHASH>;
      close(GETHASH);
      open(SMBPTMP, ">>$Smbptmp") or return "4575 Cannot append to $Smbptmp: $!";
      print SMBPTMP "$username:$uid:$smbhashes:$descr:$homedir:$shell\n";
      close(SMBPTMP);
    }
    
    return $return unless (($return = Cobalt::User::commit_passwd()) =~ /^2/o);
    Cobalt::Quota::setqlim($Userpart,$uid,$quota,($quota + $Quota_harder));
    #Cobalt::Quota::setqlim($Varpart,$uid,$quota,($quota + $Quota_harder));
    
    print STDERR "User $username complete\n" if ($Debug);
    return "2530 user $username created successfully";
}

sub user_del
# user_del: Delete a user
# Arguments: the login name of the user, optional boolean value to not delete home directory
# Side effects: removes the password file entry, and deletes the home
#   directory
# Return value: a string of the form "902 nut crunchies exploded"
# Example: Wipes user and home directory
#          Cobalt::User::user_del("joey"); 
# Example: Deletes user but leaves the user home directory as-is
#          Cobalt::User::user_del("anonftpuser",1); 
# Example: Deletes user but leaves the user home directory as-is
#          Cobalt::User::user_del("anonftpuser","/home/sites/home/ftp/");
{
    my ($username,$userhome) = @_;
    my ($return);
    my ($login,$home) = (getpwnam($username))[0,7];
    return "2516 Cannot delete $username: no such user"
	unless ($login);
    return "4502 Cannot delete $Adminuser" if ($username eq $Adminuser);
    return "4522 Cannot delete $username: prohibited user"
	if ($System_users{$username});
    
    return $return unless (($return = Cobalt::User::lock_passwd()) =~ /^2/o);

    #   clean the smbpasswd file
    if ($Features{'smb'}) {
      print STDERR "Cleaning from $Smbptmp\n" if ($Debug);
      open(SMBPTMP,$Smbptmp) 
	|| return Cobalt::User::user_abort("4576 Cannot open $Smbptmp to delete user: $!");
      open(SMBPASSWDLOCK,">$Lockdir/smbpasswd") 
	|| return Cobalt::User::user_abort("4577 Cannot clobber $Lockdir/smbpasswd: $!");
      while (<SMBPTMP>)
	{
	  print SMBPASSWDLOCK unless (/^$username/o);
	}
      close SMBPASSWDLOCK;
      close SMBPTMP;
      rename("$Lockdir/smbpasswd",$Smbptmp) 
	|| return Cobalt::User::user_abort("4578 Cannot move $Smbptmp: $!");
    }

    # clean the netatalk passwd file
    if ($Features{'atalk'}) {
      my $atalk_ret = Cobalt::Atalk::atalk_del_user($username);
      return $atalk_ret if ($atalk_ret !~ /^2/);
    }

    ### clean the password file
    print STDERR "Cleaning from $Ptmp\n" if ($Debug);
    open(PASSWD,$Ptmp) 
	|| return Cobalt::User::user_abort("4517 Cannot open $Ptmp to delete user: $!");
    open(PTMP,">$Lockdir/passwd") 
	|| return Cobalt::User::user_abort("4518 Cannot clobber $Lockdir/passwd: $!");
    "$username:" =~ /^$username:/; # pre-compile our pattern
    while (<PASSWD>)
    {
	print PTMP unless (//);
    }
    close PTMP;
    close PASSWD;
    rename("$Lockdir/passwd",$Ptmp) 
	|| return Cobalt::User::user_abort("4519 Cannot move $Ptmp: $!");
    
    unless ($Features{'pam'}) {
      # Clean the htpasswd file
      open(HTPASSWD,$Htptmp) 
	|| return Cobalt::User::user_abort("4520 Cannot open htpasswd file $Htpasswd: $!");
      open(HTP,">$Lockdir/htpasswd") 
	|| return Cobalt::User::user_abort("4521 Cannot clobber $Htptmp: $!");
      "$username:" =~ /^$username:/; # pre-compile
      while (<HTPASSWD>)
      {
	print HTP unless (//);
      }
      close HTP;
      close HTPASSWD;
      rename("$Lockdir/htpasswd",$Htptmp) 
	|| return Cobalt::User::user_abort("4540 Cannot rename $Htptmp: $!");
    }
    
    ### get rid of the directory
    unless ($userhome) 
    {
	if ($home && (-d $home)) {
	    print STDERR "Deleting $home...\n" if ($Debug);
	    Cobalt::Utility::sauce_deltree($home);
	} else {
	    print STDERR "4531 $username has no files.\n" if ($Debug);
	}
    } else
    {
        print STDERR "Deleting user but leaving directories $userhome alone...\n" if ($Debug);
    }
    unlink("$Mailspool/$username") if (-e "$Mailspool/$username");

    ### Commit
    return $return unless (($return = Cobalt::User::commit_passwd()) =~ /^2/o);
    
    ### Clean the group file
    my ($gr,$gp,$gd,$members,@ingroups);
    setgrent();
    $username =~ /\b$username\b/;  # pre-compile
    # We have to do this in two steps because the getgrent
    # stepping through the group list gets confused when we
    # do the getgrnam call within Cobalt::Group::group_del_user()
    while (($gr,$gp,$gd,$members) = getgrent)
    {
	push(@ingroups,$gr)
	    if ($members =~ //);
    }
    endgrent();
    foreach $gr (@ingroups)
    {
	Cobalt::Group::group_del_user($gr,$username);
    }

    # Make sure that we don't leave any vacation or forwarding stuff
    # around
    # I'm not going to sweat the return values here
    Cobalt::List::alias_delete($username); # forwarding alias
    Cobalt::Vacation::vacation_set_message($username,"")
	if (Cobalt::Vacation::vacation_get_message($username));
    Cobalt::Vacation::vacation_set_on($username,0) if (Cobalt::Vacation::vacation_get_on($username));
    Cobalt::Vacation::vacation_del_db($username);

    if ($Features{'vmachine'}) {
      # Remove the user from all mailing lists (including site-users) here
      # rationale: list-admin does not want to process bounce mails
      # problem: where do i find the domainname user is bound to? w/o
      #          that user@domain entries cannot be safely deleted
      # fix: use list_sites_del_mem to find the users domain!!
      my $domain = Cobalt::List::list_sites_del_mem($username);
      Cobalt::List::list_lists_del_mem("$username\@$domain");
    }
    # delete unqualified:-) user entries from all the mailing lists
    Cobalt::List::list_lists_del_mem($username);

    # Make sure email aliases are removed
    Cobalt::Email::mail_virtuser_del_byuser( $username );
    my @aliases = Cobalt::List::alias_get_bytarget($username);
    return $aliases[0] if ($aliases[0] =~ /^[134]\S*\s/);
    $return = Cobalt::List::alias_delete(@aliases);
    return $return if ($return !~ /^2/);

    print STDERR "Deletion of user $username complete\n" if ($Debug);
    return "2535 user $username deleted";
}

sub user_show
# user_show: Get information about a user
# Arguments: login name
# Return value: an array containing 
#   ("login", "full name", quota, usage, shell)
# Side effects: none
# Example: ($login,$name,$quota,$use,$shell) = Cobalt::User::user_show("mick");
{
    my ($login) = @_;
    return ("4504 no such user") unless (Cobalt::User::user_exist($login));
    my ($fullname,$shell) = (getpwnam($login))[6,8];
    my ($usage,$quota) = (Cobalt::Quota::repquota($login))[0,1];
    return ($login,$fullname,$quota,$usage,$shell);
}

sub user_list
# user_list: Return a list of users
#   Skip capitalized users, they're special.
# Arguments: none
# Return value: a sorted list of users, with admin on top
# Side effects: none
# Example: @users = Cobalt::User::user_list();
{
    my (@users,$name);
    setpwent();
    while ($name = getpwent)
    {
	next if ($name eq $Adminuser);
	next if ($name =~ /^[A-Z]/o);  # capitalized names are special
	push (@users,$name) unless ($System_users{$name});
    }
    endpwent();

    @users = ($Adminuser,sort(@users));
    return @users;
}

sub user_mod
# user_mod: modify the settings for a user.  An argument of 0 or ""
#   for any given field means no change.
# Arguments: ("login","new description","new cleartext password","new quota","new shell")
# Side effects: modifies the password file
#   If we change the password for admin, we also do it for root.
# Return value: a string of the form "895 Out of paper"
# Example: Cobalt::User::user_mod("randy","Randy the Killer Clown",0,0);
{
    my ($name,$newdescr,$newpasswd,$newquota,$newshell) = @_;
    my ($return);
    return "4523 Cannot modify $name: no such user"
	unless (Cobalt::User::user_exist($name));
    my $warn_quota = ($MaxQuota / 1024);
    return "4525 User quotas may not be set above $warn_quota" 
	if ($newquota > $MaxQuota);
    return "4524 Cannot modify $name: prohibited user"
	if ($System_users{$name});
    return "4505 Names cannot contain colons"
	if ($newdescr =~ /:/o);
    return $return unless (($return = Cobalt::User::lock_passwd()) =~ /^2/o);
    # Truncate the GCOS if it's too long
    $newdescr = substr($newdescr,0,$MaxGcosLen) 
	if (length($newdescr) > $MaxGcosLen);
    $newdescr =~ s/>/&gt;/og;  # No HTML!
    $newdescr =~ s/</&lt;/og;
    $newdescr =~ s/\'/&\#039;/og;

    ### modify the password file
    my ($name,$pass,$uid,$gid,$j1,$j2,$desc,$dir,$shell) = getpwnam($name);
    if ($newpasswd)
    {
	$pass = Cobalt::User::verify_password($newpasswd);
	if ($pass =~ /\s/o)
	{
	    # we got an error message, not a password
	    return Cobalt::User::user_abort("$pass");
	}
	
	print STDERR "Setting password for $name to $pass\n" if ($Debug);
    }
    if ($newdescr)
    {
	$desc = $newdescr;
    }
    if ($newshell)
    {
	$shell = $newshell;
    }
    if (($newdescr || $newpasswd || $newshell) && ($name eq $Adminuser))
    {
	open(PASSWD,$Ptmp) || return Cobalt::User::user_abort("4538 Cannot read $Ptmp: $!");
	open(PTMP,">$Lockdir/passwd") 
	    || return Cobalt::User::user_abort("4541 Cannot write tmpfile $Lockdir/passwd: $!");
	while (<PASSWD>)
	{
	    if (/^$Adminuser:/o)
	    {
		print PTMP "$name:$pass:$uid:$gid:$desc:$dir:$shell\n";
	    }
	    elsif (/^root:/o)
	    {
		print PTMP "root:$pass:0:0:Root:/root:/bin/sh\n";
	    }
	    else
	    {
		print PTMP;
	    }
	}
	close PASSWD;
	close PTMP;
	rename("$Lockdir/passwd",$Ptmp);
    }
    elsif ($newdescr || $newpasswd || $newshell)  # not the admin user
    {
	open(PASSWD,$Ptmp) || return Cobalt::User::user_abort("4532 Cannot read $Ptmp: $!");
	open(PTMP,">$Lockdir/passwd") 
	    || return Cobalt::User::user_abort("4533 Cannot write tmp file $Lockdir/passwd: $!");
	"$name:" =~ /^$name:/;
	while (<PASSWD>)
	{
	    if (//)
	    {
		# don't chomp
		print PTMP "$name:$pass:$uid:$gid:$desc:$dir:$shell\n";
	    }
	    else
	    {
		print PTMP;
	    }
	}
	close PASSWD;
	close PTMP;
	rename("$Lockdir/passwd",$Ptmp);
    }

    # modify the netatalk password entry
    if ($Features{'atalk'}) {
      my $atalk_ret = Cobalt::Atalk::atalk_set_passwd($name, $newpasswd);
      return $atalk_ret if ($atalk_ret !~ /^2/);
    }

    # modify the smbpasswd entry
    if ($Features{'smb'}) {
      # start by getting the new password hash
      $ENV{'SMBPASSWD'} = $newpasswd;
      open(GETHASH, "$Smb_gethash|")
	|| return Cobalt::User::user_abort("4579 Cannot run $Smb_gethash: $!");
      my $smbhashes = <GETHASH>;
      close(GETHASH);
      
      # modify smbpasswd
      open(SMBPTMP,$Smbptmp) 
	|| return Cobalt::User::user_abort("4580 Cannot open $Smbptmp to delete user: $!");
      open(SMBPASSWDLOCK,">$Lockdir/smbpasswd") 
	|| return Cobalt::User::user_abort("4582 Cannot clobber $Lockdir/smbpasswd: $!");
      "$name:" =~ /^$name:/; # pre-compile our pattern
      while (<SMBPTMP>)
	{
	  if (//) {
	    print SMBPASSWDLOCK "$name:$uid:$smbhashes:$desc:$dir:$shell\n";
	  } else {
	    print SMBPASSWDLOCK;
	  }
	}
      close SMBPASSWDLOCK;
      close SMBPTMP;
      rename("$Lockdir/smbpasswd",$Smbptmp) 
	|| return Cobalt::User::user_abort("4583 Cannot move $Smbptmp: $!");
      # done modifying smbpasswd
    }
    
    unless ($Features{'pam'}) {
      if ($newpasswd)
      {
	# Fix the htpasswd file
	open(HTPASSWD,$Htptmp) 
	    || return Cobalt::User::user_abort("4549 Cannot open htpasswd file $Htpasswd: $!");
	open(HTP,">$Lockdir/htpasswd") 
	    || return Cobalt::User::user_abort("4547 Cannot clobber $Htptmp: $!");
	"$name:" =~ /^$name:/; # pre-compile
	while (<HTPASSWD>)
	{
	    if (//)
	    {
		print HTP "$name:$pass\n";
	    }
	    else
	    {
		print HTP;
	    }
	}
	close HTP;
	close HTPASSWD;
	rename("$Lockdir/htpasswd",$Htptmp) 
	    || return Cobalt::User::user_abort("4548 Cannot rename $Htptmp: $!");
      }
    }
    return $return unless (($return = Cobalt::User::commit_passwd()) =~ /^2/o);
    
    ### Set the quota
    if ($newquota)
    {
	Cobalt::Quota::setqlim($Userpart,(getpwnam($name))[2],
		$newquota,($newquota + $Quota_harder));
	#Cobalt::Quota::setqlim($Varpart,(getpwnam($name))[2],
	#	$newquota,($newquota + $Quota_harder));
    }
    
    print STDERR "Modification of user $name complete\n" if ($Debug);
    return "2533 user $name modified successfully";
}

sub user_list_groups
# Find out what groups a user is in.
# Arguments: username
# Return value: a list of groups
# Side effects: none
# Example: @debbies_groups = Cobalt::User::user_list_groups("debbie");
{
    my ($username) = @_;
    my ($gr,$members,@grouplist);
    setgrent();
    $username =~ /\b$username\b/;  # pre-compile
    while (($gr,$members) = (getgrent)[0,3])
    {
	next if ($System_groups{$gr});
	push(@grouplist,$gr) if ($members =~ //);
    }
    endgrent();
    return sort(@grouplist);
}

sub user_list_site
# Find out what site a user is in.
# Assuming the list groupName-users contains a list of users of group groupName
#   and a user can only belong to one groupName-users list
# Arguments: username
# Return value: fully qualified domain name of the site, group name
#   Nothing if nothing found
# Side effects: none
# Example: ( $domain, $group ) = Cobalt::User::user_list_site( 'kevin' );
{
    my ( $userName )=@_;

    my $domain;
    my $group;

    my (@lists, $list);
    return $lists[0] if ((@lists = list_lists())[0] =~ /^[134]/);
    foreach $list ( @lists ) {
	if( $list=~/^(.+)-users$/ ) {
	    $group=$1;
	    if( grep /^$userName$/, list_get_mem($list) ) {
		$domain=( Cobalt::Vsite::vsite_get_bygroup($group) )[ 1 ];
		last;
	    }
	}
    }

    return ( $domain, $group );
}

sub user_info_sorted
# Get a sorted list of users, sorted by full name
# Arguments: noParseFullName
# Return value: a two dimensional array: each element of the array
#   is an array containing (full name, '', user name) if noParseFullName is true or (first name, last name, user name) otherwise
# Side effects: none
{
    my ( $noParseFullName )=@_;

    my ($login,@users,@admin);
    foreach $login (Cobalt::User::user_list())
    {
	#next if ($login eq $Adminuser);   # Don't change the admin user
	my $descr = (getpwnam($login))[6];
	my ($firstName,$lastName);

        if( !$noParseFullName ) {
	    # This way will split the first and last name by the last whitespace:
	    # @name = split(/\s+/o,$descr);
	    # $lastname = pop(@name);
	    # $firstname = join(' ',@name);

	    # This way will split the first and last name by the first whitespace:
	    if ($descr =~ /^\s*(\S+\s+)?(\S?.*\S)\s*$/o)
	    {
		($firstName,$lastName) = ($1,$2);
		# get rid of trailing whitespace
		$firstName =~ s/\s+$//og;
	    }
	}
        else {
	    $firstName=$descr;
	    $lastName='';
        }

	if ($login eq $Adminuser)
	{
	    @admin = ($firstName,$lastName,$login);
	}
	else
	{
	    my @user = ($firstName,$lastName,$login);
	    push(@users, [ @user ]);
	}
	print STDERR "My name is $firstName/$lastName/\n" if ($Debug);
    }

    # sort users 
    @users=sort { 
	# order by last name, first name, then user name 
	lc ( ${ $a }[ 1 ] ) cmp lc( ${ $b }[ 1 ] ) 
        or 
	lc( $ { $a }[ 0 ] ) cmp lc( $ { $b }[ 0 ] ) 
	or 
        lc( ${ $a }[ 2 ] ) cmp lc( ${ $b }[ 2 ] ); 
    } 
    @users; 

    # admin goes first
    #my @admin = ("",(getpwnam($Adminuser))[6],$Adminuser);
    unshift(@users, [ @admin ]);
    return @users;
}

##############################

sub lock_passwd
# Lock the password file so that nobody can collide with us.
# Arguments: none
# Side effects: copies $Passwd to $Ptmp and $Htpasswd to $Htptmp
# Return value: message of the form "2671 All of the engineers prefer MIPS"
{
    if (-e $Ptmp)
    {
      return Cobalt::User::user_abort("4507 The password file is locked.");
    }
    else
    {
	# Lock the password file by creating the ptmp file as a
	# copy of the password file.
	open(PASSWD,$Passwd) || return "4508 Cannot lock $Passwd: $!";
	open(PTMP,">$Ptmp") 
	    || return "4509 Cannot lock $Ptmp: $!";
	print PTMP while (<PASSWD>);
	close PTMP;
	close PASSWD;

	# Lock the smbpasswd file by creating the smbptmp file as a
	# copy of the password file.
	if ($Features{'smb'}) {
	  open(SMBPASSWD,$Smbpasswd) || return "4572 Cannot lock $Smbpasswd: $!";
	  open(SMBPTMP,">$Smbptmp") 
	    || return "4573 Cannot lock $Smbptmp: $!";
	  print SMBPTMP while (<SMBPASSWD>);
	  close SMBPTMP;
	  close SMBPASSWD;
	}

	# Lock the netatalk passwd file by calling atalk_lock_passwd
	# (which does the same thing as the others above)
	if ($Features{'atalk'}) {
	  my $atalk_ret = Cobalt::Atalk::atalk_lock_passwd();
	  return $atalk_ret if ($atalk_ret !~ /^2/);
	}

	# we need to lock this now in case the next opens abort
	open(LOCK,">$Lockdir/user-$$") || return Cobalt::User::user_abort("4599 Cannot get user lock: $!");
	print LOCK "$$\n$0";
	close LOCK;

    	unless ($Features{'pam'}) {
	  print STDERR "Copying htpassword file $Htpasswd to $Htptmp\n" 
	    if ($Debug);
	  if (! -e $Htpasswd)
	  {
	    # this looks like a fresh system
	    print STDERR "Creating new htpassword file $Htpasswd\n" 
	      if ($Debug);
	    open(HTPASS,">$Htpasswd")
	      || return "4574 Cannot create $Htpasswd: $!";
	    close HTPASS;
	  }
	  Cobalt::Utility::sauce_cp($Htpasswd,$Htptmp);
	}
    }

    print STDERR "Locked $Ptmp\n" if ($Debug);
    return "2511 Password file locked";
}

sub commit_passwd
# Unlocks the password file by moving ptmp into the place of passwd
# Arguments: none
# Side effects: replaces password with ptmp and htpasswd with htptmp
{
    rename($Ptmp,$Passwd) 
	|| return "4510 Cannot replace $Passwd with $Ptmp: $!";
    chmod(0644,$Passwd);
    if ($Features{'pam'}) {
        # currently sauce supports shadow passwd only
        # support for radius/nis/etc requires libpwdb
	system("$Pwconv 2>/dev/null") &&
	    return ("3999 user_cant_pwconv");
    }
    else {
    rename($Htptmp,$Htpasswd) 
	|| return "4511 Cannot replace $Htpasswd with $Htptmp: $!";
    chmod(0644,$Htpasswd);
    }
    if ($Features{'smb'})
    {
	rename($Smbptmp,$Smbpasswd)
	  || return "4581 Cannot replace $Smbpasswd with $Smbptmp: $!";
	chmod(0600,$Smbpasswd);
    }
    if ($Features{'atalk'}) {
      my $atalk_ret = Cobalt::Atalk::atalk_commit_passwd();
      return $atalk_ret if ($atalk_ret !~ /^2/);
    }

    unlink("$Lockdir/user-$$");
    endpwent();  # flush the cache
    return "2512 Password file changes committed";
}

sub unlock_passwd
# Unlocks the password file by wiping ptmp
# Arguments: none
# Side effects: wipes ptmp, htptmp
{
    unlink($Ptmp) || return "4512 Cannot delete ptmp $Ptmp: $!";
    print STDERR "Removed $Ptmp\n" if ($Debug);

    unless ($Features{'pam'}) {
      unlink($Htptmp) || return "4513 Cannot delete htptmp $Htptmp: $!";
      print STDERR "Removed $Htptmp\n" if ($Debug);
    }

    if ($Features{'smb'}) {
      unlink($Smbptmp) || return "smb_cant_unlink_smbptmp Cannot unlink $Smbptmp: $!";
    }
    if ($Features{'atalk'}) {
      my $atalk_ret = Cobalt::Atalk::atalk_unlock_passwd();
      return $atalk_ret if ($atalk_ret !~ /^2/);
    }

    unlink("$Lockdir/user-$$");
    endpwent();  # flush the cache
    return "2513 Password file unlocked";
}

sub verify_password
# Make sure the password is okay
# Arguments: the password
#   If the password is "*" then return a locked password entry
# Return value: the encrypted version of the password, or an error message.
#   Encrypted passwords will always be 13 characters; error messages
#   never will.
#TBD# SECURITY ALERT
# If we wanted to enforce strong passwords, this is where we would
# do it.  Add tests to this function to see if a password is good enough,
# and return an error if it isn't...
{
    my ($userpass) = @_;
    
    return "*" if ($userpass eq "*");  # Locked entry
    if ($userpass =~ /[\000-\037\177]/o)
    {
	return "4534 The password may not contain control characters";
    }
    if (length($userpass) > $MaxPassLen)
    {
	return "4535 The given password is too long.  Passwords must be less than $MaxPassLen characters";
    }
    elsif (length($userpass) < $MinPassLen)
    {
	return "4536 The given password is too short.  Passwords must be at least $MinPassLen characters.";
    }

    # Generate the encrypted password
    srand();
    my $salt = ('a'..'z','A'..'Z',0..9)[rand(62)] . 
	('a'..'z','A'..'Z',0..9)[rand(62)];
    my $cryptpass = crypt($userpass, $salt);
    return $cryptpass;
}

sub user_abort
# leave in a (more or less) graceful manner
# Arguments: a string to print
# Side effects: unlocks anything we've locked
{
    if (-e "$Lockdir/user-$$")
    {
	my $return = Cobalt::User::unlock_passwd();
	return $return unless ($return =~ /^2/o);
    }
    return join('',@_);
}

sub user_get_first_available_uid
# Read in the password file and figure out what the lowest
# unallocated UID is.
# Arguments: UID to start from
# Side effects: none
# Return value: a nice fresh UID to use
{
  my $lowest_id = $_[0];
  $lowest_id = $Firstuid if ($lowest_id < $Firstuid);
  
  my %ids = ();
  open(PW, "$Passwd") || return "4569 Cannot open $Passwd: $!\n";
  while (<PW>) {
    chomp;
    my ($user, $pw, $id, $rest) = split(/\:/o, $_, 4);

    # Be careful and fill in the UID holes
    if ($id == $lowest_id) {
      $lowest_id++;
    }
    elsif ($id > $lowest_id) {
      $ids{$id} = 1;
    }
  }
  close(PW);

  # Incrementing for what?
#  $lowest_id++;
  $lowest_id++ while ($ids{$lowest_id});
  return $lowest_id;
}

1;
