#
# Written by Brion Moss (brion@cobaltnet.com, brion@nda.com)
#            Kevin K.M. Chiu (kevin@cobaltnet.com)
#
# Copyright 1998 Cobalt Networks http://www.cobaltnet.com
#

package Cobalt::Vacation;
use AutoLoader 'AUTOLOAD';
BEGIN {
require Exporter;
@ISA    = qw(Exporter);
@EXPORT = qw(vacation_set_message vacation_get_message
	     vacation_set_on vacation_get_on vacation_del_db);

require Cobalt::List;
import Cobalt::List qw(alias_get alias_get_vacationless alias_add alias_set);
require Cobalt::Language;
import Cobalt::Language qw(:DEFAULT);
}

#use Cobalt::Paths;
#use strict;
1;
__END__

sub vacation_get_on
# Check if vacation forwarding is active
# Arguments: user name
# Return value: 1 for on, 0 if off
# Side effects: none
{
    my($user) = @_;
    my @alias = Cobalt::List::alias_get($user);
    return 0 if ($alias[0] =~ /^[134]\S*\s/); # error
    (grep(/$Vaca_prog/o,@alias)) ? (return 1) : (return 0);
}

sub vacation_set_on
# Turn the vacation forwarding on or off
# Arguments: user name, state -- 1 for on, 0 for off,
#   domain where the user belongs to (optional)
# Return value: the usual
# Side effects: edits the alias file
{
    my ($user,$state,$domain) = @_;
    my $emailAddr=$user;
    if( $domain ) { $emailAddr.='@'.$domain; }

    my $return;
    if ($state && Cobalt::Vacation::vacation_get_on($user))
    {
	return "2325 $user already has autoreply active";
    }
    elsif ((! $state) && (! Cobalt::Vacation::vacation_get_on($user)))
    {
	return "2326 $user already has autoreply deactivated";
    }
    elsif ($state)  # turn it up, man
    {
	my $vacaline;

	if ($Language{"japanese"})
	{
	  $vacaline = "\"|$Vaca_prog $Vaca_dir/$user.jmsg $emailAddr\"";
	}
	else
	{
	  $vacaline = "\"|$Vaca_prog $Vaca_dir/$user.msg $emailAddr\"";
	}
	$return = Cobalt::List::alias_add($user, "\\$user", $vacaline);
    }
    else  # no vacation
    {
	$return = Cobalt::List::alias_set($user,Cobalt::List::alias_get_vacationless($user));
    }
    return $return;
}

sub vacation_get_message
# Get the message being used by the vacation program
# Arguments: user name
# Return value: the message, or "" if no message
# Side effects: none
{
    my ($user) = @_;
    my $message = "$Vaca_dir/$user.msg";
    if (-r $message)
    {
	open(VACA,$message) ||
	    Cobalt::List::abort("4180 cannot read vacation message for $user: $!");
	my @vaca = <VACA>;
	close VACA;
	my $vaca = join('',@vaca);
	return $vaca;
    }
    return "";
}

sub vacation_set_message
# Set the message being used by the vacation program
# Arguments: user name, the message in a big string
#   if the string is "", then the file is deleted.
# Return value: the usual status string
# Side effects: creates the message file
{
    if (! -e $Vaca_dir)
    {
	mkdir($Vaca_dir,01777) ;
	chmod(01777,$Vaca_dir);
    }
    my ($user,$message) = @_;
    my $messagefile = "$Vaca_dir/$user.msg";
    my $messagefileJ = "$Vaca_dir/$user.jmsg" if ($Language{"japanese"});
    if ($message)
    {
	open(VACA,">$messagefile") ||
	    return("4181 cannot write vacation message for $user: $!");
	print VACA $message;
	# Brion, you just make the file longer everytime you save it
#	print VACA "\n";
	close VACA;

	# need to convert file from Shift-JIS to JIS for sendmail
	system( "$ShiftJISToJIS_prog $messagefile >$messagefileJ" )
	  if ($Language{"japanese"});

	my $uid;
	if ($uid = (getpwnam($user))[2])
	{
	    # might not be for a real user
	    chown($uid,-1,$messagefile)
		|| return "3999 vacation_cant_chown_msg_file";
	    if ($Language{"japanese"}) {
		chown($uid,-1,$messagefileJ)
		    || return "3999 vacation_cant_chown_msgj_file";
	    }
	}

	# needs to be vacation.pl readable
	chmod 0644,$messagefile;
	if ($Language{"japanese"}) { chmod 0644,$messagefileJ; }

	return "2327 Vacation message set in $messagefile";
    }
    else
    {
	unlink($messagefile) if (-e $messagefile);
	unlink("$messagefileJ") if ($Language{"japanese"} &&
				    (-e "$messagefileJ"));
	return "2328 $user now has no message file";
    }
}

sub vacation_del_db
# Deletes the vacation database (log of who has been responded to) for
#   a user or alias
# Arguments: user name
# Side effects: deletes the database
# Return value: none
{
    my ($user) = @_;
    unlink("$Vaca_dir/$user.db");
}

1;
