# This autoloader file was built automatically by 
# /usr/local/sbin/autoize from Atalk.pm

# Copyright 1998 Cobalt Networks (http://www.cobaltnet.com)

use strict;
### START HARDCODED PATHS
my $Netatalk_passwd = "/etc/atalk/passwd";
my $Netatalk_ptmp = "/etc/locks/netatalk_ptmp";
my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = ($Debug ||= 0);
### END HARDCODED PATHS

# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/Atalk.pm.  Changes made here will be lost.
package Cobalt::Atalk;

# atalk_del_user
# remove a user's record from the netatalk passwd file
# args:
#       $user = username
#
# No check is done to make sure $user is a valid user.
#
# Note: The Samba equivelent of this function is in User::user_del.
#       In retrospect, I think it was a mistake not to seperate it out
#
sub atalk_del_user {
  my ($user) = @_;
  my $line;

  print STDERR "atalk: called Cobalt::Atalk::atalk_del_user($user)\n" if ($Debug);

  # make sure files can only be read by root
  umask(0177);
  # security check
  # this is *very* important... this file contains *plaintext* passwords
  chown(0,0,$Netatalk_passwd);
  chmod(0600,$Netatalk_passwd);

  # parse & modify passwd file
  open(PTMP, ">>$Netatalk_ptmp")
    or return "4016 Cannot append to /etc/locks/netatalk_ptmp: $!";
  open(ATALKPASS, "$Netatalk_passwd")
    or return "4020 Cannot open /etc/atalk/passwd: $!";
  while (<ATALKPASS>) {
    # comment?
    if (/^\s*\#/) {
      print PTMP;
      next;
    }

    # valid password line?
    if (/^([^:]+)+:.+\n$/o) {
      # write out the line if it doesn't belong to $user
      if ($user ne $1) {
	print PTMP;
      }
    }
  }
  close(PTMP);
  close(ATALKPASS);
  return "2012 Atalk password record for user $user was removed";
}


1;
