# This autoloader file was built automatically by 
# /usr/local/sbin/autoize from Atalk.pm

# Copyright 1998 Cobalt Networks (http://www.cobaltnet.com)

use strict;
### START HARDCODED PATHS
my $Netatalk_passwd = "/etc/atalk/passwd";
my $Netatalk_ptmp = "/etc/locks/netatalk_ptmp";
my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = ($Debug ||= 0);
### END HARDCODED PATHS

# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/Atalk.pm.  Changes made here will be lost.
package Cobalt::Atalk;

# atalk_set_passwd
# sets the plaintext password in the netatalk password file
# args:
#       $user = username
#       $passwd = password
#
# Note: The Samba equivelent of this function is in User::user_add.
#       In retrospect, I think it was a mistake not to seperate it out
#
sub atalk_set_passwd {
  my ($user, $passwd) = @_;
  my $line;
  my $found = 0;

  print STDERR "atalk: called Cobalt::Atalk::atalk_set_passwd($user, $passwd)\n" if ($Debug);

  # make sure files can only be read by root
  umask(0177);

  # does $Netatalk_passwd exist?
  if (! -e $Netatalk_passwd) {
    open(ATALKPASS, ">$Netatalk_passwd")
      or return "4012 Cannot write to /etc/atalk/passwd: $!";
    close(ATALKPASS);
  }

  # security check
  # this is *very* important... this file contains *plaintext* passwords
  chown(0,0,$Netatalk_passwd);
  chmod(0600,$Netatalk_passwd);

  # parse & modify passwd file
  open(PTMP, ">>$Netatalk_ptmp")
    or return "4013 Cannot append to /etc/locks/netatalk_ptmp: $!";
  # protect cleartext password
  chmod(0600,$Netatalk_ptmp);
  open(ATALKPASS, "$Netatalk_passwd")
    or return "4014 Cannot open /etc/atalk/passwd: $!";
  my $inconfig = 0;
  while (<ATALKPASS>) {
    print STDERR "atalk: $_" if ($Debug);
    # comment?
    if (/^\s*\#/) {
      print PTMP;
      next;
    }

    # valid password line?
    if (/^([^:]+)+:.+\n$/o) {
      # write it out
      if ($user eq $1) {
	print PTMP "$user:$passwd\n";
	print STDERR "atalk: replacing $user:$passwd\n" if ($Debug);
	$found = 1;
      } else {
	print PTMP;
      }
    }
  }

  # if we've made it here and $found = 0 then the user didn't exist in the
  # the file
  if (! $found) {
    print PTMP "$user:$passwd\n";
    print STDERR "atalk: adding $user:$passwd\n" if ($Debug);
  }

  close(PTMP);
  close(ATALKPASS);
  return "2011 Set atalk password for user $user";
}


1;
