# This autoloader file was built automatically by 
# /usr/local/sbin/autoize from Dhcpd.pm

# Copyright 1998 Cobalt Networks (http://www.cobaltnet.com)

use strict;
### START HARDCODED PATHS
my $Dhcpd_conf = "/etc/dhcpd.conf";
my $Lockdir = "/etc/locks";
my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = ($Debug ||= 0);
### END HARDCODED PATHS

# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/Dhcpd.pm.  Changes made here will be lost.
package Cobalt::Dhcpd;

sub dhcpd_set_parameters
# Set the dhcpd server paramters
# Arguments: domain name, dns server 1, dns server 2,
#   gateway, netmask, maximum lease time
#   Any argument except the netmask can be empty.
# Side Effects: writes $Dhcpd_conf; creates a new one if none exists
#   This function does *not* restart dhcpd; it assumes that
#   dhcpd_clean_ranges() will be called next, and that function
#   takes care of restarting the daemon.
{
	my($domain,$dns1,$dns2,$gw,$nm,$maxlease) = @_;
	return "4049 Invalid netmask $nm"
	  if (($nm eq "0.0.0.0") ||
	      (Cobalt::Network::net_convert_netmask($nm) !~ /^\d+$/o));

	my($ipaddr) = (Cobalt::Network::net_get_ipaddr())[0];
	my($broadcast) = Cobalt::Network::net_get_broadcast($ipaddr,$nm);

	sleep 2 if (-e "$Lockdir/dhcpd.conf"); # Avoid race
	return "4050 DHCP server configuration files in use" if (-e "$Lockdir/dhcpd.conf");
	return "4051 $domain is not a valid domain name." 
	  if ($domain && (! Cobalt::Netutil::netutil_test_domainname($domain)));
	return "4052 Please enter a valid DNS server address" 
	  if ($dns1 && (! Cobalt::Netutil::netutil_test_ipaddr($dns1)));
	return "4053 Please enter a valid DNS server address" 
	  if ($dns2 && (! Cobalt::Netutil::netutil_test_ipaddr($dns2)));
	return "4054 $gw is not a valid gateway address" 
	  if ($gw && (! Cobalt::Netutil::netutil_test_ipaddr($gw)));
	return "4055 $maxlease is not valid; the maximum lease time must be an integer greater than zero" 
	  if ($maxlease && ($maxlease !~ /^\d+$/o));

	open(DHCPnew,"> $Lockdir/dhcpd.conf") 
	  || return "4056 Could not update DHCP server configuration: $!";
	if (-r $Dhcpd_conf) {
	  open(DHCPcnf, "$Dhcpd_conf") || return "4057 Could not update DHCP server configuration: $!";
	  while(<DHCPcnf>) {
	        # All arguments are considered optional...if aren't given an
	        # argument, then we comment out the corresponding line.
	        # We leave the line around as a placeholder so it's easy
	        # to add back in later.
	        if (/^\#?option domain-name\s+/o)
		{
		  print DHCPnew "#" unless ($domain); # comment it out
		  print DHCPnew "option domain-name \"$domain\";\n";
		} elsif (/^\#?option domain-name-servers/o)
		{
		  # if no nameservers are defined, leave it commented out
		  print DHCPnew "#" unless ($dns1);
		  my $dns2str;
		  $dns2str = ", $dns2" if ($dns2);
		  print DHCPnew "option domain-name-servers $dns1$dns2str;\n";
		} elsif (/^\#?option routers /o)
		{
		        print DHCPnew "#" unless ($gw); # comment out
			print DHCPnew "option routers $gw;\n";
		} elsif (/^\#?option subnet-mask /o)
		{
		        print DHCPnew "#" unless ($nm);  # comment out
			print DHCPnew "option subnet-mask $nm;\n";
		} elsif (/^\#?default-lease-time /o)
		{
		        print DHCPnew "#" unless ($maxlease);  # comment out
			print DHCPnew "default-lease-time $maxlease;\n";
		} elsif (/^\#?max-lease-time /o)
		{
		        print DHCPnew "#" unless ($maxlease);  # comment out
			print DHCPnew "max-lease-time $maxlease;\n";
		} elsif (/^\#?option broadcast-address /o)
		{
		        print DHCPnew "#" unless ($broadcast);  # comment out
			print DHCPnew "option broadcast-address $broadcast;\n";
		} elsif (/^subnet ([\d\.]+) netmask ([\d\.]+)/o)
		{
		  # Set subnet parameters based on the new netmask and gateway
		  my $subnet = Cobalt::Network::net_get_network($ipaddr,$nm);
		  print STDERR "Changing dynamic range from subnet $1/$2 to $subnet/$nm\n" if ($Debug);
		  print DHCPnew "subnet $subnet netmask $nm {\n";
		} else
		{
			print DHCPnew;
		}
	      }
	  close(DHCPcnf);
	} else {
	  # Creating a fresh new config file
	  $dns1 .= ", $dns2" if ($dns2);
	  print DHCPnew "# Configuration file for ISC dhcpd\n";
	  print DHCPnew "# Created by the Cobalt user interface\n\n";
	  print DHCPnew "server-identifier ", Cobalt::Network::net_get_hostname(), ";\n\n";
	  print DHCPnew "# common option definitions\n";
	  if ($domain) {
	    print DHCPnew "option domain-name \"$domain\";\n";
	  } else {
	    print DHCPnew "#option domain-name \n";
	  }
	  if ($dns1) {
	    print DHCPnew "option domain-name-servers $dns1;\n"
	  } else {
	    print DHCPnew "#option domain-name-servers\n"
	  }
	  if ($gw) {
	    print DHCPnew "option routers $gw;\n";
	  } else {
	    print DHCPnew "#option routers \n";
	  }
	  if ($nm) {
	    print DHCPnew "option subnet-mask $nm;\n";
	  } else {
	    print DHCPnew "#option subnet-mask \n";
	  }
	  if ($broadcast) {
	    print DHCPnew "option broadcast-address $broadcast;\n";
	  } else {
	    print DHCPnew "#option broadcast-address \n";
	  }
	  if ($maxlease) {
	    print DHCPnew "default-lease-time $maxlease;\n";
	    print DHCPnew "max-lease-time $maxlease;\n";
	  } else {
	    print DHCPnew "#default-lease-time \n";
	    print DHCPnew "#max-lease-time \n";
	  }
	  print DHCPnew "use-host-decl-names on;\n\n";
	}
	close(DHCPnew);

	rename("$Lockdir/dhcpd.conf","$Dhcpd_conf")
	  || return "4058 Could not update DHCP server configuration: $!";
	
	Cobalt::Dhcpd::dhcpd_hup();
	return "2018 DHCP server updated successfully";
}


1;
