# This autoloader file was built automatically by 
# /usr/local/sbin/autoize from Dhcpd.pm

# Copyright 1998 Cobalt Networks (http://www.cobaltnet.com)

use strict;
### START HARDCODED PATHS
my $Dhcpd_conf = "/etc/dhcpd.conf";
my $Lockdir = "/etc/locks";
my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = ($Debug ||= 0);
### END HARDCODED PATHS

# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/Dhcpd.pm.  Changes made here will be lost.
package Cobalt::Dhcpd;

sub dhcpd_set_range
# Allocate a range of addresses to give out dynamically
# Arguments: a hash table which lists a set of ranges, in the format
#          %hash = ($low1 => $high1, $low2 => $high2, ...);
# Return value: usual status string of form: "4819 Don't sleep in the subway"
# Side effects: modifies the DHCPD config file, HUPs dhcpd
{
	my(%range) = @_;
	my($setRange,$low);
	sleep 2 if (-e "$Lockdir/dhcpd.conf"); # Avoid race
	return "4037 DHCP server configuration files in use" if (-e "$Lockdir/dhcpd.conf");

	# Get the network parameters for the primary interface
	my($ipaddr,$nm) = Cobalt::Network::net_get_ipaddr();

	foreach $low (keys %range) {
	  unless (Cobalt::Network::net_network_ismember($low,$ipaddr,$nm)) {
	    return "4038 The high and low addresses of the range must both be on the same subnet as the servers network interface";
	  }
	  unless (Cobalt::Network::net_network_ismember($range{$low},$ipaddr,$nm)) {
	    return "4039 The high and low addresses of the range must both be on the same subnet as the servers network interface";
	  }
	  my @low = split(/\./o,$low);
	  my @hi  = split(/\./o,$range{$low});
	  unless (($low[3] <= $hi[3]) &&
		  ($low[2] <= $hi[2]) &&
		  ($low[1] <= $hi[1]) &&
		  ($low[0] <= $hi[0])) {
	    return "4040 The upper end of the IP address range must be higher than the lower end";
	  }
	}

	open(DHCPcnf,"$Dhcpd_conf") 
	  || return "4041 Could not read DHCP server configuration: $!";
	open(DHCPnew,"> $Lockdir/dhcpd.conf") 
	  || return "4042 Could not update DHCP server configuration: $!";
	while(<DHCPcnf>)
	{
		print DHCPnew  unless (/^\s*subnet /o ... /^\s*\}\s*$/o);
	}
	close(DHCPcnf);

	# Create a subnet block if necessary
	my ($subnet) = Cobalt::Network::net_get_network($ipaddr,$nm);
	print DHCPnew "subnet $subnet netmask $nm {\n";
	foreach $low (keys %range)
	{
		print DHCPnew "  range $low ".$range{$low}.";\n";
	}
	print DHCPnew "}\n";
	close(DHCPnew);

	rename("$Lockdir/dhcpd.conf","$Dhcpd_conf")
	  || return "4043 Could not update DHCP server configuration: $!";

	Cobalt::Dhcpd::dhcpd_hup(); # Kick it
	return "2016 DHCP server updated successfully";
}


1;
