# This autoloader file was built automatically by 
# /usr/local/sbin/autoize from Expert.pm

# Copyright 1998 Cobalt Microserver (http://www.cobaltmicro.com)

use strict;
### START HARDCODED PATHS
my $Console = "/dev/console";
my $Debug = "0";
my $Htconf = "/etc/httpd/conf";
my $Htgroup_dir = "/home/sites";
my $Htscript = "/etc/rc.d/init.d/httpd.init";
my $If_cfg_dir = "/etc/sysconfig/network-scripts";
my $Ifup = "/etc/sysconfig/network-scripts/ifup";
my $Lockdir = "/etc/locks";
my $Log_archive = "/var/log/httpd";
my $Logrotate_dir = "/etc/logrotate.d";
my $Subsys = "/var/lock/subsys";
my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = $Debug;
### END HARDCODED PATHS

# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/Expert.pm.  Changes made here will be lost.
package Cobalt::Expert;

sub vhost_set
# Configure or create the settings for a virtual interface
# Arguments: the new IP address, netmask, fully qualified host name, group name
# Side effects: edits the files in /etc/sysconfig
#   Reconfigures the network interface for the new address
#   Adds entries to the httpd.conf to listen to the new address
# Note: this doesn't handle the mail configuration aspects of setting
#   up a virtual host, only the network interface and the web server.
{
    my ($addr,$mask,$name,$group) = @_;

    $addr =~ s/\.0+/\./go;  # get rid of excess leading zeros
    # If we're given a weird IP and it's not via the web, wake the LCD
    if ($addr !~ /^(\d+\.\d+\.\d+\.\d+)$/o)
    {
	print STDERR "Got invalid IP address $addr\n" if ($Debug);
	return "4140 $addr is not a valid IP address";
    }

    # Check that the hostname is valid
    if ($name =~ /^([\w\-\.]+)$/o)
    {
	$name = $1;  # untaint
    }
    else
    {
	return "4141 invalid hostname $name";
    }

    # There must be an existing group to associate with this interface
    unless (getgrnam($group))
    {
	return "4142 Error: Group does not exist";
    }

    my $broadcast = &net_get_broadcast($addr,$mask);
    my $network = &get_network($addr,$mask);
    my $main_addr = (net_get_ipaddr())[0];
    
    # Are we modifying an existing interface or adding a new one?
    my ($netif) = (vhost_get_byaddr($addr))[0];
    print STDERR "Netif is ``$netif''\n" if ($Debug);
    unless ($netif)
    {
	# Get the number for the next interface to add
	my $ifnum;
	opendir(IFDIR,$If_cfg_dir) || 
	    return "4143 Cannot read network script directory $If_cfg_dir: $!";
	my @ifcfgs = sort(grep(/^ifcfg-eth0:\d+/o,readdir(IFDIR)));
	closedir IFDIR;
	for ($ifnum = 0; $ifnum <= $#ifcfgs; $ifnum++)
	{
	    if ($ifcfgs[$ifnum] ne "ifcfg-eth0:$ifnum")
	    {
		$netif = "eth0:$ifnum";
		last;
	    }
	}
	$netif ||= "eth0:" . ($#ifcfgs + 1);
	print STDERR "Allocating new interface $netif for $addr\n" if ($Debug);
    }

    print STDERR "New settings for $netif: $addr/$mask $name $group\n"
	if ($Debug);
    # ifcfg-eth0:(number) is our main lock file
    return "4144 Cannot configure virtual interface: ifcfg-$netif is locked"
	if (-e "$Lockdir/ifcfg-$netif");
    open(O,">$Lockdir/ifcfg-$netif")
	|| return "4145 Cannot get lock for interface $netif: $!";

    # get all our locks at once
    if (-e "$Lockdir/httpd.conf")
    {
	unlink("$Lockdir/ifcfg-$netif");
	return "3140 $Htconf/httpd.conf is locked";
    }
    unless (open(HTCTMP,">$Lockdir/httpd.conf"))
    {
	unlink("$Lockdir/ifcfg-$netif");
	return "3141 Cannot write $Lockdir/httpd.conf: $!";
    }

    print O "DEVICE=$netif\n";
    print O "IPADDR=$addr\n";
    print O "NETMASK=$mask\n";
    print O "NETWORK=$network\n";
    print O "BROADCAST=$broadcast\n";
    print O "ONBOOT=yes\nALIAS=yes\nBOOTPROTO=none\n";
    close O;

    my %listenaddrs;
    my $donv = 0;
    unless (open(HTCONF,"$Htconf/httpd.conf"))
    {
	unlink("$Lockdir/ifcfg-$netif");
	unlink("$Lockdir/httpd.conf");
	return "3142 Cannot read $Htconf/httpd.conf: $!";
    }
    while (<HTCONF>)
    {
	if (/^Listen ([^:]+)\:(\d+)/o)
	{
	    $listenaddrs{$1} = $2;
	    print HTCTMP;
	}
	elsif (/^<VirtualHost $addr>/)
	{
	    # There's already an entry for us, rewrite it
	    print STDERR "Fixing virtual host entry\n" if ($Debug);
	    print HTCTMP "<VirtualHost $addr>\n";
	    print HTCTMP "ServerName $name\n";
	    print HTCTMP "ServerAdmin $group\@$name\n";
	    print HTCTMP "DocumentRoot $Htgroup_dir/$group\n";
	    print HTCTMP "TransferLog $Log_archive/$addr-access\n";
	    print HTCTMP "ErrorLog $Log_archive/$addr-error\n";
	    print HTCTMP "</VirtualHost>\n";
	    $donv = 1;
	    
	    # Skip past the rest of the entry
	    while (<HTCONF>)
	    {
		last if (/^<\/VirtualHost>/o);
	    }
	}
	else
	{
	    print HTCTMP;
	}
    }
    print HTCTMP "Listen ${main_addr}:80\n" unless $listenaddrs{$main_addr};
    print HTCTMP "Listen ${addr}:80\n" unless $listenaddrs{$addr};
    unless ($donv)
    {
	print STDERR "Adding new virtual host entry\n" if ($Debug);
	print HTCTMP "<VirtualHost $addr>\n";
	print HTCTMP "ServerName $name\n";
	print HTCTMP "ServerAdmin $group\@$name\n";
	print HTCTMP "DocumentRoot $Htgroup_dir/$group\n";
	print HTCTMP "TransferLog $Log_archive/$addr-access\n";
	print HTCTMP "ErrorLog $Log_archive/$addr-error\n";
	print HTCTMP "</VirtualHost>\n";
    }
    close HTCONF;
    close HTCTMP;
    unless (rename("$Lockdir/httpd.conf","$Htconf/httpd.conf"))
    {
	unlink("$Lockdir/ifcfg-$netif");
	unlink("$Lockdir/httpd.conf");
	return "3143 Cannot update $Htconf/httpd.conf: $!";
    }

    unless (open(LOGROT,">$Logrotate_dir/$addr-apache"))
    {
	unlink("$Lockdir/ifcfg-$netif");
	return "3144 Cannot create $Logrotate_dir/$addr-apache: $!";
    }
    print LOGROT <<END;
/var/log/httpd/$addr-access {
   compress
   size 30M
    postrotate
        killall -USR1 httpd
    endscript
    }

/var/log/httpd/$addr-error {
    compress
    size 5M
    postrotate
        killall -USR1 httpd
    endscript
    }
END

    # unlock
    unless (rename("$Lockdir/ifcfg-$netif","$If_cfg_dir/ifcfg-$netif"))
    {
	unlink("$Lockdir/ifcfg-$netif");
	return "3145 Cannot update virtual interface config file: $!";
    }

    # Bring the interface up
    print STDERR "$Ifup $netif\n" if ($Debug);
    system("$Ifup $netif >$Console 2>&1\n");

    # Restart the web server, if it's up
    if (-f "$Subsys/httpd")
    {
	system("$Htscript stop >$Console 2>&1");
	system("$Htscript start >$Console 2>&1");
    }

    return "2140 Virtual interface for address $addr has been set up";
}

use Cobalt::Language;

1;
