# This autoloader file was built automatically by 
# /usr/local/sbin/autoize from Fetchmail.pm

# Copyright 1998 Cobalt Networks (http://www.cobaltnet.com)

use strict;
### START HARDCODED PATHS
my $Fetchmail = "/usr/bin/fetchmail";
my $Fetchmail_lock = "/etc/locks/fetching_remote_mail";
my $Fetchmailrc_dir = "/var/fetchmail";
my $Mail_cmd = "/bin/mail";
my $Sauce_log = "/var/cobalt/sauce.log";
my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = ($Debug ||= 0);
### END HARDCODED PATHS

# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/Fetchmail.pm.  Changes made here will be lost.
package Cobalt::Fetchmail;

sub fm_do_fetch
# retrieves remote mail based on fetchmail rc files in /var/fetchmail
# args: none
{
  my $file;
  my $ret;
  my $msg;

  print STDERR "fetchmail: called fm_do_fetch\n" if ($Debug);

  if (-e $Fetchmail_lock) {
    # should we check for a really old lock here to make sure a stale lock
    # isn't left around, preventing mail access?
    print STDERR "fetchmail: fetchmail is locked\n" if ($Debug);
    return "4194 Fetchmail lockfile found";
  }

  # set lockfile (this isn't race-safe, but it should be good enough)
  open(LOCK, ">$Fetchmail_lock");
  close(LOCK);
  
  # grab text messages
  my $fetchmailErrorSubject = Cobalt::Language::msg_get('fetchmailErrorSubject');
  my $fetchmailErrorMsg = Cobalt::Language::msg_get('fetchmailErrorMsg');

  # step through each fetchmail resource file
  opendir(RCDIR, $Fetchmailrc_dir);
  while ($file = readdir(RCDIR)) {
    if ($file !~ /^\./) {

      # check if proto is ETRN
      if ((Cobalt::Fetchmail::fm_get_domain_rc())[3]) {
	print STDERR "fetchmail: calling $Fetchmail -s -f $Fetchmailrc_dir/$file" if ($Debug);
	$ret = system("$Fetchmail -s -f $Fetchmailrc_dir/$file >>$Sauce_log 2>&1");
      } else {
	# use the 'a' flag to make sure all mail is retrieved
	# with 'a' read messages will be left on the server
	print STDERR "fetchmail: calling $Fetchmail -as -f $Fetchmailrc_dir/$file" if ($Debug);
	$ret = system("$Fetchmail -as -f $Fetchmailrc_dir/$file >>$Sauce_log 2>&1");
      }

      # convert to the "real" return code
      $ret=int( $ret/256 );

      if( $ret==2 ) { $msg = Cobalt::Language::msg_get('fetchmail_cant_pop'); }
      elsif( $ret==3 || $ret==9 ) { $msg = Cobalt::Language::msg_get('fetchmail_cant_auth'); }
      elsif( $ret==4 || $ret==7 ) { $msg = Cobalt::Language::msg_get('fetchmail_cant_connect'); }
      elsif( $ret==10 ) { $msg = Cobalt::Language::msg_get('fetchmail_cant_smtp'); }
      elsif( $ret==11 ) { $msg = Cobalt::Language::msg_get('fetchmail_cant_dns'); }
      elsif( ($ret != 0) && ($ret != 1) ) { $msg = Cobalt::Language::msg_get('fetchmail_err_generic'); }

      # send error report if necessary
      if ($msg ne '') {
	print STDERR "fetchmail: fetchmail threw an error: $msg\n" if ($Debug);
	
	if ($file =~ /^multidrop/) {
	  # send mail to admin
	  open(MAIL, "|$Mail_cmd -s '$fetchmailErrorSubject' sys");
	} else {
	  # send mail to user
	  open(MAIL, "|$Mail_cmd -s '$fetchmailErrorSubject' $file");
	}
	print MAIL $fetchmailErrorMsg . "\n";
	print MAIL "$msg\n";
	close(MAIL);
      }
    }
  }
  close(RCDIR);

  # remove lock
  unlink($Fetchmail_lock);
}


1;
