# This autoloader file was built automatically by 
# /usr/local/sbin/autoize from Fileman.pm

# Copyright 1998 Cobalt Networks (http://www.cobaltnet.com)

use strict;
### START HARDCODED PATHS
my $Adminuser = "admin";
my $Firstuid = "110";
my $Htgroup_dir = "/home/sites";
my $Htusers_dir = "/usr/httpd/users";
my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = ($Debug ||= 0);
### END HARDCODED PATHS

# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/Fileman.pm.  Changes made here will be lost.
package Cobalt::Fileman;

sub file_rights
# Does a user have rights to a file or directory?
# Arguments: user name, file or directory name (full path)
# Return value: 0 for no permissions, -1 for read only, 1 for read/write
#   For our purposes, there will never be write perms w/o read perms
#   An error condition (bad username, etc) means no permissions
#   No accesses outside the web publishing area are allowed.
#   The admin user is always allowed (within the web area)
# Side effects: none
# Example: &panic unless (Cobalt::Fileman::file_rights("ken","/home/httpd/html/users/ken"));
{
    my ($user,$dir) = @_;
    my $uid = getpwnam($user);
    return 0 unless ($uid >= $Firstuid);  # no uid, no permee
    return 0 unless ($dir);               # no permission to nothing

    if ((! -e $dir) && (! -l $dir))
    {
	# We're looking to create something where it doesn't exist
	$dir =~ /^(.+)\/([^\/]+)$/o;
	$dir = $1;
	print STDERR "Cobalt::Fileman::file_rights() is recursing into parent $dir\n" 
	    if ($Debug);
	return Cobalt::Fileman::file_rights($user,$dir);
    }
    my ($dir_dev,$dir_mode,$dir_uid,$dir_gid) = (stat($dir))[0,2,4,5];

    # They shouldn't be playing with anything outside the web server area
    # We check the device number because they can try to mislead us with ..'s.
    if ((($dir !~ /^$Htusers_dir/o) &&
	($dir !~ /^$Htgroup_dir/o)) ||
	($dir_dev !~ (stat($Htusers_dir))[0]))
    {
	return 0;
    }

    # The admin gets whatever the admin wants
    return 1 if ($user eq $Adminuser);

    # first check against ownership (UID match has top precedence)
    if ($dir_uid == $uid)
    {
	if    ($dir_mode & 0200) { return 1;  }  # write perms, allow anything
	elsif ($dir_mode & 0400) { return -1; }  # read perms
	else                     { return 0;  }  # no perms
    }
    
    # Now check against groupship
    my $members = (getgrgid($dir_gid))[3];
    if ($members =~ /\b$user\b/)
    {
	if    ($dir_mode & 020) { return 1;  }  # write perms, allow anything
	elsif ($dir_mode & 040) { return -1; }  # read perms
	else                    { return 0;  }  # no perms
    }

    # Finally, check against worldship
    if    ($dir_mode & 02) { return 1;  }  # write perms, allow anything
    elsif ($dir_mode & 04) { return -1; }  # read perms
    else                   { return 0;  }  # no perms
    
}


1;
