# This autoloader file was built automatically by 
# /usr/local/sbin/autoize from Fpx.pm

# Copyright 1998 Cobalt Networks (http://www.cobaltnet.com)

use strict;
### START HARDCODED PATHS
my $Adminuser = "admin";
my $Fpx_admsrvexe = "/usr/local/frontpage/currentversion/bin/fpsrvadm.exe";
my $Fpx_cnfdir = "/usr/local/frontpage";
my $Htconf = "/etc/httpd/conf";
my $Userdir = "/home/sites";
my $WWWuser = "httpd";
my $Webdir = "web";
my %Features = ("networker","1","telnetConfig","1","dhcpd","0","shiftjis","Cobalt interface message for shiftjis (We should never see this)","pagebuilder","0","www","1","mx","1","gated","1","dns","1","nfs","0","securetty","1","english","1","atalk","0","fpx","1","siteAdminRedirect","1","users","1","ntp","0","mon_textdb_only","0","search","0","anonFTP","1","specialHomeSkel","0","viewGroups","0","cobaltHomePage","0","lastNameFirst","1","warnRootOnly","1","vsites","1","pam","1","discuss","0","docorg","0","groups","1","smb","0","mail","1","nat","0","squid","0","autoGenWebUsage","0","japanese","0","cobaltRedirect","0","backups","1","vmachine","1","mon_port_testing","1","viewUsers","0","usersCGIInWizard","1","privatedirs","0","telnetDefault","1","ssl","0","siteManagement","1","adm","1","bind8","1","personalRedirect","1","webRequests","1","usersAddUser","0","adminRedirect","1","quotas","1","snmp","1","defaultUserNameGen","1","groupdirs","1","proftpd","1");
my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = ($Debug ||= 0);
### END HARDCODED PATHS

# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/Fpx.pm.  Changes made here will be lost.
package Cobalt::Fpx;

sub fpx_set_web
# Enable or disable FrontPage Server Extensions 
# Arguments: group or username, 1 for on/0 for off
# Return value: a status string of the usual form form "2534 ho hum" 
# Side effects: Calls MS's fpsrvadm.exe, modifies /etc/htpasswd using Vsite.pm, 
#
# Complications:
# fpadmsrv.exe expects an Apache configuration syntax that's wrong.
# M$ doesn't respect server syntax, fpadmsrv.exe is a joke.
# We're generating an alternate [fp98.]httpd.conf that would
# never work with Apache but is laid out how FPX would like
# it to be.  The silver lining is that we never let fpadmsrv.exe 
# modify any Apache configuration file. (See vsite_set/get_fpx) 
#
# Examples: $good = Cobalt::Fpx::fpx_set_web($group,0);
#            $bad = Cobalt::Fpx::fpx_set_web($user,1);
{
  my($group,$state)=@_; # $group may also be a user

  return "1002 Microsoft FrontPage Server Extensions are not supported in this release" unless ($Features{'fpx'});
  return "4319 The FrontPage Server Extensions configuration tools are not available" unless (-x $Fpx_admsrvexe);

  # Current state
  my $fpx_active = fpx_get_web($group); 
  return "2048 A root web is already configured" if ($state && $fpx_active);
  return "2049 The root web has already been uninstalled" unless ($state || $fpx_active);
  
  # Test user vs group/site
  my($user,$could_not_install);
  if(user_exist($group))  ### User web
  {
    $user=$group; # Clean variable names
    $group='';

    my($site)=vsite_getgroup($user);
    return "4324 Can not configure the user site web" unless ($site);

    # No user webs unless the site has a root web
    return "4325 A root web must first be enabled in the Site Settings menu" unless(fpx_get_web($site));

    # fpadmsrv.exe matches $siteName in httpd.conf
    my($siteName)=(vsite_get_bygroup($site))[1];

    my($user_home,$user_uid)=(getpwnam($user))[7,2];

    if ($state) # Enable
    {
      # Rename .htaccess files so fpadmsrv.exe doesn't die from confusion
      my(@htax) = fpx_rename_htaccess("$user_home/$Webdir");

      # FPX user web won't install without a dummy .htaccess file sitting
      #   in the MAIN SITE WEB ROOT.  'Guess nobody at Microsoft went to 
      #   sysadmin school.  Learned to program from a lawyer maybe?
      my($redmond)=("$Userdir/$site/$Webdir/.htaccess");
      if (-e $redmond) 
      {
        $redmond=0;
      } else
      {
        open(MSBS,">$redmond"); # touch 
        close(MSBS); 
      }

      # Call fpadmsrv.exe (ugh)
      system("$Fpx_admsrvexe -o install -p 80 -t apache -m \"$siteName\" -w \"\~$user\" -u \"$user\" -g \"$site\" -pw \"$$\" -s \"$Htconf/fp98.httpd.conf\" -xu \"$user\" -xg \"$site\" > /dev/null") && ($could_not_install = 1);

      # ignore anything that fpx did to this file
      unlink($redmond) if ($redmond);  

      # FPX creates a bunch of .htaccess files that need to be re-written for
      # pam/shadow support.. So, find the new .htaccess files and edit each one
      # We do this before restoring original .htaccess files
      fpx_rewrite_htaccess((getpwnam($user))[7]."/$Webdir",'user',"$user $Adminuser");

      fpx_restore_htaccess((getpwnam($user))[7]."/$Webdir",@htax); # Fix what we've started
      return "4326 A FrontPage web could not be configured successfully" if ($could_not_install);

      # Update Apache conf manually
      my(@fpx_users)=vsite_get_fpx($site);
      push(@fpx_users,$user); # Add user to site fpx users
      my($return)=vsite_set_fpx($site,1,@fpx_users);
      return $return if ($return !~ /^2/);

      # Restore lost sticky bits
      fpx_recurse_chmod( 02775, "$user_home/$Webdir");

      # SUID page counter 
      chmod(04755,(getpwnam($user))[7]."/$Webdir/_vti_bin/fpcount.exe");

      return "2050 A root web has been configured successfully";
    } else # Disable
    {
      # fpx "delete" actually rm -rf's the user web tree.  Unacceptable, so
      # we first move the user home directory out of harms way, then restore it.
      rename("$user_home/$Webdir","$user_home/fp98-$Webdir");
      
      # Uninstall
      my $fpx_failure = 0; 
      system("$Fpx_admsrvexe -o delete -p 80 -t apache -m \"$siteName\" -w \"~$user\" -s \"$Htconf/fp98.httpd.conf\" > /dev/null") && ($fpx_failure = 1); 

      # Restore the user web directory
      rename("$user_home/fp98-$Webdir","$user_home/$Webdir");

      # Now bail since we've restored functionality
      return "4327 The FrontPage web could not be uninstalled successfully" if ($fpx_failure);

      # Update Apache conf explicitly
      my(@old_users)=vsite_get_fpx($site);
      my(@fpx_users,$fpx_user);
      foreach $fpx_user (@old_users)
      {
        push(@fpx_users,$fpx_user) unless ( ($fpx_user eq $user) ||
                                            ($fpx_user eq $site) );
      }
      my($return)=vsite_set_fpx($site,1,@fpx_users);
      return $return if ($return !~ /^2/);

      # FPX files have been made unavailable to fpadmsrv.exe
      system("/bin/rm -rf $user_home/$Webdir/_vti* > /dev/null 2>&1; /bin/rm -rf $user_home/$Webdir/_private > /dev/null 2>&1\n");
      
      return "2051 The root web has been uninstalled successfully";
    }

  } else                  ### Site web
  {
    # fpadmsrv.exe matches $siteName in fp98.httpd.conf
    my($siteName)=(vsite_get_bygroup($group))[1];
    return "4328 Can not configure the site root web" unless ($siteName);
    my($ret)=fpx_hose_apacheconf();
    return "4329 The web server configuration file could not be modified" unless ($ret);

    if ($state) # Enable
    {
      # Rename .htaccess files so fpadmsrv.exe doesn't die from confusion
      my(@htax) = fpx_rename_htaccess("$Userdir/$group/$Webdir");

      my($user)=$WWWuser; # For lack of a better default

      # Call fpadmsrv.exe (ugh)
      system("$Fpx_admsrvexe -o install -p 80 -t apache -m \"$siteName\" -u \"$user\" -pw \"$$\" -s \"$Htconf/fp98.httpd.conf\" -xu \"$user\" -xg \"$group\" > /dev/null") && ($could_not_install = $!);

      # FPX creates a bunch of .htaccess files that need to be re-written for
      # pam/shadow support.. So, find the new .htaccess files and edit each one
      # We do this before restoring original .htaccess files
      fpx_rewrite_htaccess( "$Userdir/$group/$Webdir", 'group', "$group $Adminuser");

      # Fix what we've started for clean aborts
      fpx_restore_htaccess( "$Userdir/$group/$Webdir", @htax); 
      return "4330 A FrontPage web could not be configured successfully" if ($could_not_install);

      # Update Apache conf manually
      my($return)=vsite_set_fpx($group,1);
      return $return if ($return !~ /^2/);

      # Add SMTP capability to FPX forms
      if (-w "$Fpx_cnfdir/$siteName:80.cnf")
      {
        my($orig_cnf);
	open(CNF);
        while(<CNF>) { $orig_cnf .= $_; }
        close(CNF);

	unless($orig_cnf =~ /smtphost/i)
	{
	  open(NUCNF,">> $Fpx_cnfdir/$siteName:80.cnf");
	  print NUCNF "SMTPHost: 127.0.0.1\n";
	  close(NUCNF);
	}

	# Must be readable by $WWWuser
	chmod(0644,"$Fpx_cnfdir/$siteName:80.cnf");
      }

      # Restore lost sticky bits
      fpx_recurse_chmod( 02775, "$Userdir/$group/$Webdir");

      # Enable the page counter (SGID)
      chmod(02755,"$Userdir/$group/$Webdir/_vti_bin/fpcount.exe");

      return "2052 A root web has been configured successfully";

    } else # Disable
    {
      # Uninstall
      # Call fpadmsrv.exe (ugh)
      system("$Fpx_admsrvexe -o uninstall -p 80 -t apache -m \"$siteName\" -s \"$Htconf/fp98.httpd.conf\" > /dev/null") && return "4331 The FrontPage web could not be uninstalled successfully";

      # Update Apache conf manually
      my($return)=vsite_set_fpx($group,0);
      return $return if ($return !~ /^2/);

      # fpadmsrv should do this automatically 
      #  ..but we don't trusts its now does we..
      unlink("$Fpx_cnfdir/$siteName:80.cnf");

      # fpadmsrv.exe won't delete all stub cgis or old logs
      system("/bin/rm -rf $Userdir/$group/$Webdir/_vti* > /dev/null 2>&1; /bin/rm -rf $Userdir/$group/$Webdir/_private > /dev/null 2>&1\n");

      return "2059 The root web has been uninstalled successfully";
    }

    return "4332 A FrontPage web could not be configured successfully" if ($could_not_install);
    return "2060 A root web has been configured successfully";
  }
}                                               


1;
