# This autoloader file was built automatically by 
# /usr/local/sbin/autoize from Ftp.pm

# Copyright 1998 Cobalt Networks (http://www.cobaltnet.com)

use strict;
### START HARDCODED PATHS
my $Badshell = "/bin/badsh";
my $FTPUsersMax = "10";
my $Ftpd_conf = "/etc/proftpd.conf";
my $Htgroup_dir = "/home/sites";
my $Lockdir = "/etc/locks";
my $MainGroup = "home";
my $Sites_AdminGroup = "site-adm";
my $WWWuser = "httpd";
my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = ($Debug ||= 0);
### END HARDCODED PATHS

# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/Ftp.pm.  Changes made here will be lost.
package Cobalt::Ftp;

sub ftp_config
# Edit ftp configuration
# Arguments: IP, update/delete, anonymous on/off, site group name, 
#                  maximum number of users, incoming quota   
# Return value: string of the form "4999 Ouch, stop it!"
# Example: Bind to new address: $return=ftp_config("10.9.25.79",1); 
#   Enable anon ftp: $return=ftp_config("10.9.25.79",1,1,'home',10,50); 
{
    my($ip,$status,$anon,$group,$maxusers,$quota)=@_;
    my($ftpuser)= uc($group); # Anonymous ftp username
    $quota ||= 0; # No incoming dir if none specified
    
    # Basic data verification 
    return "4337 The default ftp site can not be disabled" if (($group eq $MainGroup) && !$status);
    return "4338 Invalid maximum users: $maxusers" if ($maxusers =~ /\D/o);
    $maxusers = $FTPUsersMax unless ($maxusers =~ /^\d+$/o);
    return "4339 Invalid IP address: $ip" if (!netutil_test_ipaddr($ip));

    # Group name validity is only necessary for anonymous ftp
    return "4340 Invalid group name $group" if ($anon && !group_exist($group));

    # Build default config data
    my($anonConfig,$siteConfig);
    if($anon)
    {
        $anonConfig=<<EOF;
	<Anonymous $Htgroup_dir/$group/ftp>
	  User                          $ftpuser
	  Group                         ftp
	  UserAlias                     anonymous $ftpuser
	  UserAlias                     guest $ftpuser
	  MaxClients                    $maxusers
	  <Directory *>
	    <Limit WRITE>
	      DenyAll
	    </Limit>
	  </Directory>
	  <Directory $Htgroup_dir/$group/ftp/incoming>
            Umask			002
            <Limit STOR>
              AllowAll
            </Limit>
            <Limit READ>
              DenyAll
            </Limit>
	  </Directory>
	</Anonymous>
EOF
    }
    $siteConfig=<<EOF;
<VirtualHost $ip>
	DefaultRoot		/ admin
	DefaultRoot		~/../.. $Sites_AdminGroup
	DefaultRoot		~ !$Sites_AdminGroup
	AllowOverwrite		on
	DisplayLogin		.ftphelp
$anonConfig</VirtualHost>
EOF
    $siteConfig=$anonConfig if ($group eq $MainGroup); # Minor special treatment

    # Read existing ftp configuration
    my(%ftpstatus)=ftp_list_config();

    # Disallow anon ftp if a different virtual site
    #   is using this IP for anonymous ftp
    my($siteIP,$modify);
    foreach $siteIP (keys %ftpstatus)
    {
        return "4341 This IP address is in use as an anonymous FTP server by another site" if ($ftpstatus{$ip} && $anon && ($ftpstatus{$ip} ne $group));
	$modify=1 if ($siteIP eq $ip);
    } 

    # Bail if we're deleting an undefined site
    return "2061 Virtual FTP site was never defined" if (!$modify && !$status);

    # Test/add anonymous ftp user
    if (!$anon) {
	my($ret)=user_del($ftpuser) if (user_exist($ftpuser));
    } else {
	my($ret)=user_add($ftpuser,"$group anonymous FTP user","*",$quota,$Badshell,0,"$Htgroup_dir/$group/ftp") unless (user_exist($ftpuser));
    }

    # Verify/create deletable ftp directories
    if($group && $anon) # group is not optional if anonymous ftp is not being enabled
    {
        mkdir("$Htgroup_dir/$group/ftp",02775) unless (-d "$Htgroup_dir/$group/ftp");
        chown((getpwnam($WWWuser))[2],(getgrnam($group))[2],"$Htgroup_dir/$group/ftp");
        chmod(02775,"$Htgroup_dir/$group/ftp"); # Always reset perms
        mkdir("$Htgroup_dir/$group/ftp/incoming",02770) unless (-d "$Htgroup_dir/$group/ftp/incoming");
	chown((getpwnam($WWWuser))[2],(getgrnam($group))[2],"$Htgroup_dir/$group/ftp/incoming");
    }

    # Set incoming folder permissions
    if ($quota && $anon)
    {
        chmod(02773,"$Htgroup_dir/$group/ftp/incoming");
        my($ret)=user_mod($ftpuser,'','',$quota);
        return $ret unless ($ret =~ /^2/);
    } else 
    {
        chmod(02770,"$Htgroup_dir/$group/ftp/incoming");
    }

    # Strict locks
    if (-e "$Lockdir/proftpd.conf")
    { 
       sleep 1; # Avoid race
       return "4342 FTP server in use, please try again soon" if (-e "$Lockdir/proftpd.conf");
    }
    open(FTPD,"$Ftpd_conf") || return "4343 Cannot read FTP config file /etc/proftpd.conf: $!";
    open(NEWCNF,"> $Lockdir/proftpd.conf") || return "4344 Cannot write ftp configuration file: $!";
    while(<FTPD>)
    {
        if (/<VirtualHost $ip>/ ... /<\/VirtualHost>/)
        { # Ignore data in $siteconfig...we'll be re-adding this soon enough
        } elsif (($group eq $MainGroup) && (/<Anonymous $Htgroup_dir\/$MainGroup\// ... /<\/Anonymous>/))
        { # $MainGroup does not appear in VirtualHost block
        } else 
        { # Worth keeping
            print NEWCNF;
        }
    }
    print NEWCNF $siteConfig if ($status);
    close(NEWCNF);
    close(FTPD);

    # Clobber config
    move("$Lockdir/proftpd.conf","$Ftpd_conf") || return "4345 Cannot write ftp configuration file: $!";
        
    return "2062 Virtual FTP site configured successfully";
}


1;
