# This autoloader file was built automatically by 
# /usr/local/sbin/autoize from Http.pm

# Copyright 1998 Cobalt Networks (http://www.cobaltnet.com)

use strict;
### START HARDCODED PATHS
my $Htpid = "/var/run/httpd.pid";
my $Htscript = "/etc/rc.d/init.d/httpd.init";
my $Lcdshowscript = "/etc/rc.d/init.d/showip";
my $Sauce_log = "/var/cobalt/sauce.log";
my $Subsys = "/var/lock/subsys";
my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = ($Debug ||= 0);
### END HARDCODED PATHS

# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/Http.pm.  Changes made here will be lost.
package Cobalt::Http;

sub http_restart
# Restart the HTTP server
# According to the Apache docs, sending a USR1 signal is the best
# way to signal the server to restart.  This is much faster and
# more graceful than messing with init scripts.
# See http://www.apache.org/docs/stopping.html
# Arguments: 1 if we should start up the server when it's not already up
#  (otherwise, if it's not up we'll leave it down).
# Return value: none
# Side effects: "The USR1 signal causes the parent process to advise the 
#  children to exit after their current request (or to exit immediately if 
#  they're not serving anything). The parent re-reads its configuration files 
#  and re-opens its log files. As each child dies off the parent replaces it 
#  with a child from the new generation of the configuration, which begins 
#  serving new requests immediately."
# If the server is not currently running, start it.
{
  my ($bring_it_up) = @_;

  if (! -e "$Subsys/httpd") {
    # The server is not currently running, start it up.
    return unless ($bring_it_up);
    print STDERR "Starting httpd\n" if ($Debug);
    system("$Htscript start >>$Sauce_log 2>&1");
    system("$Lcdshowscript >>$Sauce_log 2>&1") if (-x $Lcdshowscript);
  } elsif (! -e $Htpid) {
    # The init script thinks the server is running, but it isn't
    # Clear out the init script's state and start it up.
    return unless ($bring_it_up);
    print STDERR "Restarting httpd\n" if ($Debug);
    system("$Htscript stop >>$Sauce_log 2>&1");
    system("$Htscript start >>$Sauce_log 2>&1");
    system("$Lcdshowscript >>$Sauce_log 2>&1") if (-x $Lcdshowscript);
  } else {
    # The server is running and seems normal
    # Read the PID from the file and send it the signal
    open(PID,$Htpid) || return undef;
    my $pid = <PID>;
    close PID;
    chomp $pid;
    print STDERR "Signalling httpd ($pid)\n" if ($Debug);
    kill('USR1',$pid) if ($pid);
  }
}






1;
