# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/Language.pm.  Changes made here will be lost.
package Cobalt::Language;

sub msg_build_database
# Build the DB_File hash database from the text file
# The text file should be a series of lines of the form
#                 'key' => 'value',
# The quotes are optional, either single or double quotes can be
# used.
# Leading/trailing whitespace and blank lines will be ignored.
# Each value must be terminated by a trailing , (this is how
# we deal with multi-line strings).
# A comma, a space, and then three pound signs indicate the
# beginning of a comment.
# Arguments: language (e.g., "english", "japanese")
# Side effects: builds the .db file based on the .txt file
# This is pretty much only used by autoize...
{
  my ($lang) = @_;
  my %DB;
  if ($Cobalt::Language::Messages_Tied) {
    untie %Cobalt::Language::Messages;
    $Cobalt::Language::Messages_Tied = 0;
  }
  unless(tie(%DB, 'DB_File', "/etc/cobalt/messages/$lang.db",
	     O_RDWR()|O_CREAT(),0644))
  {
    return "Cannot read messages database /etc/cobalt/messages/$lang.db";
  }
  eval 'use Cobalt::Product';
  my %found;
  open(TXT,"/etc/cobalt/messages/$lang.txt") ||
    die "Can't open /etc/cobalt/messages/$lang.txt: $!\n";
  while (<TXT>) {
    next if (/^\s*\#/o);  # lines begining with comments are ignored
    next if (/^\s*$/o);   # blank lines are ignored
    my ($key,$val);

    # could also use "$line = split(/('\||\|')/)" and then switch on $#line
    if (/^\s*([^\|]+)\s*\|\s*\'(.+)\'\|$/o) {
      # whole string, right here
      ($key,$val) = ($1,$2);
    }
    elsif (/^\s*([^\|]+)\s*\|\s*\'(.+)$/o) {
      # start of a multi-line string
      ($key,$val) = ($1,"$2\n");
      while (<TXT>) {
	if (/^(.*)\'\|$/o) { # end of the string
	  $val .= $1;
	  last;
	}
	else {
	  $val .= $_;
	}
      }
    }

    print STDERR "Duplicate key `$key' in /etc/cobalt/messages/$lang.txt\n" 
      if ($found{$key});
    $found{$key} = 1;

    # Replace any Product.pm variables with their values
    my $val_copy = $val;
    while ($val_copy =~ /\$([A-Z]\w+)/o) {
      my $variable = $1;
      # There may be uppercase variables that do not appear
      # in product.pm--strip them out of the string so we don't
      # keep trying to match them
      $val_copy =~ s/\$$variable//g;
      if (defined eval('$$variable')) {
	# If it's in Product.pm, then substitute the value in
	$val =~ s/\$$variable/$$variable/g;
      }
    }

    $DB{$key} = $val;
  }
  close TXT;
  untie %DB;
  undef %found;
}

1;
