# This autoloader file was built automatically by 
# /usr/local/sbin/autoize from Modem.pm

# Copyright 1998 Cobalt Networks (http://www.cobaltnet.com)

use strict;
### START HARDCODED PATHS
my $Chat_script = "/etc/ppp/chatscript";
my $Diald_conf = "/etc/diald.conf";
my $Diald_link = "/etc/rc.d/rc3.d/S90diald";
my $Pap_secrets = "/etc/ppp/pap-secrets";
my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = ($Debug ||= 0);
### END HARDCODED PATHS

# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/Modem.pm.  Changes made here will be lost.
package Cobalt::Modem;

########################################################################
#
# PPP using DIALD config information
#
# RPMs required:  diald-0.16-3C1.mips.rpm
#                 diald-config-1.2.1-1.mips.rpm
#                 ppp-2.2.0f-5C1.mips.rpm
#
# Config Files:   /etc/diald.conf - diald and pppd options
#		  /etc/ppp/chatscript - script to authenticate connection
#		  /etc/ppp/pap-secrets - holds user name and passwd for 
#				pap authentication
#		  /etc/ppp/chap-secrets - holds user name and passwd for 
#				chap authentication
#		  /usr/sbin/diald - diald binary
#		  /etc/rc.d/init.d/diald.init  - diald initscript 
#		  /etc/rc.d/rc3.d/S90diald - diald boot script link
#
#########################################################################


sub modem_get_settings
# Descr:	Read modem settings from system files and setup a hash.
#		ie.  model => 'hayes generic'
# Arguments: 	None
# Return:	A hash containing all the modem settings
# 
#  $modem{'MODEL'} => modem model  (not stored or used anywhere at this time) 
#  $modem{'LOCALIP'} => local IP Address or NONE
#  $modem{'NETMASK'} => local netmask or NONE
#  $modem{'USERNAME'} => user name  
#  $modem{'PASSWD'} => password  
#  $modem{'PHONE'} => phone number  
#  $modem{'GWMODE'} = 1 modem is defaultroute, 0 not
#  $modem{'CONNMODE'} => 0 dialondemand, 1 always connected
#			 2 never connected (diald off)
#  $modem{'DIALMODE'} => 0 not used, 1 dial out, 
#			 2 dial in, 3 dial in/out (always 0 for 2800)
#  $modem{'INITSTR'} => modem initstring, like ATZ 
{
  my %modem;

  # get user and password from pap-secrets file
  # user and passwd will be on line that looks like - <user> * <password>
  open( PAP, $Pap_secrets)
    || return "4526 Cannot read PAP secrets /etc/ppp/pap-secrets: $!";
  while( <PAP> ) {
    if(/^([^\#]\S*)\s+\*\s+(\S+)/o) {
      ($modem{'USERNAME'},$modem{'PASSWD'}) = ($1,$2);
      last;
    }
  }
  close PAP;

  # get info from chatscript
  # initstring will be on like that looks like - "" ATZ
  # phone number will be on line that looks like - OK ATDT5551212
  open( CHAT, "<$Chat_script")
    || return "4558 Cannot read chatscript /etc/ppp/chatscript: $!";
  while( <CHAT> ) {
    if (/^OK ATDT(.+)/o) {
      $modem{'PHONE'}=$1;
      last;
    }
    elsif (/^OK (.+)/) {
      # when no phone number is set INITSTR shows up as ATDT...bogus!
      # also, if multiple AT lines exist, this gives the last one before ATDT
      $modem{'INITSTR'}=$1 unless ($1 eq "ATDT");
    }
  }
  close CHAT;

  # get info from diald.conf
  $modem{'GWMODE'} = 0;  # default, unless we see otherwise
  # if only 'up' is present then {'CONNMODE'} = 1 = always connected
  # if only 'down' is present then {'CONNMODE'} = 2 = always down = not used
  # if both 'up' & 'down' are absent then {'CONNMODE'} = 0 = dial-on-demand
  # if both are present the first line matters (always recheck w/ new rpms)
  $modem{'CONNMODE'} = 0;	# both are not present unless we see either
  open(DCONF,$Diald_conf)
    || return "4559 Cannot read diald config file /etc/diald.conf: $!";
  while (<DCONF>) {
    # if diald.conf has a line with 'netmask 255.255.255.0'
    #	-> {'NETMASK'} = '255.255.255.0'  
    if (/^netmask\s+(\S+)/o) {
      $modem{'NETMASK'} = $1;
    }
    # if diald.conf has a line with 'defaultroute'
    #	-> {'GWMODE'} = 1     - yes
    #   else 
    #	-> {'GWMODE'} = 0     - no 
    elsif (/^defaultroute$/o) {
      $modem{'GWMODE'} = 1;
    }
    # 192.168.0.1, and 192.168.0.2 are fake IP addresses that are 
    # used when we really want to dynamically assign the IP address
    # of the Qube, this also means there should be a line 
    # 'dynamic' in diald.conf
    #
    # if diald.conf has a line with 'local 192.168.0.1'
    #	-> {'LOCALIP'} = 'NONE'
    # if diald.conf has a line with 'local 10.9.25.40'
    #  where 10.9.25.40 could be any real IP address
    #	-> {'LOCALIP'} = '10.9.25.40'
    elsif (/^local\s+(\S+)/o) {
      my $addr = $1;
      if ($addr =~ /^192\.168\.0\.[12]$/o) {
	$modem{'LOCALIP'} = 'NONE';
      } else {
	$modem{'LOCALIP'} = $addr;
      }
    }
    # diald is active...get info on how active from diald.conf
    # if qube-admin adds "restrict" lines, we are screwed!
    elsif (/^up$/o) {  # always connected
      $modem{'CONNMODE'} = 1 if ($modem{'CONNMODE'} == 0);
    }
    elsif (/^down$/o) {  # not used or always down
      $modem{'CONNMODE'} = 2 if ($modem{'CONNMODE'} == 0);
    }
  }
  close DCONF;

  if (! -e $Diald_link) {
    # diald is off...override settings from $Diald_conf
    $modem{'CONNMODE'} = 2;
  }

  return(%modem);
}


1;
