# This autoloader file was built automatically by 
# /usr/local/sbin/autoize from Modem.pm

# Copyright 1998 Cobalt Networks (http://www.cobaltnet.com)

use strict;
### START HARDCODED PATHS
my $Chap_secrets = "/etc/ppp/chap-secrets";
my $Chat_script = "/etc/ppp/chatscript";
my $Diald_conf = "/etc/diald.conf";
my $Diald_link = "/etc/rc.d/rc3.d/S90diald";
my $Diald_script = "/etc/rc.d/init.d/diald.init";
my $Diald_stop0 = "/etc/rc.d/rc0.d/K55diald";
my $Diald_stop6 = "/etc/rc.d/rc6.d/K55diald";
my $Lockdir = "/etc/locks";
my $Pap_secrets = "/etc/ppp/pap-secrets";
my $Sauce_log = "/var/cobalt/sauce.log";
my $Syslog_script = "/etc/rc.d/init.d/syslog";
my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = ($Debug ||= 0);
### END HARDCODED PATHS

# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/Modem.pm.  Changes made here will be lost.
package Cobalt::Modem;

sub modem_set_settings
# Decr:		Write modem settings to system files
# Arguments:	A reference to a hash containing all the modem settings
# Return:	Return code that looks like "2000 OK"
# Ex:		$ret=Cobalt::Modem::modem_set_settings( $modemRef );
{
  my($modemref) = @_;
  my %modem = %$modemref;  # dereference the pointer (ugh)
	
  # Start/Restart diald if CONNMODE = 0 or 1
  # Stop diald if CONNMODE = 2
  # Create or delete initscripts link as appropriate
  # /etc/rc.d/init.d/diald.init (start|stop)  
  # S90diald, K55diald
  #

  if ($modem{'LOCALIP'} ne "NONE") {
    if (!Cobalt::Netutil::netutil_test_ipaddr($modem{'LOCALIP'})) {
      return "4563 Invalid local IP address";
    }
    if (!Cobalt::Netutil::netutil_test_ipaddr($modem{'NETMASK'})) {
      return "4564 Invalid netmask";
    }
  }
  if ($modem{'CONNMODE'}!=2 && $modem{'USERNAME'} =~ /\s/) {
    return "4565 Invalid username";
  }
  if ($modem{'CONNMODE'}!=2 && $modem{'PHONE'} !~ /^[\d\-\(\)\+\-\d\*\,\s\#]+$/o) {
    return "4566 Invalid phone number";
  }
  # we don't really check the password...that could be anything...

  # for chap-secrets and pap-secrets
  my $commentLine="\# The following line is automatically generated. Do not change or remove";

  # Setup pap-secrets (user,passwd)
  # add/replace last line of pap-secrets to look like:
  #  USERNAME * PASSWD
  # We replace only the line with a matching USERNAME...the name that
  # pppd looks for is determined as follows (from the pppd man page):
  #
  # if the usehostname option is given,
  #      then  the  local  name  is the hostname of this machine
  #      (with the domain appended, if given)
  # else if the name option is given,
  #      then use the argument of the first name option seen
  # else if the local IP address is specified with a hostname,
  #      then use that name
  # else use the hostname of this machine (with the domain appended,
  #      if given)
  # So...instead of using a user-defined username, maybe we should
  # just use our hostname?
  return "4567 Cannot get lock on pap secrets file, try again later" if (-e "$Lockdir/pap.secrets");
  # we use sysopen so we can specify the file mode
  sysopen(PAPOUT,"$Lockdir/pap.secrets",O_RDWR()|O_CREAT(),0600)
    || return "4568 Cannot write pap secrets lockfile: $!";
  open(PAPIN,$Pap_secrets)
    || return "3002 Cannot read PAP secrets /etc/ppp/pap-secrets: $!";

  while (<PAPIN>) {
    # remove Cobalt lines
    if( /^$commentLine$/ ) {
      # this line is the account info
      <PAPIN>;
    }
    else {
      print PAPOUT;
    }
  }
  print PAPOUT "$commentLine\n";
  print PAPOUT $modem{'USERNAME'}, " * ", $modem{'PASSWD'}, "\n";
  close PAPIN;
  close PAPOUT;
  rename("$Lockdir/pap.secrets",$Pap_secrets)
    || return "3003 Cannot write new pap secrets file: $!";
  chmod(0600,$Pap_secrets);  # make sure

  # Setup chap-secrets (user,passwd)
  # add/replace last line of chap-secrets to look like:
  #  USERNAME * PASSWD	
  return "4668 Cannot get lock on chap secrets file, try again later" if (-e "$Lockdir/chap.secrets");
  # use sysopen to set the creation mode
  sysopen(CHAPOUT,"$Lockdir/chap.secrets",O_RDWR()|O_CREAT(),0600)
    || return "4669 Cannot write chap secrets lockfile: $!";
  open(CHAPIN,$Chap_secrets)
    || return "3004 Cannot read CHAP secrets /etc/ppp/chap-secrets: $!";

  while (<CHAPIN>) {
    # remove Cobalt lines
    if( /^$commentLine$/ ) {
      # this line is the account info
      <CHAPIN>;
    }
    else {
      print CHAPOUT;
    }
  }
  print CHAPOUT "$commentLine\n";
  print CHAPOUT $modem{'USERNAME'}, " * ", $modem{'PASSWD'}, "\n";
  close CHAPIN;
  close CHAPOUT;
  rename("$Lockdir/chap.secrets",$Chap_secrets)
    || return "3005 Cannot write new chap secrets file: $!";
  chmod(0600,$Chap_secrets);

  # Setup chatscript (initstring,phone#)
  #
  # chatscript should look like:
  #
  #  TIMEOUT 5
  #  "" ATZ
  #  OK <init string>
  #  OK ATDT<phone number>
  #  ...etc...
  #
  # set phone number on line that begins with - OK ATDT
  # OK ATDT5551212
  #	
  return "3006 /etc/ppp/chatscript is locked, try again later" if ( -e "$Lockdir/chatscript" );
  open( CHATSLOCK, ">$Lockdir/chatscript" )
    || return "3007 Cannot write chatscript lock /etc/locks/chatscript: $!";
  open( CHATS, "$Chat_script")
    || return "3008 Cannot read chatscript /etc/ppp/chatscript: $!";
  while( <CHATS> ) {
    if (/^OK ATDT/) {
      # this is the number
      if ($modem{'INITSTR'} ne "NONE") {
        print CHATSLOCK "OK $modem{'INITSTR'}\n";
      }
      print CHATSLOCK "OK ATDT$modem{'PHONE'}\n";
    }
    elsif (! /^OK /) {
      print CHATSLOCK;
    }
  }
  close CHATSLOCK;
  close CHATS;
  rename( "$Lockdir/chatscript", $Chat_script ) ||
    return "3009 Cannot update /etc/ppp/chatscript: $!";
  chmod(0755,$Chat_script);

  # set up diald.conf
  #
  # if {'CONNMODE'} = 1 = always connected
  #  -> add 'up' to the first line of /etc/diald.conf
  # if {'CONNMODE'} = 0 = dial-on-demand
  #  -> remove 'up' and 'down' from /etc/diald.conf
  my ($netmasked,$gwed);
  return "3010 /etc/diald.conf is busy, try again later" if ( -e "$Lockdir/diald.conf" );
  open( DIALDLOCK, ">$Lockdir/diald.conf" )
    || return "3011 Cannot get lock for /etc/diald.conf: $!";
  open( DIALD, "$Diald_conf")
    || return "3012 Cannot read diald config file /etc/diald.conf: $!";
  while (<DIALD>) {
    if ($. == 1) { # first line of the file
      if ($modem{'CONNMODE'}==1) {
	print DIALDLOCK "up\n";   # always connected
      }
    }
    if (/^up$/o || /^down$/o) {
      # do not print this line for either CONNMODE
    }
    # replace line 'netmask xxx.xxx.xxx.xxx' -> 'netmask {'NETMASK'}'
    #  remove line 'netmask xxx.xxx.xxx.xxx' if no NETMASK specified
    elsif (/^netmask/o) {
      if ($modem{'NETMASK'} && ($modem{'NETMASK'} ne "NONE")) {
        print DIALDLOCK "netmask ", $modem{'NETMASK'}, "\n";
        $netmasked = 1;
      }
    }
    # if {'LOCALIP'} = '10.9.25.40'
    #  replace line 'local 10.9.25.40'
    # if {'LOCALIP'} = '192.168.0.1' = dynamic
    #   -> replace line 'local 192.168.0.1' 
    #   -> add line     'dynamic'
    elsif (/^local/o) {
      if ($modem{'LOCALIP'} eq "NONE") {
	print DIALDLOCK "dynamic\nlocal 192.168.0.1\n";
      }
      else {
	print DIALDLOCK "local ", $modem{'LOCALIP'}, "\n";
      }
    }
    elsif (/^dynamic/o) {
      # ignore, we'll re-add it as needed when we hit the local line
    }
    # if {'GWMODE'} = 0 = not the defaultroute
    #   the line 'defaultroute' should not exist in diald.conf
    # if {'GWMODE'} = 1 = yes the defaultroute
    #   the line 'defaultroute' should exist in diald.conf
    elsif (/^defaultroute/o) {
      print DIALDLOCK "defaultroute\n" if ($modem{'GWMODE'});
      $gwed = 1;
    }
    # the 'pppd-options user will' should become
    #   -> 'pppd-options user USERNAME'
    elsif (/^pppd-options/o) {
      # this line could have anything on it, so modify only the username
      $_ =~ s/user [\S]+/user $modem{'USERNAME'}/;
      print DIALDLOCK;
    }
    else {
      print DIALDLOCK;
    }
  }

  # These lines may not be in the configuration file, so add them if we
  # didn't find them.
  if ($modem{'NETMASK'} && ($modem{'NETMASK'} ne "NONE") && ! $netmasked) {
    print DIALDLOCK "netmask ", $modem{'NETMASK'}, "\n"
  }
  if ($modem{'GWMODE'} && ! $gwed) {
    print DIALDLOCK "defaultroute\n";
    # delete any other default routes; update GATEWAYDEV in network setup
    my $ret;
    return $ret if (($ret = net_set_gateway("none", "eth-doug")) !~ /^2/o);
  }
  close DIALDLOCK;
  close DIALD;
  rename( "$Lockdir/diald.conf", $Diald_conf )
    || return "3013 Cannot write new /etc/diald.conf: $!";

  # handle modem not used
  if ($modem{'CONNMODE'}==2) {
    # deactivate diald, activate kernel messages to console
    system("$Diald_script stop >>$Sauce_log 2>&1") 
      && return "3014 Cannot stop diald: $!"; 
    unlink($Diald_link,$Diald_stop0,$Diald_stop6);
    # klogd args
    return "3015 Syslog script is busy, try again later" if (-e "$Lockdir/syslog");
    open(SYSLOK,">$Lockdir/syslog") 
      || return "3016 Cannot write syslog lock: $!";
    open(SYSLOG,$Syslog_script) || return "3017 Cannot read syslog script: $!";
    while (<SYSLOG>) {
      if (/^\s*daemon klogd/o) {
	s/daemon klogd.*$/daemon klogd -c 1/o;
      }
      print SYSLOK;
    }
    close SYSLOG;
    close SYSLOK;
    rename("$Lockdir/syslog",$Syslog_script) 
      || return "3018 Cannot save new syslog script: $!";
    chmod(0755,$Syslog_script);

    # restart syslog
    system("$Syslog_script stop >>$Sauce_log 2>&1")
      && return "3019 Cannot stop syslog: $!"; 
    system("$Syslog_script start >>$Sauce_log 2>&1")
      && return "3020 Cannot start syslog: $!"; 
    
    return "2082 PPP configuration saved successfully";
  }

  # klogd args
  return "3021 Syslog script is busy, try again later" if (-e "$Lockdir/syslog");
  open(SYSLOK,">$Lockdir/syslog") 
    || return "3022 Cannot write syslog lock: $!";
  open(SYSLOG,$Syslog_script) || return "3023 Cannot read syslog script: $!";
  while (<SYSLOG>) {
    if (/^\s*daemon klogd/o) {
      s/daemon klogd.*$/daemon klogd/o;
    }
    print SYSLOK;
  }
  close SYSLOG;
  close SYSLOK;
  rename("$Lockdir/syslog",$Syslog_script) 
    || return "3024 Cannot save new syslog script: $!";
  chmod(0755,$Syslog_script);
  system("$Diald_script stop >>$Sauce_log 2>&1") 
    && return "3025 Cannot stop diald: $!"; 
  system("$Diald_script start >>$Sauce_log 2>&1")
    && return "3026 Cannot start diald: $!"; 
  symlink($Diald_script,$Diald_link);
  symlink($Diald_script,$Diald_stop0);
  symlink($Diald_script,$Diald_stop6);

  # restart syslog
  system("$Syslog_script stop >>$Sauce_log 2>&1")
    && return "3027 Cannot stop syslog: $!"; 
  system("$Syslog_script start >>$Sauce_log 2>&1")
    && return "3028 Cannot start syslog: $!"; 

  return "2083 PPP configuration saved successfully";
}


1;
