# This autoloader file was built automatically by 
# /usr/local/sbin/autoize from Monitor.pm

# Copyright 1998 Cobalt Networks (http://www.cobaltnet.com)

use strict;
### START HARDCODED PATHS
my %Status = ("none","0","0","none","green","1","1","green","2","yellow","red","3","3","red","yellow","2");
my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = ($Debug ||= 0);
### END HARDCODED PATHS

# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/Monitor.pm.  Changes made here will be lost.
package Cobalt::Monitor;

sub mon_build_alert
# Build the message
# Arguments: ($server,$instance,$severity,$cond_time,$update_time,$data)
#   $server:     the name of the server (or service); e.g., "mem",
#                "ftp", or "smb".
#   $instance:   a particular subsystem or instance of the server;
#                for example, the "userspace" server has an instance for
#                each user; the "mail" server has instances for pop,
#                imap, and smtp; etc.
#   $severity:   A number indicating the state of this instance 
#                1 = green, 2 = yellow, 3 = red
#   $cond_time:  The time at which this severity was entered
#   $update_time: The time when this particular report was generated
#   $data:       Additional information, defined per service and instance
# Return value: a nicely stated message string
{
    my ($server,$instance,$severity,$cond_time,$update_time,$data) = @_;
    my $return;
    my @data = split(/,/o,$data);
    $instance = lc($instance);  # lowercase everything for consistency

    $severity = -1 unless (defined($severity) && $severity);

    my $time = Cobalt::Language::msg_localtime($cond_time);
    my $servername = Cobalt::Language::msg_get("servers_$server");

    if ($server eq "cpu") { # CPU load
      if ($severity == $Status{"green"}) {
	$return = Cobalt::Language::msg_get("mon_state_cpu_green");
	$return = eval "\"$return\"";
	return $return;
      } elsif ($severity <= 0) {
	$return = Cobalt::Language::msg_get("mon_state_noneyet");
	$return = eval "\"$return\"";
	return $return;
      } elsif ($severity == $Status{"yellow"}) { 
	$return = Cobalt::Language::msg_get("mon_state_cpu_yellow");
	$return = eval "\"$return\"";
	return $return;
      } else {
	$return = Cobalt::Language::msg_get("mon_state_cpu_red");
	$return = eval "\"$return\"";
	return $return;
      }
    }
    elsif ($server eq "filesystem") {
      if ($severity <= 0)
      {
	  return Cobalt::Language::msg_get("mon_state_noneyet");
      }
      # $data[0] is the percent of capacity used
      $instance =~ s/\//slash/o;
      my $diskname = Cobalt::Language::msg_get("disks_$instance");
      $return = Cobalt::Language::msg_get("mon_state_disk");
      $return = eval "\"$return\"";
      return $return;
    }
    elsif (($server eq "userspace")  ||
	   ($server eq "groupspace") ||
	   ($server eq "mailspace")) {
      $return = Cobalt::Language::msg_get("mon_state_$server");
      $return = eval "\"$return\"";
      return $return;      
    }
    elsif ($server eq "mail") {
      $servername = $instance;
      if ($severity == $Status{"green"}) {
	$return = Cobalt::Language::msg_get("mon_state_green");
	$return = eval "\"$return\"";
	return $return;
      } elsif ($severity <= 0) {
	$return = Cobalt::Language::msg_get("mon_state_noinfo");
	$return = eval "\"$return\"";
	return $return;
      } else {
	$return = Cobalt::Language::msg_get("mon_state_red");
	$return = eval "\"$return\"";
	return $return;
      }
    }
    elsif ($server eq "mem") {
        if ($severity <= 0)
	{
	    return Cobalt::Language::msg_get("mon_state_noneyet");
	}
	# how much swapping is going on?
	if    ($instance eq "swapping")  
	{
	  if ($severity == $Status{"green"}) {
	    $return = Cobalt::Language::msg_get("mon_state_mem_swaps_green");
	    $return = eval "\"$return\"";
	    return $return;
	  } elsif ($severity <= 0) {
	    $return = Cobalt::Language::msg_get("mon_state_mem_noinfo");
	    $return = eval "\"$return\"";
	    return $return;
	  } elsif ($severity == $Status{"yellow"}) { 
	    $return = Cobalt::Language::msg_get("mon_state_mem_swaps_yellow");
	    $return = eval "\"$return\"";
	    return $return;
	  } else {
	    $return = Cobalt::Language::msg_get("mon_state_mem_swaps_red");
	    $return = eval "\"$return\"";
	    return $return;
	  }
	}
	elsif ($instance eq "vmem")  # virtual memory use
	{
	  # data[0] is the percentage of virtual memory used
	  $return = Cobalt::Language::msg_get("mon_state_mem_vmem");
	  $return = eval "\"$return\"";
	  return $return;
	}
    }
    elsif ($server eq "net") {
      # The network collision rate is a special case.  Also, it is broken,
      # has always been broken, will always be broken, and should be
      # removed from the product.  Who listens to me, though?
      my ($severity) = Cobalt::Monitor::mon_get_state($server,$instance);

	if ($severity < 0)
	{
	  return Cobalt::Language::msg_get("mon_state_noneyet");
	}

	if    ($instance eq "collisions") { # collisions since boot
	  # data is the collision rate since boot
	  if ($severity == 0) {
	    return Cobalt::Language::msg_get("mon_state_net_idle");
	  }
	  else {
	    # how long have we been up?
	    open(UPTIME,"/proc/uptime") 
	      || die "Cannot read /proc/uptime: $!\n";
	    my $uptime = (split('\s',<UPTIME>))[0];  
	    close UPTIME;
	    
	    my $since = $uptime - ($^T - $update_time);
	    my $days = int($since / 86400);
	    if ($days) {
	      ($since -= ($days * 86400));
	      $days = "$days days ";
	    }
	    else {
	      $days = "";
	    }
	    my $hours = int($since / 3600);
	    if ($hours) {
	      ($since -= ($hours * 3600));
	      $hours = "$hours hours ";
	    }
	    else {
	      $hours = "";
	    }
	    my $mins = int($since / 60);
	    ($mins) ? ($mins = "$mins minutes") : ($mins = "");
	    
	    if ($severity == $Status{"green"}) {
	      $return = Cobalt::Language::msg_get("mon_state_net_green");
	      $return = eval "\"$return\"";
	      return $return;
	    } elsif ($severity == $Status{"yellow"}) {
	      $return = Cobalt::Language::msg_get("mon_state_net15_yellow");
	      $return = eval "\"$return\"";
	      return $return;
	    } else {
	      $return = Cobalt::Language::msg_get("mon_state_net15_red");
	      $return = eval "\"$return\"";
	      return $return;
	    }
	  }
	}
    }
    elsif ($instance eq "col15") { # collisions over past 15 minutes
      # The collision rate over the past 15 minutes is stored
      # in $data3
      if (! $severity){
	return Cobalt::Language::msg_get("mon_net_idle15");
      }
      elsif ($severity == $Status{"green"}) {
	$return = Cobalt::Language::msg_get("mon_state_net15_green");
	$return = eval "\"$return\"";
	return $return;
      } elsif ($severity == $Status{"yellow"}) {
	$return = Cobalt::Language::msg_get("mon_state_net15_yellow");
	$return = eval "\"$return\"";
	return $return;
      } else {
	$return = Cobalt::Language::msg_get("mon_state_net15_red");
	$return = eval "\"$return\"";
	return $return;
      }
    }
    else { # most services work fine within this model
      if ($severity == $Status{"green"}) {
	$return = Cobalt::Language::msg_get("mon_state_green");
	$return = eval "\"$return\"";
	return $return;
      } elsif ($severity <= 0) {
	$return = Cobalt::Language::msg_get("mon_state_noinfo");
	$return = eval "\"$return\"";
	return $return;
      } else {
	$return = Cobalt::Language::msg_get("mon_state_red");
	$return = eval "\"$return\"";
	return $return;
      }
    }
}


1;
