# This autoloader file was built automatically by 
# /usr/local/sbin/autoize from Monitor.pm

# Copyright 1998 Cobalt Networks (http://www.cobaltnet.com)

use strict;
### START HARDCODED PATHS
my $MinTx = "6000";
my $Netif = "eth0";
my %Status = ("none","0","0","none","green","1","1","green","2","yellow","red","3","3","red","yellow","2");
my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = ($Debug ||= 0);
### END HARDCODED PATHS

# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/Monitor.pm.  Changes made here will be lost.
package Cobalt::Monitor;

sub mon_collwatch
# Keep an eye on the network interface and report excessive collisions
# Monitors the number of collisions since system boot, and the number
#   over just the last 15 minutes
# Arguments: yellow threshold (%), red threshold (%), optional network
#   interface (defaults to primary)
# Note: this function has little or no value.  It should never be used.
{
    my ($yelp,$redp,$netif) = @_;
    my ($tx,$colls);
    $netif ||= $Netif;
    # there seems to be some inconsistency about the format of this
    # file.  Let's hope that it's okay.
    open(NS,"/proc/net/dev") || die "Cannot read /proc/net/dev: $!\n";
    while (<NS>)
    {
        if (/^\s*$netif:\s*([\d\s]+)/)
        {
            ($tx,$colls) = (split(/\s+/o,$1))[5,9];
	    last;
        }
    }
    close NS;

    # First do the current collision rate (since the last monitor run)
    my ($condition,$cond_time,$update_time,$notified,$attempts,$reserved,$data) = 
      Cobalt::Monitor::mon_get_state("net","col15");
    my ($prevtx,$prevcols) = split(/,/o,$data);

    # If we rebooted within the last 15 minutes, our state will
    # be from the previous boot.  Ignore it and reset our counter.
    open(UPTIME,"/proc/uptime") || die "Cannot read /proc/uptime: $!\n";
    my $uptime = (split('\s',<UPTIME>))[0];
    close UPTIME;
    if (($tx < $prevtx) || ($uptime < 1000))
    {
	$prevtx = 0;
	$prevcols = 0;
    }

    my ($colpcent,$col15);  # overall, past 15 mins

    # What if there's been no activity at all?
    if (($tx - $prevtx) == 0)
    {
	# No activity == no collisions
	$col15 = 0;
	$colpcent = 0;
    }
    else
    {
	# There's been activity
	$col15 = int((($colls - $prevcols) / ($tx - $prevtx)) * 100);
	$colpcent = int(($colls / $tx) * 100);
    }

    # record the past-15-minutes state
    if (($tx - $prevtx) < $MinTx)
    {
      Cobalt::Monitor::mon_set_state("net","col15",0,"-","-","-","$tx,$colls,$col15");
    }
    elsif ($col15 >= $redp)
    {
      Cobalt::Monitor::mon_set_state("net","col15",$Status{"red"},"-","-","-",
		    "$tx,$colls,$col15");
    }
    elsif ($col15 >= $yelp)
    {
      Cobalt::Monitor::mon_set_state("net","col15",$Status{"yellow"},"-","-","-",
		    "$tx,$colls,$col15");
    }
    else
    {
      Cobalt::Monitor::mon_set_state("net","col15",$Status{"green"},"-","-","-",
		    "$tx,$colls,$col15");
    }

    # record the overall-since-boot state
    if ($tx < $MinTx)
    {
      Cobalt::Monitor::mon_set_state("net","collisions",0,"-","-","-",$colpcent);
    }
    elsif ($colpcent >= $redp)
    {
      Cobalt::Monitor::mon_set_state("net","collisions",$Status{"red"},"-","-","-",$colpcent);
    }
    elsif ($colpcent >= $yelp)
    {
      Cobalt::Monitor::mon_set_state("net","collisions",$Status{"yellow"},"-","-","-",$colpcent);
    }
    else
    {
      Cobalt::Monitor::mon_set_state("net","collisions",$Status{"green"},"-","-","-",$colpcent);
    }
}


1;
