# This autoloader file was built automatically by 
# /usr/local/sbin/autoize from Monitor.pm

# Copyright 1998 Cobalt Networks (http://www.cobaltnet.com)

use strict;
### START HARDCODED PATHS
my $DB_dir_root = "/var/cobalt";
my $Issuefile = "/etc/issue";
my $MainGroup = "home";
my $Status_db = "/var/cobalt/status";
my %Features = ("networker","1","telnetConfig","1","dhcpd","0","shiftjis","Cobalt interface message for shiftjis (We should never see this)","pagebuilder","0","www","1","mx","1","gated","1","dns","1","nfs","0","securetty","1","english","1","atalk","0","fpx","1","siteAdminRedirect","1","users","1","ntp","0","mon_textdb_only","0","search","0","anonFTP","1","specialHomeSkel","0","viewGroups","0","cobaltHomePage","0","lastNameFirst","1","warnRootOnly","1","vsites","1","pam","1","discuss","0","docorg","0","groups","1","smb","0","mail","1","nat","0","squid","0","autoGenWebUsage","0","japanese","0","cobaltRedirect","0","backups","1","vmachine","1","mon_port_testing","1","viewUsers","0","usersCGIInWizard","1","privatedirs","0","telnetDefault","1","ssl","0","siteManagement","1","adm","1","bind8","1","personalRedirect","1","webRequests","1","usersAddUser","0","adminRedirect","1","quotas","1","snmp","1","defaultUserNameGen","1","groupdirs","1","proftpd","1");
my %Status = ("none","0","0","none","green","1","1","green","2","yellow","red","3","3","red","yellow","2");
my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = ($Debug ||= 0);
### END HARDCODED PATHS

# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/Monitor.pm.  Changes made here will be lost.
package Cobalt::Monitor;

#######################################################################
#
# Database functions

sub mon_query
# Get the state of a service for display in the UI
# Arguments: service or (service,instance)
#   Given just the service return the most severe instance within
#   the service (e.g., any red, then any yellow, then any green)
# Return value: condition (red/yellow/green), message text
# Example: ($state,$text) = Cobalt::Monitor::mon_query("mem");
{
  my ($service,$instance) = @_;
  my ($condition,$cond_time,$update_time,$notified,$attempts,$reserved,$data);

  my %STATUS;
  my $fit_to_be_untied = 0;
  if (-e "$DB_dir_root/$service.state") {
    # Do it the text file way
    if (open(US, "$DB_dir_root/$service.state")) {
      while (<US>) {
	chomp;
	my @parts = split(/\|/o, $_, 2);
	$STATUS{"$service|$parts[0]"} = $parts[1];
      }
      close(US);
    }

    # Special case services that include other services
    if (($service eq "memberspace") ||
	($service eq "sitespace") ||
	($service eq "diskspace")) {
      if (open(US, "$DB_dir_root/userspace.state")) {
	while (<US>) {
	  chomp;
	  my @parts = split(/\|/o, $_, 2);
	  $STATUS{"userspace|$parts[0]"} = $parts[1];
	}
	close(US);
      }
      if (open(US, "$DB_dir_root/groupspace.state")) {
	while (<US>) {
	  chomp;
	  my @parts = split(/\|/o, $_, 2);
	  $STATUS{"groupspace|$parts[0]"} = $parts[1];
	}
	close(US);
      }
    }
    if ($service eq "diskspace") {
      if (open(US, "$DB_dir_root/filesystem.state")) {
	while (<US>) {
	  chomp;
	  my @parts = split(/\|/o, $_, 2);
	  $STATUS{"filesystem|$parts[0]"} = $parts[1];
	}
	close(US);
      }
    }
  }
  else {
    # Do it with da database...
    unless(tie(%STATUS, 'DB_File', $Status_db, O_RDONLY(),0644)) {
      return (0,"4671 Cannot read database /var/cobalt/status: $!");
    }
    $fit_to_be_untied = 1;
  }

  # If we aren't given an explicit instance, take the most severe
  # as stored in the PRIME instance
  unless ($instance) {
    my @ins = split(/\|/o,$STATUS{"$service|PRIME"});
    $instance = $ins[6];  # PRIME stores the instance name in the data field
  }

  # Yey.  Special cases.  This is how we handle nested alert lights.
  if ($service eq "memberspace") {
    # This is the condition of the given group and all its members
    # first set the condition to that of the group
    ($condition,$cond_time,$update_time,$notified,$attempts,$reserved,$data) =
      split(/\|/o,$STATUS{"groupspace|$instance"});
    $service = "groupspace";
    my $user;
    # override with the most severe condition among the users
    # (if more severe than the group condition)
    if ($Features{'vmachine'}) {
      foreach $user (Cobalt::List::list_get_mem("${instance}-users")) {
	my ($u_condition,$u_cond_time,$u_update_time,$u_notified,$u_attempts,
	    $u_reserved,$u_data) = split(/\|/o,$STATUS{"userspace|$user"});
	if ($u_condition > $condition) {
	  ($condition,$cond_time,$update_time,$notified,$attempts,
	   $reserved,$data) = 
	     ($u_condition,$u_cond_time,$u_update_time,$u_notified,$u_attempts,
	      $u_reserved,$u_data);
	  ($service,$instance) = ("userspace",$user);
	}
	last if ($condition == $Status{"red"});  # doesn't get any worse
      }
    } else {  # just a normal group of users
      foreach $user (Cobalt::List::list_get_mem("$instance")) {
	my ($u_condition,$u_cond_time,$u_update_time,$u_notified,$u_attempts,
	    $u_reserved,$u_data) = split(/\|/o,$STATUS{"userspace|$user"});
	if ($u_condition > $condition) {
	  ($condition,$cond_time,$update_time,$notified,$attempts,
	   $reserved,$data) = 
	     ($u_condition,$u_cond_time,$u_update_time,$u_notified,$u_attempts,
	      $u_reserved,$u_data);
	  ($service,$instance) = ("userspace",$user);
	}
	last if ($condition == $Status{"red"});  # doesn't get any worse
      }
    }
  }
  elsif ($service eq "sitespace") {
    my $vsite;
    my $worst = 0;
    # Check each virtual site...and also the main site, of course.
    foreach $vsite (Cobalt::Vsite::vsite_list(),[1,1,$MainGroup,'']) {
      my $sitegroup = @$vsite[2];
      # first check the group itself
      my ($g_condition) = split(/\|/o,$STATUS{"groupspace|$sitegroup"});
      if ($g_condition > $worst) {
	($service,$instance) = ("groupspace",$sitegroup);
	$worst = $g_condition;
      }
      last if ($worst == $Status{"red"});  # doesn't get any worse

      # then check each user
      my $user;
      foreach $user (Cobalt::List::list_get_mem("${instance}-users")) {
	my ($u_condition) = split(/\|/o,$STATUS{"userspace|$user"});
	if ($u_condition > $worst) {
	  ($service,$instance) = ("userspace",$user);
	  $worst = $u_condition;
	}
	last if ($worst == $Status{"red"});  # doesn't get any worse
      }
    }
    # need some defaults...
    $service ||= "groupspace"; 
    $instance ||= $MainGroup;
    ($condition,$cond_time,$update_time,$notified,$attempts,$reserved,$data) =
      split(/\|/o,$STATUS{"$service|$instance"});
  }
  elsif ($service eq "diskspace") {
    # Pick the worst of filesystem, groupspace, and userspace
    my ($groupcond,$usercond,$fscond);
    $fscond = (split(/\|/o,$STATUS{"filesystem|PRIME"}))[0];
    $groupcond = (split(/\|/o,$STATUS{"groupspace|PRIME"}))[0];
    $usercond = (split(/\|/o,$STATUS{"userspace|PRIME"}))[0];
    if (($fscond >= $usercond) && ($fscond >= $groupcond)) {
      $service = "filesystem";
    }
    elsif (($groupcond >= $usercond) && ($groupcond > $fscond)) {
      $service = "groupspace";
    }
    else {
      $service = "userspace";
    }
    
    # Okay, now that we've figured out what service is the worst,
    # figure out which instance is the worst so we can have an
    # intellegable message (not that it gets used, but it makes
    # me feel better)
    ($condition,$cond_time,$update_time,$notified,$attempts,$reserved,$data) =
      split(/\|/o,$STATUS{"$service|PRIME"});
    $instance = $data;  # PRIME stores the worst offender in its data field
    ($condition,$cond_time,$update_time,$notified,$attempts,$reserved,$data) =
      split(/\|/o,$STATUS{"$service|$instance"});
  }
  else {
    ($condition,$cond_time,$update_time,$notified,$attempts,$reserved,$data) =
      split(/\|/o,$STATUS{"$service|$instance"});
  }
  untie %STATUS if ($fit_to_be_untied);
  $condition ||= 0;

  # If we've just booted and haven't re-run monitor yet, we consider
  # ourselves to have no state information
  $condition = -1
    if ((stat($Issuefile))[10] > (stat($Status_db))[10]);

  my $message = Cobalt::Monitor::mon_build_alert($service,$instance,
						 $condition,$cond_time,
						 $update_time,$data);
  return ($condition,$message);
}


1;
