# This autoloader file was built automatically by 
# /usr/local/sbin/autoize from Monitor.pm

# Copyright 1998 Cobalt Networks (http://www.cobaltnet.com)

use strict;
### START HARDCODED PATHS
my %Status = ("none","0","0","none","green","1","1","green","2","yellow","red","3","3","red","yellow","2");
my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = ($Debug ||= 0);
### END HARDCODED PATHS

# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/Monitor.pm.  Changes made here will be lost.
package Cobalt::Monitor;

sub mon_swapwatch
# Keep an eye on the memory state and report excessive swapping
# Arguments: yellow threshold, red threshold
#   thresholds are in pages per second (in or out)
# Side effects: updates the database
# Return value: none
{
    my ($yelp,$redp) = @_;
    my ($swapped,$swapness);
    open(MEM,"/proc/stat") || die "Cannot read /proc/stat: $!\n";
    while (<MEM>)
    {
	if (/^swap\s+(\d+)\s+(\d+)/o)
	{
	    $swapped = $1 + $2;
	}
    }
    close MEM;
    
    my ($condition,$cond_time,$update_time,$notified,$attempts,
	$reserved,$data) = Cobalt::Monitor::mon_get_state("mem","swapping");
    my ($hadswapped,$swaps_per_second) = split(/,/o,$data);

    # If we rebooted within the last 15 minutes, our state will
    # be from the previous boot.  Ignore it and reset our counter.
    open(UPTIME,"/proc/uptime") || die "Cannot read /proc/uptime: $!\n";
    my $uptime = (split('\s',<UPTIME>))[0];
    close UPTIME;
    $hadswapped = 0 if ($uptime < 1200); # 15 min + margin

    unless ($update_time && $hadswapped)
    {
	# no previous state
	Cobalt::Monitor::mon_set_state("mem","swapping",0,"-","-","-",$swapped);
	return;
    }

    my $elapsed = time() - $update_time;  $elapsed ||= 1;
    my $elapsmin = int($elapsed / 60);
    $swapness = ($swapped - $hadswapped) / $elapsed;  # swaps per second
    $swapness = int($swapness * 100) / 100;   # get rid of excessive digits
    $swapness = 0 if ($swapness <= 0);

    if ($swapness >= $redp)
    {
      Cobalt::Monitor::mon_set_state("mem","swapping",$Status{"red"},"-","-","-",
		    "$swapped,$swapness");
    }
    elsif ($swapness >= $yelp)
    {
      Cobalt::Monitor::mon_set_state("mem","swapping",$Status{"yellow"},"-","-","-",
		    "$swapped,$swapness");
    }
    else
    {
      Cobalt::Monitor::mon_set_state("mem","swapping",$Status{"green"},"-","-","-",
		    "$swapped,$swapness");
    }
}


1;
