# This autoloader file was built automatically by 
# /usr/local/sbin/autoize from Monitor.pm

# Copyright 1998 Cobalt Networks (http://www.cobaltnet.com)

use strict;
### START HARDCODED PATHS
my $Console = "/dev/console";
my $MonDBDir = "/var/cobalt";
my $Telnet_bin = "/usr/sbin/in.telnetd";
my $Telnet_cmd = "/usr/bin/telnet";
my %Status = ("none","0","0","none","green","1","1","green","2","yellow","red","3","3","red","yellow","2");
my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = ($Debug ||= 0);
### END HARDCODED PATHS

# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/Monitor.pm.  Changes made here will be lost.
package Cobalt::Monitor;

sub mon_telnettest
# Test the state of the telnet server
# We use /usr/bin/telnet because getting perl to talk the telnet
# protocol is painful.
# Arguments: the name of the host to test
# Return value: none
# Side effects: makes entries in the monitor database
{
  my ($host) = @_;

  # Check that the inetd process is running
  mon_procwatch("inetd","telnet");

  # Check if the binary is there
  if (! -x $Telnet_bin) {
    Cobalt::Monitor::mon_set_state("telnet","binary",$Status{"red"},"-","-","-","-");
  } else {
    Cobalt::Monitor::mon_set_state("telnet","binary",$Status{"green"},"-","-","-","-");
  }

  # ...and that's pretty much good enough.
  return;

  #use IPC::Open2;
  #use IO::Handle;

  open(LOG,">>$MonDBDir/telnet.log") || open(LOG,">>$Console");

  # Make sure we don't hang forever trying to connect...
  alarm 10;
  $SIG{'ALRM'} = sub { 
    alarm 0;
    $SIG{'ALRM'} = 'DEFAULT';
    Cobalt::Monitor::mon_set_state("telnet","telnet",
				   $Status{"red"},"-","-","-","-");
    print LOG "Failed to send to telnet server: $!\n";
    close LOG;
    return;
  };

  # Open the session
  # open2 allows us to open a session for both reading and writing;
  # the $telnet_rdhandle handle is the read-only filehandle and 
  # $telnet_wrhandle is the write-only filehandle.  
  my $telnet_rdhandle = new IO::Handle;
  my $telnet_wrhandle = new IO::Handle;
  my $pid = open2($telnet_rdhandle,$telnet_wrhandle,"$Telnet_cmd -r -8 $host 2>&1");
  unless ($pid) {
    # the telnet command didn't work for us
    alarm 0;
    $SIG{'ALRM'} = 'DEFAULT';
    Cobalt::Monitor::mon_set_state("telnet","telnet",
				   $Status{"red"},"-","-","-","-");
    print LOG "Could not open telnet session: $!\n";
    close LOG;
    return;
  }

  # Talk to the telnet server
  while (<$telnet_rdhandle>) {
    print STDERR "telnet> $_" if ($Debug && /\S/o);
    print $telnet_wrhandle "\n";  # seems to need this...
    if (/^(login|password):/io) {
      # looks like we got a prompt...
      alarm 0;
      $SIG{'ALRM'} = 'DEFAULT';
      Cobalt::Monitor::mon_set_state("telnet","telnet",
				     $Status{"green"},"-","-","-","-");
      print $telnet_wrhandle "\n~.\n";  # close the session
      $telnet_rdhandle->close;
      $telnet_wrhandle->close;
      close LOG;
      return;
    }
  }

  # If we got here, something is wrong...
  alarm 0;
  $SIG{'ALRM'} = 'DEFAULT';
  Cobalt::Monitor::mon_set_state("telnet","telnet",$Status{"red"},"-","-","-","-");
  print $telnet_wrhandle "\n~.\n";  # close the session
  $telnet_rdhandle->close;
  $telnet_wrhandle->close;
  print LOG "No login prompt from telnet server.\n";
  close LOG;
  return;
}


1;
