# This autoloader file was built automatically by 
# /usr/local/sbin/autoize from NAS.pm

# Copyright 1998 Cobalt Networks (http://www.cobaltnet.com)

use strict;
### START HARDCODED PATHS
my $NAS_groupdiff = "10000";
my $NAS_mainShareDev = "md0";
my $NAS_mode_allnoread = "1456";
my $NAS_mode_allnoread_dir = "1528";
my $NAS_mode_allread = "1460";
my $NAS_mode_allread_dir = "1533";
my $NAS_shareOwner = "admin";
my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = ($Debug ||= 0);
### END HARDCODED PATHS

# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/NAS.pm.  Changes made here will be lost.
package Cobalt::NAS;

# nas_move_share
#
# use a shell command to copy the contents of a directory from one
# filesystem to another
#
# args: $origfs, $newfs



# nas_share_mod
#
# This is a complex function that is the heart of share management on the nas.
# Adding & modifying a share are identical-- adding a share is simply
# modifying a share that doesn't already exist.
#
# Args: a variable number of arguments using the 'arg => value' syntax.
#
# Only name is required.  All shares are uniquely identified by their name.
#
# required args:
# name => current share name (in chooser & nethood)
#
# optional args:
# newname    => string
#               new share name
# fullAccess => list
#               list of users & groups with full access
#               groups must start with a @
# allCanRead => boolean
#               specifies whether or not the rest of the users on
#               the system get read access
# quota       => integer
#               the maximum space available to the share in megabytes
# accessible => boolean
#               specifies whether or not the share is accessible
# device     => the device the share is stored on
# 

sub nas_share_mod {
  my %args = shift;
  
  my $path; # path of share
  my $ret; # generic return value
  my ($cursharedev, $shareid); # share device and id
  my ($mode, $dirmode); 
  # deal with required args
  my $arg;

  print STDERR "nas_share_mod: entered\n" if ($Debug);

  foreach $arg (('name')) {
    return "4708 No name parameter defined." if (! $args{$arg});
  }
  
  if ($args{'allCanRead'}) {
    $mode = $NAS_mode_allread;
    $dirmode = $NAS_mode_allread_dir;
  } else {
    $mode = $NAS_mode_allnoread;
    $dirmode = $NAS_mode_allnoread_dir;
  }

  # load smb.conf to see if the share exists
  my %samba;
  if (Cobalt::Samba::samba_service_exists($args{'name'} == 1)) {
    print STDERR "nas_share_mod: share $args{'name'} exists\n" if ($Debug);
    $ret = Cobalt::Samba::get_smb_service_parms($args{'name'}, \%samba);
    return $ret if ($ret !~ /^2/);
    $path = $samba{'path'};

    # location and id of share
    ($cursharedev, $shareid) = Cobalt::NAS::nas_share_loc($path);

    print STDERR "nas_share_mod: share device: $cursharedev\n" if ($Debug);
    print STDERR "nas_share_mod: share id: $shareid\n" if ($Debug);
      
    # there's a race condition here... someone could edit the same share
    # right now... only one set of changes would be committed
  } else {
    # the share doesn't already exist  - this is more complex

    print STDERR "nas_share_mod: share $args{'name'} does not exist\n" if ($Debug);

    # set default device
    if (! exists($args{'device'})) {
      $args{'device'} = $NAS_mainShareDev;
    }
    $cursharedev = $args{'device'};
    
    print STDERR "nas_share_mod: putting share on device $cursharedev\n" if ($Debug);

    $path = Cobalt::NAS::nas_share_newpath($cursharedev);
    return $path if ($path != /^2/);

    # location and id of share
    ($cursharedev, $shareid) = Cobalt::NAS::nas_share_loc($path);

    print STDERR "nas_share_mod: share device: $cursharedev\n" if ($Debug);
    print STDERR "nas_share_mod: share id: $shareid\n" if ($Debug);

    # create the share's group
    $ret = Cobalt::NAS::nas_group_new($shareid, $shareid); # don't add any members yet
    return $ret if ($ret !~ /^2/);

    print STDERR "nas_share_mod: created share group $shareid\n" if ($Debug);
    
    # create the share directory
    mkdir($path, $dirmode);
    print STDERR "nas_share_mod: created share directory $path with mode $mode\n" if ($Debug);
    
    # tell samba about the share
    $samba{'path'} = $path;
  }
      
  # now the world should look the same whether
  # or not the share already existed
  
  # set ownership & permissions
  # this can be expensive so only do it if the mode has changed
  my ($x,$x,$curmode) = stat($path);
  if ($curmode != $mode) {
    print STDERR "nas_share_mode: setting uid=$NAS_shareOwner, gid=" . $shareid + $NAS_groupdiff . " for $path\n" if ($Debug);
    print STDERR "nas_share_mode: setting fileperm=$mode, dirmode=$dirmode for $path\n" if ($Debug);
    Cobalt::NAS::nas_share_setmode($path, getpwnam($NAS_shareOwner), $shareid + $NAS_groupdiff, $mode, $dirmode);
  }
  
  # set members of share group
  if ($args{'fullAccess'}) {
    $ret = Cobalt::NAS::nas_group_addusers($shareid, Cobalt::NAS::nas_expand_memberlist($args{'fullAccess'}));
    return $ret if ($ret =~ /^2/);
  }

  print STDERR "nas_share_mode: set fullAccess list to $args{'fullAccess'}\n" if ($Debug);
  
  # set quota
  if ($args{'quota'}) {
    $ret = Cobalt::Quota::setqlim($args{'device'}, $shareid, $args{'quota'}, $args{'quota'}, 1);
    return $ret if ($ret != /^2/);
  }

  print STDERR "nas_share_mode: set quota to $args{'quota'}\n" if ($Debug);
  
  # have we moved to a different filesystem?
  if (Cobalt::NAS::nas_share_whichfs($path) ne $args{'device'}) {
    # need code here to move the share to a different filesystem
    print STDERR "nas_share_mode: filesystem changed from " . Cobalt::NAS::nas_share_whichfs($path) . " to $args{'device'}\n" if ($Debug);
  }

  # woohoo!  time to finish.
  if (! exists $args{'newname'}) {
    $args{'newname'} = $args{'name'};
    print STDERR "nas_share_mode: changing name from $args{'name'} to $args{'newname'}\n" if ($Debug);
  }

  # update atalk.  this isn't necessary if the name didn't change
  $ret = Cobalt::Atalk::atalk_set_volume($path, $args{'newname'});
  # bail out -- can't set appletalk volume
  return $ret if ($ret !~ /^2/);

  print STDERR "nas_share_mode: set atalk volume\n" if ($Debug);
  
  # update smb.conf
  $ret = Cobalt::Samba::set_smb_service_parms($args{'name'}, \%samba, $args{'newname'});
  if ($ret !~ /^2/) {
    # uh oh... can't set smb.conf
    # back out the change to appletalk - don't check return
    Cobalt::Atalk::atalk_del_volume($path);
    return $ret;
  }
  print STDERR "nas_share_mode: updated smb.conf\n" if ($Debug);

  # sweet success
  print STDERR "nas_share_mode: finished successfully\n" if ($Debug);
  return "2097 Share modified successfully";
}


1;
