# This autoloader file was built automatically by 
# /usr/local/sbin/autoize from Nat.pm

# Copyright 1998 Cobalt Networks (http://www.cobaltnet.com)

use strict;
### START HARDCODED PATHS
my $Filter_cmd = "/sbin/ipfwadm";
my $Filter_script = "/etc/rc.d/init.d/ipfilters";
my $Lockdir = "/etc/locks";
my %Features = ("networker","1","telnetConfig","1","dhcpd","0","shiftjis","Cobalt interface message for shiftjis (We should never see this)","pagebuilder","0","www","1","mx","1","gated","1","dns","1","nfs","0","securetty","1","english","1","atalk","0","fpx","1","siteAdminRedirect","1","users","1","ntp","0","mon_textdb_only","0","search","0","anonFTP","1","specialHomeSkel","0","viewGroups","0","cobaltHomePage","0","lastNameFirst","1","warnRootOnly","1","vsites","1","pam","1","discuss","0","docorg","0","groups","1","smb","0","mail","1","nat","0","squid","0","autoGenWebUsage","0","japanese","0","cobaltRedirect","0","backups","1","vmachine","1","mon_port_testing","1","viewUsers","0","usersCGIInWizard","1","privatedirs","0","telnetDefault","1","ssl","0","siteManagement","1","adm","1","bind8","1","personalRedirect","1","webRequests","1","usersAddUser","0","adminRedirect","1","quotas","1","snmp","1","defaultUserNameGen","1","groupdirs","1","proftpd","1");
my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = ($Debug ||= 0);
### END HARDCODED PATHS

# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/Nat.pm.  Changes made here will be lost.
package Cobalt::Nat;

sub nat_enable_masq
# Enable IP masquerading for an interface.
# In theory this function should handle virtual interfaces just fine.
# Arguments: interface name or IP and netmask
#   We accept both forms because there isn't really a network interface
#   associated with the modem (at least, not one the sauce understands).
# Return value: message of the form "2671 A jester unemployed is nobody's fool"
# Side effects: Adds a rule to the IP filtering rules to turn on masquerading
#   for this interface
# Examples: nat_enable_masq("eth0");
#            nat_enable_masq("192.168.45.0","255.255.255.0");
#            nat_enable_masq("192.168.45.0","24");
{
  return "1005 IP filtering is not supported in this product." unless ($Features{'nat'});

  # Handle arguments
  my ($n_addr,$n_mask);
  if ($#_) {
    # Arguments given were IP and netmask
    ($n_addr,$n_mask) = @_;
    $n_mask = Cobalt::Network::net_convert_netmask($n_mask) 
      if ($n_mask =~ /\./o); # decimalize dotted quad netmasks
  } else {
    # Argument was the interface name
    my ($netif) = @_;
    ($n_addr,$n_mask) = Cobalt::Network::net_get_ipaddr($netif);
  }

  return "4750 The filtering rules are being edited.  Try again later." if (-e "$Lockdir/ipfilters");
  open(NATOUT,">$Lockdir/ipfilters")
    || return "4751 Cannot write new filtering rules: $!";
  open(NATIN,"$Filter_script") || return "4752 Cannot read IP filters configuration: $!";
  my ($in_masq_section,$masq_set);
  # Scan through the ip filters rule for the masquerading section.
  # If this interface is already being masqueraded in the masquerading
  # section, then we don't need to do anything.
  # Otherwise, add it to the end.
  while (<NATIN>) {
    if (/^\t?\# Begin masquerading rules/o) {
      $in_masq_section = 1;
    }
    elsif (/^\t?\# End masquerading rules/o) {
      $in_masq_section = 0;
      print NATOUT "\t$Filter_cmd -F -a accept -m -S $n_addr/$n_mask # masq\n";
    }
    elsif ($in_masq_section && (/-S $n_addr\/$n_mask \# masq/)) {
      # Oh look, it's already here.  Never mind, let's just clean up and go.
      close NATOUT;
      close NATIN;
      unlink("$Lockdir/ipfilters");
      return "2110 Masquerading already enabled for this interface";
    }
    print NATOUT;
  }
  close NATOUT;
  close NATIN;
  rename("$Lockdir/ipfilters",$Filter_script)
    || return "4753 Cannot write IP filter rules: $!";
  # The chmod will be done by nat_set_rules

  # Now rebuild the rules to make sure all the -m's are in the
  # right place.
  return Cobalt::Nat::nat_set_rules(Cobalt::Nat::nat_get_default(),
				    Cobalt::Nat::nat_get_rules());
}


1;
