# This autoloader file was built automatically by 
# /usr/local/sbin/autoize from Nat.pm

# Copyright 1998 Cobalt Networks (http://www.cobaltnet.com)

use strict;
### START HARDCODED PATHS
my $Filter_cmd = "/sbin/ipfwadm";
my $Filter_script = "/etc/rc.d/init.d/ipfilters";
my $Lockdir = "/etc/locks";
my %Features = ("networker","1","telnetConfig","1","dhcpd","0","shiftjis","Cobalt interface message for shiftjis (We should never see this)","pagebuilder","0","www","1","mx","1","gated","1","dns","1","nfs","0","securetty","1","english","1","atalk","0","fpx","1","siteAdminRedirect","1","users","1","ntp","0","mon_textdb_only","0","search","0","anonFTP","1","specialHomeSkel","0","viewGroups","0","cobaltHomePage","0","lastNameFirst","1","warnRootOnly","1","vsites","1","pam","1","discuss","0","docorg","0","groups","1","smb","0","mail","1","nat","0","squid","0","autoGenWebUsage","0","japanese","0","cobaltRedirect","0","backups","1","vmachine","1","mon_port_testing","1","viewUsers","0","usersCGIInWizard","1","privatedirs","0","telnetDefault","1","ssl","0","siteManagement","1","adm","1","bind8","1","personalRedirect","1","webRequests","1","usersAddUser","0","adminRedirect","1","quotas","1","snmp","1","defaultUserNameGen","1","groupdirs","1","proftpd","1");
my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = ($Debug ||= 0);
### END HARDCODED PATHS

# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/Nat.pm.  Changes made here will be lost.
package Cobalt::Nat;

sub nat_set_rules
# Save the ruleset to the filter script file
# Arguments: default policy, list of rules
#   each rule is a list consisting of:
#   source ip, source mask, source port, dest ip, dest mask, dest port, 
#              proto, policy, weight
#
#   Policy: "accept" or "deny".
#   Proto: "tcp", "udp", "icmp", or "all".  You must specify a protocol if
#         you specify a port.  You cannot specify a port when "all" is the
#         protocol.
#   Port: a port number or a range of ports seperated by a colon
#          (100:150 would be ports 100 through 150)
#         or "all" for all ports.
#   Netmask: should be in decimal notation (24 instead of 255.255.255.0)
#   Weight: an arbitrary decimal number used in sorting.
#
#   Why do you have to specify a default policy?  Because that's how
#   ipfwadm works.  Don't blame me.
#
#   The rules get sorted automatically by user-specified weight.
#
# Side effects: builds the script which sets the filtering and NAT rules,
#   and sets IP forwarding on in the kernel if we're doing any masquerading.
#   The script is actually built with three clauses--one for if filtering
#   is on, one for if masquerading is on, and one for if filtering and
#   masquerading are on.  A variable at the top is used to switch between
#   them.
# Return value: status string of the form "4811 Can't handle the truth"
# Example: Cobalt::Nat::nat_set_rules("deny",
#        [
#         ("207.9.25.0",16,"all","0.0.0.0",0,"all","all","accept",10),
#         ("0.0.0.0",0,"all","207.9.25.1",24,25,"tcp","accept", 5),
#        ]);
#
#    In this example, all outgoing traffic from the 207.9.25.0 network
#    is allowed; incoming traffic to the mail port on 207.9.25.1 is allowed;
#    and all else is denied by default.
#
# NOTE: There is no way to do name-based filtering.
# NOTE 2: This should not be used as a substitute for a real firewall package.
{
  return "1004 IP filtering is not supported in this product." unless ($Features{'nat'});
  my ($defpol,@rulelist) = @_;

  my ($localip,$localmask) = Cobalt::Network::net_get_ipaddr();
  my $filter_mode = Cobalt::Nat::nat_get_on();

  my $we_do_forwarding = 0; # should IP forwarding be on in the kernel?

  my ($r,@rules,@rules_nomasq,@masq_rules,$rulenum);
  # Build the list of addresses and interfaces for this system
  my @all_addresses;          # All the IP addresses this system answers to
  # Doing hashes of arrays doesn't work, so we do this...yeesh.
  my %masqed_interfaces_addr; # The interfaces that get masqueraded
  my %masqed_interfaces_mask; # The netmasks of interfaces that get masqueraded
  my $netif;
  # On a system that supports virtual interfaces, this should be
  # net_get_vinterfaces() or something.
  foreach $netif (Cobalt::Network::net_get_interfaces()) {
    my ($n_addr,$n_mask) = Cobalt::Network::net_get_ipaddr($netif);

    # some interface may not be setup with an IP address
    if (!$n_addr || $n_addr eq '0.0.0.0') { next; }

    push(@all_addresses,$n_addr);
    if (Cobalt::Nat::nat_get_masq($netif)) {
      $masqed_interfaces_addr{$netif} = $n_addr;
      $masqed_interfaces_mask{$netif} = $n_mask;
      push(@masq_rules,
	   "$Filter_cmd -F -a accept -m -S $n_addr/$n_mask # masq $netif");
      # Turn on IP forwarding if masquerading is on
      $we_do_forwarding = 1;
    }
  }

  # Handle each rule in turn
  foreach $r (@rulelist) {
    my @rule = @$r;
    $rulenum++;  # keep track, for error messages
    print STDERR "Rule $rulenum: ", join(' ',@rule), "\n" if ($Debug);
    my ($src_ip,$src_mask,$src_port,$dst_ip,$dst_mask,$dst_port,$proto,
	$policy,$weight) = @rule;

    # Set defaults, fix the arguments
    # Arguments will default to "everyone/everything" if not specified.
    $proto = "all" if ((!$proto) || ($proto =~ /^a/o));
    $src_ip =~ s/(\.|^)0+(\d)/$1$2/go;  # get rid of excess leading zeros
    $src_ip = "0.0.0.0" unless ($src_ip);
    $src_mask = 0 unless ($src_mask);
    $src_mask = Cobalt::Network::net_convert_netmask($src_mask) 
      if ($src_mask =~ /\./o); # decimalize dotted quad netmasks
    $src_port = "" if ((!$src_port) || ($src_port =~ /^a/o));
    $src_port =~ s/-/:/o;  # make dash-specified ranges fit syntax
    $dst_ip =~ s/(\.|^)0+(\d)/$1$2/go;  # get rid of excess leading zeros
    $dst_ip = "0.0.0.0" unless ($dst_ip);
    $dst_mask = 0 unless ($dst_mask);
    $dst_mask = Cobalt::Network::net_convert_netmask($dst_mask) 
      if ($dst_mask =~ /\./o); # decimalize dotted quad netmasks
    $dst_port = "" if ((!$dst_port) || ($dst_port =~ /^a/o));
    $dst_port =~ s/-/:/o;  # make dash-specified ranges fit syntax
    $policy = "accept" if ($policy =~ /^a/oi); # deal with synonyms
    $defpol ||= "accept";
    $weight ||= 50;

    print STDERR "Adding rule $src_ip/$src_mask:$src_port => $dst_ip/$dst_mask:$dst_port [$policy] # $weight\n" if ($Debug);
    
    # Check our arguments, thoroughly
    if ($src_ip !~ /^(\d+\.\d+\.\d+\.\d+)$/o)
    {
      return "4731 Invalid source IP address $src_ip in rule $rulenum";
    }
    elsif (($src_mask !~ /^\d+$/o) || ($src_mask < 0) || ($src_mask > 32))
    {
      return "4732 Invalid source netmask in rule $rulenum";
    }
    elsif ($src_port !~ /^[\d\:]*$/o)
    {
      return "4733 Invalid source port in rule $rulenum";
    }
    elsif ($dst_ip !~ /^(\d+\.\d+\.\d+\.\d+)$/o)
    {
      return "4734 Invalid destination IP $dst_ip in rule $rulenum";
    }
    elsif (($dst_mask !~ /^\d+$/o) || ($dst_mask < 0) || ($dst_mask > 32))
    {
      return "4735 Invalid destination netmask in rule $rulenum";
    }
    elsif ($dst_port !~ /^[\d\:]*$/o)
    {
      return "4736 Invalid destination port in rule $rulenum";
    }
    elsif ($proto !~ /^(tcp|udp|icmp|all)$/oi)
    {
      return "4737 Unknown protocol $proto in rule $rulenum";
    }
    elsif ($policy !~ /^(accept|deny)$/o)
    {
      return "4738 Invalid policy $policy";
    }
    elsif ($weight !~ /^\d+$/o)
    {
      return "4739 IP filtering rule priority is not valid. It should be an integer greater than zero.";
    }  
    elsif (($proto eq "all") && ($src_port || $dst_port))
    {
      return "4740 ICMP protocol (ALL protocols includes ICMP) does not work with a specified port.";
    }

    # Figure out wether this rule should apply to incoming traffic
    # or outgoing traffic (ipfwadm makes a distinction).
    my ($dirflag,$forward);
    for (@all_addresses) {
      if (($dst_mask == 32) && ($dst_ip eq $_)) {
	# aimed specifically at this system (dest == my IP/32)
	# Use the -V flag to establish the interface via which
	# this packet is coming (otherwise the rule will apply
	# to all interfaces, which would suck).
	$dirflag = "-I -V $_";
	$forward = 0;
	last;
      }
      elsif (($src_mask == 32) && ($src_ip eq $_)) {
	# coming specifically from this system (source == my IP/32)
	$dirflag = "-O -V $_";
	$forward = 0;
	last;
      }
      elsif (Cobalt::Network::net_network_ismember($_,$dst_ip,$dst_mask)) {
	# this rule involves this system itself (dest == my IP/!32)
	$dirflag = "-I -V $_";
	$forward = 1;
      }
      elsif (Cobalt::Network::net_network_ismember($_,$src_ip,$src_mask)) {
	# this rule involves this system itself (source == my IP/!32)
	$dirflag = "-O -V $_";
	$forward = 1;
      }
    }

    # If we haven't matched anything, then the rule doesn't involve
    # the qube itself...so this must be just a forwarding rule
    # (no -I or -O)
    $forward = 1 unless ($dirflag);

    # Check if this rule is for an address on a masqueraded network--
    # if it is, the forwarding rules need the -m argument
    # Only accepted packets get masqueraded--denied packets just get
    # dropped.
    my ($netif,$masq);
    foreach $netif (keys %masqed_interfaces_addr) {
      my $n_addr = $masqed_interfaces_addr{$netif};
      my $n_mask = $masqed_interfaces_mask{$netif};
      if (($policy eq "accept") &&
	  ((Cobalt::Netutil::netutil_test_network_in_network($src_ip,$src_mask,
							     $n_addr,$n_mask)
	    ||
	    Cobalt::Netutil::netutil_test_network_in_network($dst_ip,$dst_mask,
							     $n_addr,$n_mask)))
	  )
      {
	# Either the source or the destination address is on an interface
	# that is being masqueraded
	$masq = "-m";
      }
    }

    # If a rule has multiple commands added for its sake (for example,
    # if this is a input and forwarding rule), then when
    # we read them in later it's hard to figure out that they should
    # all be the same rule.  So, we set things up so that any rule that
    # is grouped with an earlier rule is indented a few spaces, and when
    # we read the file in we just ignore or merge any such indented rules.
    # Note that we use space, not tab.
    my $indent = "";

    if ($dirflag) {  # is it an Input or Output rule?
      push(@rules,"$Filter_cmd $dirflag -a $policy -P $proto -S $src_ip/$src_mask $src_port -D $dst_ip/$dst_mask $dst_port # $weight");
      push(@rules_nomasq,"$Filter_cmd $dirflag -a $policy -P $proto -S $src_ip/$src_mask $src_port -D $dst_ip/$dst_mask $dst_port # $weight");
      $indent = "   ";
    }
    if ($forward) {  # is it a forwarding rule?
      push(@rules,"$indent$Filter_cmd -F -a $policy $masq -P $proto -S $src_ip/$src_mask $src_port -D $dst_ip/$dst_mask $dst_port # $weight");
      push(@rules_nomasq,"$indent$Filter_cmd -F -a $policy -P $proto -S $src_ip/$src_mask $src_port -D $dst_ip/$dst_mask $dst_port # $weight");
    }
  }

  @rules = sort {
    # sort by weight
    my $a_weight = $1 if ($a =~ /\#\s*(\d+)/o);
    $a_weight ||= 50;
    my $b_weight = $1 if ($b =~ /\#\s*(\d+)/o);
    $b_weight ||= 50;
    return ($a_weight <=> $b_weight);
  } @rules;

  @rules_nomasq = sort {
    # sort by weight
    my $a_weight = $1 if ($a =~ /\#\s*(\d+)/o);
    $a_weight ||= 50;
    my $b_weight = $1 if ($b =~ /\#\s*(\d+)/o);
    $b_weight ||= 50;
    return ($a_weight <=> $b_weight);
  } @rules_nomasq;

  # Now build the script
  return "4741 The filtering rules are being edited.  Try again later." if (-e "$Lockdir/ipfilters");
  open(NAT,">$Lockdir/ipfilters") || return "4742 Cannot write new filtering rules: $!";
  print NAT "#!/bin/sh\n\n";
  print NAT "# This script sets the IP filtering rules\n";
  print NAT "# This file is edited automatically by the Cobalt admin\n";
  print NAT "# interface.  Any changes made here may be overwritten.\n\n";
  $defpol = "accept" if ($defpol eq "masq");
  print NAT "# 0 = off; 1 = filter only; 2 = masq only; 3 = masq+filter\n";
  print NAT "MODE=$filter_mode\n\n";

  print NAT "# Flush all existing state\n";
  print NAT "$Filter_cmd -F -f\n";
  print NAT "$Filter_cmd -I -f\n";
  print NAT "$Filter_cmd -O -f\n";

  # If filtering and masquerading are off, then just set the default
  # policy to accept; otherwise, set it to the actual default policy.
  # This check is actually done in the shell script, not here.
  print NAT "\nif [ \$MODE = 0 ]; then\n";
  print NAT "\t# Filters are off; set the default policy to accept\n"; 
  print NAT "\t$Filter_cmd -F -p accept\n";
  print NAT "\t$Filter_cmd -I -p accept\n";
  print NAT "\t$Filter_cmd -O -p accept\n";
  print NAT "\texit\n";
  print NAT "else\n";
  print NAT "\t# Set the default policy\n"; 
  print NAT "\t$Filter_cmd -F -p $defpol\n";
  print NAT "\t$Filter_cmd -I -p $defpol\n";
  print NAT "\t$Filter_cmd -O -p $defpol\n";
  print NAT "fi\n";

  print NAT "\n# The rules will be slightly different depending on\n";
  print NAT "# if masquerading is on or not.  Switch between them.\n";
  print NAT "case \"\$MODE\" in\n";
  print NAT "  1)\n";
  print NAT "\t# Filters only\n";
  print NAT join("\n\t","\t# Begin rules", @rules_nomasq, "# End rules\n");
  print NAT "  ;;\n";

  print NAT "  3)\n";
  print NAT "\t# Masquerade and filters\n";
  print NAT join("\n\t","\t# Begin rules", @rules, "# End rules\n");
  print NAT "  ;;\n";
  print NAT "esac\n";  # end case

  print NAT "\n# Add the general maquerading rules, if masquerading is on\n";
  print NAT "if [ \$MODE -gt 1 ]; then\n";
  print NAT join("\n\t",
		 "\t# Begin masquerading rules", 
		 @masq_rules, 
		 "# End masquerading rules\n",
		);
  print NAT "\t/bin/true\n" if ($#masq_rules == -1);
  print NAT "fi\n";
  print NAT "\n";
  close NAT;
  rename("$Lockdir/ipfilters",$Filter_script)
    || return "4745 Cannot write IP filter rules: $!";
  chmod(0755,$Filter_script);

  # Make sure forwarding is turned on
  Cobalt::Network::net_set_forwarding(1) if ($we_do_forwarding);

  return "2107 Filter rules set";
}


1;
