# This autoloader file was built automatically by 
# /usr/local/sbin/autoize from Netutil.pm

# Copyright 1998 Cobalt Networks (http://www.cobaltnet.com)

use strict;
### START HARDCODED PATHS
my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = ($Debug ||= 0);
### END HARDCODED PATHS

# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/Netutil.pm.  Changes made here will be lost.
package Cobalt::Netutil;

sub netutil_test_network_in_network
# Test if one IP network is a subset of another network
#   This is not only for testing if an address is on a network--
#   it also tests wether a subnet is within a larger subnet
#   (e.g., 1.2.3.0/24 is a subnet of 1.2.0.0/16)
#   (generally: is the first network within the second?)
# argument 0 is the IP address of the first network
# argument 1 is the netmask of the first network
# argument 2 is the IP address of the second network
# argument 3 is the netmask of second network
# IP addresses do not have to have all 4 octets; any octets that are not
#   defined will be assumed to be 0 (e.g., "10.10.0.0" is equivalent
#   to "10.10".)
# The netmask can be given either in dotted quad (255.255.0.0) or in 
#   decimal (16).  If in dotted quad, all four quads must be defined
#   (e.g., "255.255" is not acceptable--you must use "255.255.0.0")
# Return value: 1 if the first network is entirely within the second
#   network or if the two networks are the same; 0 otherwise.
#   Note that if the second network is entirely within the first,
#   we will return 0.
# Example: netutil_test_network_in_network("192.168.45.0",24,"192.168.0.0",16);
{
    my ($ip1, $netmask1, $ip2, $netmask2) = @_;

    # convert the IP addresses from dotted quad (e.g., 192.168.123.45) 
    # to binary (e.g., 11000000101010000111101100101101)
    $ip1 = unpack('B32',pack('C4',split('\.',$ip1)));
    $ip2 = unpack('B32',pack('C4',split('\.',$ip2)));

    # If the netmask is given in dotted quad, convert to decimal
    $netmask1 = Cobalt::Network::net_convert_netmask($netmask1) 
      if ($netmask1 =~ /\./o);
    $netmask2 = Cobalt::Network::net_convert_netmask($netmask2)
      if ($netmask2 =~ /\./o);

    # chop off the bits according to the netmask; if the netmask
    # is 32, then no chopping is done (it's exactly that IP)
    # (To return all but the last character, use substr($string, 0, -1))
    $ip1 = substr($ip1,0,$netmask1) if ($netmask1 != 32);
    $ip2 = substr($ip2,0,$netmask2) if ($netmask2 != 32);

    # Now do the comparisons...
    if ($netmask1 >= $netmask2) {
      # the second network is as large or larger than the first
      # (a smaller subnet mask means a larger network)
      return 1 if ($ip1 =~ /^$ip2/);
    }
    # The following test would be if we wanted to check if the second
    # network is within the first...but we don't do that.
    #else {
    #  # the first network is larger (smaller mask)
    #  return -1 if ($ip2 =~ /^$ip1/);
    #}
    return 0;
}


1;
