# This autoloader file was built automatically by 
# /usr/local/sbin/autoize from Network.pm

# Copyright 1998 Cobalt Networks (http://www.cobaltnet.com)

use strict;
### START HARDCODED PATHS
my $Lockdir = "/etc/locks";
my $Net_cfg = "/etc/sysconfig/network";
my $Netif = "eth0";
my $Route_cmd = "/sbin/route";
my $Sauce_log = "/var/cobalt/sauce.log";
my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = ($Debug ||= 0);
### END HARDCODED PATHS

# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/Network.pm.  Changes made here will be lost.
package Cobalt::Network;

sub net_set_gateway
# Sets the default gateway
# Arguments: the address of the gateway, optional the network interface.
#   If given an argument of "none", undefine the gateway.
#   The network interface will default to eth0 if not given.
# Return value: a status string of the form "984 The dentist escaped"
# Side effects: edits the network configuration file and modifies
#   the routing information
{
    my ($gaw,$netif) = @_;

    if ($gaw =~ /^[0\.]+$/o) { return "4371 Invalid gateway $gaw"; }

    my ($gw,$gwline,$gwdevline,$oldgw,$oldgwdev,$setgw,$setgwdev);
    $netif ||= $Netif;

    print STDERR "Setting gateway to $gaw ...\n " if ($Debug);
    # Note that "0.0.0.0" is considered a valid gateway, because
    # that's the gateway the modem points things to.  0.0.0.0's
    # therefore have to be filtered out as bad input by the UI or
    # the LCD--we can't do it.
    if ($gaw eq "none")
    {
	$gw = "";
	$gwline = undef;
	$gwdevline = undef;
    }
    elsif ($gaw =~ /^(\d+\.\d+\.\d+\.\d+)$/o)
    {
	$gw = $1;
	$gwline = "GATEWAY=$gw\n";
	$gwdevline = "GATEWAYDEV=$netif\n";
    }
    else
    {
	print STDERR "Invalid gateway $gaw\n" if ($Debug);
	return "4356 Invalid gateway $gaw";
    }
    $gw =~ s/(\.|^)0+(\d)/$1$2/go;  # get rid of excess leading zeros
    my ($ipee) = Cobalt::Network::net_get_ipaddr();
    return "4357 Host cannot be its own gateway" if ($gw eq $ipee);

    return "4367 /etc/sysconfig/network is locked, cannot set gateway" 
	if (-e "$Lockdir/network");
    open(O,">$Lockdir/network")
	|| return "4369 Cannot modify /etc/sysconfig/network: $!";
    open(I,$Net_cfg) 
	|| return Cobalt::Network::net_abort("4368 Cannot read /etc/sysconfig/network: $!");
    while (<I>)
    {
        if (/^GATEWAY=(.*)$/o)
        {
	    $oldgw = $1;
	    $_ = $gwline;
	    $setgw = 1;
        }
	if (/^GATEWAYDEV=(.*)$/o) {
		$oldgwdev = $1;
		$_ = $gwdevline;
		$setgwdev = 1;
	}
	print O;
    }
    print O $gwline if ((! $setgw) && $gwline);
    print O $gwdevline if ((! $setgwdev) && $gwdevline);
    close O;
    close I;

    print STDERR "$Route_cmd del default\n" if ($Debug);
    system("$Route_cmd del default >>$Sauce_log 2>&1");
    if ($gw)
    {
	print STDERR "$Route_cmd add default gw $gw $netif\n" if ($Debug);
	# Make sure we remove the lock file before returning
	if (system("$Route_cmd add default gw $gw $netif 1>&2")) {
	  unlink("$Lockdir/network");
	  return "3416 Cannot add routing table entry";
	}
    }

    # unlock and save
    rename("$Lockdir/network",$Net_cfg) ||
	return "3409 Cannot update network file: $!";

    return "2411 Gateway changed from $oldgw to $gw";
}


1;
