# This autoloader file was built automatically by 
# /usr/local/sbin/autoize from Network.pm

# Copyright 1998 Cobalt Networks (http://www.cobaltnet.com)

use strict;
### START HARDCODED PATHS
my $AdminConf = "/etc/admserv/conf";
my $Adminuser = "admin";
my $Atalkscript = "/etc/rc.d/init.d/atalk";
my $Dhcpd_conf = "/etc/dhcpd.conf";
my $Dns_cgi = "/usr/admserv/cgi-bin/.cobalt/dns/index.cgi";
my $Hostfile = "/etc/hosts";
my $Hostname_cmd = "/bin/hostname";
my $Hostnamefile = "/etc/HOSTNAME";
my $Htconf = "/etc/httpd/conf";
my $Htpid = "/var/run/httpd.pid";
my $Ifconfig_cmd = "/sbin/ifconfig";
my $Lcdshowscript = "/etc/rc.d/init.d/showip";
my $Lockdir = "/etc/locks";
my $Mailscript = "/etc/rc.d/init.d/sendmail";
my $MaxHostnameLen = "40";
my $Net_cfg = "/etc/sysconfig/network";
my $Route_cmd = "/sbin/route";
my $Sauce_log = "/var/cobalt/sauce.log";
my $Smbscript = "/etc/rc.d/init.d/smb";
my $Subsys = "/var/lock/subsys";
my %Features = ("networker","1","telnetConfig","1","dhcpd","0","shiftjis","Cobalt interface message for shiftjis (We should never see this)","pagebuilder","0","www","1","mx","1","gated","1","dns","1","nfs","0","securetty","1","english","1","atalk","0","fpx","1","siteAdminRedirect","1","users","1","ntp","0","mon_textdb_only","0","search","0","anonFTP","1","specialHomeSkel","0","viewGroups","0","cobaltHomePage","0","lastNameFirst","1","warnRootOnly","1","vsites","1","pam","1","discuss","0","docorg","0","groups","1","smb","0","mail","1","nat","0","squid","0","autoGenWebUsage","0","japanese","0","cobaltRedirect","0","backups","1","vmachine","1","mon_port_testing","1","viewUsers","0","usersCGIInWizard","1","privatedirs","0","telnetDefault","1","ssl","0","siteManagement","1","adm","1","bind8","1","personalRedirect","1","webRequests","1","usersAddUser","0","adminRedirect","1","quotas","1","snmp","1","defaultUserNameGen","1","groupdirs","1","proftpd","1");
my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = ($Debug ||= 0);
### END HARDCODED PATHS

# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/Network.pm.  Changes made here will be lost.
package Cobalt::Network;

sub net_set_hostname
# Set the hostname
# Arguments: the new name for the computer, a boolean of wether
#    to force a reconfig (normally there will be none if the
#    hostname hasn't changed)
# Return value: a string containing a return code and a message
#   (e.g. "908 Name change complete")
# Side effects: edits the network settings in /etc/sysconfig, the
#   /etc/HOSTNAME file, and in the hosts file; then tells the system
#   its new name (which then takes effect immediately)
#   Also reconfigure the web server and admin server for the new names
#   (and turn hostnamelookups off while we're there)
#   Restarts the web servers and the mail server
{
    my ($name,$force) = @_;
    return "4372 Cannot reconfigure network: system busy"
	unless(Cobalt::Network::net_lock_network());

    if(length( $name ) > $MaxHostnameLen ) { return Cobalt::Network::net_abort("4388 Hostname cannot be longer than 40 characters"); }

    if ($name =~ /^([\w\-\.]+)$/o)
    {
	# untaint (we used to do thorough taint checking.  Those were
	#          the days...)
	$name = $1;
    }
    else
    {
	return Cobalt::Network::net_abort("4370 Invalid hostname: $name");
    }

    # Localhost is blocked by the UI; down at this level it's okay
    #return Cobalt::Network::net_abort("4371 reserved name: localhost")
	#if ($name eq "localhost");

    my ($fullname,$shortname,$oldname,$domain,$changed);
    # fullname is the fully qualified domain name
    $domain = (Cobalt::Network::net_get_dnsclient())[0];
    if ($domain && ($name !~ /\.$domain$/))
    {
	$fullname = "$name.$domain";
	$shortname = $name;
    }
    else
    {
	$fullname = $name;
	$shortname = $name;
	$shortname =~ s/\.$domain$//;
    }

    # Localhost should never have a domain name
    if ($shortname =~ /^localhost(\..+)?$/o)
    {
      $fullname = "localhost";
      $shortname = "localhost";
    }

    return Cobalt::Network::net_abort("4362 /etc/sysconfig/network is locked, cannot set hostname")
	if (-e "$Lockdir/network");
    open(O,">$Lockdir/network")
	|| return Cobalt::Network::net_abort("4361 Cannot modify /etc/sysconfig/network: $!");
    open(I,$Net_cfg) 
	|| return Cobalt::Network::net_abort("4350 Cannot read /etc/sysconfig/network: $!","$Lockdir/network");
    while (<I>)
    {
        if (/^HOSTNAME=(.*)$/o)
        {
	    $oldname = $1;
	    $_ = "HOSTNAME=$fullname\n";
        }
	print O;
    }
    close O;
    close I;
    rename("$Lockdir/network",$Net_cfg) ||
	return Cobalt::Network::net_abort("3407 Cannot update network file: $!","$Lockdir/network");

    # If nothing changed, don't do anything
    # Special case: if the new name is "localhost" we must be
    # doing a reset
    if ((($fullname eq $oldname) && ($fullname ne "localhost"))
	&& (! $force))
    {
	Cobalt::Network::net_unlock_network();
	return "2067 No update to hostname or domain name needed";
    }

    my ($ip,$mask) = Cobalt::Network::net_get_ipaddr();

    # Rebuild the hosts file
    # Actually, we don't fix the hosts file; we clobber it entirely.
    if (($ip ne "127.0.0.1") && ($name ne "localhost"))
    {
        return Cobalt::Network::net_abort("3406 The hosts file is locked.  Please try again later")
	    if (-e "$Lockdir/hosts");
	open(O,">$Lockdir/hosts")
	    || return Cobalt::Network::net_abort("3405 Cannot modify hosts file: $!");
	open(I,$Hostfile)
	  || return Cobalt::Network::net_abort("3437 Cannot read hosts file: $!");
	my $printed = 0;
	while (<I>) 
	{
	  if ((/^$ip\s/) || (/\s$fullname(\s|$)/)) {
	    # found a line with old info about this host
	    # the linux dnsdomainname command gets its domainname from
	    # reading /etc/hosts.
	    print O "$ip\t$fullname \t# main interface\n" unless ($printed);
	    #print O "$ip\t$shortname $fullname \t# main interface\n" unless ($printed);
	    $printed = 1;
	  } else {
	    print O;
	  }
	}
	print O "$ip\t$fullname \t# main interface\n" unless ($printed);
	close O;
	close I;
	rename("$Lockdir/hosts",$Hostfile) ||
	    return Cobalt::Network::net_abort("3417 Cannot update hosts file: $!","$Lockdir/hosts");
    }
	
    # I don't know who uses this file.  I don't really much care.
    if (open (HN,">$Hostnamefile"))
    {
	print HN "$fullname\n";
	close HN;
    }
    else
    {
	return Cobalt::Network::net_abort("3412 Cannot change hostname file: $!");
    }
    
    # Change the servername for the web server, if there is one
    if ($Features{'www'}) {
      return Cobalt::Network::net_abort("3418 /etc/httpd/conf/httpd.conf is locked, try again later")
	if (-e "$Lockdir/httpd.conf");
      open(O,">$Lockdir/httpd.conf")
        || return Cobalt::Network::net_abort("3420 Cannot modify /etc/httpd/conf/httpd.conf: $!");
      open(I,"$Htconf/httpd.conf")
	|| return Cobalt::Network::net_abort("3419 Cannot read /etc/httpd/conf/httpd.conf: $!",
			"$Lockdir/httpd.conf");
      my $nomore;
      while (<I>) {
	if ($nomore) {
	  print O;
	  next;
	}
        elsif (/^ServerName /o) {
	  $_ = "ServerName $fullname\n";
        }
        elsif (/^ServerAdmin /o) {
	  $_ = "ServerAdmin $Adminuser\@$fullname\n";
        }
        elsif (/^\#ServerName new.host.name/o) {
	  $_ = "ServerName $fullname\n";
        }
	# take advantage of the typo, and fix it as a semaphore
	elsif (/^\# See alto the BindAddress entry./o) {
	  print O "# See also the BindAddress entry.\n\n";
	  print O "### Anything below this will be left untouched by\n";
	  print O "### The Cobalt administration interface.\n";
	  print O "### If adding virtual hosts, also add Listen directives\n";
	  $_ = "### as described above.\n\n";
	  $nomore = 1;
	}
	elsif (/<VirtualHost/) {
	  # We're in the virtual host area, leave it alone
	  $nomore = 1;
	}
	elsif (/^\#\#\# Anything below this will be left untouched by/o) {
	  # We're in the virtual host area, leave it alone
	  $nomore = 1;
	}
	print O;
      }
      close I;
      close O;
      rename("$Lockdir/httpd.conf","$Htconf/httpd.conf") ||
	return Cobalt::Network::net_abort("3404 Cannot update /etc/httpd/conf/httpd.conf: $!");
    } # end changes to $Htconf
	
    # Change the servername for the admin server, if any
    if ($Features{'adm'}) {
      return Cobalt::Network::net_abort("3422 /etc/admserv/conf/httpd.conf is locked, try again later")
	if (-e "$Lockdir/adm_httpd.conf");
      open(O,">$Lockdir/adm_httpd.conf")
        || return Cobalt::Network::net_abort("3424 Cannot modify /etc/admserv/conf/httpd.conf: $!");
      open(I,"$AdminConf/httpd.conf")
        || return Cobalt::Network::net_abort("3423 Cannot read /etc/admserv/conf/httpd.conf: $!",
			"$Lockdir/httpd.conf");
      while (<I>) {
	# We fix the ServerAdmin, but we don't restart the web server.
	# Restarting the web server causes badness, because we're in
	# the middle of talking to the bloody thing.
        if (/^ServerAdmin /o) {
	    $_ = "ServerAdmin $Adminuser\@$fullname\n";
        }
	elsif (/^ServerName /o) { # replace literal localhost
	  $_ = "ServerName $fullname\n";
	}
	
	print O;
      }
      close I;
      close O;
      rename("$Lockdir/adm_httpd.conf","$AdminConf/httpd.conf") ||
	return Cobalt::Network::net_abort("3425 Cannot update /etc/admserv/conf/httpd.conf: $!",
		     "Lockdir/httpd.conf");
    } # end admin server changes

    # Set the hostname for the running system
    system("$Hostname_cmd $fullname 1>&2") &&
	return Cobalt::Network::net_abort("3413 Cannot change name without rebooting: $!");

    # Update aliases from sendmail's /etc/virtusertable
    my($ret) = Cobalt::Email::mail_virtuser_change_domain($oldname,$fullname);
    return Cobalt::Network::net_abort($ret) if ($ret !~ /^2/o);

    # Update the server identifier in dhcpd.conf
    if ($Features{'dhcpd'} && (-e $Dhcpd_conf)) {
      my($ret) = Cobalt::Dhcpd::dhcpd_set_name($fullname);
      return Cobalt::Network::net_abort($ret) if ($ret !~ /^2/o);
    }

    # then fork a child to restart the daemons
    my $pid;
    if (!defined($pid = fork)) {
	die "cannot fork: $!";
    } elsif ($pid) {
	# i'm the parent
	print STDERR "forking a child to restart daemons\n" if ($Debug);
	return "2064 Daemons restarted";
    }
    else
    {
	# otherwise I'm the child
	open(STDERR,">>$Sauce_log");
	open(STDOUT,">>$Sauce_log");

	# fix the DNS entries and restart
	if (Cobalt::Services::get_dns_server_on())
	{
	  # Signal the script to rebuild the files and update the server
	  # The CGI will take care of restarting the server
	  system("$Dns_cgi -update >>$Sauce_log 2>&1");
	}

	# restart appletalk
	if (-e "$Subsys/atalk")
	{
	  my $netif;
	  foreach $netif (Cobalt::Network::net_get_interfaces())
	    {
	      # Turn off atalk and reset the interface
	      my ($gw,$gwif) = Cobalt::Network::net_get_gateway();
	      my ($addr,$mask) = Cobalt::Network::net_get_ipaddr($netif);
	      my ($broadcast) = Cobalt::Network::net_get_broadcast($addr,$mask);
	      my $network = Cobalt::Network::net_get_network($addr,$mask);
	      system("$Atalkscript stop >>$Sauce_log 2>&1");
	      system("$Ifconfig_cmd $netif down 1>&2");
	      system("$Ifconfig_cmd $netif inet $addr netmask $mask broadcast $broadcast 1>&2");
	      system("$Route_cmd add -net $network netmask $mask $netif 1>&2");
	      Cobalt::Network::net_set_gateway($gw,$gwif) if ($gw);
	    }

	    system("$Atalkscript start >>$Sauce_log 2>&1");
	}

	# Everything else can be concurrent
	Cobalt::Network::net_unlock_network();

	# restart the web server
	if (-f "$Subsys/httpd")
	{
	    my ($pid,$l);
	    # Note that we die instead of returning because we're a child
	    # process...return()ing from here could result in some really
	    # bad juju.
	    open(PID,$Htpid) 
	        || die Cobalt::Network::net_abort("4017 Cannot read httpd PID file /var/run/httpd.pid: $!");
	    $l = <PID>;
	    if ($l =~ /^(\d+)$/o)
	    {
	        $pid = $1;
	    }
	    close PID;
	    kill('HUP',$pid) if ($pid);
	}

	# restart mail
	if (-f "$Subsys/sendmail")
	{
	    system("$Mailscript stop >>$Sauce_log 2>&1 ");
	    system("$Mailscript start >>$Sauce_log 2>&1 ");
	}

	# restart samba
	if (-f "$Subsys/smb")
	{
	    system("$Smbscript stop >>$Sauce_log 2>&1 ");
	    system("$Smbscript start >>$Sauce_log 2>&1 ");
	}

	system ("$Lcdshowscript start >>$Sauce_log 2>&1")
	    if (-x $Lcdshowscript);
	exit;
    }
    return "2412 Name successfully changed from $oldname to $fullname";
}


1;
