# This autoloader file was built automatically by 
# /usr/local/sbin/autoize from Network.pm

# Copyright 1998 Cobalt Networks (http://www.cobaltnet.com)

use strict;
### START HARDCODED PATHS
my $Hostfile = "/etc/hosts";
my $Htconf = "/etc/httpd/conf";
my $If_cfg_dir = "/etc/sysconfig/network-scripts";
my $Ifconfig_cmd = "/sbin/ifconfig";
my $Ifup = "/etc/sysconfig/network-scripts/ifup";
my $Lockdir = "/etc/locks";
my $Netif = "eth0";
my $Route_cmd = "/sbin/route";
my $Sauce_log = "/var/cobalt/sauce.log";
my %Features = ("networker","1","telnetConfig","1","dhcpd","0","shiftjis","Cobalt interface message for shiftjis (We should never see this)","pagebuilder","0","www","1","mx","1","gated","1","dns","1","nfs","0","securetty","1","english","1","atalk","0","fpx","1","siteAdminRedirect","1","users","1","ntp","0","mon_textdb_only","0","search","0","anonFTP","1","specialHomeSkel","0","viewGroups","0","cobaltHomePage","0","lastNameFirst","1","warnRootOnly","1","vsites","1","pam","1","discuss","0","docorg","0","groups","1","smb","0","mail","1","nat","0","squid","0","autoGenWebUsage","0","japanese","0","cobaltRedirect","0","backups","1","vmachine","1","mon_port_testing","1","viewUsers","0","usersCGIInWizard","1","privatedirs","0","telnetDefault","1","ssl","0","siteManagement","1","adm","1","bind8","1","personalRedirect","1","webRequests","1","usersAddUser","0","adminRedirect","1","quotas","1","snmp","1","defaultUserNameGen","1","groupdirs","1","proftpd","1");
my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = ($Debug ||= 0);
### END HARDCODED PATHS

# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/Network.pm.  Changes made here will be lost.
package Cobalt::Network;

sub net_set_ipaddr
# Set the IP address and netmask
# Arguments: the new IP address and netmask (in dotted quad), and an 
#   optional network interface name (e.g. "eth1").  The network 
#   interface argument will default to the primary (eth0) interface.
# Side effects: edits the files in /etc/sysconfig
#   Reconfigures the network interface for the new address
#   Sets the .user redirects in the admin server, and restarts the admin server
{
    my ($addr,$mask,$netif) = @_;
    my $broadcast = Cobalt::Network::net_get_broadcast($addr,$mask);
    my $network = Cobalt::Network::net_get_network($addr,$mask);
    my ($dida,$didn,$didm,$didb);

    $netif ||= $Netif; # default the interface

    my ($oldaddr,$oldmask) = Cobalt::Network::net_get_ipaddr($netif);
    return "2111 No change to IP address or netmask"
      if (($addr eq $oldaddr) && ($mask eq $oldmask));

    ## Check arguments
    $addr =~ s/(\.|^)0+(\d)/$1$2/go;  # get rid of excess leading zeros
    # If we're given a weird IP and it's not via the web, wake the LCD
    if ($addr !~ /^(\d+\.\d+\.\d+\.\d+)$/o)
    {
	print STDERR "Got invalid IP address $addr\n" if ($Debug);
	exec("/sbin/lcdstart") unless $ENV{'REMOTE_ADDR'};
	return "4360 Invalid ip address: $addr";  # not reached
    }
    if (Cobalt::Network::net_network_ismember($addr,"224.0.0.0","240.0.0.0")) {
      # This is an address on the multicast network
      return "4760 $addr cannot be used as your IP address, as it is reserved for multicast use";
    }

    ## Get background information
    my $name = Cobalt::Network::net_get_hostname($netif);
    my ($gw,$gwif) = Cobalt::Network::net_get_gateway();
    print STDERR "My name is $name and my gateway is $gw on $gwif\n" if ($Debug);

    my ($fullname,$domain);
    if (($domain = (Cobalt::Network::net_get_dnsclient())[0])
	&& ($name !~ /$domain$/))
    {
	$fullname = "$name.$domain";
    }
    else
    {
	$fullname = "";
    }

    ## Rewrite the interface configuration file
    # need the general lock NO MATTER WHAT
    return "4375 Cannot reconfigure network: system busy"
      unless(Cobalt::Network::net_lock_network());
    # get the lock specific to this interface
    # the general lock this shouldn't be an issue)
    return Cobalt::Network::net_abort("4364 Cannot set IP address: ifcfg-$netif is locked")
      if (-e "$Lockdir/ifcfg-$netif");
    open(O,">$Lockdir/ifcfg-$netif")
	|| return Cobalt::Network::net_abort("4366 Cannot modify interface config ifcfg-$netif: $!");
    if (! -e "$If_cfg_dir/ifcfg-$netif") {
      # This interface hasn't been configured yet
      print O "DEVICE=$netif\n";
      print O "ONBOOT=yes\n";
    }
    else {
      open(I,"$If_cfg_dir/ifcfg-$netif") 
	|| return Cobalt::Network::net_abort("4365 Cannot read interface config ifcfg-$netif: $!");
      while (<I>) {
        if (/^IPADDR=(.*)$/o) {
	  $oldaddr = $1;
	  print O "IPADDR=$addr\n";
	  $dida = 1;
        }
        elsif (/^NETWORK=(.*)$/o) {
	  print O "NETWORK=$network\n";
	  $didn = 1;
        }
        elsif (/^NETMASK=(.*)$/o) {
	  print O "NETMASK=$mask\n";
	  $didm = 1;
        }
        elsif (/^BROADCAST=(.*)$/o) {
	    print O "BROADCAST=$broadcast\n";
	    $didb = 1;
        }
	elsif (/^BOOTPROTO/o) {
	  # get rid of this; if we're in here, we're hard-coded
	  ;
	}
	else {
	  print O;
	}
      }
      close I;
    }
    print O "IPADDR=$addr\n" unless ($dida);
    print O "NETWORK=$network\n" unless ($didn);
    print O "NETMASK=$mask\n" unless ($didm);
    print O "BROADCAST=$broadcast\n" unless ($didb);
    close O;

    ## Rebuild the hosts file
    if (($name ne "localhost") && ($addr ne "127.0.0.1"))
    {
      my %hostlist;
      return Cobalt::Network::net_abort("3402 Cannot write hosts file: hosts file is locked") 
	if (-e "$Lockdir/hosts");
      open(HOUT,">$Lockdir/hosts")
	|| return Cobalt::Network::net_abort("3415 Cannot modify hosts file: $!");
      open(HIN,$Hostfile)
	|| return Cobalt::Network::net_abort("3401 Cannot read /etc/hosts: $!");
      #while (<HIN>) {
      #  next if (/^\#/o);  # we don't deal well with comments
      #  my ($addr,$name) = split(/\s+/o,$_,2);
      #  $hostlist{$addr} = $name;
      #}
      #undef $hostlist{$oldaddr} if ($oldaddr ne $addr);
      #unless ($hostlist{$addr}) {
      #  $hostlist{$addr} = "$name $fullname \t# interface $netif\n";
      #}
      #print HOUT "# This file is generated and clobbered by the Cobalt interface.\n";
      #print HOUT "127.0.0.1\t", $hostlist{"127.0.0.1"};
      #undef $hostlist{"127.0.0.1"};
      #my $a;
      #foreach $a (sort keys %hostlist) {
      #  print HOUT "$a\t", $hostlist{$a} if ($hostlist{$a});
      #}
      my $printed;
      while (<HIN>) 
      {
	if ((/^$addr\s/) || (/\s$fullname(\s|$)/)) {
	  # found a line with old info about this host
	  #print HOUT "$addr\t$name $fullname \t# main interface\n" unless ($printed);
	  print HOUT "$addr\t$fullname \t# main interface\n" unless ($printed);
	  $printed = 1;
	} else {
	  print HOUT;
	}
      }
      #print HOUT "$addr\t$name $fullname \t# main interface\n" unless ($printed);
      print HOUT "$addr\t$fullname \t# main interface\n" unless ($printed);
      close HOUT;
      close HIN;
      rename("$Lockdir/hosts",$Hostfile) ||
	return Cobalt::Network::net_abort("3408 Cannot update hosts file: $!");
    }

    ## Reconfigure the interface
    print STDERR "$Ifconfig_cmd $netif inet $addr netmask $mask broadcast $broadcast\n"
	if ($Debug);
    print STDERR "$Route_cmd add -net $network netmask $mask $netif\n"
	if ($Debug);
    system("$Ifconfig_cmd $netif down 1>&2");
    system("$Ifconfig_cmd $netif inet $addr netmask $mask broadcast $broadcast 1>&2");
    system("$Route_cmd add -net $network netmask $mask $netif 1>&2");
    # We need to re-add the broadcast route if we reset eth0
    system("$Route_cmd add -net 255.255.255.255 netmask 255.255.255.255 $Netif 1>&2") if ($netif eq $Netif);
    print STDERR "Now, My name is $name and my gateway is $gw\n" if ($Debug);
    Cobalt::Network::net_set_gateway($gw,$gwif) if ($gw);

    print STDERR "Changed IP address for $netif to $addr from $oldaddr\n"
      if ($Debug);

    # Change the ip of the web server Listen directive, if there is one
    if ($Features{'www'}) {
    
      return Cobalt::Network::net_abort("3421 /etc/httpd/conf/httpd.conf is locked")
        if (-e "$Lockdir/httpd.conf");
     
      open(O,">$Lockdir/httpd.conf")
        || return Cobalt::Network::net_abort("3432 Cannot modify /etc/httpd/conf/httpd.conf: $!");
      open(I,"$Htconf/httpd.conf")
        || return Cobalt::Network::net_abort("3430 Cannot read /etc/httpd/conf/httpd.conf: $!",
			"$Lockdir/httpd.conf");

      # There are cases when we don't know the $oldaddr, 
      #   after a lcd reset network for one, so we relax this a bit..
      my $searchstring = "Listen ";
      $searchstring = "Listen $oldaddr:80" if ($oldaddr);

      my $nomore;
      while (<I>) {
        if ($nomore) {
          print O;
	  next;
        }
        elsif (/^$searchstring/o) {
          $_ = "Listen $addr:80\n";
	  $nomore = 1;
        }
        print O;
      }
      close I;
      close O;
      rename("$Lockdir/httpd.conf","$Htconf/httpd.conf") ||
        return Cobalt::Network::net_abort("3414 Cannot update /etc/httpd/conf/httpd.conf: $!");
    } # end changes to $Htconf

    # Bring all the virtual interfaces associated with this
    # physical interface back up
    my $iface;
    opendir(IFDIR,$If_cfg_dir) || 
      return Cobalt::Network::net_abort("4175 Cannot read network script dir /etc/sysconfig/network-scripts: $!");
    my @ifcfgs = grep(/^ifcfg-$netif:\d+/,readdir(IFDIR));
    closedir IFDIR;
    foreach $iface (@ifcfgs) {
      $iface =~ s/^ifcfg-//o;

      # Set the netmask, broadcast, and network for each virtual interface
      open(VIRTIN,"$If_cfg_dir/ifcfg-$iface") || 
	  Cobalt::Network::net_abort("4172 Cannot read /etc/sysconfig/network-scripts/ifcfg-$iface: $!");
      open(VIRTOUT,">$Lockdir/ifcfg-$iface") ||
	  Cobalt::Network::net_abort("4174 Cannot write /etc/locks/ifcfg-$iface: $!");
      my $vaddr; # the IP of the virtual site
      while (<VIRTIN>) {

	  # This counts on the fact that IPADDR shows up in the file before
	  # NETWORK and BROADCAST.  As the sauce is written right now, we
	  # can count on that, assuming nobody hand-edits the files.
	  # $mask is the $mask we were using for the main interface
	  if (/^IPADDR=([\d\.]+)/o) {
	      $vaddr = $1;
	      print VIRTOUT "IPADDR=$vaddr\n";
	  }
	  elsif (/^NETMASK/o) {
	      print VIRTOUT "NETMASK=$mask\n";
	  }
	  elsif (/^NETWORK/o) {
	      my $network = Cobalt::Network::net_get_network($vaddr,$mask);
	      print VIRTOUT "NETWORK=$network\n";
	  }
	  elsif (/^BROADCAST/o) {
	      my $broadcast = Cobalt::Network::net_get_broadcast($vaddr,$mask);
	      print VIRTOUT "BROADCAST=$broadcast\n";
	  }
	  else {
	      print VIRTOUT;
	  }
      }

      close VIRTIN;
      close VIRTOUT;
      rename("$Lockdir/ifcfg-$iface","$If_cfg_dir/ifcfg-$iface") ||
	  Cobalt::Network::net_abort("4173 Cannot set new ifcfg-$iface file: $!");

      # Tell the mail server to allow relaying from our network and domain
      my($ret) = Cobalt::Email::mail_add_relay($network);
      return Cobalt::Network::net_abort($ret) if ($ret !~ /^2/o);

      # Now bring the interface up (unless we're still booting up)
      system("$Ifup $iface >> $Sauce_log") if (Cobalt::Utility::sauce_booted());
    }

    # unlock
    rename("$Lockdir/ifcfg-$netif","$If_cfg_dir/ifcfg-$netif") ||
	return Cobalt::Network::net_abort("3403 Cannot update interface config file: $!");
    Cobalt::Network::net_unlock_network();
    return "2410 Changed IP address from $oldaddr to $addr";
}


1;
