# This autoloader file was built automatically by 
# /usr/local/sbin/autoize from Network.pm

# Copyright 1998 Cobalt Networks (http://www.cobaltnet.com)

use strict;
### START HARDCODED PATHS
my $If_cfg_dir = "/etc/sysconfig/network-scripts";
my $Ifup = "/etc/sysconfig/network-scripts/ifup";
my $Lockdir = "/etc/locks";
my $Netif = "eth0";
my $Sauce_log = "/var/cobalt/sauce.log";
my $Sites_Max = "250";
my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = ($Debug ||= 0);
### END HARDCODED PATHS

# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/Network.pm.  Changes made here will be lost.
package Cobalt::Network;

sub net_set_vinterface
# Configure virtual network interfaces - Automatically invoked by Vsite.pm
# Arguments: the new IP address, netmask, 
#   optional physical interface name (defaults to eth0)
# Side effects: edits the files in /etc/sysconfig
#   Reconfigures the network interface for the new address
# Return value: status string of the form "4346 You don't get a wish
#   because you don't have your own brain."
{
    my ($addr,$mask,$netif) = @_;

    $netif ||= $Netif;

    # $addr =~ s/(\.|^)0+(\d)/$1$2/go;  # get rid of excess leading zeros
    
    if( !Cobalt::Netutil::netutil_test_ipaddr( $addr ) )
    {
        print STDERR "Got invalid IP address $addr\n" if ($Debug);
        return "4140 $addr is not a valid IP address";
    }
    if (Cobalt::Network::net_network_ismember($addr,"224.0.0.0","240.0.0.0")) {
      # This is an address on the multicast network
      return "4766 $addr cannot be used as an IP address, as it is reserved for multicast use";
    }

    my $broadcast = Cobalt::Network::net_get_broadcast($addr,$mask);
    my $network = Cobalt::Network::net_get_network($addr,$mask);
    my $main_addr = (Cobalt::Network::net_get_ipaddr())[0];

    # Verify $addr is not already in use
    my(@all_IPs) = Cobalt::Network::net_get_alladdrs();
    my($ip_instance,%active_IPs);
    foreach $ip_instance (@all_IPs) { $active_IPs{"$ip_instance"}=1; }
    return "4767 The IP address $addr is already in use by a virtual network interface" 
      if ($active_IPs{$addr});

    # This is a new interface
    my @siteslist = Cobalt::Vsite::vsite_list();
    return "4598 This Cobalt server is limited to 250 virtual sites for maximum performance."
      if (($#siteslist + 1) >= $Sites_Max);  # plus one to un-zero-index

    # Get the number for the next interface to add
    my ($ifnum,$virtif);
    opendir(IFDIR,$If_cfg_dir) ||
      return "4143 Cannot read network script directory /etc/sysconfig/network-scripts: $!";
    my @ifcfgs = sort {$a <=> $b } (grep(/^ifcfg-$netif:\d+$/o,readdir(IFDIR)));
    closedir IFDIR;
    for ($ifnum = 0; $ifnum <= $#ifcfgs; $ifnum++)
    {
        if ($ifcfgs[$ifnum] ne "ifcfg-$netif:$ifnum")
        {
            $virtif = "$netif:$ifnum";
            last;
        }
    }
    $virtif ||= "$netif:" . ($#ifcfgs + 1);
    print STDERR "Allocating new interface $virtif for $addr\n"
      if ($Debug);

    print STDERR "New settings for $virtif: $addr/$mask\n"
        if ($Debug);
    # need the general lock NO MATTER WHAT
    return "4379 Cannot reconfigure network: system busy"
      unless(Cobalt::Network::net_lock_network());
    return Cobalt::Network::net_abort("4144 Cannot configure virtual interface: ifcfg-$netif is locked")
      if (-e "$Lockdir/ifcfg-$virtif");
    open(O,">$Lockdir/ifcfg-$virtif")
      || return Cobalt::Network::net_abort("4145 Cannot get lock for interface $netif: $!");

    print O "DEVICE=$virtif\n";
    print O "IPADDR=$addr\n";
    print O "NETMASK=$mask\n";
    print O "NETWORK=$network\n";
    print O "BROADCAST=$broadcast\n";
    print O "ONBOOT=yes\nALIAS=yes\nBOOTPROTO=none\n";
    close O;

    # unlock
    unless (rename("$Lockdir/ifcfg-$virtif","$If_cfg_dir/ifcfg-$virtif"))
    {
        return Cobalt::Network::net_abort("3145 Cannot update virtual interface config file: $!",
                         "$Lockdir/ifcfg-$virtif");
    }

    # Bring the interface up
    print STDERR "$Ifup $virtif\n" if ($Debug);
    system("$Ifup $virtif >>$Sauce_log 2>&1\n");

    Cobalt::Network::net_unlock_network();

    return "2140 Virtual interface for address $addr has been set up";
}


1;
