# This autoloader file was built automatically by 
# /usr/local/sbin/autoize from Repair.pm

# Copyright 1998 Cobalt Networks (http://www.cobaltnet.com)

use strict;
### START HARDCODED PATHS
my $AdminConf = "/etc/admserv/conf";
my $Admscript = "/etc/rc.d/init.d/admserv.init";
my $Ahttpd = "/usr/sbin/ahttpd";
my $Lcdshowscript = "/etc/rc.d/init.d/showip";
my $MonDBDir = "/var/cobalt";
my $Sauce_log = "/var/cobalt/sauce.log";
my $Stat_lim_expire = "86400";
my $Stat_relimit = "7";
my %Features = ("networker","1","telnetConfig","1","dhcpd","0","shiftjis","Cobalt interface message for shiftjis (We should never see this)","pagebuilder","0","www","1","mx","1","gated","1","dns","1","nfs","0","securetty","1","english","1","atalk","0","fpx","1","siteAdminRedirect","1","users","1","ntp","0","mon_textdb_only","0","search","0","anonFTP","1","specialHomeSkel","0","viewGroups","0","cobaltHomePage","0","lastNameFirst","1","warnRootOnly","1","vsites","1","pam","1","discuss","0","docorg","0","groups","1","smb","0","mail","1","nat","0","squid","0","autoGenWebUsage","0","japanese","0","cobaltRedirect","0","backups","1","vmachine","1","mon_port_testing","1","viewUsers","0","usersCGIInWizard","1","privatedirs","0","telnetDefault","1","ssl","0","siteManagement","1","adm","1","bind8","1","personalRedirect","1","webRequests","1","usersAddUser","0","adminRedirect","1","quotas","1","snmp","1","defaultUserNameGen","1","groupdirs","1","proftpd","1");
my %Servers = ("smb","Windows file sharing (SMB)","mail","Email","filesystem","Disk","www","Web Server","dns","Domain Name Service (DNS)","net","Network","nfs","Unix file sharing (NFS)","cpu","CPU","ftp","File Transfer Protocol (FTP)","userspace","Quota","telnet","Telnet server","atalk","Apple file sharing (AppleShare)","modem","Modem","adm","Administrative Web Server","ntp","Network Time Protocol (NTP)","ssl","Secure Web Server","groupspace","Quota","snmp","Simple Network Management Protocol (SNMP)","mem","Memory");
my %Status = ("none","0","0","none","green","1","1","green","2","yellow","red","3","3","red","yellow","2");
my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = ($Debug ||= 0);
### END HARDCODED PATHS

# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/Repair.pm.  Changes made here will be lost.
package Cobalt::Repair;

sub repair_adm
# Repair the admin server
# Arguments: none
# Side effects: restarts, etc, the server; updates the database
# Return value: status string
{
    my $service = "adm";
    my $servertext = $Servers{$service};

    return "1701 No administrative web server on this system" unless ($Features{'adm'});

    my ($condition,$cond_time,$update_time,$notified,$attempts,$reserved,
	$data) = Cobalt::Monitor::mon_get_state($service);

    print STDERR "Attempting restart of $service: $condition\n" if ($Debug);
    my $hostname = Cobalt::Network::net_get_hostname();

    open(LOG,">>$MonDBDir/$service.log") ||
      return "3710 Cannot write log file /var/cobalt/$service.log: $!";

    # First check some basics
    if (! -x $Ahttpd) {
      print LOG "<FONT COLOR=\"red\">$servertext, ",
      Cobalt::Language::msg_localtime(time()),
      ": $servertext binary is missing!</FONT>\n";
    }
    if (! -e $AdminConf) {
      print LOG "<FONT COLOR=\"red\">$servertext, ",
      Cobalt::Language::msg_localtime(time()),
      ": $servertext config file(s) are missing!</FONT>\n";
    }
    if (! -x $Admscript) {
      print LOG "<FONT COLOR=\"red\">$servertext, ",
      Cobalt::Language::msg_localtime(time()),
      ": $servertext startup script is missing!</FONT>\n";
    }

    # Attempt #1: restart.
    if ($attempts <= $Stat_relimit)
      {
	my ($bad_config) = 0;	# flag if we got config file errors

	print LOG "<FONT COLOR=\"$Status{$condition}\">$servertext, ",
	Cobalt::Language::msg_localtime(time()), ": attempting restart</FONT>\n";
	system ("$Admscript stop >>$Sauce_log 2>&1");

	# We watch the output of the startup script to see if there
	# are config file errors.
	my ($badfile);
	open(SCRIPTOUT,"$Admscript start 2>&1 |")
	  || return "3711 Cannot run admin server startup script: $!";
	while (<SCRIPTOUT>)
	  {
	    if (/Syntax error on line \d+ of (.+):/o)
	      {
		$badfile = $1;
	      }
	    elsif (/^Invalid/o)
	      {
		chomp;
		print LOG "<FONT COLOR=\"red\">$servertext, ",
		Cobalt::Language::msg_localtime(time()),
		": $servertext error in $badfile: $_</FONT>\n";
	      }
	  }
	close SCRIPTOUT;
	Cobalt::Monitor::mon_set_state($service,"","-","-",($attempts + 1),"-","-");

	if ($badfile)
	  {
	    # take punitive action
	    print LOG "<FONT COLOR=\"$Status{$condition}\">$servertext, ",
	    Cobalt::Language::msg_localtime(time()),
	    ": Rebuilding config file $badfile from master copy</FONT>\n";
	    rename($badfile,"$badfile.$^T");
	    open(SRC,"$badfile.master")
	      || return "3712 Cannot read master file $badfile.master: $!";
	    open(DEST,">$badfile")
	      || return "3713 Cannot write admin config file $badfile: $!";
	    print DEST while (<SRC>);
	    close DEST;
	    close SRC;

	    # Do our edits, as needed
	    Cobalt::Network::net_set_ipaddr(Cobalt::Network::net_get_ipaddr())
	      if (($service eq "adm") && ($badfile =~ /srm.conf$/o));
	    Cobalt::Network::net_set_hostname($hostname,1)
	      if ($badfile =~ /httpd.conf$/o);

	    # okay, NOW try it
	    system ("$Admscript start >>$Sauce_log 2>&1");
	  }

	system ("$Lcdshowscript start")
	  if (-x $Lcdshowscript);
      }
    elsif (($cond_time - $^T) > $Stat_lim_expire)
      {
	# we hit the limit over a day ago, reset the restart counter
	print LOG "<FONT COLOR=\"$Status{$condition}\">$servertext, ",
	Cobalt::Language::msg_localtime(time()), ": resetting restart counter</FONT>\n";
	Cobalt::Monitor::mon_set_state($service,"","-","-",0,"-","-");
      }
    else
      {
	print LOG "<FONT COLOR=\"$Status{$condition}\">$servertext, ",
	Cobalt::Language::msg_localtime(time()), ": too many restarts</FONT>\n";
      }

    close LOG;

    # Check the pulse
    Cobalt::Repair::repair_examine($service);
    ($condition,$cond_time,$update_time,$notified,$attempts,$reserved,$data) =
      Cobalt::Monitor::mon_get_state($service);

    if ($condition == $Status{'green'})
      {
	return "2710 admin server restarted successfully";
      }
    else
      {
	return "4711 Unable to restart administrative web server";
      }
}


1;
