# This autoloader file was built automatically by 
# /usr/local/sbin/autoize from Repair.pm

# Copyright 1998 Cobalt Networks (http://www.cobaltnet.com)

use strict;
### START HARDCODED PATHS
my $AdminServer_port = "81";
my $CollRed = "4015";
my $CollYel = "2700";
my $DiskRed = "90";
my $DiskYel = "80";
my $LoadRed = "6";
my $LoadYel = "3";
my $Netif = "eth0";
my $Subsys = "/var/lock/subsys";
my $SwapRed = "40";
my $SwapYel = "20";
my $VmemRed = "75";
my $VmemYel = "50";
my %Features = ("networker","1","telnetConfig","1","dhcpd","0","shiftjis","Cobalt interface message for shiftjis (We should never see this)","pagebuilder","0","www","1","mx","1","gated","1","dns","1","nfs","0","securetty","1","english","1","atalk","0","fpx","1","siteAdminRedirect","1","users","1","ntp","0","mon_textdb_only","0","search","0","anonFTP","1","specialHomeSkel","0","viewGroups","0","cobaltHomePage","0","lastNameFirst","1","warnRootOnly","1","vsites","1","pam","1","discuss","0","docorg","0","groups","1","smb","0","mail","1","nat","0","squid","0","autoGenWebUsage","0","japanese","0","cobaltRedirect","0","backups","1","vmachine","1","mon_port_testing","1","viewUsers","0","usersCGIInWizard","1","privatedirs","0","telnetDefault","1","ssl","0","siteManagement","1","adm","1","bind8","1","personalRedirect","1","webRequests","1","usersAddUser","0","adminRedirect","1","quotas","1","snmp","1","defaultUserNameGen","1","groupdirs","1","proftpd","1");
my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = ($Debug ||= 0);
### END HARDCODED PATHS

# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/Repair.pm.  Changes made here will be lost.
package Cobalt::Repair;

sub repair_examine
# Generic function for checking the state of a subsystem
# Arguments: Service name
# Side effects: Does its best to see if that service is working properly
#    Updates the database with the result.
# Return value: none
{
    my ($service) = @_;
    print STDERR "Checking the state of $service\n" if ($Debug);

    # Get our IP address so we know what address to connect to
    my $myipaddr = (Cobalt::Network::net_get_ipaddr($Netif))[0];

    # Big nasty case statement; do the right thing for whatever service
    # we're examining.
    if    ($service eq "adm")
      {
	# Just see if the port answers
	Cobalt::Monitor::mon_portwatch("adm","$myipaddr",$AdminServer_port,"adm");
      }
    elsif ($Features{'atalk'} && ($service eq "atalk"))
      {
        if (-e "$Subsys/atalk")	# check if it's all the way up
	  {
	    Cobalt::Monitor::mon_atalktest();	# check using nbplkup
	    Cobalt::Monitor::mon_procwatch("atalkd","atalk"); # verify the procs are there
	    Cobalt::Monitor::mon_procwatch("afpd","atalk");
	  }
	else
	  {
	    mon_set_state('atalk','PRIME',0,'-','-','-','-');
	    mon_set_state('atalk','nbplkup',0,'-','-','-','-');
	    mon_set_state('atalk','atalkd',0,'-','-','-','-');
	    mon_set_state('atalk','afpd',0,'-','-','-','-');
	  }
      }
    elsif ($service eq "cpu")
      {
	Cobalt::Monitor::mon_loadwatch($LoadYel,$LoadRed); # Check load average
      }
    elsif ($service eq "crond")
      {
	Cobalt::Monitor::mon_procwatch("crond","crond");
	Cobalt::Monitor::mon_cronwatch();
      }
    elsif ($service eq "custo")
      {
	Cobalt::Monitor::mon_procwatch("custodiat","custo");
      }
    elsif ($service eq "modem")
      {
	# Check that the modem is doing its thing
	my %modem_settings = Cobalt::Modem::modem_get_settings();
	my $conn_mode = Cobalt::Modem::modem_get_connmode(\%modem_settings);
	if ($conn_mode != 2) {
	  Cobalt::Monitor::mon_modem();
	}
	else {
	  # register the modem as off
	  mon_set_state('modem','PRIME',0,'-','-','-','-');
	  mon_set_state('modem','diald',0,'-','-','-','-');
	}
      }
    elsif ($service eq "filesystem")
      {
	# Check the overall usage for each partition
	Cobalt::Monitor::mon_diskwatch("/",$DiskYel,$DiskRed);
	Cobalt::Monitor::mon_diskwatch("/var",$DiskYel,$DiskRed);
	Cobalt::Monitor::mon_diskwatch("/home",$DiskYel,$DiskRed);
      }
    elsif (($Features{'users'}) && ($service eq "userspace"))
      {
	# Check the usage for each user (quota and mail spool)
	my $name;
	Cobalt::Monitor::mon_quotawatch("*USERS");
	#foreach $name (Cobalt::User::user_list()) {
	#Cobalt::Monitor::mon_quotawatch($name,$UsrQuota,$QuotYel,$QuotRed);
	#Cobalt::Monitor::mon_spoolwatch($name) if (-e "$Mailspool/$name");
	#}
      }
    elsif (($Features{'groups'}) && ($service eq "groupspace"))
      {
	# Check the quota usage for each group
	my ($name);
	Cobalt::Monitor::mon_quotawatch("*GROUPS");
	#foreach $name (Cobalt::Group::group_list()) {
	#Cobalt::Monitor::mon_quotawatch($name,$GrpQuota,$QuotYel,$QuotRed);
	#}
      }
    elsif ($Features{'dhcpd'} && ($service eq "dhcpd"))
      {
	# Check that the DHCP server is running
	if (Cobalt::Services::get_dhcp_server_on())
	  {
	    Cobalt::Monitor::mon_procwatch("dhcpd","dhcpd");
	  }
	else
	  {
            mon_set_state('dhcpd','PRIME',0,'-','-','-','-');
            mon_set_state('dhcpd','dhcpd',0,'-','-','-','-');
	  }
      }
    elsif ($Features{'dns'} && ($service eq "dns"))
      {
	if (Cobalt::Services::get_dns_server_on())
	  {
	    Cobalt::Monitor::mon_portwatch("dns","$myipaddr","53","dns");
	    Cobalt::Monitor::mon_procwatch("named","dns");
	  }
        else
        {
            mon_set_state('dns','PRIME',0,'-','-','-','dns');
            mon_set_state('dns','dns',0,'-','-','-','-');
            mon_set_state('dns','named',0,'-','-','-','-');
        }
      }
    elsif ($service eq "inetd")
      {
	Cobalt::Monitor::mon_procwatch("inetd","inetd");
      }
    elsif ($service eq "lcd")
      {
	Cobalt::Monitor::mon_procwatch("lcdsleep","lcd"); # LCD panel process
      }
    elsif ($Features{'mail'} && ($service eq "mail"))
      {
	if (Cobalt::Services::get_mail_server_on())
	  {
	    Cobalt::Monitor::mon_portwatch("mail","$myipaddr","110","pop3");
	    Cobalt::Monitor::mon_portwatch("mail","$myipaddr","143","imap");
	    Cobalt::Monitor::mon_portwatch("mail","$myipaddr","25","smtp");
	  }
        else
        {
            mon_set_state('mail','PRIME',0,'-','-','-','smtp');
            mon_set_state('mail','pop3',0,'-','-','-','-');
            mon_set_state('mail','imap',0,'-','-','-','-');
            mon_set_state('mail','smtp',0,'-','-','-','-');
        }
      }
    elsif ($service eq "mem")
      {
	Cobalt::Monitor::mon_swapwatch($SwapYel,$SwapRed);
	Cobalt::Monitor::mon_memwatch($VmemYel,$VmemRed);
      }
    elsif ($service eq "net")
      {
	Cobalt::Monitor::mon_collwatch($CollYel,$CollRed);
      }
    elsif ($Features{'nfs'} && ($service eq "nfs"))
      {
	# NFS is not, technically, supported
	if (Cobalt::Services::get_nfs_server_on())
	  {
	    Cobalt::Monitor::mon_nfstest();
	    Cobalt::Monitor::mon_procwatch("rpc.nfsd","nfs");
	    Cobalt::Monitor::mon_procwatch("rpc.mountd","nfs");
	  }
        else
        {
            mon_set_state('nfs','PRIME',0,'-','-','-','-');
            mon_set_state('nfs','rpc.nfsd',0,'-','-','-','-');
            mon_set_state('nfs','rpc.mountd',0,'-','-','-','-');
        }
      }
    elsif (($service eq "ntp") && $Features{'ntp'})
      {
	if (Cobalt::Services::get_ntp_server_on())
	  {
	    Cobalt::Monitor::mon_procwatch("ntpd","ntp");
	  }
       else
       {
        mon_set_state('ntp','PRIME',0,'-','-','-','-');
        mon_set_state('ntp','ntpd',0,'-','-','-','-');
       }
      }
    elsif ($service eq "portmap")
      {
	# Only used if we're doing NFS (or NIS!?!)
	if (Cobalt::Services::get_nfs_server_on())
	  {
	    Cobalt::Monitor::mon_procwatch("portmap","portmap");
	  }
        else
        {
            mon_set_state('portmap','PRIME',0,'-','-','-','-');
            mon_set_state('portmap','portmap',0,'-','-','-','-');
         }
      }
    elsif ($Features{'smb'} && ($service eq "smb"))
      {
	if (Cobalt::Services::get_smb_server_on())
	  {
	    Cobalt::Monitor::mon_procwatch("smbd","smb");
	    Cobalt::Monitor::mon_procwatch("nmbd","smb");
	    Cobalt::Monitor::mon_portwatch("smb","$myipaddr","139","smb");
	  }
        else
        {
            mon_set_state('smb','PRIME',0,'-','-','-','smb');
            mon_set_state('smb','smb',0,'-','-','-','-');
            mon_set_state('smb','smbd',0,'-','-','-','-');
            mon_set_state('smb','nmbd',0,'-','-','-','-');
        }
      }
    elsif ($Features{'snmp'} && ($service eq "snmp"))
      {
	if (Cobalt::Services::get_snmp_server_on())
	  {
	    Cobalt::Monitor::mon_procwatch("snmpd","snmp");
	  }
       else
       {
         mon_set_state('snmp','PRIME',0,'-','-','-','-');
         mon_set_state('snmp','snmpd',0,'-','-','-','-');
       }
      }
    elsif ($Features{'ssl'} && ($service eq "ssl"))
      {
	Cobalt::Monitor::mon_portwatch("ssl","$myipaddr","443","ssl");
      }
    elsif ($service eq "telnet")
      {
	if (Cobalt::Services::get_telnet_server_on())
	  {
	    Cobalt::Monitor::mon_telnettest("$myipaddr");
	  }
       else
       {
         mon_set_state('telnet','PRIME',0,'-','-','-','telnet');
         mon_set_state('telnet','telnet',0,'-','-','-','-');
         mon_set_state('telnet','binary',0,'-','-','-','-');
         mon_set_state('telnet','inetd',0,'-','-','-','-');
       }
      }
    elsif ($service eq "www")
      {
	Cobalt::Monitor::mon_portwatch("www","$myipaddr","www","www");
      }
    elsif ($service eq "ftp")
      {
	# ftp is usually run from inetd, so generally needn't be
	# checked by itself.  We just check it for the UI's sake,
	# or in case we're running it standalone
	if (Cobalt::Services::get_ftp_server_on())
	  {
	    Cobalt::Monitor::mon_portwatch("ftp","$myipaddr","ftp","ftp");
	  }
       else
       {
         mon_set_state('ftp','PRIME',0,'-','-','-','ftp');
         mon_set_state('ftp','ftp',0,'-','-','-','-');
       }
      }
    else
      {
	# Don't print a message unless debugging is on, because
	# we may just be trying to check on an unsupported
	# service
	print STDERR "Unsupported service $service\n" if ($Debug);
      }
}


1;
