# This autoloader file was built automatically by 
# /usr/local/sbin/autoize from Repair.pm

# Copyright 1998 Cobalt Networks (http://www.cobaltnet.com)

use strict;
### START HARDCODED PATHS
my $Inetd_script = "/etc/rc.d/init.d/inet";
my $Lcdshowscript = "/etc/rc.d/init.d/showip";
my $MonDBDir = "/var/cobalt";
my $Sauce_log = "/var/cobalt/sauce.log";
my $Stat_lim_expire = "86400";
my $Stat_relimit = "7";
my %Servers = ("smb","Windows file sharing (SMB)","mail","Email","filesystem","Disk","www","Web Server","dns","Domain Name Service (DNS)","net","Network","nfs","Unix file sharing (NFS)","cpu","CPU","ftp","File Transfer Protocol (FTP)","userspace","Quota","telnet","Telnet server","atalk","Apple file sharing (AppleShare)","modem","Modem","adm","Administrative Web Server","ntp","Network Time Protocol (NTP)","ssl","Secure Web Server","groupspace","Quota","snmp","Simple Network Management Protocol (SNMP)","mem","Memory");
my %Status = ("none","0","0","none","green","1","1","green","2","yellow","red","3","3","red","yellow","2");
my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = ($Debug ||= 0);
### END HARDCODED PATHS

# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/Repair.pm.  Changes made here will be lost.
package Cobalt::Repair;

sub repair_generic
# Generic server restart script
# Arguments: service,path to server binary, path to server config,
#   path to startup script, and restart inetd flag (1=yes,0=no)
# Side effects: restarts, etc, the server; updates the database
# Return value: status string
{
    my ($service,$server_bin,$server_conf,$server_script,$restart_inetd) = @_;

    my ($condition,$cond_time,$update_time,$notified,$attempts,$reserved,$data) =
      Cobalt::Monitor::mon_get_state($service);

    my $servertext = $Servers{$service};

    print STDERR "Attempting restart of $service: $condition\n" if ($Debug);

    open(LOG,">>$MonDBDir/$service.log") ||
      return "3705 Cannot write log file /var/cobalt/$service.log: $!";

    # First check some basics
    if ($server_bin && (! -x $server_bin)) {
      print LOG "<FONT COLOR=\"red\">$servertext, ",
      Cobalt::Language::msg_localtime(time()),
      ": $servertext binary is missing!</FONT>\n";
    }
    if ($server_conf && (! -e $server_conf)) {
      print LOG "<FONT COLOR=\"red\">$servertext, ",
      Cobalt::Language::msg_localtime(time()),
      ": $servertext config file(s) are missing!</FONT>\n";
    }
    if ($server_script && (! -x $server_script)) {
      print LOG "<FONT COLOR=\"red\">$servertext, ",
      Cobalt::Language::msg_localtime(time()),
      ": $servertext startup script is missing!</FONT>\n";
    }

    my $actiontext;
    if ($attempts <= $Stat_relimit) # not too many tries yet
      {
	$actiontext = "attempting restart";

	if ($server_script) {
	  # there may be no server script, just inetd
	  system ("$server_script stop >>$Sauce_log 2>&1");
	  system ("$server_script start >>$Sauce_log 2>&1");
	}

	# inetd is often a special case; we don't restart the server
	# itself, we start the master daemon that runs it
	if ($restart_inetd)
	  {
	    system ("$Inetd_script stop >>$Sauce_log 2>&1");
	    system ("$Inetd_script start >>$Sauce_log 2>&1");
	  }

	# record the additional attempt
	Cobalt::Monitor::mon_set_state($service,"","-","-",($attempts + 1),"-","-");

	system ("$Lcdshowscript start")
	  if (-x $Lcdshowscript);
      }
    elsif (($cond_time - $^T) > $Stat_lim_expire)
      {
	# we hit the limit over a day ago, reset the restart counter
	$actiontext = "resetting restart counter";
	Cobalt::Monitor::mon_set_state($service,"","-","-",0,"-","-");
      }
    else
      {
	$actiontext = "too many restarts";
      }

    # look at our state now
    Cobalt::Repair::repair_examine($service);
    ($condition,$cond_time,$update_time,$notified,$attempts,$reserved,$data) =
      Cobalt::Monitor::mon_get_state($service);

    # "How we doin'?"
    if ($condition == $Status{'green'})
      {
	print LOG "<FONT COLOR=\"green\">$servertext, ", Cobalt::Language::msg_localtime(time()),
	": $actiontext ... $service restarted successfully</FONT>\n";
	close LOG;
	return "2703 $servertext successfully restarted";
      }
    else
      {
	print LOG "<FONT COLOR=\"red\">$servertext, ", Cobalt::Language::msg_localtime(time()),
	": Couldn't restart $service ... $actiontext</FONT>\n";
	close LOG;
	return "4703 $servertext remained down";
      }
}


1;
