# This autoloader file was built automatically by 
# /usr/local/sbin/autoize from Repair.pm

# Copyright 1998 Cobalt Networks (http://www.cobaltnet.com)

use strict;
### START HARDCODED PATHS
my $Adminuser = "admin";
my $AlertList = "sys";
my $MonDBDir = "/var/cobalt";
my $Sendmail = "/usr/sbin/sendmail";
my $ShiftJISToJIS_prog = "/usr/local/sbin/nkf";
my %Features = ("networker","1","telnetConfig","1","dhcpd","0","shiftjis","Cobalt interface message for shiftjis (We should never see this)","pagebuilder","0","www","1","mx","1","gated","1","dns","1","nfs","0","securetty","1","english","1","atalk","0","fpx","1","siteAdminRedirect","1","users","1","ntp","0","mon_textdb_only","0","search","0","anonFTP","1","specialHomeSkel","0","viewGroups","0","cobaltHomePage","0","lastNameFirst","1","warnRootOnly","1","vsites","1","pam","1","discuss","0","docorg","0","groups","1","smb","0","mail","1","nat","0","squid","0","autoGenWebUsage","0","japanese","0","cobaltRedirect","0","backups","1","vmachine","1","mon_port_testing","1","viewUsers","0","usersCGIInWizard","1","privatedirs","0","telnetDefault","1","ssl","0","siteManagement","1","adm","1","bind8","1","personalRedirect","1","webRequests","1","usersAddUser","0","adminRedirect","1","quotas","1","snmp","1","defaultUserNameGen","1","groupdirs","1","proftpd","1");
my %Language = ("english","1","shiftjis","Cobalt interface message for shiftjis (We should never see this)","lastNameFirst","1","japanese","0");
my %Status = ("none","0","0","none","green","1","1","green","2","yellow","red","3","3","red","yellow","2");
my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = ($Debug ||= 0);
### END HARDCODED PATHS

# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/Repair.pm.  Changes made here will be lost.
package Cobalt::Repair;

##########

sub repair_notify
# Mail a report of what happened
# Arguments: The server, the instance,  and (optional) an list of
#    additional addresses to which to send the notification
# Note that this should only get called with an instance argument if we
#    want seperate notifications for each instance of a service.  In
#    practice, this means only for quota problems (users and groups).
#    Actually, this goes for the alsotell argument as well.
# This function checks to see if a notification has already been sent,
#    and does not repeat if one has.  It does *not* check the state of
#    the service to be sure that a notification is called for.
# Side effects: sends email to admin, sets the notified flag in the
#    database
# Doesn't warn about CPU problems if swapping is high
{
    my ($service,$instance,@alsotell) = @_;
    my $alsotell = join(',',@alsotell);
    $alsotell ||= '';		# just to be picky
    print STDERR "Notify $service/$instance ($alsotell)\n"
      if ($Debug);

    # We want the mail to appear to come from the site the user
    # is associated with
    my $adminEmail = $Adminuser;
    if (($Features{'vmachine'})) {
       if( $alsotell[0] ) {
           $adminEmail .= '@' . (Cobalt::User::user_list_site($alsotell[0]))[0];
       }

       my $i;
       for ($i=0; $i<=$#alsotell; $i++) {
           $alsotell[$i] .= '@' . (Cobalt::User::user_list_site($alsotell[$i]))[0];
       }
    }

    my $alsotell = join(',',@alsotell);
    $alsotell ||= '';  # just to be picky
    print STDERR "Notify $service/$instance ($alsotell)\n" if ($Debug);

    if ($service eq "cpu")
      {
	# special case: don't warn about load if it's really
	# a swapping issue
	my ($swap_sev) = Cobalt::Monitor::mon_get_state("mem","swapping");
	return if ($swap_sev != $Status{'green'});
      }

    my ($condition,$cond_time,$update_time,$notified,$attempts,$reserved,$data) =
      Cobalt::Monitor::mon_get_state($service,$instance);
    print STDERR "Notify $service/$instance: $condition\n" if ($Debug);
    return if ($notified);

    my $subject = Cobalt::Repair::repair_get_subject($service,$instance);
    my $header = "From: $adminEmail\nTo: $AlertList\n";
    $header.="Cc: $alsotell\n" if ($alsotell);
    $header.="Subject: $subject\n\n";

    if ($Language{'japanese'}) {
       # convert header from Shift-JIS to JIS for sendmail
       my $tmpShiftJISFile="/tmp/header.sjis.$$";

       open( SHIFTJIS, ">$tmpShiftJISFile" );
       print SHIFTJIS $header;
       close( SHIFTJIS );

       open( JIS, "$ShiftJISToJIS_prog $tmpShiftJISFile|")
           || die "Can't run $ShiftJISToJIS_prog $tmpShiftJISFile: $!\n";
       $header=join '', <JIS>;
       close( JIS );

       # cleanup
       unlink( $tmpShiftJISFile );
    }

    my $advice = Cobalt::Repair::repair_get_message($service,$instance);
    print STDERR "Sending mail to $AlertList $alsotell ($condition)\n"
      if ($Debug);
    if (open (MAIL, "|$Sendmail -oi -t"))
      {
	print MAIL $header;
	print MAIL "$advice\n";
	close MAIL;
	if ($service eq "net")
	  {
	    # special case: set the notified flag for both collisions and col15
	    Cobalt::Monitor::mon_set_state($service,"collisions","-",1,"-","-","-");
	    Cobalt::Monitor::mon_set_state($service,"col15","-",1,"-","-","-");
	  }
	Cobalt::Monitor::mon_set_state($service,$instance,"-",1,"-","-","-");
	open(LOG,">>$MonDBDir/$service.log") ||
	  return "3051 Cannot write log file /var/cobalt/$service.log: $!";
	print LOG "<FONT COLOR=\"$Status{$condition}\">$service, ",
	Cobalt::Language::msg_localtime($^T),
	": Sending alert mail to $Adminuser $alsotell</FONT><!-- $instance ($data)-->\n";
	close LOG;
      }
    else
      {
	return "4705 Cannot send mail: $!";
      }
}


1;
