# This autoloader file was built automatically by 
# /usr/local/sbin/autoize from Services.pm

# Copyright 1998 Cobalt Networks (http://www.cobaltnet.com)

use strict;
### START HARDCODED PATHS
my $Lcdshowscript = "/etc/rc.d/init.d/showip";
my $Lockdir = "/etc/locks";
my $SNMPconf = "/etc/snmpd.conf";
my $SNMPscript = "/etc/rc.d/init.d/snmpd";
my $Sauce_log = "/var/cobalt/sauce.log";
my $Subsys = "/var/lock/subsys";
my %Features = ("networker","1","telnetConfig","1","dhcpd","0","shiftjis","Cobalt interface message for shiftjis (We should never see this)","pagebuilder","0","www","1","mx","1","gated","1","dns","1","nfs","0","securetty","1","english","1","atalk","0","fpx","1","siteAdminRedirect","1","users","1","ntp","0","mon_textdb_only","0","search","0","anonFTP","1","specialHomeSkel","0","viewGroups","0","cobaltHomePage","0","lastNameFirst","1","warnRootOnly","1","vsites","1","pam","1","discuss","0","docorg","0","groups","1","smb","0","mail","1","nat","0","squid","0","autoGenWebUsage","0","japanese","0","cobaltRedirect","0","backups","1","vmachine","1","mon_port_testing","1","viewUsers","0","usersCGIInWizard","1","privatedirs","0","telnetDefault","1","ssl","0","siteManagement","1","adm","1","bind8","1","personalRedirect","1","webRequests","1","usersAddUser","0","adminRedirect","1","quotas","1","snmp","1","defaultUserNameGen","1","groupdirs","1","proftpd","1");
my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = ($Debug ||= 0);
### END HARDCODED PATHS

# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/Services.pm.  Changes made here will be lost.
package Cobalt::Services;

sub set_snmp_community
# Sets the community with read and write access
# Arguments: read-only community, read/write community
#   If a community name is set to empty, then disable that community
# Return value: a status string of the form "666 Got a raise. Blow a hole in the account."
# Example: $ret = set_snmp_community("mypublic","");  (this disables the 
#   r/w community and sets the read-only community to "mypublic").
{
    return "1404 SNMP not implemented in this release"
	unless ($Features{'snmp'});

    my ( $read_community, $write_community )=@_;

    # The SNMP RFC just says that a community should be an "octet string";
    # but I know that would break the config file, so I'm going to
    # restrict to alphanumeric, underscore, dash, and dots until I find 
    # documenation to say otherwise.
    return "4790 Invalid SNMP read community $read_community"
      if ($read_community !~ /^[\w\.\-]*$/o);
    return "4791 Invalid SNMP write community $write_community"
      if ($write_community !~ /^[\w\.\-]*$/o);

    return "4472 /etc/snmpd.conf is locked" if ( -e "$Lockdir/snmp.conf" );
    open( SNMPLOCK, ">$Lockdir/snmp.conf" )
	|| return "4473 Cannot write SNMP config file /etc/locks/snmp.conf: $! ";
    open( SNMP, "$SNMPconf") 
	|| return "3477 Cannot read SNMP config file /etc/snmpd.conf: $! ";
    my $done=0;
    while( <SNMP> ) {
      if (/^[\#\s]*community\s+(\S+)\s+(xmini.+)/o ) {
	# Change the read-only community
	if ($read_community) {
	  $_ = "community\t$read_community\t$2\n";
	}
	elsif (/^c/o) {
	  # only comment it out if it isn't already
	  $_ = "#$_";
	}
      } 
      elsif (/^[\#\s]*community\s+(\S+)\s+(mini.+)/o) {
	# Change the read/write community (commented out by default)
	if ($write_community) {
	  $_ = "community\t$write_community\t$2\n";
	}
	elsif (/^c/o) {
	  # only comment it out if it isn't already
	  $_ = "#$_";
	}
      }
      elsif (/^trap community:/o) {
	# Trap community is not documented.  We don't want to just
	# leave it as "public" if we changed community names...
	my $trap_community;
	# first try setting it to the write community
	$trap_community = $write_community;
	# if write_community wasn't set, set it to the read community
	$trap_community ||= $read_community;
	# otherwise...well, set it to something.  No communities are
	# set, so we'll set it to the default of "public".
	$trap_community ||= "public";

	$_ = "trap community:\t$trap_community\n";
      }
      print SNMPLOCK;
    }
    close SNMPLOCK;
    close SNMP;
    rename( "$Lockdir/snmp.conf", $SNMPconf ) ||
	return "3478 Cannot update /etc/snmpd.conf: $!";
    chmod(0600, "$SNMPconf");

    # Start a child process to restart the SNMP agent
    my $pid;
    if( !defined( $pid=fork ) ) { die "cannot fork: $!"; }
    elsif( $pid ) {
	# i'm the parent
	print STDERR "forking a child to restart daemons\n" if ($Debug);
    }
    else {
	# otherwise I'm the child
	open( STDERR, ">>$Sauce_log" );
	open( STDOUT, ">>$Sauce_log" );
	# restart SNMP
	if( -f "$Subsys/snmpd" ) {
	    system( "$SNMPscript stop >>$Sauce_log 2>&1 " );
	    system( "$SNMPscript start >>$Sauce_log 2>&1 " );
	}
	system( "$Lcdshowscript start >>$Sauce_log 2>&1" )
	    if( -x $Lcdshowscript );
	exit;
    }

    # done!
    return "2116 Set the SNMP community strings successfully";
}


1;
