# This autoloader file was built automatically by 
# /usr/local/sbin/autoize from Syswatch.pm

# Copyright 1998 Cobalt Networks (http://www.cobaltnet.com)

use strict;
### START HARDCODED PATHS
my $MinTx = "6000";
my $Netif = "eth0";
my %Status = ("none","0","0","none","green","1","1","green","2","yellow","red","3","3","red","yellow","2");
my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = ($Debug ||= 0);
### END HARDCODED PATHS

# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/Syswatch.pm.  Changes made here will be lost.
package Cobalt::Syswatch;

sub collwatch
# Keep an eye on the network interface and report excessive collisions
# Monitors the number of collisions since system boot, and the number
#   over just the last 15 minutes
# Arguments: yellow threshold (%), red threshold (%), optional network
#   interface (defaults to primary)
{
    my ($yelp,$redp,$netif) = @_;
    my ($tx,$colls);
    $netif ||= $Netif;
    # there seems to be some inconsistency about the format of this
    # file.  Let's hope that it's okay.
    open(NS,"/proc/net/dev") || die "Cannot read /proc/net/dev: $!\n";
    while (<NS>)
    {
        if (/^\s*$netif:\s*([\d\s]+)/)
        {
            ($tx,$colls) = (split(/\s+/o,$1))[5,9];
	    last;
        }
    }
    close NS;

    # First do the current collision rate (since the last monitor run)
    my ($sev,$ctime,$status,$prevtx,$prevcols) 
	= db_prev_entry("net","col15");

    # If we rebooted within the last 15 minutes, our state will
    # be from the previous boot.  Ignore it and reset our counter.
    open(UPTIME,"/proc/uptime") || die "Cannot read /proc/uptime: $!\n";
    my $uptime = (split('\s',<UPTIME>))[0];
    close UPTIME;
    if (($tx < $prevtx) || ($uptime < 1000))
    {
	$prevtx = 0;
	$prevcols = 0;
    }

    my ($colpcent,$col15);  # overall, past 15 mins

    # What if there's been no activity at all?
    if (($tx - $prevtx) == 0)
    {
	# No activity == no collisions
	$col15 = 0;
	$colpcent = 0;
    }
    else
    {
	# There's been activity
	$col15 = int((($colls - $prevcols) / ($tx - $prevtx)) * 100);
	$colpcent = int(($colls / $tx) * 100);
    }

    if (($tx - $prevtx) < $MinTx)
    {
	db_add_entry("net",0,0,time(),"0",
		     " : $colls - $prevcols / $tx - $prevtx  [idle network]",
		     "col15",0,$tx,$colls,$col15);
    }
    elsif ($col15 >= $redp)
    {
	db_add_entry("net",$Status{"red"},0,time(),"130",
		     " : $colls - $prevcols / $tx - $prevtx  [$col15% collision rate]",
		     "col15",0,$tx,$colls,$col15);
    }
    elsif ($col15 >= $yelp)
    {
	db_add_entry("net",$Status{"yellow"},0,time(),"130",
		     " : $colls - $prevcols / $tx - $prevtx [$col15% collision rate]",
		     "col15",0,$tx,$colls,$col15);
    }
    else
    {
	db_add_entry("net",$Status{"green"},0,time(),"0",
		     " : $colls - $prevcols / $tx - $prevtx [$col15% collision rate]",
		     "col15",0,$tx,$colls,$col15);
    }

    if ($tx < $MinTx)
    {
	db_add_entry("net",0,0,time(),"130",
		     "int(($colls / $tx) * 100) [idle network]",
		     "collisions",0,$colpcent);
    }
    elsif ($colpcent >= $redp)
    {
	db_add_entry("net",$Status{"red"},0,time(),"130",
		     "int(($colls / $tx) * 100) [$colpcent% collision rate]",
		     "collisions",0,$colpcent);
    }
    elsif ($colpcent >= $yelp)
    {
	db_add_entry("net",$Status{"yellow"},0,time(),"130",
		     "int(($colls / $tx) * 100) [$colpcent% collision rate]",
		     "collisions",0,$colpcent);
    }
    else
    {
	db_add_entry("net",$Status{"green"},0,time(),0,
		     "int(($colls / $tx) * 100) [$colpcent% collision rate]",
		     "collisions",0,$colpcent);
    }
}

use Cobalt::Language;

1;
