# This autoloader file was built automatically by 
# /usr/local/sbin/autoize from Syswatch.pm

# Copyright 1998 Cobalt Networks (http://www.cobaltnet.com)

use strict;
### START HARDCODED PATHS
my %Status = ("none","0","0","none","green","1","1","green","2","yellow","red","3","3","red","yellow","2");
my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = ($Debug ||= 0);
### END HARDCODED PATHS

# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/Syswatch.pm.  Changes made here will be lost.
package Cobalt::Syswatch;

sub swapwatch
# Keep an eye on the memory state and report excessive swapping
# Arguments: yellow threshold, red threshold
#   thresholds are in pages per second (in or out)
# Side effects: updates the database
# Return value: none
{
    my ($yelp,$redp) = @_;
    my ($swapped,$sev,$ctime,$mtime,$swapness);
    open(MEM,"/proc/stat") || die "Cannot read /proc/stat: $!\n";
    while (<MEM>)
    {
	if (/^swap\s+(\d+)\s+(\d+)/o)
	{
	    $swapped = $1 + $2;
	}
    }
    close MEM;
    
    my ($sev,$ctime,$status,$mtime,$hadswapped) 
	= db_prev_entry("mem","swapping");

    # If we rebooted within the last 15 minutes, our state will
    # be from the previous boot.  Ignore it and reset our counter.
    open(UPTIME,"/proc/uptime") || die "Cannot read /proc/uptime: $!\n";
    my $uptime = (split('\s',<UPTIME>))[0];
    close UPTIME;
    $hadswapped = 0 if (($uptime - 1200) < 0); # 15 min + margin

    unless ($mtime && $hadswapped)
    {
	# no previous state
	db_add_entry("mem",$Status{"green"},0,time(),0,
		     "No previous state","swapping",0,time(),$swapped);
	return;
    }

    my $elapsed = time - $mtime;  $elapsed = 1 unless ($elapsed);
    my $elapsmin = int($elapsed / 60);
    $swapness = ($swapped - $hadswapped) / $elapsed;  # swaps per second
    $swapness = int($swapness * 100) / 100;   # get rid of excessive digits
    $swapness = 0 if ($swapness <= 0);

    if ($swapness >= $redp)
    {
	db_add_entry("mem",$Status{"red"},0,time(),150,
		     "$swapness - $hadswapped / $elapsed [$swapness swaps/second]",
		     "swapping",0,time(),$swapped,$swapness);
    }
    elsif ($swapness >= $yelp)
    {
	db_add_entry("mem",$Status{"yellow"},0,time(),150,
		     "$swapness - $hadswapped / $elapsed [$swapness swaps/second]",
		     "swapping",0,time(),$swapped,$swapness);
    }
    else
    {
	db_add_entry("mem",$Status{"green"},0,time(),0,
		     "$swapness - $hadswapped / $elapsed [$swapness swaps/second]",
		     "swapping",0,time(),$swapped,$swapness);
    }
}

use Cobalt::Language;

1;
