# This autoloader file was built automatically by 
# /usr/local/sbin/autoize from User.pm

# Copyright 1998 Cobalt Networks (http://www.cobaltnet.com)

use strict;
### START HARDCODED PATHS
my $Htpasswd = "/etc/htpasswd";
my $Htptmp = "/etc/locks/htptmp";
my $Lockdir = "/etc/locks";
my $Passwd = "/etc/passwd";
my $Ptmp = "/etc/locks/ptmp";
my $Smbpasswd = "/etc/smbpasswd";
my $Smbptmp = "/etc/locks/smbptmp";
my %Features = ("networker","1","telnetConfig","1","dhcpd","0","shiftjis","Cobalt interface message for shiftjis (We should never see this)","pagebuilder","0","www","1","mx","1","gated","1","dns","1","nfs","0","securetty","1","english","1","atalk","0","fpx","1","siteAdminRedirect","1","users","1","ntp","0","mon_textdb_only","0","search","0","anonFTP","1","specialHomeSkel","0","viewGroups","0","cobaltHomePage","0","lastNameFirst","1","warnRootOnly","1","vsites","1","pam","1","discuss","0","docorg","0","groups","1","smb","0","mail","1","nat","0","squid","0","autoGenWebUsage","0","japanese","0","cobaltRedirect","0","backups","1","vmachine","1","mon_port_testing","1","viewUsers","0","usersCGIInWizard","1","privatedirs","0","telnetDefault","1","ssl","0","siteManagement","1","adm","1","bind8","1","personalRedirect","1","webRequests","1","usersAddUser","0","adminRedirect","1","quotas","1","snmp","1","defaultUserNameGen","1","groupdirs","1","proftpd","1");
my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = ($Debug ||= 0);
### END HARDCODED PATHS

# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/User.pm.  Changes made here will be lost.
package Cobalt::User;

##############################

sub lock_passwd
# Lock the password file so that nobody can collide with us.
# Arguments: none
# Side effects: copies $Passwd to $Ptmp and $Htpasswd to $Htptmp
# Return value: message of the form "2671 All of the engineers prefer MIPS"
{
    if (-e $Ptmp)
    {
      return Cobalt::User::user_abort("4507 The password file is locked.");
    }
    else
    {
	# Lock the password file by creating the ptmp file as a
	# copy of the password file.
	open(PASSWD,$Passwd) || return "4508 Cannot lock /etc/passwd: $!";
	open(PTMP,">$Ptmp") 
	    || return "4509 Cannot lock /etc/locks/ptmp: $!";
	print PTMP while (<PASSWD>);
	close PTMP;
	close PASSWD;

	# Lock the smbpasswd file by creating the smbptmp file as a
	# copy of the password file.
	if ($Features{'smb'}) {
	  open(SMBPASSWD,$Smbpasswd) || return "4572 Cannot lock /etc/smbpasswd: $!";
	  open(SMBPTMP,">$Smbptmp") 
	    || return "4573 Cannot lock /etc/locks/smbptmp: $!";
	  print SMBPTMP while (<SMBPASSWD>);
	  close SMBPTMP;
	  close SMBPASSWD;
	}

	# Lock the netatalk passwd file by calling atalk_lock_passwd
	# (which does the same thing as the others above)
	if ($Features{'atalk'}) {
	  my $atalk_ret = Cobalt::Atalk::atalk_lock_passwd();
	  return $atalk_ret if ($atalk_ret !~ /^2/);
	}

	# we need to lock this now in case the next opens abort
	open(LOCK,">$Lockdir/user-$$") || return Cobalt::User::user_abort("4599 Cannot get user lock: $!");
	print LOCK "$$\n$0";
	close LOCK;

    	unless ($Features{'pam'}) {
	  print STDERR "Copying htpassword file $Htpasswd to $Htptmp\n" 
	    if ($Debug);
	  if (! -e $Htpasswd)
	  {
	    # this looks like a fresh system
	    print STDERR "Creating new htpassword file $Htpasswd\n" 
	      if ($Debug);
	    open(HTPASS,">$Htpasswd")
	      || return "4574 Cannot create /etc/htpasswd: $!";
	    close HTPASS;
	  }
	  Cobalt::Utility::sauce_cp($Htpasswd,$Htptmp);
	}
    }

    print STDERR "Locked $Ptmp\n" if ($Debug);
    return "2511 Password file locked";
}


1;
