# This autoloader file was built automatically by 
# /usr/local/sbin/autoize from Vhost.pm

# Copyright 1998 Cobalt Networks (http://www.cobaltnet.com)

use strict;
### START HARDCODED PATHS
my $Dns_notification = "/etc/dns.notification";
my $Htconf = "/etc/httpd/conf";
my $If_cfg_dir = "/etc/sysconfig/network-scripts";
my $Ifdown = "/etc/sysconfig/network-scripts/ifdown";
my $Lockdir = "/etc/locks";
my $Logrotate_boot = "/etc/logrotate.onboot";
my $Logrotate_dir = "/etc/logrotate.d";
my $Sauce_log = "/var/cobalt/sauce.log";
my $Weblog_dir = "/var/log/httpd";
my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = ($Debug ||= 0);
### END HARDCODED PATHS

# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/Vhost.pm.  Changes made here will be lost.
package Cobalt::Vhost;

sub vhost_del
# Remove an IP-based virtual interface
# This only applies to IP-based virtual sites
# Arguments: an interface name, the associated group name
# Side effects: removes the ifcfg-eth0:xx file, the /etc/logrotate.d file,
#   cleans httpd.conf
{
    my ($virtif,$group) = @_;
    unless (open(IFCFG,"$If_cfg_dir/ifcfg-$virtif"))
    {
	return "4788 Cannot read $If_cfg_dir/ifcfg-$virtif: $!";
    }
    my $addr;
    while (<IFCFG>)
    {
	if (/^IPADDR=(.+)/o)
	{
	    $addr = $1;
	    last;
	}
    }
    close IFCFG;
    # get the main network lock file (and the rest of our locks)
    return "4789 Cannot reconfigure network: system busy"
      unless(Cobalt::Network::net_lock_network());
    return "4790 Cannot remove interface: ifcfg-$virtif is locked"
	if (-e "$Lockdir/ifcfg-$virtif");
    open(O,">$Lockdir/ifcfg-$virtif")
	|| return "4791 Cannot lock interface config ifcfg-$virtif: $!";
    close O;
    if (-e "$Lockdir/httpd.conf")
    {
      return Cobalt::Network::net_abort("4149 $Htconf/httpd.conf is locked",
		       "$Lockdir/ifcfg-$virtif");
    }
    unless (open(HTCTMP,">$Lockdir/httpd.conf"))
    {
      return Cobalt::Network::net_abort("4152 Cannot write $Lockdir/httpd.conf: $!",
		       "$Lockdir/ifcfg-$virtif");
    }
    unless (open(HTCONF,"$Htconf/httpd.conf"))
    {
      return Cobalt::Network::net_abort("4153 Cannot read $Htconf/httpd.conf: $!",
		       "$Lockdir/ifcfg-$virtif",
		       "$Lockdir/httpd.conf");
    }
    while (<HTCONF>)
    {
	if (/^Listen $addr:80/o)
	{
	    next;
	}
	elsif (/^<VirtualHost $addr>/o)
	{
	    while (<HTCONF>)
	    {
		last if (/^<\/VirtualHost>/o);
	    }
	}
	elsif (/^\# VirtualHost label $virtif /) 
	{
	    next;
	}
	else
	{
	    print HTCTMP;
	}
    }
    close HTCONF;
    close HTCTMP;
    unless (rename("$Lockdir/httpd.conf","$Htconf/httpd.conf"))
    {
      return Cobalt::Network::net_abort("3149 Cannot update $Htconf/httpd.conf: $!",
		       "$Lockdir/ifcfg-$virtif",
		       "$Lockdir/httpd.conf");
    }

    unlink("$Logrotate_dir/$addr-apache");
    unlink("$Logrotate_boot/$addr-apache");
    system("$Ifdown $virtif >>$Sauce_log 2>&1");
    unlink("$If_cfg_dir/ifcfg-$virtif");
    unlink("$Lockdir/ifcfg-$virtif");

    if ($group)
    {
      # conditional for backwards compatibility Cobalt::Vhost::vhost_del(nic)
      unlink("$Weblog_dir/$group-access") 
	if (-e "$Weblog_dir/$group-access");
      unlink("$Weblog_dir/$group-error") 
	if (-e "$Weblog_dir/$group-error");
    }

    Cobalt::Network::net_unlock_network();

    if (-r $Dns_notification)
    {
	my $main_addr = (Cobalt::Network::net_get_ipaddr())[0];
        
	my($subject) = Cobalt::Language::msg_get('notify_vhost_del_subject');
        $subject =~ s/\$addr/$addr/; $subject =~ s/\$main_addr/$main_addr/; $subject =~ s/\\n/\n/g;
        
        my($msgbody) = Cobalt::Language::msg_get('notify_vhost_del_body');
        $msgbody =~ s/\$addr/$addr/; $msgbody =~ s/\$main_addr/$main_addr/; $msgbody =~ s/\\n/\n/g;
       
	Cobalt::Vhost::inform_BigBrother($subject,$msgbody);
    }

    return "2124 Virtual interface deleted successfully";
}


1;
