# This autoloader file was built automatically by 
# /usr/local/sbin/autoize from Vhost.pm

# Copyright 1998 Cobalt Networks (http://www.cobaltnet.com)

use strict;
### START HARDCODED PATHS
my $AdminServer_port = "81";
my $Dns_notification = "/etc/dns.notification";
my $Htconf = "/etc/httpd/conf";
my $Htgroup_dir = "/home/sites";
my $Htscript = "/etc/rc.d/init.d/httpd.init";
my $If_cfg_dir = "/etc/sysconfig/network-scripts";
my $Ifup = "/etc/sysconfig/network-scripts/ifup";
my $Lcdshowscript = "/etc/rc.d/init.d/showip";
my $Lockdir = "/etc/locks";
my $Logrotate_boot = "/etc/logrotate.onboot";
my $Logrotate_dir = "/etc/logrotate.d";
my $Netif = "eth0";
my $Sauce_log = "/var/cobalt/sauce.log";
my $Sites_Max = "50";
my $Subsys = "/var/lock/subsys";
my $Weblog_dir = "/var/log/httpd";
my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = ($Debug ||= 0);
### END HARDCODED PATHS

# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/Vhost.pm.  Changes made here will be lost.
package Cobalt::Vhost;

sub vhost_set
# Configure or create the settings for an IP-based virtual interface,
#   setting up the web server and the network interface.
# This is specifically for IP-based virtual sites.
# Arguments: the new IP address, netmask, fully qualified host name,
#   group name, optional physical interface name (defaults to eth0)
# Side effects: edits the files in /etc/sysconfig
#   Reconfigures the network interface for the new address
#   Adds entries to the httpd.conf to listen to the new address
# Return value: status string of the form "4346 You don't get a wish
#   because you don't have your own brain."
# Note: this doesn't handle the mail configuration aspects of setting
#   up a virtual host, only the network interface and the web server.
{
    my ($addr,$mask,$name,$group,$netif) = @_;
    my $cgis_enabled;

    $netif ||= $Netif;

    # Picky, but not worth returning an error
    $name =~ tr/A-Z/a-z/;

#    $addr =~ s/(\.|^)0+(\d)/$1$2/go;  # get rid of excess leading zeros
    # If we're given a weird IP and it's not via the web, wake the LCD
#    if ($addr !~ /^(\d+\.\d+\.\d+\.\d+)$/o)
    if( !Cobalt::Netutil::netutil_test_ipaddr( $addr ) )
    {
	print STDERR "Got invalid IP address $addr\n" if ($Debug);
	return "4781 $addr is not a valid IP address";
    }
    if (Cobalt::Network::net_network_ismember($addr,"224.0.0.0","240.0.0.0")) {
      # This is an address on the multicast network
      return "4782 $addr cannot be used as an IP address, as it is reserved for multicast use";
    }

    # Check that the hostname is valid
    if ($name =~ /^([\w\-\.]+)$/o)
    {
	$name = $1;  # untaint
    }
    else
    {
	return "4138 invalid hostname $name";
    }

    # There must be an existing group to associate with this interface
    unless (getgrnam($group))
    {
	return "4142 Error: Group does not exist";
    }

    my $broadcast = Cobalt::Network::net_get_broadcast($addr,$mask);
    my $network = Cobalt::Network::net_get_network($addr,$mask);
    my $main_addr = (Cobalt::Network::net_get_ipaddr())[0];
    
    # Are we modifying an existing interface or adding a new one?
    my ($virtif) = (Cobalt::Vhost::vhost_get_byaddr($addr))[0];
    print STDERR "Virtual interface is ``$virtif''\n" if ($Debug);
    unless ($virtif)
    {
      # This is a new interface
      my @siteslist = Cobalt::Vhost::vsite_list();
      return "4783 This Cobalt microserver is limited to $Sites_Max virtual sites for maximum performance."
	if (($#siteslist + 1) >= $Sites_Max);  # plus one to un-zero-index
      
	# Get the number for the next interface to add
	my $ifnum;
	opendir(IFDIR,$If_cfg_dir) || 
	    return "4784 Cannot read network script directory $If_cfg_dir: $!";
	my @ifcfgs = sort {$a <=> $b } (grep(/^ifcfg-$netif:\d+$/o,readdir(IFDIR)));
	closedir IFDIR;
	for ($ifnum = 0; $ifnum <= $#ifcfgs; $ifnum++)
	{
	    if ($ifcfgs[$ifnum] ne "ifcfg-$netif:$ifnum")
	    {
		$virtif = "$netif:$ifnum";
		last;
	    }
	}
	$virtif ||= "$netif:" . ($#ifcfgs + 1);
	print STDERR "Allocating new interface $virtif for $addr\n"
	  if ($Debug);
    }
    else {
      # this is an existing interface
      $cgis_enabled = Cobalt::Vhost::vsite_get_cgis($virtif);
    }

    print STDERR "New settings for $virtif: $addr/$mask $name $group\n"
	if ($Debug);
    # need the general lock NO MATTER WHAT
    return "4785 Cannot reconfigure network: system busy"
      unless(Cobalt::Network::net_lock_network());
    return Cobalt::Network::net_abort("4786 Cannot configure virtual interface: ifcfg-$netif is locked")
      if (-e "$Lockdir/ifcfg-$virtif");
    open(O,">$Lockdir/ifcfg-$virtif")
      || return Cobalt::Network::net_abort("4787 Cannot get lock for interface $netif: $!");

    # get all our locks at once
    if (-e "$Lockdir/httpd.conf")
    {
	return Cobalt::Network::net_abort("3140 $Htconf/httpd.conf is locked",
			 "$Lockdir/ifcfg-$virtif");
    }
    unless (open(HTCTMP,">$Lockdir/httpd.conf"))
    {
	return Cobalt::Network::net_abort("3141 Cannot write $Lockdir/httpd.conf: $!",
			 "$Lockdir/ifcfg-$virtif");
    }

    print O "DEVICE=$virtif\n";
    print O "IPADDR=$addr\n";
    print O "NETMASK=$mask\n";
    print O "NETWORK=$network\n";
    print O "BROADCAST=$broadcast\n";
    print O "ONBOOT=yes\nALIAS=yes\nBOOTPROTO=none\n";
    close O;

    my %listenaddrs;
    my $donv = 0;
    unless (open(HTCONF,"$Htconf/httpd.conf"))
    {
	return Cobalt::Network::net_abort("3142 Cannot read $Htconf/httpd.conf: $!",
			"$Lockdir/ifcfg-$virtif",
			"$Lockdir/httpd.conf");
    }
    while (<HTCONF>)
    {
	if (/^Listen ([^:]+)\:(\d+)/o)
	{
	    $listenaddrs{$1} = $2;
	    print HTCTMP;
	}
	elsif (/^<VirtualHost $addr>/)
	{
	    # There's already an entry for us, rewrite it
	    print STDERR "Fixing virtual host entry\n" if ($Debug);
	    print HTCTMP<<END;
# VirtualHost label $virtif Please do not edit or remove this comment
<VirtualHost $addr>
ServerName $name
ServerAdmin $group\@$name
DocumentRoot $Htgroup_dir/$group
CustomLog $Weblog_dir/$group-access "%h %l %u %t \\"%r\\" %s %b \\"%{Referer}i\\" \\"%{User-Agent}i\\""
ErrorLog $Weblog_dir/$group-error
Redirect /siteadmin http://$addr:$AdminServer_port/.cobalt/siteManage/$group
Redirect /personal http://$addr:$AdminServer_port/.cobalt/personal
END
;
            if ($cgis_enabled) {
               print "AddHandler cgi-wrapper .cgi\nAddHandler cgi-wrapper .pl\n";
            }
            else {
               print "#AddHandler cgi-wrapper .cgi\n#AddHandler cgi-wrapper .pl\n";
            }
            print "</VirtualHost>\n";
	    $donv = 1;
	    
	    # Skip past the rest of the entry
	    while (<HTCONF>)
	    {
		last if (/^<\/VirtualHost>/o);
	    }
	}
	else
	{
	    print HTCTMP;
	}
    }
    print HTCTMP "Listen ${main_addr}:80\n" unless $listenaddrs{$main_addr};
    print HTCTMP "Listen ${addr}:80\n" unless $listenaddrs{$addr};
    unless ($donv)
    {
	print STDERR "Adding new virtual host entry\n" if ($Debug);
	print HTCTMP<<END; 
# VirtualHost label $virtif Please do not edit or remove this comment
<VirtualHost $addr>
ServerName $name
ServerAdmin $group\@$name
DocumentRoot $Htgroup_dir/$group
CustomLog $Weblog_dir/$group-access "%h %l %u %t \\"%r\\" %s %b \\"%{Referer}i\\" \\"%{User-Agent}i\\""
ErrorLog $Weblog_dir/$group-error
Redirect /siteadmin http://$addr:$AdminServer_port/.cobalt/siteManage/$group
Redirect /personal http://$addr:$AdminServer_port/.cobalt/personal
#AddHandler cgi-wrapper .cgi
#AddHandler cgi-wrapper .pl
</VirtualHost>
END
    }
    close HTCONF;
    close HTCTMP;
    unless (rename("$Lockdir/httpd.conf","$Htconf/httpd.conf"))
    {
      return Cobalt::Network::net_abort("3143 Cannot update $Htconf/httpd.conf: $!",
		       "$Lockdir/ifcfg-$virtif",
		       "$Lockdir/httpd.conf");
    }

    unless (open(LOGROT,">$Logrotate_dir/$addr-apache"))
    {
      return Cobalt::Network::net_abort("3144 Cannot create $Logrotate_dir/$addr-apache: $!",
		       "$Lockdir/ifcfg-$virtif",
		       "$Lockdir/httpd.conf");
    }
    print LOGROT <<END;
$Weblog_dir/$group-access {
   compress
   size 30M
    postrotate
        killall -USR1 httpd
    endscript
    }

$Weblog_dir/$group-error {
    compress
    size 5M
    postrotate
        killall -USR1 httpd
    endscript
    }
END

    unless (open(LOGROT,">$Logrotate_boot/$addr-apache"))
    {
      return Cobalt::Network::net_abort("3146 Cannot create $Logrotate_boot/$addr-apache: $!",
		       "$Lockdir/ifcfg-$virtif",
		       "$Lockdir/httpd.conf");
    }
    print LOGROT <<END;
$Weblog_dir/$group-access {
   compress
   size 30M
    }

$Weblog_dir/$group-error {
    compress
    size 5M
    }
END

    # unlock
    unless (rename("$Lockdir/ifcfg-$virtif","$If_cfg_dir/ifcfg-$virtif"))
    {
	return Cobalt::Network::net_abort("3057 Cannot update virtual interface config file: $!",
			 "$Lockdir/ifcfg-$virtif");	 
    }

    # Bring the interface up
    print STDERR "$Ifup $virtif\n" if ($Debug);
    system("$Ifup $virtif >>$Sauce_log 2>&1\n");

    # Restart the web server, if it's up
    if (-f "$Subsys/httpd")
    {
	system("$Htscript stop >>$Sauce_log 2>&1");
	system("$Htscript start >>$Sauce_log 2>&1");
	system("$Lcdshowscript") if (-x $Lcdshowscript);
    }

    Cobalt::Network::net_unlock_network();

    if (-r $Dns_notification)
    {
        my($subject) = Cobalt::Language::msg_get('notify_vhost_add_subject');
        $subject =~ s/\$addr/$addr/; $subject =~ s/\$main_addr/$main_addr/; $subject =~ s/\\n/\n/g;
        my($msgbody) = Cobalt::Language::msg_get('notify_vhost_add_body');
        $msgbody =~ s/\$addr/$addr/; $msgbody =~ s/\$main_addr/$main_addr/; $msgbody =~ s/\\n/\n/g;
        Cobalt::Vhost::inform_BigBrother($subject,$msgbody);
    }

    return "2123 Virtual interface for address $addr has been set up";
}


1;
