# This autoloader file was built automatically by 
# /usr/local/sbin/autoize from Vhost.pm

# Copyright 1998 Cobalt Networks (http://www.cobaltnet.com)

use strict;
### START HARDCODED PATHS
my $AdminServer_port = "81";
my $Htconf = "/etc/httpd/conf";
my $Htgroup_dir = "/home/sites";
my $Lockdir = "/etc/locks";
my $Sites_Max = "50";
my $Weblog_dir = "/var/log/httpd";
my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = ($Debug ||= 0);
### END HARDCODED PATHS

# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/Vhost.pm.  Changes made here will be lost.
package Cobalt::Vhost;

sub vname_add
# Adds a name-based virtual web server
# This function is specifically for name-based virtual sites
# Arguments: the IP address, server name, associated group name, admin email
# Return value: string of the form "3310 High Anxiety"
# Side effects: Adds NameVirtualHost entry to the conf file and <VirtualHost>
#    block
{
	my ($addr,$name,$group,$serveradmin) = @_;
	my($found_interface,$found_namev,$new_conf);

	# read in the current configuration to see if this virtual
	# site already exists and if the interface is there
	open(HTCNF,"$Htconf/httpd.conf") || 
	  return "4587 Cannot open web configuration";
        while(<HTCNF>)
        {
		$found_interface = 1 if (/VirtualHost $addr>/);
		$found_namev = 1 if (/NameVirtualHost $addr$/);
		$new_conf .= $_;
        }
        close(HTCNF);

	if ($found_interface) 
	{
	  $new_conf .= <<EOF
		  
# VirtualHost label name Please do not edit or remove this comment
<VirtualHost $addr>
ServerName $name
ServerAdmin $serveradmin
DocumentRoot $Htgroup_dir/$group
ErrorLog $Weblog_dir/$group-error
CustomLog $Weblog_dir/$group-access "%h %l %u %t \\"%r\\" %s %b \\"%{Referer}i\\" \\"%{User-Agent}i\\""
Redirect /siteadmin http://$addr:$AdminServer_port/.cobalt/siteManage/$group
Redirect /personal http://$addr:$AdminServer_port/.cobalt/personal
#AddHandler cgi-wrapper .cgi
#AddHandler cgi-wrapper .pl
</VirtualHost>
EOF
	} else {
		return "4595 IP address is not in use.  Please create an IP-bound server.";
	}

	if (!$found_namev) 
	{
	  # This is a wholly new site
	  my @siteslist = Cobalt::Vhost::vsite_list();
	  return "4597 This Cobalt microserver is limited to $Sites_Max virtual sites for maximum performance."
	    if (($#siteslist + 1) >= $Sites_Max); # +1 to un-zero-index
	  $new_conf =~ s/(Listen $addr:80)/NameVirtualHost $addr\n$1/;
	}

	# Now get our locks
	return "4377 Cannot reconfigure network: system busy"
	  unless(Cobalt::Network::net_lock_network());
	return Cobalt::Network::net_abort("4588 Web configuration in use, please try again")
	  if (-e "$Lockdir/httpd.conf");
	open(HTLOCK,"> $Lockdir/httpd.conf") || 
	  return Cobalt::Network::net_abort("4593 Cannot update the web server configuration file");
	print HTLOCK $new_conf;	
	close(HTLOCK);

	rename("$Lockdir/httpd.conf","$Htconf/httpd.conf") ||
	  return Cobalt::Network::net_abort("4589 Cannot update the web server configuration file",
			  "Lockdir/httpd.conf");
	Cobalt::Network::net_unlock_network();
	return "2660 Name-based virtual server added successfully"; 
}


1;
