# This autoloader file was built automatically by 
# /usr/local/sbin/autoize from Vhost.pm

# Copyright 1998 Cobalt Networks (http://www.cobaltnet.com)

use strict;
### START HARDCODED PATHS
my $Htconf = "/etc/httpd/conf";
my $Htgroup_dir = "/home/sites";
my $If_cfg_dir = "/etc/sysconfig/network-scripts";
my $MainGroup = "home";
my $Netif = "eth0";
my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = ($Debug ||= 0);
### END HARDCODED PATHS

# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/Vhost.pm.  Changes made here will be lost.
package Cobalt::Vhost;

sub vsite_get
# Get the information about an virtual site
# This function works for either IP-based or name-based sites
# Arguments: Interface name (eth0:4 or whatever) or site name
# Return value: IP address, netmask, fully qualified host name, group,
#     webmaster email
#   Returns 0 if the given site doesn't exist
# Side effects: reads the ifcfg-NAME and httpd.conf files
# Example: ($addr,$mask,$host,$group,$admin) = Cobalt::Vhost::vsite_get("eth1:3");
{
    my ($virtif) = @_;
    my ($addr,$mask,$host,$group,$admin_email);

    if ($virtif eq $Netif) # Special case for eth0 compatibility
    {
	my $nul;
	($addr,$mask,$nul) = Cobalt::Network::net_get_ipaddr();
	$host = Cobalt::Network::net_get_hostname();
        my $domain = (net_get_dnsclient())[0];
        $host .= '.' . $domain if ($domain);
	$group = $MainGroup;
	$admin_email = ''; # Not yet supported

	return($addr,$mask,$host,$group,$admin_email); 
    }

    my $inentry = 0;
    open(HTCONF,"$Htconf/httpd.conf") 
	|| return "4147 Cannot read $Htconf/httpd.conf: $!";
    while (<HTCONF>)
    {
        if (/^\# VirtualHost label\s*(name)?\s*([a-z\:\d\.]+) /o)
	{
	  my $site = $2;
	  $inentry = 1 if ($site eq $virtif);
	}
	elsif (($inentry) && (/^<VirtualHost ([\d\.]+)>/o))
	{
	    $addr = $1;
	    print STDERR "Found address for $addr\n" if ($Debug);
	}
	elsif (($inentry) && (/^ServerAdmin ([\w\@\-\.]+)/o))
	{
	    $admin_email = $1;
	    print STDERR "Found admin email is $admin_email\n" if ($Debug);
	}
	elsif (($inentry) && (/^ServerName (.+)/o))
	{
	    $host = $1;
	    print STDERR "Found host entry for $addr: $host\n" if ($Debug);
	}
	elsif (($inentry) && (/^DocumentRoot $Htgroup_dir\/(.+)/o))
	{
	    $group = $1;
	    close HTCONF;
	    last;
	}
	elsif (/^<\/VirtualHost/o)
	{
	    $inentry = 0;  # end of entry
	}
    }

    return 0 unless ($addr);

    if (-e "$If_cfg_dir/ifcfg-$virtif") {
      # IP-based
      print STDERR "Getting information about interface $virtif\n" if ($Debug);
      open(IFCFG,"$If_cfg_dir/ifcfg-$virtif")
	|| return "4146 Cannot read interface config ifcfg-$virtif: $!";
      while (<IFCFG>) {
	if (/^IPADDR=(.+)/o)     { $addr = $1; }
	elsif (/^NETMASK=(.+)/o) { $mask = $1; }
      }
      close IFCFG;
    }
    else {
      # name-based
      $mask = "N/A";   # mask isn't important
    }

    return ($addr,$mask,$host,$group,$admin_email);
}


1;
