# This autoloader file was built automatically by 
# /usr/local/sbin/autoize from Vhost.pm

# Copyright 1998 Cobalt Networks (http://www.cobaltnet.com)

use strict;
### START HARDCODED PATHS
my $Htconf = "/etc/httpd/conf";
my $Htscript = "/etc/rc.d/init.d/httpd.init";
my $Lcdshowscript = "/etc/rc.d/init.d/showip";
my $Lockdir = "/etc/locks";
my $MainGroup = "home";
my $Netif = "eth0";
my $Sauce_log = "/var/cobalt/sauce.log";
my $Subsys = "/var/lock/subsys";
my $Userdir = "/home/sites";
my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = ($Debug ||= 0);
### END HARDCODED PATHS

# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/Vhost.pm.  Changes made here will be lost.
package Cobalt::Vhost;

sub vsite_set_cgis
# Enable or disable the use of CGIs for a virtual site
# This affects the entire site
# Arguments: site name, 1 to enable/0 to disable
# Return value: status string of the form "8750 I can't hear you, 
#   there's a bannana in my ear!"
# Side effects: modifies the virtual host entry in httpd.conf.
# Example: Cobalt::Vhost::vsite_set_cgis("eth0:0",1);
{
  my ($vsite,$cgis_enabled) = @_;
  my $inentry = 0;
  my $on;

  # verify and grab our locks
  return "4792 Cannot reconfigure network: system busy"
    unless(Cobalt::Network::net_lock_network());
  if (-e "$Lockdir/httpd.conf")
  {
    return Cobalt::Network::net_abort("4793 $Htconf/httpd.conf is locked");
  }
  # special case for the primary network interface
  if (($vsite eq $Netif) || ($vsite eq $MainGroup)) {
    unless (open(HTCTMP,">$Lockdir/access.conf"))
    {
      return Cobalt::Network::net_abort("4794 Cannot write $Lockdir/access.conf: $!",
		       "$Lockdir/access.conf");       
    }

    unless (open(HTCONF,"$Htconf/access.conf")) {
      return "4795 Cannot read $Htconf/access.conf: $!";
    }
    while (<HTCONF>) {
      if ($inentry == 2)
      {
	# already did our work; let's not do pattern matching we don't
	# have to do.
	print HTCTMP;
      }
      elsif (/^<Directory $Userdir\/$MainGroup>/o)
      {
	$inentry = 1;
	print HTCTMP;
	print STDERR "vsite_set_cgis found $vsite\n" if ($Debug);
      }
      elsif ($inentry && (/^(\#)?AddHandler cgi-wrapper /o))
      {
	# clear the existing AddHandler entries 
	print STDERR "vsite_set_cgis found AddHandler\n" if ($Debug);
      }
      elsif ($inentry && (/^<\/Directory/o))
      {
	# Put in the correct new AddHandler entries
	if ($cgis_enabled) {
	  print HTCTMP "AddHandler cgi-wrapper .cgi\n";
	  print HTCTMP "AddHandler cgi-wrapper .pl\n";
	  $on = Cobalt::Language::msg_get('On_string');
	}
	else {
	  print HTCTMP "#AddHandler cgi-wrapper .cgi\n";
	  print HTCTMP "#AddHandler cgi-wrapper .pl\n";
	  $on = Cobalt::Language::msg_get('Off_string');
	}
	$inentry = 2;  # end of entry flag
	print HTCTMP;
      }
      else
      {
	print HTCTMP;
      }
    }
    unless ($inentry) {
      # Haven't seen it, add it to the end
      print HTCTMP "<Directory $Userdir/$MainGroup>\n";
      print HTCTMP "Options Indexes SymLinksIfOwnerMatch\n";
      print HTCTMP "AllowOverride AuthConfig Limit\n";
      if ($cgis_enabled) {
	print HTCTMP "AddHandler cgi-wrapper .cgi\n";
	print HTCTMP "AddHandler cgi-wrapper .pl\n";
	$on = Cobalt::Language::msg_get('On_string');
      }
      else {
	print HTCTMP "#AddHandler cgi-wrapper .cgi\n";
	print HTCTMP "#AddHandler cgi-wrapper .pl\n";
	$on = Cobalt::Language::msg_get('Off_string');
      }
      $inentry = 2;  # end of entry flag
      print HTCTMP "</Directory>\n";
    }
    close HTCONF;
    close HTCTMP;

    # move the temp file in place
    unless (rename("$Lockdir/access.conf","$Htconf/access.conf"))
    {
      return Cobalt::Network::net_abort("4796 Cannot update $Htconf/access.conf: $!",
		       "$Lockdir/access.conf");       
    }
  }
  else {
    # General virtual host case
    unless (open(HTCTMP,">$Lockdir/httpd.conf"))
    {
      return Cobalt::Network::net_abort("4797 Cannot write $Lockdir/httpd.conf: $!",
		       "$Lockdir/httpd.conf");       
    }

    unless (open(HTCONF,"$Htconf/httpd.conf"))  {
      return Cobalt::Network::net_abort("4798 Cannot read $Htconf/httpd.conf: $!",
		       "$Lockdir/httpd.conf");       
    }
    
    # fix the httpd.conf file
    while (<HTCONF>) {
      if ($inentry == 2) {
	# already did our work; let's not do pattern matching we don't
	# have to do.
	print HTCTMP;
      }
      elsif (/^\# VirtualHost label\s*(name)?\s*([a-z\:\d\.]+) /o)
      {
	$inentry = 1 if ($2 eq $vsite);
	print HTCTMP;
	print STDERR "vsite_set_cgis found $vsite\n" if ($Debug);
      }
      elsif ($inentry && (/^(\#)?AddHandler cgi-wrapper /o))
      {
	# clear the existing AddHandler entries 
	print STDERR "vsite_set_cgis found AddHandler\n" if ($Debug);
      }
      elsif ($inentry && (/^<\/VirtualHost/o))
      {
	# Put in the correct new AddHandler entries
	if ($cgis_enabled) {
	  print HTCTMP "AddHandler cgi-wrapper .cgi\n";
	  print HTCTMP "AddHandler cgi-wrapper .pl\n";
	  $on = Cobalt::Language::msg_get('On_string');
	}
	else {
	  print HTCTMP "#AddHandler cgi-wrapper .cgi\n";
	  print HTCTMP "#AddHandler cgi-wrapper .pl\n";
	  $on = Cobalt::Language::msg_get('Off_string');
	}
	$inentry = 2;  # end of entry flag
	print HTCTMP;
      }
      else
      {
	print HTCTMP;
      }
    }
    close HTCONF;
    close HTCTMP;

    unless ($inentry) {
      # never found the virtual host
      return Cobalt::Network::net_abort("4799 No virtual host entry for $vsite",
		       "$Lockdir/httpd.conf"); 
    }

    # move the temp file in place
    unless (rename("$Lockdir/httpd.conf","$Htconf/httpd.conf"))
    {
      return Cobalt::Network::net_abort("4800 Cannot update $Htconf/httpd.conf: $!",
		       "$Lockdir/httpd.conf");
    }
  }
  # Restart the web server, if it's up
  if (-f "$Subsys/httpd")
  {
    print STDERR "Restarting web server" if ($Debug);
    system("$Htscript stop >>$Sauce_log 2>&1");
    system("$Htscript start >>$Sauce_log 2>&1");
    system("$Lcdshowscript") if (-x $Lcdshowscript);
  }

  Cobalt::Network::net_unlock_network();
  return "2144 CGI use on this virtual site is now $on";
}


1;
