# This autoloader file was built automatically by 
# /usr/local/sbin/autoize from Vhost.pm

# Copyright 1998 Cobalt Networks (http://www.cobaltnet.com)

use strict;
### START HARDCODED PATHS
my $Htconf = "/etc/httpd/conf";
my $Htscript = "/etc/rc.d/init.d/httpd.init";
my $Lcdshowscript = "/etc/rc.d/init.d/showip";
my $Lockdir = "/etc/locks";
my $MainGroup = "home";
my $Netif = "eth0";
my $Sauce_log = "/var/cobalt/sauce.log";
my $Subsys = "/var/lock/subsys";
my $Userdir = "/home/sites";
my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = ($Debug ||= 0);
### END HARDCODED PATHS

# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/Vhost.pm.  Changes made here will be lost.
package Cobalt::Vhost;

sub vsite_set_ssi
# Enable or disable the use of SSIs for a virtual site
# This affects the entire site
# Arguments: site group name, 1 to enable/0 to disable
# Return value: status string of the form "4400 not going to happen in java"
# Side effects: modifies the virtual host entry in httpd.conf
#               or the main site in access.conf
# Example: Cobalt::Vhost::vsite_set_ssi("home",1);
{
  my ($vsite,$ssi_enabled) = @_;
  my $inentry = 0;
  my $on;

  # verify and grab our locks
  return "4387 Cannot reconfigure network: system busy"
    unless(Cobalt::Network::net_lock_network());
  if (-e "$Lockdir/httpd.conf")
  {
    return Cobalt::Network::net_abort("4165 $Htconf/httpd.conf is locked");
  }
  # special case for the primary network interface
  if (($vsite eq $Netif) || ($vsite eq $MainGroup)) {
    unless (open(HTCTMP,">$Lockdir/access.conf"))
    {
      return Cobalt::Network::net_abort("4164 Cannot write $Lockdir/access.conf: $!",
                       "$Lockdir/access.conf");
    }

    unless (open(HTCONF,"$Htconf/access.conf")) {
      return "4168 Cannot read $Htconf/access.conf: $!";
    }
    while (<HTCONF>) {
      if ($inentry == 2)
      {
        # already did our work; let's not do pattern matching we don't
        # have to do.
        print HTCTMP;
      }
      elsif (/^<Directory $Userdir\/$MainGroup>/o)
      {
        $inentry = 1;
        print HTCTMP;
        print STDERR "vsite_set_ssi found $vsite\n" if ($Debug);
      }
      elsif ($inentry && (/^(\#)?AddHandler server-parsed /o))
      {
        # clear the existing AddHandler entries
        print STDERR "vsite_set_ssi found AddHandler .shtml\n" if ($Debug);
      }
      elsif ($inentry && (/^(\#)?AddType\s+text\/html\s+.shtml/o))
      {
        # clear the existing AddHandler entries
        print STDERR "vsite_set_ssi found AddType .shtml\n" if ($Debug);
      }
      elsif ($inentry && (/^<\/Directory/o))
      {
        # Put in the correct new AddHandler entries
        if ($ssi_enabled) {
          print HTCTMP "AddHandler server-parsed .shtml\n";
          print HTCTMP "AddType    text/html     .shtml\n";
          $on = Cobalt::Language::msg_get('On_string');
        }
        else {
          print HTCTMP "#AddHandler server-parsed .shtml\n";
          print HTCTMP "#AddType    text/html     .shtml\n";
          $on = Cobalt::Language::msg_get('Off_string');
        }
        $inentry = 2;  # end of entry flag
        print HTCTMP;
      }
      else
      {
        print HTCTMP;
      }
    }
    unless ($inentry) {
      # Haven't seen it, add it to the end
      print HTCTMP "<Directory $Userdir/$MainGroup>\n";
      print HTCTMP "Options Includes Indexes SymLinksIfOwnerMatch\n";
      print HTCTMP "AllowOverride All\n";
      if ($ssi_enabled) {
        print HTCTMP "AddHandler server-parsed .shtml\n";
        print HTCTMP "AddType    text/html     .shtml\n";
        $on = Cobalt::Language::msg_get('On_string');
      }
      else {
        print HTCTMP "#AddHandler server-parsed .shtml\n";
        print HTCTMP "#AddType    text/html     .shtml\n";
        $on = Cobalt::Language::msg_get('Off_string');
      }
      $inentry = 2;  # end of entry flag
      print HTCTMP "</Directory>\n";
    }
    close HTCONF;
    close HTCTMP;

    # move the temp file in place
    unless (rename("$Lockdir/access.conf","$Htconf/access.conf"))
    {
      return Cobalt::Network::net_abort("4170 Cannot update $Htconf/access.conf: $!",
                       "$Lockdir/access.conf");
    }
  }
  else {
    # General virtual host case
    unless (open(HTCTMP,">$Lockdir/httpd.conf"))
    {
      return Cobalt::Network::net_abort("4171 Cannot write $Lockdir/httpd.conf: $!",
                       "$Lockdir/httpd.conf");
    }

    unless (open(HTCONF,"$Htconf/httpd.conf"))  {
      return Cobalt::Network::net_abort("4163 Cannot read $Htconf/httpd.conf: $!",
                       "$Lockdir/httpd.conf");
    }

    # fix the httpd.conf file
    while (<HTCONF>) {
      if ($inentry == 2) {
        # already did our work; let's not do pattern matching we don't
        # have to do.
        print HTCTMP;
      }
      elsif (/^DocumentRoot $Userdir\/$vsite\s*$/o) # Sensitive to vsite GROUP name
      {
        $inentry = 1;
        print HTCTMP;
        print STDERR "vsite_set_ssi found $vsite\n" if ($Debug);
      }
      elsif ($inentry && (/^(\#)?AddHandler server-parsed .shtml/o))
      {
        # clear the existing AddHandler .shtml entries
        print STDERR "vsite_set_ssi found AddHandler .shtml\n" if ($Debug);
      }
      elsif ($inentry && (/^(\#)?AddType\s+text\/html\s+.shtml/o))
      {
        # clear the existing AddType .shtml entries
        print STDERR "vsite_set_ssi found AddType .shtml\n" if ($Debug);
      }
      elsif ($inentry && (/^<\/VirtualHost/o))
      { 
        # Put in the correct new AddHandler entries
        if ($ssi_enabled) {
          print HTCTMP "AddHandler server-parsed .shtml\n";
          print HTCTMP "AddType    text/html     .shtml\n";
          $on = Cobalt::Language::msg_get('On_string');
        }
        else {
          print HTCTMP "#AddHandler server-parsed .shtml\n";
          print HTCTMP "#AddType    text/html     .shtml\n";
          $on = Cobalt::Language::msg_get('Off_string');
        }
        $inentry = 2;  # end of entry flag
        print HTCTMP;
      }
      else
      {
        print HTCTMP;
      }
    }
    close HTCONF;
    close HTCTMP;

    unless ($inentry) {
      # never found the virtual host
      return Cobalt::Network::net_abort("4167 No virtual host entry for $vsite",
                       "$Lockdir/httpd.conf");
    }

    # move the temp file in place
    unless (rename("$Lockdir/httpd.conf","$Htconf/httpd.conf"))
    {
      return Cobalt::Network::net_abort("4162 Cannot update $Htconf/httpd.conf: $!",
                       "$Lockdir/httpd.conf");
    }
  }
  # Restart the web server, if it's up
  if (-f "$Subsys/httpd")
  {
    print STDERR "Restarting web server" if ($Debug);
    system("$Htscript stop >>$Sauce_log 2>&1");
    system("$Htscript start >>$Sauce_log 2>&1");
    system("$Lcdshowscript") if (-x $Lcdshowscript);
  }

  Cobalt::Network::net_unlock_network();
  return "2125 Site SSI permissions are now $on";
}


1;
