# This autoloader file was built automatically by 
# /usr/local/sbin/autoize from Vsite.pm

# Copyright 1998 Cobalt Networks (http://www.cobaltnet.com)

use strict;
### START HARDCODED PATHS
my $AdminPages_dir = "/usr/admserv/html/.cobalt";
my $MainGroup = "home";
my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = ($Debug ||= 0);
### END HARDCODED PATHS

# NOTE: Derived from /usr/lib/perl5/site_perl/Cobalt/Vsite.pm.  Changes made here will be lost.
package Cobalt::Vsite;

sub vsite_sync
# Updates virtualhost configuration to match non-virtual network configuration
# Critical for Netif-bound web url rewrites, sharing the Netif ip address
# Arguments: none
# Return value: true for success, false for failure
{
    # Obtain stale configuration to remove symlinks, etc..
    my($origip,$orighost,$origgroup,$webmaster) = vsite_get_bygroup($MainGroup);

    # Fetch old site configuration, to be re-added
    my($fpx_state)=fpx_get_web($MainGroup);
    my(@fpx_webs)=vsite_get_fpx($MainGroup) if ($fpx_state);
    my($site_ssi)=vsite_get_ssi($MainGroup);
    my($site_cgi)=vsite_get_cgis($MainGroup);
    my($ftpAnon,$ftpMaxUsers,$ftpQuota)=ftp_get_anonymous($MainGroup);

    vsite_del($MainGroup) if ($origip); # Return value insignificant

    # Symlink management to facilitate /siteadmin/ redirects
    unlink("$AdminPages_dir/siteManage/$orighost")
	if (-l "$AdminPages_dir/siteManage/$orighost");
    unlink("$AdminPages_dir/siteManage/$origip")
	if (-l "$AdminPages_dir/siteManage/$origip");
    
    # Determine our real net settings
    my $hostname = Cobalt::Network::net_get_hostname();

    # The first time the system is booted it has no hostname; the name
    # defaults to "localhost".  If the main web server is configured as
    # a virtual host, and you try to access the system by name
    # before the system knows its own name, the web server returns a
    # "unknown host" error.  So, in the special case that we don't have
    # a real hostname, we don't add the virtual host entry.
    return 1 if ($hostname eq "localhost");

    my $domainname = (Cobalt::Network::net_get_dnsclient())[0];
    my $fqdn = "$hostname.$domainname";

    my($ip,$netmask,$mac)=Cobalt::Network::net_get_ipaddr();

    # Migrate virtual sites
    if ($ip ne $origip)
    {
      # Delete the superflous virtual network interface that results from
      # changing a real network interface's IP address to one that was in
      # use as a virtual network interface.
      my(@all_sites)=vsite_list();
      my($site_instance);
      foreach $site_instance (@all_sites)
      {
        net_del_vinterface( $ip ) if ( @$site_instance[0] eq $ip );
        # We ignore the return value as $ip could already be in use by a
        # real network interface
      }
      vsite_migrate_ipaddr($origip,$ip); 
    }

    # Re-enable the Netif's vsite
    my($return)=vsite_add($ip,$fqdn,$MainGroup,$webmaster,$netmask);
    return 0 if ($return !~ /^2/);

    # Restore site services (httpd configuration only)
    # Do not return on a failed service configuration, move on to the next
    vsite_set_fpx($MainGroup,1,@fpx_webs) if ($fpx_state);
    vsite_set_ssi($MainGroup,1) if ($site_ssi);
    vsite_set_cgis($MainGroup,1) if ($site_cgi);
    my($return)=ftp_config($ip,1,$ftpAnon,$MainGroup,$ftpMaxUsers,$ftpQuota);

    # Recreate symlinks
    symlink("$AdminPages_dir/siteManage/$MainGroup",
	    "$AdminPages_dir/siteManage/$fqdn");
    symlink("$AdminPages_dir/siteManage/$MainGroup",
	    "$AdminPages_dir/siteManage/$ip");

    Cobalt::Http::http_restart();

    return 0 if ($return !~ /^2/);
    return 1;
}


1;
